#include "AttributeList/AttributeListSpecification.h"
#include "AttributeList/AttributeExceptions.h"
#include <sstream>

pool::AttributeListSpecification::AttributeListSpecification() {}

pool::AttributeListSpecification::~AttributeListSpecification() {}

void pool::AttributeListSpecification::push_back(std::string attr_name, pool::typename_string_t type_name)
{
  if( index(attr_name) != pool::IDX_NOT_FOUND) 
    throw pool::attribute_duplicate(attr_name);

  m_index.push_back(pool::AttributeSpecification(attr_name, type_name));

  m_specmap[attr_name] = m_index.size()-1;
}


void
pool::AttributeListSpecification::push_back(std::string attr_name, const std::type_info& typeInfo )
{
  if( index(attr_name) != pool::IDX_NOT_FOUND) 
    throw pool::attribute_duplicate(attr_name);

  m_index.push_back(pool::AttributeSpecification(attr_name, typeInfo));

  m_specmap[attr_name] = m_index.size()-1;
}


const pool::AttributeSpecification& pool::AttributeListSpecification::operator[](std::string attr_name) const
{
  std::map<std::string, size_t>::const_iterator it = m_specmap.find(attr_name);

  if(it == m_specmap.end()) 
    throw pool::attribute_not_found(attr_name);
  else
    return m_index[it->second];

}

const pool::AttributeSpecification& pool::AttributeListSpecification::operator[](size_t i) const
{
  if( i >= m_index.size() ) {
    std::ostringstream os;
    os << i;
    throw pool::attribute_not_found( os.str() );
  }
  else
    return m_index[i];
}

size_t pool::AttributeListSpecification::index(std::string attr_name) const
{
  std::map<std::string, size_t>::const_iterator it = m_specmap.find(attr_name);

  return (it == m_specmap.end()) ? pool::IDX_NOT_FOUND : it->second;
}

size_t pool::AttributeListSpecification::size() const
{
  return m_specmap.size();
}

pool::AttributeListSpecification::const_ordered_index_iterator pool::AttributeListSpecification::ordered_index_begin() const
{
  return m_specmap.begin();
}

pool::AttributeListSpecification::const_ordered_index_iterator pool::AttributeListSpecification::ordered_index_end() const
{
  return m_specmap.end();
}

pool::AttributeListSpecification::ordered_index_iterator pool::AttributeListSpecification::ordered_index_begin()
{
  return m_specmap.begin();
}

pool::AttributeListSpecification::ordered_index_iterator pool::AttributeListSpecification::ordered_index_end()
{
  return m_specmap.end();
}

pool::AttributeListSpecification::iterator pool::AttributeListSpecification::begin()
{
  return m_index.begin();
}

pool::AttributeListSpecification::iterator pool::AttributeListSpecification::end()
{
  return m_index.end();
}

pool::AttributeListSpecification::const_iterator pool::AttributeListSpecification::begin() const
{
  return m_index.begin();
}

pool::AttributeListSpecification::const_iterator pool::AttributeListSpecification::end() const
{
  return m_index.end();
}

bool pool::AttributeListSpecification::equals(const AttributeListSpecification& rhs) const
{
  return (*this) == rhs;
}

bool pool::AttributeListSpecification::operator != (const AttributeListSpecification& rhs) const
{
  return !((*this) == rhs);
}

bool pool::AttributeListSpecification::operator == (const AttributeListSpecification& rhs) const
{
  if(size() != rhs.size()) return false;

  try
  {
    for(const_ordered_index_iterator it = ordered_index_begin(); it != ordered_index_end(); ++it)
      if(m_index[it->second] != rhs[it->first]) return false;
  }
  catch(const pool::attribute_not_found& /* x */)
  {
    return false;
  }

  return true;
}

bool pool::AttributeListSpecification::includes(const AttributeListSpecification& rhs) const
{
  // elements may not occur more than once so this test is enough
  if(size() < rhs.size()) return false;

  try
  {
    for(const_ordered_index_iterator it = rhs.ordered_index_begin(); it != rhs.ordered_index_end(); ++it)
      if((*this)[it->first] != rhs.m_index[it->second]) return false;
  }
  catch(const pool::attribute_not_found& /* x */)
  {
    return false;
  }

  return true;
}

pool::AttributeListSpecification pool::AttributeListSpecification::append_and_merge(const AttributeListSpecification& rhs) const
{
  AttributeListSpecification r(*this);

  for(const_iterator it = rhs.begin(); it != rhs.end(); ++it)
  {
    size_t idx = index(it->name());

    if(idx == pool::IDX_NOT_FOUND) 
      r.push_back(it->name(), it->type_name());
    else
      // it is enough to compare types here but using == operator does no harm
      if(*it != (*this)[idx])
        throw pool::attribute_duplicate(it->name());
      else
        ; // merge the attribute -> leave original one and do not append anything
  }
  return r;
}

void  pool::AttributeListSpecification::print(std::ostream& os) const
{
  bool first = true;

  for(const_iterator it=begin(); it!=end(); ++it)
  {
    if(!first) 
      os << ", ";
    it->print(os);
    first = false;

  }
}
