# include "AttributeList/AttributeList.h"
# include "AttributeList/AttributeListSpecification.h"
# include "AttributeList/AttributeExceptions.h"

# include "AttributeList/AttributePredefinedTypes.h" 

# include "AttributeList/AttributeValueAccessor.h"

# include <iostream>

pool::AttributeList* make_list()
{
  std::vector< std::pair<std::string, pool::typename_string_t> > columns;

  columns.push_back( std::make_pair("A","int"));
  columns.push_back( std::make_pair("B","int"));
  columns.push_back( std::make_pair("X", "string"));

  // -- create attribute specification

  boost::shared_ptr<pool::AttributeListSpecification> spec_ptr(new pool::AttributeListSpecification());

  for(unsigned int i=0; i<columns.size(); ++i)
    spec_ptr->push_back( columns[i].first, columns[i].second );


  pool::AttributeList * alist = new  pool::AttributeList(spec_ptr);

  return alist;
}
      

int use_list(pool::AttributeList& alist)
{
  try
    {  
      // -- create attribute list for the specification

      alist["A"].setValue(10);
      alist["B"].setValue(20);
      alist["X"].setValue(std::string("xxx"));
      
      alist.print(std::cout); std::cout << std::endl;

      pool::AttributeBaseValue &av = pool::AttributeValueAccessor(alist["A"]);

      std::cout << "current value = "; av.print(std::cout);

      int *Aptr = reinterpret_cast<int*>(av.getMemoryAddress());

      *Aptr = 20;

      alist.print(std::cout); std::cout << std::endl;

      std::cout << std::endl << "OK" << std::endl;

    }
  catch(pool::attribute_bad_type x)
    {
      std::cerr << "Type mismatch" << std::endl << x.what() << std::endl;
    }
  catch(pool::attribute_not_found x)
    {
      std::cerr << "Not found" << std::endl << x.what() << std::endl;
    }
  
  return 0;
}

int main()
{
  pool::AttributeList *alist = make_list();

  int x = use_list(*alist);

  delete alist;

  return x;
}
