#include "AttributeList/AttributeListSpecification.h"
#include "StorageSvc/DbReflex.h"
#include "POOLCore/Token.h"

#include "DataSvc/IDataSvc.h"
#include "Reflex/Base.h"


template< class T > inline
pool::Collection<T>::Collection( pool::IDataSvc* dataSvc,
				 pool::ICollection* icoll ):
      m_proxy( dataSvc, icoll )
{}


template< class T > inline
pool::Collection<T>::Collection( const pool::CollectionProxy& proxy ):
      m_proxy( proxy )
{}



template< class T > inline bool
pool::Collection<T>::add( const Ref<T>& pref,
			  const AttributeList& attributeList )
{
   return m_proxy.add( pref, attributeList);
}


//----------------------------- Iterator

template< class T > inline
pool::Collection<T>::Iterator::Iterator():
  m_thisClassType( ROOT::Reflex::Type::ByTypeInfo( typeid(T) ) ),
  m_ref(),
  m_proxy()
{}


template< class T > inline
pool::Collection<T>::Iterator::Iterator( const CollectionProxy::Iterator& iter ) :
  m_thisClassType( ROOT::Reflex::Type::ByTypeInfo( typeid(T) ) ),
  m_ref( iter.dataSvc() ),
  m_proxy( iter )
{}


template< class T > inline
pool::Collection<T>::Iterator::Iterator( const Iterator& rhs ):
  m_thisClassType( rhs.m_thisClassType ),
  m_ref( rhs.m_proxy.dataSvc() ),
  m_proxy( rhs.m_proxy )
{}


template< class T > inline const T&
pool::Collection<T>::Iterator::operator*() const {
  return m_ref.operator*();
}

template< class T > inline T&
pool::Collection<T>::Iterator::operator*() {
  return m_ref.operator*();
}

template< class T > inline const T*
pool::Collection<T>::Iterator::operator->() const {
  return m_ref.operator->();
}

template< class T > inline T*
pool::Collection<T>::Iterator::operator->() {
  return m_ref.operator->();
}


template< class T > inline
pool::Ref<T>&
pool::Collection<T>::Iterator::ref() {
   return m_ref;
}

template< class T > inline const
pool::Ref<T>&
pool::Collection<T>::Iterator::ref() const {
   return m_ref;
}


template< class T > inline bool
pool::Collection<T>::Iterator::next()
{
   while( m_proxy.next() ) {
      if( m_proxy.template isCurrentObjectOfType< T >() ) {
	 m_ref = m_proxy.template ref< T >();
	 return true;
      }
   }
   return false;
}


template< class T > inline 
const pool::AttributeList&
pool::Collection<T>::Iterator::attributeList() const
{
   return m_proxy.attributeList();
}


template< class T > inline bool
pool::Collection<T>::Iterator::isValid() const {
   return m_proxy.isValid();
}
