#include <string>
#include <iostream>
#include <exception>

#include "POOLCore/Exception.h"
#include "Collection/Collection.h"
#include "Collection/CollectionProxy.h"
#include "Collection/CollectionSvc.h"

#include "SealBase/ShellEnvironment.h"

#include "FileCatalog/IFileCatalog.h"
#include "FileCatalog/IFCAction.h"

using namespace std;
using namespace pool;
using namespace seal;

int main( int, char**){
  
  try {
    
    const string catalogURI = "xmlcatalog_file:CollectionCatalog.xml";
    
    const string collType = "RootCollection";
    const string collName = "Collection1";
    const string collConnection = "./";
    
    MetaDataEntry* metaData = 0;
    const string logicalName = "LogicalName1";
    const string physicalName = collType+"|"+collConnection+"|"+collName;;
    FileCatalog::FileID guid;
    
    const bool authorizeReplace = true;
    //const bool readOnly = true;
    
    IDataSvc* dataSvc = 0;
    
    ShellEnvironment shellEnv;
    shellEnv.set("POOL_COLLECTION_WRITE_CATALOG", catalogURI);
    
    
    {
      cout << "Create and register Collection<T> ...";
      Collection<double> collection = 
        CollectionSvc::createAndRegister<double> ( dataSvc,
                                                   collType,
                                                   collConnection,
                                                   collName,
                                                   logicalName,
                                                   metaData,
                                                   authorizeReplace);
      collection.attributeListSpecification() = AttributeListSpecification();
      cout << (collection.open() ? "OK":"KO") << endl;
    }
    
    
    {
      cout << "Create and register CollectionProxy ...";
      CollectionProxy collection = 
        CollectionSvc::createAndRegister( dataSvc,
                                          collType,
                                          collConnection,
                                          collName,
                                          logicalName,
                                          metaData,
                                          authorizeReplace); 
      collection.attributeListSpecification() = AttributeListSpecification();
      cout << (collection.open() ? "OK":"KO") << endl;
    }
    
    
    
    {
      cout << "Create and register CollectionProxy the crude way ...";
      CollectionProxy collection( dataSvc,
                                  "PHYSICAL_NAME",	// aka collection type
                                  logicalName,		// aka collection connection
                                  physicalName,		// aka collection name
                                  ICollection::CREATE_AND_OVERWRITE );
      collection.attributeListSpecification() = AttributeListSpecification();
      cout << (collection.open() ? "OK":"KO") << endl;
      collection.commit();
    }
    
    
    
    { // retrieve the guid ...
      IFileCatalog catalog;
      catalog.addReadCatalog(catalogURI);
      catalog.connect();
      IFCAction act;
      catalog.setAction(act);
      catalog.start();
      act.lookupFileByLFN( logicalName, guid);
      catalog.commit();
      catalog.disconnect();
    }
    
    
    
    {
       cout << endl << "Open Collection<T> with physical name ...";
      Collection<double> collection = CollectionSvc::openWithPhysicalName<double> ( dataSvc, physicalName );
      cout << (collection.open() ? "OK":"KO") << endl << endl;
    }
    { 
      cout << "Open Collection<T> with logical name ...";
      Collection<double> collection = CollectionSvc::openWithLogicalName<double> ( dataSvc, logicalName );
      cout << (collection.open() ? "OK":"KO") << endl<< endl;
    } 
    { 
      cout << "Open Collection<T> with guid ...";
      Collection<double> collection = CollectionSvc::openWithGuid<double> ( dataSvc, guid );
      cout << (collection.open() ? "OK":"KO") << endl<< endl;
    }
    

    
    {
      cout << "Open CollectionProxy with physical name ...";
      CollectionProxy collection = CollectionSvc::openWithPhysicalName( dataSvc, physicalName );
      cout << (collection.open() ? "OK":"KO") << endl;
    } 
    { 
      cout << "Open CollectionProxy with logical name ...";
      CollectionProxy collection = CollectionSvc::openWithLogicalName( dataSvc, logicalName );
      cout << (collection.open() ? "OK":"KO") << endl;
    } 
    { 
      cout << "Open CollectionProxy with guid ...";
      CollectionProxy collection = CollectionSvc::openWithGuid( dataSvc, guid );
      cout << (collection.open() ? "OK":"KO") << endl;
    }
    
    
    
    {
      cout << "Open CollectionProxy with physical name the crude way ...";
      CollectionProxy collection( dataSvc,
                                  "PHYSICAL_NAME",
                                  "",
                                  physicalName,
                                  ICollection::READ);
      cout << (collection.open() ? "OK":"KO") << endl;
    }
    
    {
      cout << "Open CollectionProxy with logical name the crude way ...";
      CollectionProxy collection( dataSvc,
                                  "LOGICAL_NAME",
                                  "",
                                  logicalName,
                                  ICollection::READ);
      cout << (collection.open() ? "OK":"KO") << endl; 
    }
    
    {
      cout << "Open CollectionProxy guid name the crude way ...";
      CollectionProxy collection( dataSvc,
                                  "GUID",
                                  "",
                                  guid,
                                  ICollection::READ);
      cout << (collection.open() ? "OK":"KO") << endl; 
    }
  }
  catch ( std::exception& e ) {
    std::cerr << e.what() << std::endl;
    return 1;
  }
}
