#include <string>
#include <iostream>
#include <exception>

#include "POOLCore/Token.h"
#include "AttributeList/AttributeList.h"

#include "FileCatalog/IFileCatalog.h"
#include "FileCatalog/IFCContainer.h"
#include "FileCatalog/IFCAction.h"
#include "FileCatalog/FCException.h"

#include "CollectionBase/ICollection.h"
#include "CollectionBase/ICollectionFactory.h"

#include "SealBase/ShellEnvironment.h"
#include "POOLCore/POOLContext.h"
#include <stdlib.h>

using namespace std;
using namespace pool;
using namespace seal;

int main(int argc, char** ){
  
  POOLContext::setMessageVerbosityLevel( seal::Msg::Fatal );
  
  
  bool crude = argc>1;
  
  const string collConnection = "./";
  const string collType = "RootCollection";
  MetaDataEntry* collMetaData = 0;
  
  Token* token = new Token();
  token->fromString("[DB=AEC1DFE2-010B-D811-9832-000347F31C25]"
                    "[CNT=TestContainer]"
                    "[CLID=12345678-1234-42AF-9CB6-567A94781A78]"
                    "[TECH=00000202]"
                    "[OID=00000003-00000002]");
  
  AttributeList attList;
  ShellEnvironment shellEnv;
  ICollectionFactory* factory = ICollectionFactory::get();
  
  try{
    
    cout << "Create 3 collections and register them in 3 separate catalogs" << endl;
    
    for(int i=0; i<3 ; ++i){
      char catalogName[200];
      sprintf( catalogName, "xmlcatalog_file:CollectionCatalog%i.xml", i);
      shellEnv.set( "POOL_COLLECTION_WRITE_CATALOG", catalogName);
      
      // reset the default catalog of the factory
      factory->setDefaultCatalog(0);
      
      char collName[200];
      sprintf( collName, "Collection%i", i); 
      
      char logicalName[200];
      sprintf( logicalName, "logicalName%i",i);
      
      bool authorizeReplace = true;
      
      ICollection* collection = 0;
      if(crude){
        string physicalName = collType+"|"+collConnection+"|"+collName; 
        // factory->create( type, session, connection, name, openMode);
        collection = factory->create( "PHYSICAL_NAME",
                                      0,		
                                      logicalName,	
                                      physicalName,
                                      ICollection::CREATE_AND_OVERWRITE);
      }else
        collection = factory->createAndRegister( collType,
                                                 collConnection,
                                                 collName,
                                                 logicalName,
                                                 collMetaData,
                                                 authorizeReplace );
      
      collection->add(*token,attList);
      collection->commit();
      delete collection;
    }
    
    factory->setDefaultCatalog(0);
    
    // retrieve the guid of collection 1 and register a replica for it.
    FileCatalog::FileID guid;   // FileCatalog::FileID is a std::string
    {
      string logicalName = "logicalName1";
      string physicalName = "RootCollection|./|Collection1";
      IFileCatalog catalog;
      catalog.setWriteCatalog("xmlcatalog_file:CollectionCatalog1.xml");
      catalog.connect();
      FCregister act;
      catalog.setAction(act);
      catalog.start();
      act.lookupFileByLFN( logicalName, guid);
      act.addReplicaPFN( physicalName, physicalName+"REPLICA");
      catalog.commit();
      catalog.disconnect();
    }
    
    shellEnv.set( "POOL_COLLECTION_WRITE_CATALOG", "xmlcatalog_file:CollectionCatalog0.xml");
    shellEnv.set( "POOL_COLLECTION_READ_CATALOGS", "xmlcatalog_file:CollectionCatalog1.xml xmlcatalog_file:CollectionCatalog2.xml");
    
    ICollection* collection = 0;
    
    
    
    cout << "Open collection 0 with physical name .... ";
    if(crude)
      collection = factory->create("PHYSICAL_NAME", 0, "", "RootCollection|./|Collection0", ICollection::READ);
    else
      collection = factory->openWithPhysicalName("RootCollection|./|Collection0");
    cout << (collection ? "OK":"KO") << endl;
    
    
    
    cout << "Open collection 1 with guid .... ";
    if(crude)
      collection = factory->create("GUID", 0, "", guid, ICollection::READ);
    else
      collection = factory->openWithGuid(guid);
    cout << (collection ? "OK":"KO") << endl;
    
    
    
    cout << "Open collection 2 with logical name .... ";
    if(crude)
      collection = factory->create("LOGICAL_NAME", 0, "", "logicalName2", ICollection::READ);
    else
      collection = factory->openWithLogicalName("logicalName2");
    cout << (collection ? "OK":"KO") << endl;
    
    
 
    cout << "Try to create the existing collection 0 ... ";
    try{
      if(crude)
        collection = factory->create("PHYSICAL_NAME",
                                     0,
                                     "",
                                     "RootCollection|./|Collection0",
                                     ICollection::CREATE);
      else
        collection = factory->createAndRegister( collType,
                                                 collConnection,
                                                 string("Collection0"));
      cout << "ERROR: this shouldn't work" << endl;
    }catch( std::exception&){
      cout << "OK, exception has been thrown" << endl;
    }
    
    
    
    cout << "Try to replace the replicated collection 1 ... ";
    try{
      if(crude)
        collection = factory->create("PHYSICAL_NAME",
                                     0,
                                     "",
                                     "RootCollection|./|Collection1",
                                     ICollection::CREATE_AND_OVERWRITE);
      else
        collection = factory->createAndRegister( collType,
                                                 collConnection,
                                                 "Collection1",	// collection name
                                                 "",		// logical name
                                                 collMetaData,
                                                 true);		// replace authorization
      cout << "ERROR: this shouldn't work" << endl;
    }catch( std::exception&){
      cout << "OK, exception has been thrown" << endl;
    }
    
    
    
    cout << "Try to replace unique collection 0 ... ";
    try{
      if(crude)
        collection = factory->create("PHYSICAL_NAME",
                                     0,
                                     "",
                                     "RootCollection|./|Collection0",
                                     ICollection::CREATE_AND_OVERWRITE);
      else
        collection = factory->createAndRegister( collType,
                                                 collConnection,
                                                 "Collection0",	// collection name
                                                 "",		// logical name
                                                 collMetaData,
                                                 true);	       	// replace authorization
      cout << "OK, this works" << endl;
    }catch( std::exception& e){
      cout << "ERROR, this should work" << endl;
      std::cout << "Exception: "<<e.what()<<std::endl;
    }
    
    
    
    cout << "Try to update the replicated collection 1 ... ";
    try{
      if(crude)
        collection = factory->create("LOGICAL_NAME",
                                     0,
                                     "",
                                     "logicalName1",
                                     ICollection::UPDATE);
      else
        collection = factory->openWithLogicalName( "logicalName1", // logical name 
                                                   false );        // readOnly
      cout << "ERROR: this shouldn't work" << endl;
    }catch( std::exception&){
      cout << "OK, exception has been thrown" << endl;
    }
    
    
    
    cout << "Try to update unique collection 0 ... ";
    try{
      if(crude)
        collection = factory->create("LOGICAL_NAME",
                                     0,
                                     "",
                                     "logicalName0",
                                     ICollection::UPDATE);
      else
        collection = factory->openWithLogicalName( "logicalName0", // logical name 
                                                   false );        // readOnly
      cout << "OK it works" << endl;
    }catch( std::exception&){
      cout << "ERROR, this should work" << endl;
    }
    
    
  }catch( std::exception& e ){
    std::cerr << "Exception: " << e.what() << std::endl;
  }
  token->release();
}
