#define POOL_CACHE_CPP 1

#include "Cache.h"
#include "CacheEntryImp.h"

pool::Cache::Cache() {
  // create a new Cache table
  m_table = new CacheTable();
}

pool::Cache::Cache(const ObjectDeletePolicy& aPolicy) {
  // create a new Cache table
  m_table = new CacheTable();
  m_deletePolicy = aPolicy;
}

pool::Cache::~Cache() {
  delete m_table;
}

pool::CacheEntryImp* pool::Cache::getEntry(const SharedAnyPtr& anObjPtr) {
  CacheEntryImp* theCacheEntry = m_table->getEntry(anObjPtr.get());
  if(!theCacheEntry){
    if(anObjPtr.get()){
      theCacheEntry = new CacheEntryImp(anObjPtr,m_deletePolicy.onCache());
      m_table->insert(theCacheEntry);
      m_table->setEntry(anObjPtr.get(),theCacheEntry);
    }
  }
  return theCacheEntry;
}

pool::CacheEntryImp* pool::Cache::getEntry(const Token& aToken) {
  CacheEntryImp* theCacheEntry = m_table->getEntry(aToken);
  if(!theCacheEntry){
    theCacheEntry = new CacheEntryImp(aToken,m_deletePolicy.onCache());
    m_table->insert(theCacheEntry);
    m_table->setEntry(aToken,theCacheEntry);
  }
  return theCacheEntry;
}

bool pool::Cache::findEntry(const CacheEntryImp* aCacheEntry){
  bool ret = false;
  if(m_table){
    ret = m_table->find(aCacheEntry);
  }
  return ret;
}

bool pool::Cache::addEntry(CacheEntryImp* aCacheEntry) {
  if(findEntry(aCacheEntry)) return false;
  bool ret = false;
  if(aCacheEntry){
    ret = m_table->insert(aCacheEntry);
    m_table->setEntry(aCacheEntry->object().get(),aCacheEntry);
    const Token *tok = aCacheEntry->token();
    if(tok){
      m_table->setEntry(*tok,aCacheEntry);
    }
  }
  return ret;
}

bool pool::Cache::updateEntry(CacheEntryImp* aCacheEntry,
                                const SharedAnyPtr& anObjPtr){ 
  bool ret = false;
  if(aCacheEntry){
    if(!m_table->find(aCacheEntry)){
      return false;
    }
    removeObject(aCacheEntry);
    aCacheEntry->setObject(anObjPtr);
    ret =  m_table->setEntry(anObjPtr.get(),aCacheEntry);
  }
  return ret;
}

bool pool::Cache::updateEntry(CacheEntryImp* aCacheEntry,const Token& aToken) {
  bool ret = false;
  if(aCacheEntry){
    if(!m_table->find(aCacheEntry)){
      return false;
    }
    removeToken(aCacheEntry);
    aCacheEntry->setToken(&aToken);
    ret =  m_table->setEntry(aToken,aCacheEntry);
  }
  return ret;
}		                         
		                         
bool pool::Cache::removeObject(CacheEntryImp* aCacheEntry) {
  bool ret = false;
  if(aCacheEntry){
    if(!m_table->find(aCacheEntry)){
      return false;
    }
    if(!aCacheEntry->isMarkedWrite()){
      ret = m_table->unsetEntry(aCacheEntry->object().get());
      aCacheEntry->resetObject();
    }
  }
  return ret;
}

bool pool::Cache::deleteObject(CacheEntryImp* aCacheEntry) {
  bool ret = false;
  if(aCacheEntry){
    if(!m_table->find(aCacheEntry)){
      return false;
    }
    if(!aCacheEntry->isMarkedWrite()){
      ret = m_table->unsetEntry(aCacheEntry->object().get());
      aCacheEntry->deleteObject();
    }
  }
  return ret;
}

bool pool::Cache::removeToken(CacheEntryImp* aCacheEntry) {
  bool ret = false;
  if(aCacheEntry){
    if(!m_table->find(aCacheEntry)){
      return false;
    } 
    const Token *tok = aCacheEntry->token();
    if(tok){ 
      ret = m_table->unsetEntry(*tok);
      aCacheEntry->setToken(0);
    }
  }
  return ret;
}

bool pool::Cache::removeEntry(CacheEntryImp* aCacheEntry) {
  bool ret = false;
  if(aCacheEntry){
    ret = m_table->remove(aCacheEntry);
    if(ret){    
      const Token *tok = aCacheEntry->token();
      if(tok){ 
        m_table->unsetEntry(*tok);
      }
      m_table->unsetEntry(aCacheEntry->object().get());
      delete aCacheEntry;
    }
  }
  return ret;
}

pool::Cache::const_list_iterator pool::Cache::list_begin(){
  return m_table->list_begin();
}

pool::Cache::const_list_iterator pool::Cache::list_end(){
  return m_table->list_end();
}

pool::Cache::const_ptr_iterator pool::Cache::ptr_iter_begin(){
  return m_table->ptr_iter_begin();
}

pool::Cache::const_ptr_iterator pool::Cache::ptr_iter_end(){
  return m_table->ptr_iter_end();
}

pool::Cache::const_token_iterator pool::Cache::token_iter_begin(){
  return m_table->token_iter_begin();
}

pool::Cache::const_token_iterator pool::Cache::token_iter_end(){
  return m_table->token_iter_end();
}

void pool::Cache::clearTable() {
  if(m_table)
    m_table->clear();
}

unsigned int pool::Cache::cacheSize(){
  return m_table->size();
}





