// $Header: /cvs/POOL/pool/DataSvc/src/DataSvc.h,v 1.31 2003/11/27 10:52:43 govi Exp $
#ifndef POOL_DATASVC_H
#define POOL_DATASVC_H

// Include files
#include "DataSvc/IDataSvc.h"

namespace pool    {

  class IPersistencySvc;
  class ITokenValidator;
  class ITransactionObserver;
  class DataSvcContext;
  class ObjectDeletePolicy;

  /** @class DataSvc DataSvc.h src/DataSvc.h
      DataSvc is the abstract interface for retrieving/registering 
      persitable objects using the storage services.

      @author Giacomo Govi
      @date 15/07/2002
  */
  class DataSvc : public pool::IDataSvc {

    public:

      /** constructor with an already created PersistencySvc
      */
      DataSvc(IPersistencySvc* aPersSvc);

      DataSvc(IFileCatalog*,ITokenValidator*,
              IDataTransform*,const ObjectDeletePolicy&);

      DataSvc(const DataSvcContext&);

      /// Standard destructor
      virtual ~DataSvc();

      /// Returns a reference to the IDataTransform object.
      IDataTransform& dataTransform();

      /// Return a referenc to the IFileCatalog instance in use.
      IFileCatalog& fileCatalog();

      /// Returns the underlying global session
      ISession& session();

      /// Returns the current transaction
      ITransaction& transaction();

      // Clear the cache content 
      void clearCache();

      // Reset the cache content (delete only pointers) 
      void resetCache();

      // Returns a reference to the ICacheSvc object.
      ICacheSvc& cacheSvc();

    private:

      IPersistencySvc* m_persSvc;

      ICacheSvc* m_cacheSvc;

      ITransactionObserver* m_transactionObserver;

      bool m_owned;

      void _setPersistencySvc(IFileCatalog* aFileCatalog,
                              ITokenValidator* aTokenValidator,
                              IDataTransform* dataTransform);
      DataSvc(){};

  };   // class DataSvc
}      // namespace pool

#endif  // POOL_DATASVC_H











