#define POOL_DATASVCFACTORY_CPP 1

#include "DataSvc/DataSvcFactory.h"
#include "DataSvc/IDataSvcFactory.h"
#include "DataSvc.h"
#include "DataSvc/DataSvcException.h"
#include "DataSvc/DataSvcContext.h"

pool::IDataSvc* pool::DataSvcFactory::instance(IPersistencySvc* aPersSvc){
  return new DataSvc(aPersSvc);
}

pool::IDataSvc* pool::DataSvcFactory::instance(IFileCatalog* aFileCatalog,
                                               ITokenValidator* aTokenValidator,
                                               IDataTransform* aDataTransform,
                                               ObjectDeletePolicy aDeletePolicy){
  IDataSvc* svc = 0;
  pool::IDataSvcFactory* psfactory = pool::IDataSvcFactory::get();
  if ( ! psfactory ) {
    throw DataSvcException("Could not retrieve an IDataSvc factory");
  }
  DataSvcContext ctx;
  ctx.setFileCatalog(aFileCatalog);
  ctx.setTokenValidator(aTokenValidator);
  ctx.setDataTransform(aDataTransform);
  ctx.setObjectDeletePolicy(aDeletePolicy);
  svc = psfactory->create( "DataSvc" ,ctx);
  return svc;
}

pool::IDataSvc* pool::DataSvcFactory::instance(const DataSvcContext& aCtx){
  IDataSvc* svc = 0;
  pool::IDataSvcFactory* psfactory = pool::IDataSvcFactory::get();
  if ( ! psfactory ) {
    throw DataSvcException("Could not retrieve an IDataSvc factory");
  }
  svc = psfactory->create( "DataSvc" ,aCtx);
  return svc;
}

pool::IDataSvc* pool::DataSvcFactory::create(IPersistencySvc* aPersSvc){
  return new DataSvc(aPersSvc);
}

pool::IDataSvc* pool::DataSvcFactory::create(IFileCatalog* aFileCatalog,
                                               ITokenValidator* aTokenValidator,
                                               IDataTransform* aDataTransform,
                                               ObjectDeletePolicy aDeletePolicy){
  IDataSvc* svc = 0;
  pool::IDataSvcFactory* psfactory = pool::IDataSvcFactory::get();
  if ( ! psfactory ) {
    throw DataSvcException("Could not retrieve an IDataSvc factory");
  }

  DataSvcContext ctx;
  ctx.setFileCatalog(aFileCatalog);
  ctx.setTokenValidator(aTokenValidator);
  ctx.setDataTransform(aDataTransform);
  ctx.setObjectDeletePolicy(aDeletePolicy);
  svc = psfactory->create( "DataSvc" ,ctx);
  return svc;
}

pool::IDataSvc* pool::DataSvcFactory::create(const DataSvcContext& aCtx){
  IDataSvc* svc = 0;
  pool::IDataSvcFactory* psfactory = pool::IDataSvcFactory::get();
  if ( ! psfactory ) {
    throw DataSvcException("Could not retrieve an IDataSvc factory");
  }
  svc = psfactory->create( "DataSvc" ,aCtx);
  return svc;
}
