// $Header: /cvs/POOL/pool/DataSvc/src/Hmap_ktoken_table.h,v 1.14 2003/10/29 17:18:54 govi Exp $
#ifndef POOL_HMAP_KTOKEN_TABLE_H
#define POOL_HMAP_KTOKEN_TABLE_H 
   
#include "Hmap_k_table.h"
#include "POOLCore/Token.h"
#include <sstream>
  
namespace pool  {

  /** @struct KeyPtr Hmap_ktoken_table.h
      Dummy struct implemented to encapsulate the token to be used as a 
      key for the search in the table.
      
      @author Giacomo Govi
      @date 25/07/2002
  **/
  struct KeyTok {
      
    const Token& theToken;

    // constructor
    KeyTok(const Token& aToken) : theToken(aToken) {
    }
    /// Copy constructor
    KeyTok(const KeyTok& copy) : theToken(copy.theToken) {
    }
    ~KeyTok() {
    }

     bool operator==(const KeyTok &kr) const {     
       return theToken == kr.theToken;    
     }
     bool operator<(const KeyTok &kr) const {     
       return theToken < kr.theToken;    
     }

    std::string toString() const {
      std::ostringstream os;
      os << "!" << theToken.oid().second
         << "!" << theToken.contID()
         << "!" << theToken.dbID()
         << "!" << theToken.classID().toString();
      return os.str();
    }

  };

}

namespace EXTSTD {

#ifndef __ICC

#ifdef __GNUC__
  template <> struct hash<std::string> {
    unsigned int operator()(const std::string& s) const { 
      return chash(s.c_str());
    }
  private: 
    hash<const char *> chash;
  };
#endif

   // Standard __gnu_cxx hash mechanism.
  template <> struct hash<pool::KeyTok>  {
    unsigned int operator()(const pool::KeyTok& kt) const {
      return hash<std::string>().operator()(kt.toString());
    }
  };

  // Hash functions for Windows
  inline unsigned int hash_value(const pool::KeyTok& kt)  {
    return hash<pool::KeyTok>().operator()(kt);
  }

#else
  // ICC (dinkumware) hash mechanism.
  template <> struct less<pool::KeyTok> {
    bool operator()(const pool::KeyTok& k1, const pool::KeyTok& k2) const {
      return k1<k2;
    }
  };  
 
  template <> class hash_compare<pool::KeyTok,less<pool::KeyTok> > {
    less<pool::KeyTok> comp;

    public:
    enum { // parameters for hash table
	  bucket_size = 4,	// 0 < bucket_size
	  min_buckets = 8       // min_buckets = 2 ^^ N, 0 < N
    };	

    hash_compare(): comp(){}

    hash_compare(less<pool::KeyTok> pred): comp(pred){}

    size_t operator()(const pool::KeyTok& kt) const {
      return hash_compare<std::string>().operator()(kt.toString());
    }

    bool operator()(const pool::KeyTok& key1, const pool::KeyTok& key2) const {
      return comp(key1,key2);
    }
  };

#endif

}

/**
    POOL namespace
*/
namespace pool  {

  /** @class Hmap_ktoken_table Hmap_ktoken_table.h
      Templated container class. Designed to be a look up table to map a set of pointers
      to an other set. The implementation uses an Hmap_k_table<KeyToken> where the KeyPtr is
      a structure holding the pointer used as akey.

      @author Giacomo Govi
      @date 25/07/2002
   **/
  template <class T> class Hmap_ktoken_table : public Hmap_k_table<KeyTok,T> {    

    public:

      // default constructor
      Hmap_ktoken_table() : Hmap_k_table<KeyTok,T>() {}

      // constructor with initial capacity
      Hmap_ktoken_table(unsigned int init_size) : Hmap_k_table<KeyTok,T>(init_size){}

  };  

}

#endif  // POOL_HMAP_KTOKEN_TABLE_H








