#include <iostream>
#include "DataSvc/SharedAnyPtr.h" 
#include "PtrHolder.h"
#include "Hmap_kptr_table.h"

typedef pool::Hmap_kptr_table<pool::PtrHolder*> PtrTable;

static PtrTable ptrTable; 

// default ctor
pool::SharedAnyPtr::SharedAnyPtr(): m_ptr(),m_deletePolicy(true){
}

pool::SharedAnyPtr::SharedAnyPtr(bool deletePolicy):
                                 m_ptr(), m_deletePolicy(deletePolicy){
}

pool::SharedAnyPtr::SharedAnyPtr(const AnyPtr& aPtr, bool deletePolicy):
                                 m_ptr(),m_deletePolicy(deletePolicy){
  m_ptr = _register(aPtr);
}
                     
pool::SharedAnyPtr::SharedAnyPtr(const SharedAnyPtr& aPtr): 
                         m_ptr(aPtr.m_ptr),m_deletePolicy(aPtr.m_deletePolicy){
  if(m_ptr) m_ptr->incrementCount();
}

pool::SharedAnyPtr::~SharedAnyPtr(){
  reset();
}

pool::SharedAnyPtr& pool::SharedAnyPtr::operator=(const SharedAnyPtr& aPtr){
  if(this == &aPtr){
    return *this;
  }
  if(m_ptr != aPtr.m_ptr){
    reset();
    m_ptr = aPtr.m_ptr;
    if(m_ptr){
      m_ptr->incrementCount();
    }
  } 
  m_deletePolicy = aPtr.m_deletePolicy;
  return *this;
}

pool::SharedAnyPtr& pool::SharedAnyPtr::copy(const SharedAnyPtr& aPtr){
  if(this == &aPtr){
    return *this;
  }
  if(m_ptr != aPtr.m_ptr){
    reset();
    m_ptr = aPtr.m_ptr;
    if(m_ptr){
      m_ptr->incrementCount();
    }
  }  
  return *this;
}

pool::SharedAnyPtr& pool::SharedAnyPtr::operator=(const AnyPtr& aPtr){
 SharedAnyPtr tmp(aPtr,m_deletePolicy);
 operator=(tmp);
 return *this;
}

bool pool::SharedAnyPtr::operator==(const SharedAnyPtr& anObj) const {
  return get() == anObj.get();
}

bool pool::SharedAnyPtr::operator!=(const SharedAnyPtr& anObj) const {
  return !(operator==(anObj));
}

pool::SharedAnyPtr::operator bool() const {
  return get();
}

const pool::AnyPtr pool::SharedAnyPtr::get() const {
  AnyPtr ret;
  if(m_ptr){
    ret = *m_ptr;
  }
  return ret;
}

const void* pool::SharedAnyPtr::constPtr() const {
  return get().constPtr();
} 

void* pool::SharedAnyPtr::ptr() const {
  return get().ptr();
} 

//void* pool::SharedAnyPtr::ptr(const std::type_info& asType) const {
void* pool::SharedAnyPtr::ptr(const AnyType& asType) const {
  return get().ptr(asType);
} 

//bool pool::SharedAnyPtr::isOfType(const std::type_info& type) const {
bool pool::SharedAnyPtr::isOfType(const AnyType& type) const {
  return get().isOfType(type);
}

void pool::SharedAnyPtr::reset(){
  unsigned int count = 0;
  if(m_ptr){
    count = m_ptr->decrementCount();
    if(count == 0){
      KeyPtr k(m_ptr->ptr());
      ptrTable.del(k);
      if(m_deletePolicy){
        m_ptr->forceDelete();
      }
      delete m_ptr;
    }
    m_ptr = 0;
  }
}

unsigned int pool::SharedAnyPtr::useCount(){
  unsigned int ret = 0;
  if(m_ptr){
    ret = m_ptr->refCount();
  } 
  return ret;
}

void pool::SharedAnyPtr::forceDelete() {
  if(m_ptr){
    if(m_ptr->ptr()){
      KeyPtr k(m_ptr->ptr());
      ptrTable.del(k);
    }
    m_ptr->incrementCount();
    m_ptr->forceDelete();
    if(m_ptr->decrementCount()==0){
      delete m_ptr;      
    }
  }
}

bool pool::SharedAnyPtr::deletePolicy(){
  return m_deletePolicy;
}

unsigned int pool::SharedAnyPtr::numberOfInstances(){
  return ptrTable.size();
}
    
void pool::SharedAnyPtr::dumpRegister(){
  PtrTable::const_iterator iter;
  std::cout << "***** Object Register *************\n";
  std::cout << "Object listed: " << ptrTable.size() << std::endl;
  for(iter=ptrTable.begin();iter!=ptrTable.end();++iter){
    unsigned int nref  = 0;
    PtrHolder *hold = (PtrHolder*)((*iter).second);
    if(hold) nref = hold->refCount();
    std::cout <<"ptr = " <<(*iter).first.thePtr << " n.ref="<<nref<<"\n";
  }
  std::cout << "*************************************\n";
}

pool::PtrHolder* pool::SharedAnyPtr::_register(const AnyPtr& aPtr) {
  PtrHolder* ret = 0;
  if(aPtr){
    KeyPtr k(aPtr.ptr());
    PtrTable::Result res = ptrTable.get(k);
    bool found = res.first;
    if(found){
      PtrHolder* hold = res.second;
      if(hold){
        ret = hold;
      }
    } else {
      PtrHolder* hold = new PtrHolder(aPtr);
      ptrTable.put(k,hold);
      ret = hold;
    }
  }
  if(ret) ret->incrementCount();
  return ret;
}

