#define POOL_DATASVCTEST_CPP 1

#include <iostream>
#include "src/DataSvc.h"
#include "src/CacheSvc.h"
#include "src/ICache.h"
#include "src/CacheEntryImp.h"
#include "../Common/TestDataObject.h"
#include "../Common/FakeToken.h"
#include "../Common/DummyPersistencySvc.h"
#include "../Common/TestKernel.h"
#include "PersistencySvc/Placement.h"

using namespace pool;

class DataSvcTest : public TestKernel {

public:
  typedef TestDataObject T; 

  IDataSvc* dataSvc;
  ICacheSvc* cacheSvc;
  ICache* cache;
  
  DataSvcTest(): TestKernel(){
    IPersistencySvc* thePersSvc = new DummyPersistencySvc;
    dataSvc = new DataSvc(thePersSvc);
    cacheSvc = &dataSvc->cacheSvc();
    cache = (dynamic_cast<CacheSvc*>(cacheSvc))->cache();
  }

  virtual ~DataSvcTest(){
    delete dataSvc;
  }

  void registerTokenTest(){
    printM("Testing register, get Token methods");
    unsigned int NLOC = 100;
    CacheEntry* ens[100];
    Token* tok;
    unsigned int i;
    for(i=0;i<NLOC;i++){
      tok = new FakeToken(i);        
      ens[i] = cacheSvc->registerToken(*tok);
      if(!ens[i]){
        printError("test ERROR: register Token failed.");      
      }
      if(cacheSvc->getToken(ens[i]) != tok){
        printError("test ERROR: the token is not in the cache entry");
      }
    }
    for(i=0;i<NLOC;i++){
      tok = new FakeToken(i);
      CacheEntry* ce = cacheSvc->registerToken(*tok);
      if(ce!=ens[i]){
        printError("test ERROR: entry retrieved different from expected.");
      }
      if(!cacheSvc->linkCacheEntry(ens[i])){
        printError("test ERROR: link entry failed.");
      }
      CacheEntryImp* entry = dynamic_cast<CacheEntryImp*>(ens[i]);
      if(!entry){
        printError("test ERROR: cache entry not resolved.");
      }
      if(entry->refCount() != 1){
        printError("test ERROR: cache entry ref count mismatch.");
      }
    }
    for(i=0;i<NLOC;i++){
      tok = new FakeToken(i);
      CacheEntryImp *cacheEntry = dynamic_cast<CacheEntryImp*>(ens[i]);
      SharedAnyPtr p(new T(99));
      cache->updateEntry(cacheEntry,p);
      if(!cacheEntry->object()){
        printError("test ERROR: update object failed");
      }
      CacheEntry* ce= cacheSvc->registerObject(p);
      if(ce!=ens[i]){
        printError("test ERROR: the ptr has not been registred.");
      }
      if(!cache->findEntry(cacheEntry)){
        printError("test ERROR: entry not found");
      }
      if(!cacheSvc->unlinkCacheEntry(ens[i])){
        printError("test ERROR: unlink cache Entry failed.");      
      }
      if(cache->findEntry(cacheEntry)){
        printError("test ERROR: the entrye has not been deleted.");
        CacheEntryImp* entry = dynamic_cast<CacheEntryImp*>(ens[i]);
	if(entry) std::cout <<"Ref count = "<<entry->refCount()<<std::endl;
      }
      // only avoid that the same address can be used for the new locs
      CacheEntryImp* tmp = new CacheEntryImp();
      ce= cacheSvc->registerObject(p);
      cacheEntry = dynamic_cast<CacheEntryImp*>(ce);
      if(cacheEntry->token()){
        printError("test ERROR: still found token for entry deleted");
      }
      if(ce==ens[i]){
        printError("test ERROR: the entry has not been deleted.");
      }
      delete tmp;
    }
    cacheSvc->clearCache();
  }

  void registerPtrTest(){
    printM("Testing register, get ptr methods");
    unsigned int NLOC = 100;
    CacheEntry* ens[100];
    T* obj[100];
    unsigned int i;
    for(i=0;i<NLOC;i++){
      obj[i] = new T(i);        
      SharedAnyPtr ptr(obj[i]);
      ens[i] = cacheSvc->registerObject(ptr);
      if(!ens[i]){
        printError("test ERROR: register object failed.");      
      }
      if(cacheSvc->getObject(ens[i]) != ptr){
        printError("test ERROR: the token is not in the cache entry");
      }
    }
    for(i=0;i<NLOC;i++){
      SharedAnyPtr ptr(obj[i]);
       CacheEntry* ce = cacheSvc->registerObject(ptr);
      if(ce!=ens[i]){
        printError("test ERROR: entry retrieved different from expected.");
      }
      if(!cacheSvc->linkCacheEntry(ens[i])){
        printError("test ERROR: link entry failed.");
      }
      CacheEntryImp* entry = dynamic_cast<CacheEntryImp*>(ens[i]);
      if(!entry){
        printError("test ERROR: cache entry not resolved.");
      }
      if(entry->refCount() != 1){
        printError("test ERROR: cache entry ref count mismatch.");
      }
    }
    if(cacheSvc->cacheSize() != NLOC){
      printError("test ERROR: cache content different from expected");
    }
    for(i=0;i<NLOC;i++){
      SharedAnyPtr ptr(obj[i]);
      CacheEntryImp* entry = dynamic_cast<CacheEntryImp*>(ens[i]);
      if(!cacheSvc->unlinkCacheEntry(ens[i])){
        printError("test ERROR: unlink cache Entry failed.");      
      }      
      // only avoid that the same address can be used for the new locs
      if(cache->findEntry(entry)){
        printError("test ERROR: the entrye has not been deleted.");
	if(entry) std::cout <<"Ref count = "<<entry->refCount()<<std::endl;
      } 
      
    }
    if(cacheSvc->cacheSize() != 0){
      printError("test ERROR: cache content has not been removed");
    }
    cacheSvc->clearCache();
  }

  void clearCacheTest(){
    printM("Testing clearCache method");
    Token* t = new FakeToken(9999);
    CacheEntry* c0= cacheSvc->registerToken(*t);
    CacheEntry* c1 = cacheSvc->registerToken(*t);
    if(c0 != c1){
        printError("test ERROR: unexpected entry retrieved");            
    } 
    T* o = new T(9999);
    SharedAnyPtr p(o);
    CacheEntry* c2= cacheSvc->registerObject(p);
    CacheEntry* c3 = cacheSvc->registerObject(p);
    if(c2 != c3){
        printError("test ERROR: unexpected entry retrieved");            
    } 
    if(cacheSvc->cacheSize()!=2) {
      printError("test ERROR: cache size different from expected");
      std::cout << "CacheSize="<<cacheSvc->cacheSize()<<std::endl;
    }
    cacheSvc->clearCache();
    if(cacheSvc->cacheSize()!=0) {
      printError("test ERROR: cache size not null");
    }
    c1 = cacheSvc->registerToken(*t);
    if(c0 == c1){
        printError("test ERROR: unexpected entry retrieved");            
    } 
    c3 = cacheSvc->registerObject(p);
    if(c2 == c3){
        printError("test ERROR: unexpected entry retrieved");            
    } 
    cacheSvc->clearCache();
    if(cacheSvc->cacheSize()!=0) {
      printError("test ERROR: cache size not null");
    }
  }

  void retrieveObjectTest(){
    printM("Testing registerToken, retrieve Ptr methods");
    unsigned int NLOC = 100;
    unsigned int i;
    std::vector<CacheEntry*> entry(NLOC);
    for(i=0;i<NLOC;i++){
      Token* t = new FakeToken(i);
      entry[i] = cacheSvc->registerToken(*t);
      if(!entry[i]){
        printError("test ERROR: register Token failed.");      
      }
    }
    for(i=0;i<NLOC;i++){
      const SharedAnyPtr& theObj = cacheSvc->getObject(entry[i]);
      CacheEntryImp* ce = dynamic_cast<CacheEntryImp*>(entry[i]);
      const T* obj = 0;
      if(ce) {
        obj = reinterpret_cast<const T*>(theObj.constPtr());
        if(obj == 0){
          printError("test ERROR: retrieved pointer is null");
          return;
        }      
        if(obj->getOId() != (long)i){
          printError("test ERROR: retrieved object id different from expected.");
        }
        const Token *tk = ce->token();
	if(tk){	 
          CacheEntry* ce1 = cacheSvc->registerToken(*tk);
          if(ce1!=entry[i]){
            printError("test ERROR: entry for token different from the pointer one"); 
          }
        } else {
          printError("test ERROR: the token is null");
        }
      } else {
        printError("test ERROR: cache entry not resolved");
      }
    }
    cacheSvc->clearCache();
  }

  void registerForWriteTest(){
    printM("Testing object write methods");
    unsigned int NLOC = 100;
    CacheEntry* ens[100];
    std::vector<T*> obj(NLOC);
    unsigned int i;
    for(i=0;i<NLOC;i++){
      obj[i] = new T(i);
      SharedAnyPtr ptr(obj[i]);
      const Placement p;
      ens[i] = cacheSvc->registerObject(ptr);
      if(!cacheSvc->registerForWrite(p,ens[i])){
        printError("test ERROR: register write Object failed.");      
      }
      CacheEntryImp* ce = dynamic_cast<CacheEntryImp*>(ens[i]);
      if(!ce){
         printError("test ERROR: cache entry not resolved");
      } else {
        if(!ce->token()){
          printError("test ERROR: the token is not in the cache entry");
        }
      }
    }
    cacheSvc->resetCache();
    for(i=0;i<NLOC;i++){
      CacheEntryImp* ce = dynamic_cast<CacheEntryImp*>(ens[i]);
      if(!ce){
         printError("test ERROR: cache entry not resolved");
      } else {
        if(ce->object()){
          printError("test ERROR: the Object pointer has not been deleted.");
        }
        cacheSvc->getObject(ens[i]);
        if(!ce->object()){
          printError("test ERROR: the Object pointer has not been retrieved.");
        }
        const T* o = reinterpret_cast<const T*>(ce->object().constPtr());
        if(o == 0){
          printError("test ERROR: retrieved pointer is null");
        }      
        if(o->getOId() != (long)i){
          printError("test ERROR: retrieved object id different from expected.");
        }
      }      
    }
    cacheSvc->clearCache();
  }

};

int main(int, const char*)
{
  DataSvcTest test;
  test.registerTokenTest();
  test.registerPtrTest();
  test.clearCacheTest();
  test.retrieveObjectTest();
  //test.registerForWriteTest();
  test.dumpError();
  return 0;  
}
