//$Id: FCEntry.h,v 1.8 2004/02/09 13:46:09 xiezhen Exp $
#ifndef POOL_FCENTRY_H
#define POOL_FCENTRY_H
#include <string>
#include <iostream>
#include "AttributeList/AttributeList.h"

namespace pool{
  /** @class PFNEntry FCEntry.h FileCatalog/FCEntry.h
      Structure of one PFN table entry
      @author Zhen Xie
  */
  class FCEntry{
  public:
    FCEntry() : m_spec(new AttributeListSpecification) {}
    const std::string guid() const{
      return (*m_attrs)["guid"].getValueAsString();
    }
    virtual ~FCEntry(){}
    virtual void print(std::ostream& os)const=0;
    AttributeList& attrs()const{
      return *m_attrs;
    }
    AttributeListSpecification& spec() const{
      return *m_spec;
    }
    size_t size() const{
      return m_spec->size();
    }
  protected:
    void initAttrs()
      {
	m_attrs.reset(new AttributeList(m_spec));
      }
    boost::shared_ptr<AttributeListSpecification> m_spec;
    boost::shared_ptr<AttributeList> m_attrs;
  };
  
 
  class PFNEntry:public FCEntry{
  public:
    ///default constructor
    PFNEntry();
    ~PFNEntry(){}
    /**PFNEntry constructor
       @param ipfname [IN] PFN 
       @param iguid [IN] fileid
       @param ifiletype [IN] file technology type
     */ 
    PFNEntry(const std::string& ipfname, 
             const std::string& iguid,
	     const std::string& ifiletype 
	     );
    ///copy constructor
    PFNEntry(const PFNEntry& pentry);
    /**fill the PFNEntry struct
     */
    void setValue(const std::string& ipfname, 
		  const std::string& iguid, 
		  const std::string& ifiletype 
		  );
    virtual void print(std::ostream& os)const;
    const std::string pfname() const;
    const std::string filetype() const;
  };
  
  inline std::ostream& operator << (std::ostream& s, const PFNEntry& p){
    p.attrs().print(s);
    return s;
  } 
  
  /** @class LFNEntry FCEntry.h FileCatalog/FCEntry.h
      Structure of one LFN table entry
      @author Zhen Xie
  */
  class LFNEntry :public FCEntry {
  public:
    ///default constructor
    LFNEntry();
    /**constructor
       @param ilfname [IN] LFN 
       @param iguid [IN] fileid
    */
    LFNEntry(const std::string& ilfname, const std::string& iguid);
    LFNEntry(const LFNEntry& lentry);
    void setValue( const std::string& ilfname, const std::string& iguid);
    virtual void print(std::ostream& os)const;
    const std::string lfname() const;
  };
  
  inline std::ostream& operator << (std::ostream& s, const LFNEntry& l){
    l.attrs().print(s);
    return s;
  } 

  class MetaDataEntry :public FCEntry {
  public:
    MetaDataEntry();
    MetaDataEntry( const MetaDataEntry& mentry );
    void addAttributeSpec(const std::string& attr_name, const typename_string_t& type_name );
    void setGuidValue(const std::string& iguid);
    template<class T>
      void setAttributeValue( const std::string& attr_name, const T& t){
      (*m_attrs)[attr_name].setValue(t);
    }
    void setAttributeValueAsString( const std::string& attr_name, const std::string& t );
    std::string getGuidValue() const;
    std::string getAttributeValueAsString( const std::string& attr_name ) const;
    MetaDataEntry append_and_merge_spec( MetaDataEntry& mentry );
    template<class T>
      void getAttributeValue( const std::string& attr_name, T&v ) const{
      (*m_attrs)[attr_name].getValue(v);
    }
    virtual void print(std::ostream& os)const;
  };

  inline std::ostream& operator << (std::ostream& s, const MetaDataEntry& m){
    m.attrs().print(s);
    return s;
  } 

  
}//ns pool
#endif
