#include "FileCatalog/FCEntry.h"
//#include <iostream>
namespace pool{
  PFNEntry::PFNEntry(){
    //std::cout<<"default constructor"<<std::endl;
    m_spec->push_back( "guid", "string");
    m_spec->push_back( "pfname", "string");
    m_spec->push_back( "filetype", "string");
    initAttrs();
  }
  PFNEntry::PFNEntry(const std::string& ipfname, 
		     const std::string& iguid, 
	             const std::string& ifiletype){
    //std::cout<<"PFNEntry constructor"<<std::endl;
    m_spec->push_back( "guid", "string");
    m_spec->push_back( "pfname", "string");
    m_spec->push_back( "filetype", "string");
    initAttrs();
    this->setValue(ipfname,iguid,ifiletype);
  }
  PFNEntry::PFNEntry(const PFNEntry& pentry):FCEntry(){
    //std::cout<<"PFNEntry copy constructor"<<std::endl;
    std::string ipfname=pentry.attrs()["pfname"].getValueAsString();
    std::string iguid=pentry.attrs()["guid"].getValueAsString();
    std::string ifiletype=pentry.attrs()["filetype"].getValueAsString();
    m_spec = pentry.m_spec;
    initAttrs(); 
    this->setValue(ipfname,iguid,ifiletype);
  }
  void PFNEntry::setValue(const std::string& ipfname, 
			  const std::string& iguid, 
			  const std::string& ifiletype
			  ){
    (*m_attrs)["guid"].setValue(iguid);
    (*m_attrs)["pfname"].setValue(ipfname);
    (*m_attrs)["filetype"].setValue(ifiletype);
  }
  void PFNEntry::print(std::ostream& os)const{
    m_attrs->print(os);
  }
  const std::string PFNEntry::pfname() const{
    return (*m_attrs)["pfname"].getValueAsString();
  }
  const std::string PFNEntry::filetype() const{
    return (*m_attrs)["filetype"].getValueAsString();
  }

  LFNEntry::LFNEntry(){
    //std::cout<<"LFNEntry default constructor"<<std::endl;
    m_spec->push_back( "guid", "string");
    m_spec->push_back( "lfname", "string");
    initAttrs();
  }
  LFNEntry::LFNEntry(const std::string& ilfname, const std::string& iguid){
    //std::cout<<"LFNEntry constructor"<<std::endl;
    m_spec->push_back( "guid", "string");
    m_spec->push_back( "lfname", "string");
    initAttrs();
    this->setValue(ilfname,iguid);
  }
  LFNEntry::LFNEntry(const LFNEntry& lentry): FCEntry(){
    //std::cout<<"LFNEntry copy constructor"<<std::endl;
    std::string ilfname=lentry.attrs()["lfname"].getValueAsString();
    std::string iguid=lentry.attrs()["guid"].getValueAsString();
    m_spec = lentry.m_spec;
    initAttrs(); 
    //    std::cout<<"m_attrs "<<m_attrs<<std::endl; 
    this->setValue(ilfname,iguid);
    //std::cout<<"fini"<<std::endl;
  }
  void LFNEntry::setValue( const std::string& ilfname, const std::string& iguid){
    (*m_attrs)["guid"].setValue(iguid);
    (*m_attrs)["lfname"].setValue(ilfname);
  }
  void LFNEntry::print(std::ostream& os)const{
    m_attrs->print(os);
  }
  const std::string LFNEntry::lfname() const{
    return (*m_attrs)["lfname"].getValueAsString();
}
  
  MetaDataEntry::MetaDataEntry(){
    m_spec->push_back( "guid", "string");
    initAttrs();
  }
  
  MetaDataEntry::MetaDataEntry( const MetaDataEntry& mentry ) : FCEntry(){
    AttributeListSpecification spec=mentry.spec();
    for(AttributeListSpecification::iterator i=spec.begin(); i!=spec.end(); ++i){
      m_spec->push_back( (*i).name(), (*i).type_name() );
    }
    initAttrs();
    for(AttributeListSpecification::iterator j=spec.begin(); j!=spec.end(); ++j){
      std::string colname=(*j).name();
      (*m_attrs)[colname]=(mentry.attrs())[colname];
    }
  }

  void MetaDataEntry::addAttributeSpec(const std::string& attr_name, const typename_string_t& type_name ){
    m_spec->push_back( attr_name, type_name);
    initAttrs();
  }
  void MetaDataEntry::setGuidValue(const std::string& iguid){
    (*m_attrs)["guid"].setValue(iguid);
  }
  void MetaDataEntry::setAttributeValueAsString( const std::string& attr_name, const std::string& t ){
    (*m_attrs)[attr_name].setValueAsString(t);
  }
  std::string MetaDataEntry::getGuidValue() const {
    return (*m_attrs)["guid"].getValueAsString();
  }
  std::string MetaDataEntry::getAttributeValueAsString( const std::string& attr_name ) const {
    return (*m_attrs)[attr_name].getValueAsString();
  }
  MetaDataEntry MetaDataEntry::append_and_merge_spec( MetaDataEntry& mentry ){
    MetaDataEntry result;
    AttributeListSpecification spec;
    spec=(this->spec()).append_and_merge( mentry.spec() );
    for( AttributeListSpecification::iterator i=spec.begin(); i!=spec.end(); ++i){
      if( (*i).name()!="guid" ){
	result.addAttributeSpec((*i).name(), (*i).type_name() );
      }
    }
    return result;
  }
  void MetaDataEntry::print(std::ostream& os)const{
    m_attrs->print(os);
  } 
}//ns pool
