#include "FileCatalog/FCImpl.h"
#include "FileCatalog/URIParser.h"
#include "FileCatalog/FCImplPluginFactory.h"
#include "FileCatalog/FCMetaImplPluginFactory.h"
#include "POOLCore/PoolMessageStream.h"
#include "FileCatalog/FCException.h"

namespace pool{
  FCImpl* FCImplBuilder::BuildFCImpl(const std::string& contact){
    FCImpl* fcimpl=0;
    URIParser p(contact);
    std::string uriprefix,url;
    try{
      p.parse();
      uriprefix=p.prefix();
      url=p.url();
      if(uriprefix.empty()||uriprefix=="XMLFileCatalog"){
        fcimpl=FCImplPluginFactory::get()->create( "xmlcatalog" );
      }else{
        fcimpl=FCImplPluginFactory::get()->create( uriprefix );
      }
      if(fcimpl==0)
        throw pool::Exception(std::string("failed to load file catalog plugin: ")+url ,"FCImplBuilder::BuildFCImpl ",
                              "FileCatalog" );
      fcimpl->setURL(url);
      fcimpl->setType(uriprefix);
      return fcimpl;
    }catch( ... ){
      throw;
    }
  }

  FCMetaImpl* FCImplBuilder::BuildFCMetaImpl( FCImpl* fc){
    PoolMessageStream fclog( "FCImplBuilder::BuildFCMetaImpl", seal::Msg::Nil);
    FCMetaImpl* metaimpl=0;
    try{
      std::string mytype=fc->implType();
      if( mytype.empty()||mytype=="XMLFileCatalog"){
        metaimpl=FCMetaImplPluginFactory::get()->create("xmlcatalog_meta",fc);
      }else{
        metaimpl=FCMetaImplPluginFactory::get()->create(mytype+std::string("_meta"),fc);
      }
      if(!metaimpl){
        fclog<<seal::Msg::Info<<"no metadata catalog plugin found for "<<mytype<<seal::endmsg;
      }
    }catch(...){
      throw;
    }
    return metaimpl;
  }
}//ns pool
