//$Id: MySQLFileCatalog.h,v 1.37 2005/04/07 14:03:11 xiezhen Exp $
#ifndef POOL_MYSQLFILECATALOG_H
#define POOL_MYSQLFILECATALOG_H

#ifndef POOL_FCIMPL_H
#include "FileCatalog/FCImpl.h"
#endif
#include "FileCatalog/FCBuf.h"

#include <memory>
namespace pool {
  ///forward declaration
  class MySQLMetaDataCatalog;
  /**@class MySQLFileCatalog MySQLFileCatalog.h MySQLCatalog/MySQLFileCatalog.h
     Interface to the concrete implementation of the MySQL catalog. 
     @author: Zhen Xie
  */
  class MySQLFileCatalog: public FCImpl{
  public:
    ///Constructor
    MySQLFileCatalog();
    ///Destructor
    virtual ~MySQLFileCatalog();
    
    /** MySQLFileCatalog connection method, 
	connect to the mysql server and set AUTOCOMMIT=0
	throw exception in case of connection problem
	@param  url DB connection parameters in url format
    */
    
    virtual void connect();
    /** close the connection to the mysql server, throw exception on error 
	(eg connection not valid)	
    */
    virtual void disconnect() const;
    //
    //transaction control methods.
    //
    ///Start the catalog transaction
    virtual void start()const;
    ///Commit the catalog transaction
    virtual void commit(const FileCatalog::CommitMode cm=FileCatalog::REFRESH) const;
    ///Rollback the catalog transaction
    virtual void rollback() const; 
    /**Register a PFN when a new file is created, 
       returns the corresponding FileID 
       @param pfn [IN] the Physical file name
       @param filetype [IN] file technology type
       @param fid [OUT] File ID of the file
       @param fsta [IN] file status
    */
    virtual void registerPFN( const std::string& pfn, const std::string& filetype, FileCatalog::FileID& fid ) const;  
    /**Register LFN 
       @param pf [IN] PFN
       @param lf [IN] LFN
    */
    virtual void registerLFN(const std::string& pf, const std::string& lf) const;
    /**Add a replica file name in the catalog.
       @param pf [IN] the PFN of the master copy file.
       @param rpf [IN] the PFN of the replica to be added in the catalog. 
    */
    virtual void addReplicaPFN(const std::string& pf, const std::string& rpf) const;
    virtual void addPFNtoGuid(const FileCatalog::FileID& guid, const std::string& pf, const std::string& filetype) const;
    /**Rename a PFN in the catalog. The use case is the file is moved.
       @param pf [IN] the old PFN
       @param newpf [IN] the new PFN
    */
    virtual void renamePFN(const std::string& pf, const std::string& newpf) const;
    virtual void lookupFileByPFN(const std::string& pf, FileCatalog::FileID& fid, std::string& ftype) const;
    virtual void lookupFileByLFN(const std::string& lf, FileCatalog::FileID& fid) const;
    virtual void lookupBestPFN(const FileCatalog::FileID& fid, const FileCatalog::FileOpenMode& omode, const FileCatalog::FileAccessPattern& amode, std::string& pf, std::string& filetype) const;
    virtual void insertPFN(PFNEntry& pentry) const;
    virtual void insertLFN(LFNEntry& lentry) const;
    virtual void deletePFN(const std::string&  pf) const;
    virtual void deleteLFN(const std::string& lf) const;
    virtual void deleteEntry(const FileCatalog::FileID& guid) const;
    virtual bool isReadOnly() const;
    virtual bool retrievePFN(const std::string& query, FCBuf<PFNEntry>& buf, const size_t& start );
    virtual bool retrieveLFN(const std::string& query, FCBuf<LFNEntry>& buf, const size_t& start );
    virtual bool retrieveGuid(const std::string& query, FCBuf<FileCatalog::FileID>& buf, const size_t& start );
 private:
    friend class MySQLMetaDataCatalog;
    struct MySQLImpl; //forward declaration
    std::auto_ptr<MySQLImpl> m_pimpl; //opaque pointer to private functions
  };   
} 
#endif



