//$Id: MySQLMetaDataCatalog.cpp,v 1.7 2006/03/14 17:56:07 xiezhen Exp $
/*
  Concrete implementation of the MySQL metadata catalog. 
  @author: Zhen Xie
*/
#include "MySQLMetaDataImpl.h"
#ifndef POOL_GUID_H
#include "POOLCore/Guid.h"
#endif
#ifndef POOL_FCMYSQLEXCEPTION_H
#include "MySQLCatalog/FCMySQLException.h"
#endif
#include "MySQLHelpers.h"
#include "FileCatalog/FCImpl.h"
#include "MySQLImpl.h"
#include "FileCatalog/FCEntry.h"
#include "POOLCore/PoolMessageStream.h"
#include "FileCatalog/FCSystemTools.h"
#include "QueryParser.h"
//#include <iostream>
namespace pool {
  MySQLMetaDataCatalog::MySQLMetaDataCatalog(FCImpl* fc):
    FCMetaImpl(fc),m_pimpl(new MySQLMetaDataCatalog::MySQLMetaDataImpl){
  }
  MySQLMetaDataCatalog::~MySQLMetaDataCatalog(){}
  /**
     create MetaData table and metadataspec table. 
  */
  void MySQLMetaDataCatalog::createMetaDataSpec(MetaDataEntry& schemadef)const {
    this->syncConnection();
    MySQLExecute e(m_pimpl->m_con);
    AttributeListSpecification myschemadef;
    myschemadef=schemadef.spec();
    if(myschemadef.size()<=1) return;
    std::string d("drop table if exists t_metaspec;");
    std::string p("create table t_metaspec( colname varbinary(250) primary key, coltype varbinary(40) ) type=innodb;");
    try{
      e(d);
      e(p);
    }catch(mysqlpp::BadQuery&er){
      throw FCMySQLQueryException("MySQLFileCatalog::createMetaDataSpec",er.error);
    }
    std::string q("create table t_meta ( guid varbinary(40) primary key, ");
    std::string coltype;
    std::string colname;
    AttributeListSpecification::iterator it;
    //translate type into MySQL type
    try{
      for(it=myschemadef.begin(); it!=myschemadef.end(); ++it){
	colname=(*it).name();
	if(colname != "guid" ){
	  std::string insertd("insert into t_metaspec(colname, coltype) values");
	  insertd+="('"+colname+"','"+(*it).type_name()+"')";
	  e(insertd);
	  if((*it).type_name()=="string"){ 
	    coltype="blob";
	  }
	  if((*it).type_name()=="int" ){
	    coltype="int";
	  }
	  if(it != myschemadef.end()-1 ){
	    q+= colname + " " + coltype + ", ";
	  }else{
	    q+= colname + " " + coltype + " ) type=innodb;";
	  }
	}
      }
      e(q);
    }catch(mysqlpp::BadQuery& er){
      if(er.error.find(std::string("Table 't_meta' already exists")) !=er.error.npos ) {
	throw FCduplicatemetadataspecException("MySQLFileCatalog::createMetaDataSpec");
      }else{
	throw FCMySQLQueryException("MySQLFileCatalog::createMetaDataSpec",er.error);
      }
    }
  }
  
  void MySQLMetaDataCatalog::updateMetaDataSpec(MetaDataEntry& schemadef, const FileCatalog::MetaUpdateMode metamode)const{
    this->syncConnection();
    try{
      MySQLExecute e(m_pimpl->m_con);
      m_pimpl->loadMetaDataSpecification();
      AttributeListSpecification myschemadef=schemadef.spec();
      if( *(m_pimpl->m_metaDataSpecification)==myschemadef ){
	return;
      }
      if ( m_pimpl->m_metaDataSpecification->size() == 0 ) { // No meta data yet. Create tables from scratch.
	this->createMetaDataSpec( schemadef );
	return;
      }
      std::string coltype,colname;
      std::string q,p;
      ///
      ///In the case FileCatalog::DELETE_REDUNDANT switch is on
      ///drop existing columns if they are not present in the new spec
      ///This removes a column and all of its data permanently. 
      ///
      if( metamode==FileCatalog::DELETE_REDUNDANT ){
	for( AttributeListSpecification::iterator it=m_pimpl->m_metaDataSpecification->begin(); it!=m_pimpl->m_metaDataSpecification->end(); ++it){
	  colname=it->name();
	  size_t idx=myschemadef.index( colname );
	  if( idx == pool::IDX_NOT_FOUND ){
	    q+="alter table t_meta drop column "+colname+";";
	    p="delete from t_metaspec where colname='"+colname+"';";
	    e(p);
	  }
	}
      }
      ///
      ///if oldspec doesnot contain a field in the new spec,expand the oldspec 
      ///in any case
      ///
      p="";
      for( AttributeListSpecification::iterator it=myschemadef.begin(); it!=myschemadef.end(); ++it){
	if( it->name()=="guid" ) continue;
	size_t idx=m_pimpl->m_metaDataSpecification->index( it->name() ) ;
	//fixme:should turn all to lower case
	if( idx == pool::IDX_NOT_FOUND ){
	  q+="alter table t_meta add (";
	  if( it->type()==typeid(std::string) ){ 
	    coltype="blob";
	  }
	  if(it->type()==typeid(int) ){
	    coltype="int";
	  }
	  colname=(*it).name();
	  q+= colname + " " + coltype + " );";
	  p="insert into t_metaspec(colname, coltype) values('"+colname+"',"+"'"+(*it).type_name()+"');";
	  e(p);
	}
      }
      if(!q.empty()){
	e(q);
      }
      m_pimpl->loadMetaDataSpecification();
    }catch(mysqlpp::BadQuery& er){
      throw FCMySQLQueryException("MySQLFileCatalog::updateMetaDataSpec",er.error);  
    }
  }
  
  void MySQLMetaDataCatalog::getMetaDataSpec( MetaDataEntry& spec) const{
    this->syncConnection();
    try{
      m_pimpl->loadMetaDataSpecification();
    }catch(mysqlpp::BadQuery& er){
      throw FCMySQLQueryException("MySQLFileCatalog::getMetaDataSpec", er.error);
    }
    if ( ! m_pimpl->m_metaDataSpecification ) return;
    for ( pool::AttributeListSpecification::const_iterator iAttributeSpecification = m_pimpl->m_metaDataSpecification->begin(); iAttributeSpecification != m_pimpl->m_metaDataSpecification->end(); ++iAttributeSpecification ) {
      spec.addAttributeSpec( iAttributeSpecification->name(), iAttributeSpecification->type_name() );
    }
  }

  void MySQLMetaDataCatalog::dropMetaDataSpec()const{
    this->syncConnection();
    try{
      MySQLExecute e(m_pimpl->m_con);
      e("drop table if exists t_metaspec;");
      e("drop table if exists t_meta; ");
      if ( m_pimpl->m_metaDataSpecification ) delete m_pimpl->m_metaDataSpecification;
      m_pimpl->m_metaDataSpecification=0;
    }catch(mysqlpp::BadQuery& er){
      throw FCMySQLQueryException("MySQLFileCatalog::dropMetaDataSpec",er.error);
    } 
  }
  void MySQLMetaDataCatalog::insertMetaData(MetaDataEntry& mentry)const{
    this->syncConnection();
    MySQLconvertString c;
    mysqlpp::Query q=m_pimpl->m_con->query();
    AttributeListSpecification arow=mentry.spec();
    AttributeListSpecification::iterator it;
    if (arow.size()==0) return;
    std::string columnname;
    q<<"insert into t_meta (";
    for(it=arow.begin(); it!=arow.end(); ++it){
      columnname=(*it).name();
      q<<columnname;
      if(it!=(arow.end()-1)){
	q<<",";
      }else{
	q<<")";
      }
    }
    q<<"values ( ";
    std::string columntype;
    for(it=arow.begin(); it!=arow.end(); ++it){
      columnname=(*it).name();
      columntype=(*it).type_name();
      if(columnname=="guid"){
	std::string g(mentry.getGuidValue());
	q<<c(g);
      }else{
	if(columntype == "int"){
	  int intme;
	  mentry.getAttributeValue(columnname,intme);
	  q<<intme;
	}
	if(columntype == "string"){
	  std::string strme;
	  mentry.getAttributeValue(columnname,strme);  
	  q<<c(strme);
	}
      }
      if(it!=arow.end()-1){
	q<<",";
      }else{
	q<<");";
      }
    }
    
    try{
      q.execute();
    }catch (mysqlpp::BadQuery& er){
      if(er.error.find(std::string("Duplicate entry "))!=er.error.npos){
	throw FCduplicatemetadataException("MySQLFileCatalog::insertMetaData");
      }
      throw FCMySQLQueryException("MySQLFileCatalog::insertMetaData",er.error);
    }
  }
  void MySQLMetaDataCatalog::deleteMetaData( const FileCatalog::FileID& fid ) const{
    this->syncConnection();
    MySQLconvertString c;
    MySQLExecute e(m_pimpl->m_con);
    std::string myquery("delete from t_meta where guid=");
    myquery+=c(fid);
    try{
      e(myquery); 
    } catch (mysqlpp::BadQuery& er){
      if( er.error.find(std::string("t_meta' doesn't exist")) !=er.error.npos ){
      }else{
	throw FCMySQLQueryException("MySQLFileCatalog::deleteMetaData",er.error);
      }
    }
  }
  /**queries retrieve pfn involve metadata attribute
   **/
  bool MySQLMetaDataCatalog::retrievePFN(const std::string& query, FCBuf<PFNEntry>& buf, const size_t& start ){
    this->syncConnection();
    buf.clear();
    std::string maxs=FCSystemTools::itostr( buf.maxsize() );
    std::string starts=FCSystemTools::itostr( start );
    std::string myquery;
    try{
      std::vector<std::string> metavar;
      m_pimpl->loadMetaDataSpecification();
      if( m_pimpl->m_metaDataSpecification!=0 ){
	for( AttributeListSpecification::iterator it=m_pimpl->m_metaDataSpecification->begin(); it!=m_pimpl->m_metaDataSpecification->end(); ++it){
	  metavar.push_back( it->name() );
	} 
      }
      QueryParser parser(query, metavar,"t_pfn");
      parser.parse();
      const std::string& tabs=parser.getTablesAsString();
      const std::string& whereClause=parser.parseResult();
      if(!whereClause.empty()){
	myquery="select t_pfn.* from "+tabs+" where "+whereClause;
      }else{
	myquery="select t_pfn.* from "+tabs;
      }
      const std::vector<std::string>& s=parser.getTables();
      if( s.size()>1 ){
	for(std::vector<std::string>::const_iterator it=s.begin();
	    it!=s.end(); ++it){
	  if( (*it)!="t_pfn" ){
	    myquery+=" AND t_pfn.guid="+(*it)+".guid";
	  }
	}
      }
      mysqlpp::Query q=m_pimpl->m_con->query();
      q<<myquery;
      q<<" limit "<<start<<" ," <<maxs;
      mysqlpp::Result res=q.store();
      if(res.size()==0) return false;
      mysqlpp::Result::iterator i;
      for(i=res.begin(); i!=res.end() ;++i){
	mysqlpp::Row row=*i;
	std::string mypfn(row.lookup_by_name("pfname"));
	std::string myguid(row.lookup_by_name("guid"));
	std::string filetype(row.lookup_by_name("filetype"));
	PFNEntry pentry(mypfn,myguid,filetype);
	buf.push_back( pentry );
      }
      res.purge();
    }catch(mysqlpp::BadQuery& er){
      throw FCMySQLQueryException("MySQLFileCatalog::retrievePFN",er.error);
    }
    return true;
  }
  /**queries retrieve lfn involve metadata attribute
   **/
  bool MySQLMetaDataCatalog::retrieveLFN(const std::string& query, FCBuf<LFNEntry>& buf, const size_t& start ){
    this->syncConnection();
    buf.clear();
    std::string maxs=FCSystemTools::itostr( buf.maxsize() );
    std::string starts=FCSystemTools::itostr( start );
    std::string myquery;
    try{
      std::vector<std::string> metavar;
      m_pimpl->loadMetaDataSpecification();
      if( m_pimpl->m_metaDataSpecification!=0 ){
	for( AttributeListSpecification::iterator it=m_pimpl->m_metaDataSpecification->begin(); it!=m_pimpl->m_metaDataSpecification->end(); ++it){
	  metavar.push_back( it->name() );
	}
      } 
      QueryParser parser(query, metavar,"t_lfn");
      parser.parse();
      const std::string& tabs=parser.getTablesAsString();
      const std::string& whereClause=parser.parseResult();
      if(!whereClause.empty()){
	myquery="select t_lfn.* from "+tabs+" where "+whereClause;
      }else{
	myquery="select t_lfn.* from "+tabs;
      }
      const std::vector<std::string>& s=parser.getTables();
      if( s.size()>1 ){
	for(std::vector<std::string>::const_iterator it=s.begin();
	    it!=s.end(); ++it){
	  if( (*it)!="t_lfn" ){
	    myquery+=" AND t_lfn.guid="+(*it)+".guid";
	  }
	}
      }
      mysqlpp::Query q=m_pimpl->m_con->query();
      q<<myquery;
      q<<" limit "<<start<<" ," <<maxs;
      mysqlpp::Result res=q.store();
      if(res.size()==0) return false;
      mysqlpp::Result::iterator i;
      for(i=res.begin(); i!=res.end() ;++i){
	mysqlpp::Row row=*i;
	std::string mylfn(row.lookup_by_name("lfname"));
	std::string myguid(row.lookup_by_name("guid"));
	LFNEntry lentry(mylfn,myguid);
	buf.push_back(lentry);
      }
      res.purge();
    }catch(mysqlpp::BadQuery& er){
      throw FCMySQLQueryException("MySQLFileCatalog::retrieveLFN",er.error);
    }
    return true;
  }
  /**queries retrieve metadata
   **/
  bool MySQLMetaDataCatalog::retrieveMetaData(const std::string& query, FCBuf<MetaDataEntry>& buf, const size_t& start ){
    this->syncConnection();
    MySQLtableExist exist( m_pimpl->m_con );
    if( !exist("t_meta") ) return false;
    buf.clear();
    std::string maxs=FCSystemTools::itostr( buf.maxsize() );
    std::string starts=FCSystemTools::itostr( start );
    std::string myquery;
    try{
      std::vector<std::string> metavar;
      m_pimpl->loadMetaDataSpecification();
      if( m_pimpl->m_metaDataSpecification==0 ) return false;
      for( AttributeListSpecification::iterator it=m_pimpl->m_metaDataSpecification->begin(); it!=m_pimpl->m_metaDataSpecification->end(); ++it){
	metavar.push_back( it->name() );
      } 
      QueryParser parser(query, metavar,"t_meta");
      parser.parse();
      const std::string& tabs=parser.getTablesAsString();
      const std::string& whereClause=parser.parseResult();
      if(!whereClause.empty()){
	myquery="select t_meta.* from "+tabs+" where "+whereClause;
      }else{
	myquery="select t_meta.* from "+tabs;
      }
      const std::vector<std::string>& s=parser.getTables();
      if( s.size()>1 ){
	for(std::vector<std::string>::const_iterator it=s.begin();
	    it!=s.end(); ++it){
	  if( (*it)!="t_meta" ){
	    myquery+=" AND t_meta.guid="+(*it)+".guid";
	  }
	}
      }
      mysqlpp::Query q=m_pimpl->m_con->query();
      q<<myquery;
      q<<" limit "<<start<<" ," <<maxs;
      mysqlpp::Result res=q.store();
      if(res.size()==0) return false;
      mysqlpp::Result::iterator i;
      if( !m_pimpl->m_metaDataSpecification ) m_pimpl->loadMetaDataSpecification();
      for(i=res.begin(); i!=res.end() ;++i){
	MetaDataEntry mentry;
	AttributeListSpecification::iterator j;
	for( j=m_pimpl->m_metaDataSpecification->begin(); j!=m_pimpl->m_metaDataSpecification->end();++j ){
	  if((*j).name()!="guid"){
	    mentry.addAttributeSpec((*j).name(),(*j).type_name());
	  }
	}
	mysqlpp::Row row=*i;
	mentry.setGuidValue(std::string(row.lookup_by_name("guid")));
	AttributeListSpecification::iterator it;
	for( it=m_pimpl->m_metaDataSpecification->begin(); it!=m_pimpl->m_metaDataSpecification->end();++it ){
	  std::string colname=(*it).name();
	  std::string coltype=(*it).type_name();
	  if(coltype=="string"){
	    std::string colvalue=std::string(row.lookup_by_name(colname.c_str()));
	    mentry.setAttributeValue(colname,colvalue);
	  }else if(coltype=="int"){
	    if(std::string(row.lookup_by_name(colname.c_str()))!="NULL"){
	      int colvalue=int(row.lookup_by_name(colname.c_str()));
	      mentry.setAttributeValue(colname,colvalue);
	    }
	  }else if(coltype=="float"){
	    if(std::string(row.lookup_by_name(colname.c_str()))!="NULL"){
	      float colvalue=float(row.lookup_by_name(colname.c_str()));
	      mentry.setAttributeValue<float>(colname,colvalue);
	    }
	  }
	}
	buf.push_back(mentry);
      }
      res.purge();
    }catch(mysqlpp::BadQuery& er){
      throw FCMySQLQueryException("MySQLMetaDataCatalog::retrieveMetaData",er.error);
    }catch(const pool::Exception& er){
      throw er;
    }
    return true;
  }
  /**queries retrieve guid involve metadata
   **/
  bool MySQLMetaDataCatalog::retrieveGuid(const std::string& query, FCBuf<FileCatalog::FileID>& buf, const size_t& start ){
    this->syncConnection();
    buf.clear();
    std::string maxs=FCSystemTools::itostr( buf.maxsize() );
    std::string starts=FCSystemTools::itostr( start );
    std::string myquery;
    try{
      std::vector<std::string> metavar;
      m_pimpl->loadMetaDataSpecification();
      if( m_pimpl->m_metaDataSpecification!=0 ){
	for( AttributeListSpecification::iterator it=m_pimpl->m_metaDataSpecification->begin(); it!=m_pimpl->m_metaDataSpecification->end(); ++it){
	  metavar.push_back( it->name() );
	} 
      }
      QueryParser parser(query, metavar,"t_meta");
      parser.parse();
      const std::string& tabs=parser.getTablesAsString();
      const std::string& whereClause=parser.parseResult();
      if( !whereClause.empty() ){
	myquery="select distinct t_meta.guid from "+tabs+" where "+whereClause;
      }else{
	myquery="select distinct t_meta.guid from "+tabs;
      }
      const std::vector<std::string>& s=parser.getTables();
      if( s.size()>1 ){
	for(std::vector<std::string>::const_iterator it=s.begin();
	    it!=s.end(); ++it){
	  if( (*it)!="t_meta" ){
	    myquery+=" AND t_meta.guid="+(*it)+".guid";
	  }
	}
      }
      mysqlpp::Query q=m_pimpl->m_con->query();
      q<<myquery;
      q<<" limit "<<start<<" ," <<maxs;
      mysqlpp::Result res=q.store();
      if(res.size()==0) return false;
      mysqlpp::Result::iterator i;
      for(i=res.begin(); i!=res.end() ;++i){
	mysqlpp::Row row=*i;
	std::string myguid(row.lookup_by_name("guid"));
	buf.push_back(myguid);
      }
      res.purge();
    }catch(mysqlpp::BadQuery& er){
      throw FCMySQLQueryException("MySQLMetaDataCatalog::retrieveGuid",er.error);
    }
    return true;
  }

  void MySQLMetaDataCatalog::syncConnection() const {
    if(!m_pimpl->m_con||m_pimpl->m_con!=static_cast<MySQLFileCatalog*>(m_fc)->m_pimpl->m_con){
      m_pimpl->m_con=static_cast<MySQLFileCatalog*>(m_fc)->m_pimpl->m_con;
    }
  }
} // ns pool 









