#include "MySQLURIParser.h"
#include <iostream>
namespace pool{

  MySQLURIParser::MySQLURIParser():m_hostport(3306){
  }

  MySQLURIParser::MySQLURIParser(const std::string& url)
    :m_url(url),m_hostport(3306)
  {
  }

  void MySQLURIParser::dump() const{
    //std::cout << "prefix: "<<m_prefix<<std::endl;
    std::cout << "url: "<< m_url<<std::endl;
    std::cout << "protocol: " << m_protocol<<std::endl;
    std::cout << "username: " << m_username<<std::endl;
    std::cout << "passwd: " << m_passwd<<std::endl;
    std::cout << "host: " << m_host<<std::endl;
    std::cout << "hostport: " << m_hostport<<std::endl;
    std::cout << "path: " << m_path<<std::endl;
  }

  void MySQLURIParser::setURI(const std::string& url){
    m_url=url;
    m_protocol="";
    m_username="";
    m_passwd="";
    m_host="";
    m_hostport=3306;
    m_path="";
  }

  void MySQLURIParser::parse() throw (FCillegalContactStringException){
    if(m_url.empty()){
      throw FCillegalContactStringException("MySQLURIParser::parse","empty url");
    }
    
    //
    //look up separators
    //
    //size_t tmppos=m_url.find_first_of(':');
    size_t colpos=m_url.find("://");
    size_t startpos=0;
    //
    //protocol parsing
    //
    if(colpos != m_url.npos ){
      m_protocol=m_url.substr(startpos,colpos);
    }
      
    //parsing optional userinfo
    size_t atpos=m_url.find_first_of('@',colpos+3);
    size_t nextstart;
    //size_t hostend;
    std::string userinfo;
    if(atpos == m_url.npos ){
      //no user info
      nextstart=colpos+3;
    }else{
      userinfo=m_url.substr(colpos+3,atpos-colpos-3);
      size_t ucolpos=userinfo.find(':',0);
      nextstart=atpos+1;
      if(ucolpos == userinfo.npos){
	// no passwd 
	m_username=userinfo.substr(0,ucolpos);
      }else{
	m_username=userinfo.substr(0,ucolpos);
	m_passwd=userinfo.substr(ucolpos+1,userinfo.length()-ucolpos-1);
      }
    }
    
    //parsing the rest of the string 
    std::string nextstr=m_url.substr(nextstart,m_url.length()-nextstart);
    //parse path
    size_t pathpos=nextstr.find_first_of('/');
    if(pathpos==nextstr.npos){ //if no host info
      m_path=nextstr;
    }else{
      m_path=nextstr.substr(pathpos+1,nextstr.length()-pathpos-1);
      if(m_path.find('/')!=m_path.npos){
	m_path.insert(0,"/"); //if absolute path
      }
      //parse hostinfo
      std::string hostinfo=nextstr.substr(0,pathpos); 
      size_t hcolpos=hostinfo.find(':');
      if(hcolpos==hostinfo.npos){
	//no port info 
	m_host=hostinfo;
      }else{
	std::string hostport=hostinfo.substr(hcolpos+1,hostinfo.length()-hcolpos-1);
	m_host=hostinfo.substr(0,hcolpos);
	m_hostport=atoi(hostport.data());
      }  
    }
  }
      
  const std::string MySQLURIParser::protocol() const{
    return m_protocol;
  }

  const std::string MySQLURIParser::username() const{
    return m_username;
  }

  const std::string MySQLURIParser::passwd() const{
    return m_passwd;
  }

  const std::string MySQLURIParser::host() const{
    return m_host;
  }
  
  const unsigned int MySQLURIParser::hostport() const{
    return m_hostport;
  }
  
  const std::string MySQLURIParser::path() const{
    return m_path;
  }

  const std::string MySQLURIParser::url() const{
    return m_url;
  }

  MySQLURIParser::~MySQLURIParser(){
  }
  
}/* ns pool */
