#include "QueryParser.h"
//#include <iostream>
namespace pool {
  QueryParser::QueryParser( const std::string& whereClause, const std::string& destTab ):m_destTab(destTab),m_parseResult(whereClause){
  }
  QueryParser::QueryParser( const std::string& whereClause, std::vector<std::string>&metavariables, const std::string& destTab ):
    m_metavariables(metavariables),
    m_destTab(destTab),
    m_parseResult(whereClause)
  {
  }
  void QueryParser::parse(){
    m_tables.push_back(m_destTab);
    this->addTablename("pfname", "t_pfn");
    if(m_destTab!="t_pfn" && m_parseResult.find("t_pfn.")!=m_parseResult.npos){ m_tables.push_back("t_pfn");
    }   
    this->addTablename("lfname", "t_lfn");
    if(m_destTab!="t_lfn" && m_parseResult.find("t_lfn.")!=m_parseResult.npos) m_tables.push_back("t_lfn");
    this->addTablename("guid", m_destTab);
    if( m_metavariables.empty() ) return;
    for( std::vector<std::string>::const_iterator it=m_metavariables.begin(); it!= m_metavariables.end(); ++it){
      this->addTablename(*it, "t_meta");
    }
    if(m_destTab!="t_meta" && m_parseResult.find("t_meta.")!=m_parseResult.npos){ m_tables.push_back("t_meta");
    }
  }
  
  void QueryParser::addTablename(const std::string& varname, const std::string& tabname){
    size_t pos=0;
    size_t varpos=0;
    while(true){
      varpos=m_parseResult.find( varname,pos );
      if(varpos==m_parseResult.npos) {
	return;
      }
      if( !isWithinQuotes(varpos) && isWord(varname, varpos) ){
	m_parseResult.insert( varpos, tabname+".");
	pos=varpos+(tabname+"."+varname).length();
      }else{
	pos=varpos+varname.length();
      }
    }
    return;
  }

  bool QueryParser::isWithinQuotes( std::string::size_type& varpos ) const{
    bool withinQuotes = false;
    for ( std::string::size_type i = 0; i < m_parseResult.size(); ++i ) {
      char c = m_parseResult[i];
      if ( c == '\'' ) {
	if ( withinQuotes ) {
	  withinQuotes = false;
	}else {
	  withinQuotes = true;
	}
      }
      if( varpos==i ) return withinQuotes;
    }
    return false;
  }
  
  bool QueryParser::isWord( const std::string& word,
			    std::string::size_type& varpos ) const{
    //    std::cout<<"varpos "<<varpos<<std::endl;
    //std::cout<<"word "<<word<<std::endl;
    
    if(varpos==0){
      char wend=m_parseResult[varpos+word.length()];
      if( wend==' '|| wend=='='||
	  wend=='!'|| wend=='<'||
	  wend=='>'
	  ){ 
	//	std::cout<<"word "<<word<<" is word "<<std::endl;
	return true;
      }else{
	return false;
      }
    }else{
      char wstart=m_parseResult[varpos-1];
      char wend=m_parseResult[varpos+word.length()];
      if( wstart==' ' && 
	  (wend==' '|| wend=='='|| 
	   wend=='!'|| wend=='<'||
	   wend=='>'
	   )
	  ){ 
	//std::cout<<"word "<<word<<" is word "<<std::endl;
	return true;
      }else{
	return false;
      }
    }
  }
  const std::string& QueryParser::parseResult() const{
    return m_parseResult;
  }

  const std::string QueryParser::getTablesAsString() const{
    std::string result;
    for(std::vector<std::string>::const_iterator it=m_tables.begin();
	it!=m_tables.end(); ++it){
      if( it!=m_tables.begin() ){
	result+=',';
      }
      result+=*it;
    }
    return result;
  }
  
  const std::vector<std::string>& QueryParser::getTables() const{
    return m_tables;
  }

}//ns pool
