//$Id: MySQLtest.cpp,v 1.22 2007/02/09 13:21:39 ioannis Exp $
/*
  CPPUnit test of MySQL Catalog.
  Author: Zhen Xie
*/
#include<cppunit/extensions/HelperMacros.h>

#ifndef POOL_MYSQLFILECATALOG_H
#include "MySQLCatalog/MySQLFileCatalog.h"
#endif
#include "FileCatalog/IFCAction.h"
#include "FileCatalog/FCCommon.h"
using namespace pool;

class MySQLtest: public CppUnit::TestFixture{
  CPPUNIT_TEST_SUITE( MySQLtest );
  CPPUNIT_TEST( testRegister );
  //CPPUNIT_TEST( testDelete );
  CPPUNIT_TEST_SUITE_END();
public:
  IFileCatalog* mycatalog;
  std::vector<std::string> names;
  FileCatalog::FileID fid;
  IFCAction a;
  FCregister r;
  FClookup l;
  std::string pfn,ftype;
  void setUp(){
    mycatalog=new IFileCatalog;
    mycatalog->setWriteCatalog("mysqlcatalog_mysql://xiezhen@localhost/zhendb");
    std::string bigname;
    std::string sname("abcdefghij");
    for (unsigned int i=0; i<25; ++i){
      bigname+=sname;
    }
    //bigname+="hello";
    names.push_back(bigname);
    names.push_back("this-is-a-pfn-test1");
    names.push_back("this-is-a-pfn-test2");
    names.push_back("this-is-a-pfn-test3");
  }

  void tearDown() {
    if(mycatalog!=0) delete mycatalog;
    names.clear();
  }

  void testRegister(){
    std::cout<<"[OVAL]TEST --> testIFCAction"<<std::endl;
    try{
      mycatalog->setAction(a);
      mycatalog->setAction(r);
      mycatalog->connect();
      mycatalog->start();
      std::cout<<"[OVAL] registerFile "<<std::endl;
      for(size_t i=0; i<names.size(); ++i){
	r.registerPFN(names[i],"root tree", fid);
	r.registerLFN(names[i], std::string("lfN")+fid); 
	r.registerLFN(names[i], std::string("lfn1")+fid);
	r.registerLFN(names[i], std::string("LFN1")+fid);
	std::cout<<"[OVAL] lookupBestPFN "<<i<<std::endl;
	r.lookupBestPFN(fid,FileCatalog::READ,FileCatalog::SEQUENTIAL,pfn,ftype);
	CPPUNIT_ASSERT_MESSAGE("wrong PFN",names[i]==pfn);  
	CPPUNIT_ASSERT_MESSAGE("wrong filetype","root tree"==ftype);
	std::cout<<"[OVAL] result pfn: "<<pfn<<" filetype: "<<ftype<<std::endl;
      }
      std::cout<<"[OVAL] lookupFileByPFN"<<std::endl;
      FileCatalog::FileID myfid;
      a.lookupFileByPFN(pfn,myfid,ftype);
      CPPUNIT_ASSERT_MESSAGE("wrong fid",fid==myfid);
      CPPUNIT_ASSERT_MESSAGE("wrong filetype","root tree"==ftype);
      std::cout<<"pfn :"<<pfn<<" fid: "<<myfid<<" filetype "<<ftype<<std::endl;
      std::cout<<"[OVAL] lookupFileByLFN"<<std::endl;
      a.lookupFileByLFN(std::string("lfN")+fid,myfid);
      std::cout<<"[OVAL] addReplicaFilename"<<std::endl;
      r.addReplicaPFN(pfn,"replicapfn");
      std::cout<<"[OVAL] renamePFN"<<std::endl;
      r.renamePFN("replicapfn","movedreplicapfn");
      mycatalog->commit(); 
      mycatalog->disconnect();
    }catch(const std::exception& er){
      std::cerr << er.what() << std::endl;
    }
  }
};
CPPUNIT_TEST_SUITE_REGISTRATION(MySQLtest);
#include<CppUnit_testdriver.cpp>
