//$Id: crosstableQuery.cpp,v 1.3 2007/02/09 13:21:40 ioannis Exp $
/*
  CPPUnit for MySQL Catalog.
  Author: Zhen Xie
*/
#include<cppunit/extensions/HelperMacros.h>
#ifndef POOL_MYSQLFILECATALOG_H
#include "MySQLCatalog/MySQLFileCatalog.h"
#endif
#include "FileCatalog/IFCContainer.h"
#include "FileCatalog/IFCAction.h"
#include "FileCatalog/FCSystemTools.h"
#include "FileCatalog/FCException.h"
using namespace pool;

class crossTableQueryTest: public CppUnit::TestFixture{
  CPPUNIT_TEST_SUITE( crossTableQueryTest );
  CPPUNIT_TEST( testLookup );
  //CPPUNIT_TEST( testDelete );
  CPPUNIT_TEST_SUITE_END();
public:
  IFileCatalog* mycatalog;
  std::vector<std::string> names;
  FileCatalog::FileID fid;
  std::string pfn,ftype;
  MetaDataEntry abx;
  void setUp(){
    mycatalog=new IFileCatalog;
    mycatalog->setWriteCatalog("mysqlcatalog_mysql://xiezhen@localhost/zhendb");
    abx.addAttributeSpec("a","string");
    abx.addAttributeSpec("b","string");
    abx.addAttributeSpec("x","int");
    names.push_back("fid1");
    names.push_back("fid2");
    names.push_back("fid3");
  }
  void tearDown() {
    if(mycatalog!=0) delete mycatalog;
    names.clear();
  }
  void testLookup(){
    std::cout<<"[OVAL]TEST --> testLookup"<<std::endl;
    try{
      FCAdmin a;
      FCregister r;
      FClookup l;
      mycatalog->setAction(a);
      mycatalog->setAction(r);
      mycatalog->setAction(l);
      mycatalog->connect();
      mycatalog->start();
      try{
	a.createMetaDataSpec(abx);
      }catch(const pool::FCduplicatemetadataspecException& er){
	std::cout<<"metadata schema already exists, drop it and create it again"<<std::endl;
	a.dropMetaDataSpec();
	a.createMetaDataSpec(abx);
      }
      MetaDataEntry meta;
      r.getMetaDataSpec(meta);
      int n=0;
      for(std::vector<std::string>::iterator it=names.begin(); it!=names.end(); ++it){
	++n;
	FileCatalog::FileID fid;
	r.registerPFN("pfn"+*it, "root-all", fid);
	r.registerLFN("pfn"+*it,"lfn"+*it );
	PFNContainer mypfns(mycatalog,3);
	l.lookupPFNByQuery("guid=\'"+fid+"\'", mypfns);
	//with or without metadata the result should be the same!
	while(mypfns.hasNext()){
	  PFNEntry pentry=mypfns.Next();
	  CPPUNIT_ASSERT_MESSAGE("wrong PFN",("pfn"+*it)==pentry.pfname());
	  std::cout<<pentry<<std::endl;
	}
	meta.setAttributeValue("a",*it+"a");
	meta.setAttributeValue("b",*it+"b");
	meta.setAttributeValue("x",n);
	r.registerMetaData(fid, meta);
	PFNContainer mypfns2(mycatalog,3);
	l.lookupPFNByQuery("guid=\'"+fid+"\'", mypfns2);
	while(mypfns2.hasNext()){
	  PFNEntry pentry=mypfns2.Next();
	  CPPUNIT_ASSERT_MESSAGE("wrong PFN",("pfn"+*it)==pentry.pfname());
	  std::cout<<pentry<<std::endl;
	}
      }
      mycatalog->commit();
      //start new transaction, test cross table query
      mycatalog->start();
      PFNContainer mypfns3(mycatalog,3);
      l.lookupPFNByQuery("pfname=\'pfn"+names[0]+"\' AND a=\'"+names[0]+"a\'", mypfns3);
      unsigned int counter=0;
      while(mypfns3.hasNext()){
	++counter;
	PFNEntry pentry=mypfns3.Next();
	CPPUNIT_ASSERT_MESSAGE("wrong PFN","pfn"+names[0]==pentry.pfname());
	std::cout<<pentry<<std::endl;
      }
      CPPUNIT_ASSERT_MESSAGE("wrong number of PFN",counter==1);
      
      LFNContainer mylfns3(mycatalog,3);
      l.lookupLFNByQuery("pfname=\'pfn"+names[0]+"\' AND a=\'"+names[0]+"a\'", mylfns3);
      unsigned int lcounter=0;
      while(mylfns3.hasNext()){
	++lcounter;
	LFNEntry lentry=mylfns3.Next();
	CPPUNIT_ASSERT_MESSAGE("wrong LFN","lfn"+names[0]==lentry.lfname());
	std::cout<<lentry<<std::endl;
      }
      CPPUNIT_ASSERT_MESSAGE("wrong number of LFN",lcounter==1);

      mycatalog->commit();
      mycatalog->disconnect();
    }catch(const std::exception& er){
      std::cerr << er.what() << std::endl;
    }
  }
};
  
CPPUNIT_TEST_SUITE_REGISTRATION( crossTableQueryTest );
#include<CppUnit_testdriver.cpp>
