//$Id: importTest.cpp,v 1.20 2007/02/09 13:21:41 ioannis Exp $
/*
  cross catalog operations test for MySQL Catalog.
  Author: Zhen Xie
*/
#include<cppunit/extensions/HelperMacros.h>
#include "FileCatalog/IFileCatalog.h"
#ifndef POOL_FCSYSTEMTOOLS_H
#include "FileCatalog/FCSystemTools.h"
#endif
#include "FileCatalog/IFCAction.h"
#include "FileCatalog/FCEntry.h"
#include "FileCatalog/FCException.h"
#include "FileCatalog/IFCContainer.h"
#include <iostream>
using namespace pool;
class importTest: public CppUnit::TestFixture{
  CPPUNIT_TEST_SUITE( importTest );
  CPPUNIT_TEST( testImport );
  //CPPUNIT_TEST( testSchemadiffImport );
  //CPPUNIT_TEST( testDelete );
  CPPUNIT_TEST_SUITE_END();
public:
  IFileCatalog* sourcecatalog;
  IFileCatalog* destcatalog;
  std::vector<std::string> names;
  FileCatalog::FileID fid, selecfid;
  std::string selecpfn, seleclfn; 
  void setUp(){
    sourcecatalog=new IFileCatalog;
    sourcecatalog->setWriteCatalog("mysqlcatalog_mysql://xiezhen@localhost/zhendb");
    destcatalog=new IFileCatalog;
    destcatalog->setWriteCatalog("mysqlcatalog_mysql://xiezhen@localhost:3306/FCtest");
    for(size_t i=0; i<20; ++i){
      names.push_back("this-is-a-physical-file-name"+FCSystemTools::itostr(i));
    }
  }
  
  void tearDown() {
    if(sourcecatalog!=0) delete sourcecatalog;
    if(destcatalog!=0) delete destcatalog; 
    names.clear();
  }
  
  void testImport(){
    std::cout<<"[OVAL] TEST --> testImport"<<std::endl;
    try{
      FCregister destr;
      destcatalog->connect();
      destcatalog->start();
      destcatalog->setAction(destr);
      for(size_t i=0; i<names.size(); ++i){
	FileCatalog::FileID fid;
	destr.registerPFN(names[i],"root",fid);
	destr.registerLFN(names[i],names[i]+"destlf1");
	destr.registerLFN(names[i],names[i]+"destlf2");
      }
      destcatalog->commit(); 
      std::cout<<"[OVAL] dest catalog populated"<<std::endl;
      
      sourcecatalog->connect();
      FCregister asource;
      FCAdmin asourceadmin;
      MetaDataEntry metaspec;
      metaspec.addAttributeSpec("jobid", "int");
      metaspec.addAttributeSpec("owner", "string");
      sourcecatalog->setAction(asourceadmin);
      sourcecatalog->start();
      try{
	asourceadmin.createMetaDataSpec(metaspec);
      }catch(const pool::FCduplicatemetadataspecException& er){
	std::cout<<"metadata schema already exists, drop it and create it again"<<std::endl;
	asourceadmin.dropMetaDataSpec();
	asourceadmin.createMetaDataSpec(metaspec);
      }
      sourcecatalog->commit();
      std::string pfn,lfn;
      sourcecatalog->start();
      sourcecatalog->setAction(asource);
      for( int n=12; n<20; ++n){
	MetaDataEntry myentry(metaspec);
	myentry.setAttributeValue("jobid",n);
	myentry.setAttributeValue("owner", std::string("cmsprod"));
	selecpfn=std::string("sourcepfn")+FCSystemTools::itostr(n);
	seleclfn=std::string("sourcelfn")+FCSystemTools::itostr(n);
	asource.registerPFN(selecpfn,"root tree",fid);
	if(n==15) selecfid=fid;
	asource.registerLFN(selecpfn,seleclfn);
	asource.registerMetaData(fid,myentry);
      }
      sourcecatalog->commit();
      std::cout<<"[OVAL] source catalog schema created"<<std::endl;

      FCAdmin adestadmin;
      FClookup destl;
      destcatalog->setAction(adestadmin);
      destcatalog->setAction(destl);
      destcatalog->start();
      std::string qstr=std::string("guid=\"")+selecfid+("\"");
      std::cout<<"[OVAL] querystring"<<qstr<<std::endl;
      try{
	adestadmin.createMetaDataSpec(metaspec);
      }catch(const pool::FCduplicatemetadataspecException& er){
	std::cout<<"metadata schema already exists, drop it and create it again"<<std::endl;
	adestadmin.dropMetaDataSpec();
	adestadmin.createMetaDataSpec(metaspec);
      }
      destcatalog->commit();
      std::cout<<"[OVAL] importing sourcecatalog to destcatalog"<<std::endl;
      destcatalog->start();
      destcatalog->importCatalog(sourcecatalog,qstr);
      MetaDataContainer me(destcatalog,100);
      destl.lookupMetaDataByQuery("",me);
      while( me.hasNext() ){
	MetaDataEntry mentry;
	mentry=me.Next();
	CPPUNIT_ASSERT_MESSAGE("wrong fid",mentry.guid()==selecfid );
      }
      destcatalog->commit();
      sourcecatalog->disconnect();
      destcatalog->disconnect();
    }catch (const std::exception& er){
      std::cerr<<er.what()<<std::endl;
    }
  }
};

CPPUNIT_TEST_SUITE_REGISTRATION(importTest);
#include<CppUnit_testdriver.cpp>
