//$Id: insertTest.cpp,v 1.21 2007/02/09 13:21:41 ioannis Exp $
/*
  CPPUnit test for MySQL Catalog.
  Author: Zhen Xie
*/
#include<cppunit/extensions/HelperMacros.h>
#include <string>
#include <iostream>
#include "FileCatalog/IFileCatalog.h"
#include "FileCatalog/FCLeaf.h"
#include "FileCatalog/FCEntry.h"
#include "FileCatalog/FCImpl.h"
#include "FileCatalog/IFCAction.h"
#include "FileCatalog/FCException.h"
using namespace pool;

class insertTest: public CppUnit::TestFixture{
  CPPUNIT_TEST_SUITE( insertTest );
  CPPUNIT_TEST( testInsert );
  // CPPUNIT_TEST( testDelete );
  CPPUNIT_TEST_SUITE_END();
public:
  IFileCatalog* mycatalog;
  std::vector<std::string> names;
  FileCatalog::FileID fid;
  std::string pfn,ftype;

  void setUp(){
    mycatalog=new IFileCatalog;
    mycatalog->setWriteCatalog("mysqlcatalog_mysql://xiezhen@localhost/zhendb");
    std::string bigname;
    std::string sname("abcdefghij");
    for (unsigned int i=0; i<24; ++i){
      bigname+=sname;
    }
    names.push_back(bigname);
    names.push_back("this-is-a-pfn-test1");
    names.push_back("this-is-a-pfn-test2");
    names.push_back("this-is-a-pfn-test3");
  }
   void tearDown() {
    if(mycatalog!=0) delete mycatalog;
    names.clear();
  }
  void testInsert(){
    std::cout<<"[OVAL] TEST --> testInsert"<<std::endl;
    try{ 
      mycatalog->connect();
      mycatalog->start();
      for(size_t i=0; i<names.size(); ++i){
	PFNEntry pentry(names[i],"1234","root/tree");
	LFNEntry lentry(names[i]+":lfn","1234");
	try{
	  static_cast<FCLeaf*>(mycatalog->getWriteCatalog())->getImpl()->insertPFN(pentry);
	  std::cout<<"[OVAL] PFN inserted: "<<names[i]<<std::endl;
	}catch(const FCduplicatePFNException& per){
	  static_cast<FCLeaf*>(mycatalog->getWriteCatalog())->getImpl()->deletePFN(names[i]);
	  std::cout<<"[OVAL] duplicate PFN removed: "<<names[i]<<std::endl;
	  static_cast<FCLeaf*>(mycatalog->getWriteCatalog())->getImpl()->insertPFN(pentry);
	  std::cout<<"[OVAL] duplicate PFN inserted: "<<names[i]<<std::endl;
	}
	try{
	  static_cast<FCLeaf*>(mycatalog->getWriteCatalog())->getImpl()->insertLFN(lentry);
	  std::cout<<"[OVAL] LFN inserted: "<<names[i]+":lfn"<<std::endl;
	}catch(const FCduplicateLFNException& ler){
	  static_cast<FCLeaf*>(mycatalog->getWriteCatalog())->getImpl()->deleteLFN(names[i]+":lfn");
	  std::cout<<"[OVAL] duplicated LFN removed: "<<names[i]+":lfn"<<std::endl;
	  static_cast<FCLeaf*>(mycatalog->getWriteCatalog())->getImpl()->insertLFN(lentry);
	  std::cout<<"[OVAL] duplicated LFN inserted: "<<names[i]+":lfn"<<std::endl;
	}
      }
      mycatalog->commit(); 
    }catch(const std::exception& er){
      std::cerr << er.what() << std::endl;
    }
  }
  
  void testDelete(){
    std::cout<<"[OVAL]TEST --> testDelete"<<std::endl;
    try{
      FCAdmin a;
      mycatalog->setAction(a);
      mycatalog->connect();
      mycatalog->start();
      for(size_t i=0; i<names.size(); ++i){
	a.deletePFN(names[i]);
	std::cout<<"[OVAL] PFN deleted: "<<names[i]<<std::endl;
	a.deleteLFN(names[i]+":lfn");
	std::cout<<"[OVAL] LFN deleted: "<<names[i]+":lfn"<<std::endl;
    }
      mycatalog->commit();
    }catch(const std::exception& er){
      std::cerr << er.what() << std::endl;
    }
  }
};

CPPUNIT_TEST_SUITE_REGISTRATION(insertTest);
#include<CppUnit_testdriver.cpp>
