//$Id: lookupTest.cpp,v 1.21 2007/02/09 13:21:41 ioannis Exp $
/*
  CPPUnit for MySQL Catalog.
  Author: Zhen Xie
*/
#include<cppunit/extensions/HelperMacros.h>
#ifndef POOL_MYSQLFILECATALOG_H
#include "MySQLCatalog/MySQLFileCatalog.h"
#endif
#include "FileCatalog/IFCContainer.h"
#include "FileCatalog/IFCAction.h"
#include "FileCatalog/FCSystemTools.h"
using namespace pool;

class lookupTest: public CppUnit::TestFixture{
  CPPUNIT_TEST_SUITE( lookupTest );
  CPPUNIT_TEST( testLookup );
  //CPPUNIT_TEST( testDelete );
  CPPUNIT_TEST_SUITE_END();
public:
  IFileCatalog* mycatalog;
  std::vector<std::string> names;
  FileCatalog::FileID fid;
  std::vector<FileCatalog::FileID> fids;
  std::string pfn,ftype;

  void setUp(){
    mycatalog=new IFileCatalog;
    mycatalog->setWriteCatalog("mysqlcatalog_mysql://xiezhen@localhost/zhendb");
    names.push_back("this-is-a-pfn-test1");
    names.push_back("this-is-a-pfn-test2");
    names.push_back("this-is-a-pfn-test3");
  }
  void tearDown() {
    if(mycatalog!=0) delete mycatalog;
    names.clear();
  }
  void testLookup(){
    std::cout<<"[OVAL]TEST --> testRegister"<<std::endl;
    try{
      mycatalog->connect();
      mycatalog->start();
      FCregister r;
      mycatalog->setAction(r);
      for(size_t i=0; i<names.size(); ++i){
	pfn=names[i]+std::string("0");
	std::cout<<"[OVAL] registerFile "<<i<<std::endl;
	r.registerPFN(pfn,"root tree", fid);
	fids.push_back(fid);
	r.addReplicaPFN(pfn,names[i]+"1");
        r.addReplicaPFN(pfn,names[i]+"2");
	r.registerLFN(pfn,names[i]+"lfn"+"0");
        r.registerLFN(pfn,names[i]+"lfn"+"1");
      }
      FClookup l;
      mycatalog->setAction(l);
      for(size_t i=0; i<fids.size(); ++i){
	std::cout<<"[OVAL] lookupPFN"<<std::endl;
	PFNContainer mypfns(mycatalog,3);
	l.lookupPFN(fids[i],mypfns);
	unsigned int counter=0;
	while(mypfns.hasNext()){
	  PFNEntry pentry=mypfns.Next();
	  pfn=pentry.pfname();
	  ftype=pentry.filetype();
	  CPPUNIT_ASSERT_MESSAGE("wrong PFN",(names[i]+FCSystemTools::itostr(counter))==pfn);
	  CPPUNIT_ASSERT_MESSAGE("wrong guid",fids[i]==pentry.guid());
	  CPPUNIT_ASSERT_MESSAGE("wrong type","root tree"==ftype);
	  std::cout<<pentry<<std::endl;
	  ++counter;
	}
	std::cout<<"number of pfns "<<counter<<std::endl;
	CPPUNIT_ASSERT_MESSAGE("wrong number of pfn",counter==3);

	std::cout<<"[OVAL] lookupLFN "<<std::endl;
	LFNContainer mylfns(mycatalog,3);
	l.lookupLFN(fids[i],mylfns);
	unsigned int lfcounter=0;
	while(mylfns.hasNext()){
	  LFNEntry lentry=mylfns.Next();
	  CPPUNIT_ASSERT_MESSAGE("wrong LFN",(names[i]+"lfn"+FCSystemTools::itostr(lfcounter))==lentry.lfname());
	  CPPUNIT_ASSERT_MESSAGE("wrong guid",fids[i]==lentry.guid());
	  std::cout<<lentry<<std::endl;
	  ++lfcounter;
	}
	std::cout<<"number of lfns "<<lfcounter<<std::endl;
	CPPUNIT_ASSERT_MESSAGE("wrong number of lfn",lfcounter==2);
      }

      std::string query;
      std::cout<<"[OVAL] lookupPFNByQuery"<<query<<std::endl;
      PFNContainer pfs(mycatalog);
      l.lookupPFNByQuery(query,pfs);
      unsigned int n=0;
      while(pfs.hasNext()){
	PFNEntry pentry=pfs.Next();
	std::cout<<pentry.pfname()<<std::endl;
	++n;
      }
      std::cout<<"number of pfns "<<n<<std::endl;
      CPPUNIT_ASSERT_MESSAGE("wrong number of pfn",n==9);

      query=std::string("guid=")+"'"+fids[0]+"'";
      std::cout<<"[OVAL] lookupLFNByQuery"<<std::endl;
      LFNContainer lfs(mycatalog);
      l.lookupLFNByQuery(query,lfs);
      n=0;
      while(lfs.hasNext()){
	LFNEntry lentry=lfs.Next();
	std::cout<<lentry.lfname()<<std::endl;
	++n;
      }
      std::cout<<"number of lfns "<<n<<std::endl;
      CPPUNIT_ASSERT_MESSAGE("wrong number of lfn",n==2);

      std::cout<<"[OVAL] lookupFileByQuery"<<std::endl;
      GuidContainer myguids(mycatalog);
      l.lookupFileByQuery("",myguids);
      n=0;
      while(myguids.hasNext()){
	FileCatalog::FileID g=myguids.Next();
	++n;
      }
      CPPUNIT_ASSERT_MESSAGE("wrong number of guid",n==3);

      mycatalog->commit(); 
      mycatalog->disconnect();
    }catch(const std::exception& er){
      std::cerr << er.what() << std::endl;
    }
  }
};
  
CPPUNIT_TEST_SUITE_REGISTRATION(lookupTest);
#include<CppUnit_testdriver.cpp>
