//$Id: metadataTest.cpp,v 1.19 2007/02/09 13:21:42 ioannis Exp $
/*
  CPPUnit test of metadata for MySQL Catalog.
  Author: Zhen Xie
*/
#include<cppunit/extensions/HelperMacros.h>
#include <string>
#include <vector>
#include <iostream>
#ifndef POOL_MYSQLFILECATALOG_H
#include "MySQLCatalog/MySQLFileCatalog.h"
#endif
#ifndef POOL_FCSYSTEMTOOLS_H
#include "FileCatalog/FCSystemTools.h"
#endif
#include "FileCatalog/FCEntry.h"
#include "FileCatalog/IFCAction.h"
#include "FileCatalog/FCException.h"
#include "FileCatalog/IFCContainer.h"
using namespace pool;
class metadataTest: public CppUnit::TestFixture{
  CPPUNIT_TEST_SUITE( metadataTest );
  CPPUNIT_TEST( testCreateSchema );
  CPPUNIT_TEST( testUpdateSchema );
  CPPUNIT_TEST( testMetadata );
  //CPPUNIT_TEST( testDelete );
  CPPUNIT_TEST_SUITE_END();
public:
  IFileCatalog* mycatalog;
  std::vector<std::string> names;
  FileCatalog::FileID fid;
  MetaDataEntry abx;
  MetaDataEntry abc;
  void setUp(){
    mycatalog=new IFileCatalog;
    mycatalog->setWriteCatalog("mysqlcatalog_mysql://xiezhen@localhost/zhendb");
    for(size_t i=0; i<20; ++i){
      names.push_back("this-is-a-physical-file-name"+FCSystemTools::itostr(i));
    }
    abx.addAttributeSpec("a","int");
    abx.addAttributeSpec("b","int");
    abx.addAttributeSpec("x", "string");
    abc.addAttributeSpec("a","int");
    abc.addAttributeSpec("b","int");
    abc.addAttributeSpec("c", "string");
  }
  void tearDown() {
    if(mycatalog!=0) delete mycatalog;
    names.clear();
  }
  /*Test createMetaDataSpec, dropMetaDataSpec and 
    FCduplicatemetadataspecException.
    inner try block: create metadataspec
      -if it's nonexistant, create from scratch
      -if it exists, handle exception thrown from createMetaDataSpec:
          first drop the metadatspec then create a new one. 
  **/
  void testCreateSchema(){
    std::cout<<"[OVAL] TEST -->testCreateSchema"<<std::endl;
    try{
      FCAdmin a;
      mycatalog->setAction(a);
      mycatalog->connect();
      mycatalog->start();
      try{
	a.createMetaDataSpec(abx);
      }catch(const pool::FCduplicatemetadataspecException& er){
	std::cout<<"metadata schema already exists, drop it and create it again"<<std::endl;
	a.dropMetaDataSpec();
	a.createMetaDataSpec(abx);
      }
      MetaDataEntry mentry;
      a.getMetaDataSpec(mentry);
      mycatalog->commit();
      mycatalog->disconnect();
    }catch (const std::exception& er){
      std::cerr << er.what() << std::endl;
    }
  }

  void testUpdateSchema(){
    std::cout<<"[OVAL] TEST -->testUpdateSchema"<<std::endl; 
    try{
      FCAdmin a;
      mycatalog->setAction(a);
      mycatalog->connect();
      MetaDataEntry abcx,result1,result2,result3;
      abcx=abx.append_and_merge_spec(abc);
      mycatalog->start();
      a.updateMetaDataSpec(abcx);
      a.getMetaDataSpec(result1);
      CPPUNIT_ASSERT_MESSAGE("case 1 failed",abcx.spec()==result1.spec());
      mycatalog->commit();
      
      mycatalog->start();
      a.updateMetaDataSpec(abx);
      a.getMetaDataSpec(result2);
      CPPUNIT_ASSERT_MESSAGE("case 2 failed",abcx.spec()==result2.spec());
      mycatalog->commit();
      mycatalog->start();
      a.updateMetaDataSpec(abx,FileCatalog::DELETE_REDUNDANT);
      a.getMetaDataSpec(result3);
      CPPUNIT_ASSERT_MESSAGE("case 3 failed",abx.spec()==result3.spec());
      mycatalog->commit();

      mycatalog->disconnect();
    }catch (const std::exception& er){
      std::cerr << er.what() << std::endl;
    }
  }
  /*Test Metadata queries
  **/
  void testMetadata(){
    std::cout<<"[OVAL] TEST -->testMetadata"<<std::endl; 
    try{
      FCAdmin a;
      FCregister r;
      FClookup l;
      mycatalog->setAction(a);
      mycatalog->setAction(r);
      mycatalog->connect();
      MetaDataEntry mspec;
      mspec.addAttributeSpec("jobid", "int");
      mspec.addAttributeSpec("owner", "string");
      mycatalog->start();
      a.dropMetaDataSpec();
      a.createMetaDataSpec(mspec);
      mycatalog->commit();
      mycatalog->start();
      for( int n=12; n<20; ++n){
	MetaDataEntry mentry( mspec );
	mentry.setAttributeValue("jobid",n);
	mentry.setAttributeValue("owner",std::string("cmsprod"));
	std::string guidvalue=std::string("guid")+FCSystemTools::itostr(n);
	mentry.setGuidValue( guidvalue );
	r.registerMetaData(guidvalue, mentry);
      }
      mycatalog->commit();
      mycatalog->start();
      MetaDataEntry myspec;
      a.getMetaDataSpec(myspec);
      MetaDataContainer metas(mycatalog,3);
      l.lookupMetaDataByQuery("",metas);
      while( metas.hasNext() ){
	MetaDataEntry mm( metas.Next() );
      }
      mycatalog->commit();
      mycatalog->disconnect();
    }catch (const std::exception& er){
      std::cerr << er.what() << std::endl;
    }
  }
  void testDelete(){
  }  
};
CPPUNIT_TEST_SUITE_REGISTRATION(metadataTest);
#include<CppUnit_testdriver.cpp>
