#include "POOLCore/PoolMessageStream.h"
#include "POOLCore/POOLContext.h"
#include <cstdlib>

namespace {
  struct _Init {
    const char* m_log_level;
    seal::Msg::Level m_out_level;
    _Init()  {
      m_log_level = ::getenv( "POOL_OUTMSG_LEVEL" );
      if ( m_log_level )  {
        switch( ::toupper(m_log_level[0]) )  {
          case 'N':
            m_out_level = seal::Msg::Nil;
             break;
          case 'V':
            m_out_level = seal::Msg::Verbose;
            break;
          case 'D':
            m_out_level = seal::Msg::Debug;
            break;
          case 'I':
            m_out_level = seal::Msg::Info;
            break;
          case 'W':
            m_out_level = seal::Msg::Warning;
            break;
          case 'E':
            m_out_level = seal::Msg::Error;
            break;
          case 'F':
            m_out_level = seal::Msg::Fatal;
            break;
          default:
            m_out_level = static_cast < seal::Msg::Level > ( ::atoi(m_log_level) );
            break;
        }
      }
    }
  };
  static _Init s_init;
}

namespace pool  {
  void setLogLevel(seal::Msg::Level lvl)  {
    s_init.m_out_level = lvl;
  }
  seal::Msg::Level getLogLevel()   {
    return s_init.m_out_level;
  }
}

pool::PoolMessageStream::PoolMessageStream( const std::string& streamIdentifier,
					    seal::Msg::Level threshold ):
  seal::MessageStream( pool::POOLContext::context(),
		       streamIdentifier,
		       threshold )
{
  this->setThreshold( s_init.m_out_level );
}
