#ifndef POOL_ISHAPETRANSFORMATIONREGISTRY_H
#define POOL_ISHAPETRANSFORMATIONREGISTRY_H

namespace ROOT { namespace Reflex { class Type; } }

/** 
    pool namespace.
*/
namespace pool  {
  class Guid;
  class DataCallBack;

  /** class IShapeTransformationRegistry
   *  Abstract class for the registring of callback objects
   *  for the transient and persistent shapes.
   */
  class IShapeTransformationRegistry {
  public:
    virtual ~IShapeTransformationRegistry() {}

    /// Register a new transformation object. Returns false if a transformation already exists, true otherwise.
    virtual bool registerTransformation( const pool::Guid& object_type, 
                                         const ROOT::Reflex::Type& shape_type,
                                         const DataCallBack& callback ) = 0;

    /// Register a new transformation object. Returns false if a transformation already exists, true otherwise.
    virtual bool registerTransformation( const ROOT::Reflex::Type& object_type, 
                                         const ROOT::Reflex::Type& shape_type,
                                         const DataCallBack& callback ) = 0;

    /// Deregisters an existing transformation object. Returns false if a transformation does not exist, true otherwise.
    virtual bool deregisterTransformation( const pool::Guid& object_type,
                                           const ROOT::Reflex::Type& shape_type ) = 0;

    /// Deregisters an existing transformation object. Returns false if a transformation does not exist, true otherwise.
    virtual bool deregisterTransformation( const ROOT::Reflex::Type& object_type,
                                           const ROOT::Reflex::Type& shape_type ) = 0;
  };
}
#endif // POOL_ISHAPETRANSFORMATIONREGISTRY_H
