#ifndef INCLUDE_PERSISTENCYSVC_ITRANSACTIONOBSERVER_H
#define INCLUDE_PERSISTENCYSVC_ITRANSACTIONOBSERVER_H

namespace pool {

  /** @class ITransactionObserver ITransactionObserver.h PersistencySvc/ITransactionObserver.h
   *
   *  ITransactionObserver is the interface class for observers of the transaction
   *  activities. Each observer which is registered with the IGlobalTransaction gets
   *  called after every transaction operation.
   */
  class ITransactionObserver {
  public:
    /// Default destructor
    virtual ~ITransactionObserver() {}

    /// Responds to a start of a read transaction
    virtual bool respondToStartRead() = 0;

    /// Responds to a start of an update transaction
    virtual bool respondToStartUpdate() = 0;

    /// Responds to a committing of a transaction
    virtual bool respondToCommit() = 0;

    /// Responds to an aborting of a transaction
    virtual bool respondToRollBack() = 0;

    /// Responds to the reverting of a transaction into update mode
    virtual bool respondToUpdate() = 0;
  };
}

#endif
