#ifndef INCLUDE_PERSISTENCYSVC_GLOBALTRANSACTION_H
#define INCLUDE_PERSISTENCYSVC_GLOBALTRANSACTION_H

#include "IGlobalTransaction.h"
#include "DatabaseRegistry.h"
#include <list>

namespace pool {
  namespace PersistencySvc {

    /** @class GlobalTransaction
     *
     *  GlobalTransaction is an implementation of the IGlobalTransaction interface.
     *
     */
    class GlobalTransaction : virtual public IGlobalTransaction {
    public:
      /// Constructor
      GlobalTransaction( DatabaseRegistry& registry );

      /// Destructor
      virtual ~GlobalTransaction();

      /// Registers an observer
      bool registerObserver( ITransactionObserver& observer );

      /// Deregisters an observer
      bool deregisterObserver( ITransactionObserver& observer );

      /// Starts a new transaction. Returns the success of the operation
      bool start( ITransaction::Type type = READ );

      /// Commits the transaction.
      bool commit();

      /// Commits and holds the transaction.
      bool commitAndHold();

      /// Rolls back the transaction
      void rollback();

      /// Checks if the transaction is active
      bool isActive() const;

      /// Reverts to update mode
      void update();

      /// Returns the transaction type
      ITransaction::Type type() const;

    private:
      /// The current transaction type
      ITransaction::Type m_type;

      /// The registered databases.
      DatabaseRegistry&  m_databases;

      /// The registered observers
      std::list< ITransactionObserver* > m_observers;
    };
  }
}

#endif
