//$Id: PersistencySvc.cpp,v 1.48 2006/01/24 10:36:56 govi Exp $

// includes
#include "PersistencySvc.h"
#include "UserSession.h"
#include "IGlobalTransaction.h"
#include "PersistencySvcConfiguration.h"
#include "UserDatabase.h"
#include "DatabaseHandler.h"
#include "PersistencySvc/DatabaseSpecification.h"
#include "PersistencySvc/ITokenValidator.h"
#include "POOLCore/Token.h"

pool::PersistencySvc::PersistencySvc::PersistencySvc( pool::IFileCatalog& fileCatalog ):
  pool::IPersistencySvc( fileCatalog ),
  m_session( new pool::PersistencySvc::UserSession( fileCatalog ) )
{}

pool::PersistencySvc::PersistencySvc::~PersistencySvc()
{
  delete m_session;
}

void
pool::PersistencySvc::PersistencySvc::setFileCatalog( pool::IFileCatalog& catalog )
{
  m_session->configuration().setFileCatalog( catalog );
}

pool::IFileCatalog&
pool::PersistencySvc::PersistencySvc::fileCatalog()
{
  return m_session->configuration().fileCatalog();
}

void
pool::PersistencySvc::PersistencySvc::setTokenValidator( pool::ITokenValidator& validator )
{
  m_session->configuration().setTokenValidator( validator );
}

pool::ITokenValidator&
pool::PersistencySvc::PersistencySvc::tokenValidator()
{
  return m_session->configuration().tokenValidator();
}

void
pool::PersistencySvc::PersistencySvc::setDataTransform( pool::IDataTransform& transform )
{
  m_session->configuration().setDataTransform( transform );
}

pool::IDataTransform&
pool::PersistencySvc::PersistencySvc::dataTransform()
{
  return m_session->configuration().dataTransform();
}

std::pair< void*, ROOT::Reflex::Type >
pool::PersistencySvc::PersistencySvc::readObject( const pool::Guid& transientType,
                                                  const pool::Token& token,
                                                  pool::DataHandler* handler )
{
  std::pair< void*, ROOT::Reflex::Type > result( 0, 0 );
  if ( m_session->transaction().isActive() ) {
    pool::Token* pToken = m_session->configuration().tokenValidator().validate( token );
    UserDatabase db( m_session->technologyDispatcher(),
                     m_session->configuration(),
                     m_session->transaction(),
                     m_session->registry(),
                     pToken->dbID(),
                     pool::DatabaseSpecification::FID );
    if ( db.openMode() == pool::IDatabase::CLOSED ) {
      db.setTechnology( pToken->technology() );
      db.connectForRead();
    }
    result = db.databaseHandler().readObject( m_session->configuration().dataTransform(),
                                              transientType,
                                              *pToken,
                                              handler );
    pToken->release();
  }
  return result;
}

std::pair< void*, ROOT::Reflex::Type >
pool::PersistencySvc::PersistencySvc::readObject( const ROOT::Reflex::Type& transientType,
                                                  const pool::Token& token,
                                                  pool::DataHandler* handler )
{
  std::pair< void*, ROOT::Reflex::Type > result( 0, 0 );
  if ( m_session->transaction().isActive() ) {
    pool::Token* pToken = m_session->configuration().tokenValidator().validate( token );
    UserDatabase db( m_session->technologyDispatcher(),
                     m_session->configuration(),
                     m_session->transaction(),
                     m_session->registry(),
                     pToken->dbID(),
                     pool::DatabaseSpecification::FID );
    if ( db.openMode() == pool::IDatabase::CLOSED ) {
      db.setTechnology( pToken->technology() );
      db.connectForRead();
    }
    result = db.databaseHandler().readObject( m_session->configuration().dataTransform(),
                                              transientType,
                                              *pToken,
                                              handler );
    pToken->release();
  }
  return result;
}

std::pair< void*, ROOT::Reflex::Type >
pool::PersistencySvc::PersistencySvc::readObject( const pool::Token& token,
                                                  pool::DataHandler* handler )
{
  std::pair< void*, ROOT::Reflex::Type > result( 0, 0 );
  if ( m_session->transaction().isActive() ) {
    pool::Token* pToken = m_session->configuration().tokenValidator().validate( token );
    UserDatabase db( m_session->technologyDispatcher(),
                     m_session->configuration(),
                     m_session->transaction(),
                     m_session->registry(),
                     pToken->dbID(),
                     pool::DatabaseSpecification::FID );
    if ( db.openMode() == pool::IDatabase::CLOSED ) {
      db.setTechnology( pToken->technology() );
      db.connectForRead();
    }
    result = db.databaseHandler().readObject( m_session->configuration().dataTransform(),
                                              *pToken,
                                              handler );
    pToken->release();
  }
  return result;
}

pool::Token*
pool::PersistencySvc::PersistencySvc::registerForWrite( const pool::Placement& place,
                                                        const void* object,
                                                        const ROOT::Reflex::Type& type,
                                                        pool::DataHandler* handler )
{
  if ( ! m_session->transaction().isActive() ||
       m_session->transaction().type() != pool::ITransaction::UPDATE ) return 0;
  UserDatabase db( m_session->technologyDispatcher(),
                   m_session->configuration(),
                   m_session->transaction(),
                   m_session->registry(),
                   place.databaseName(),
                   place.databaseNameType() );
  pool::IDatabase::OpenMode openMode = db.openMode();
  switch ( openMode ) {
  case pool::IDatabase::CLOSED:
    db.setTechnology( place.technology() );
    db.connectForWrite();
    break;
  case pool::IDatabase::READ:
    db.revertMode( pool::IDatabase::UPDATE );
    break;
  default:
    break;
  };
  return db.databaseHandler().writeObject( m_session->configuration().dataTransform(),
                                           place.containerName(),
                                           place.shapeID(),
                                           place.technology(),
                                           object,
                                           type,
                                           handler );
}

bool
pool::PersistencySvc::PersistencySvc::updateObject( const void* object,
                                                    const ROOT::Reflex::Type& type,
                                                    const pool::Token& token,
                                                    pool::DataHandler* handler )
{
  if ( ! m_session->transaction().isActive() ||
       m_session->transaction().type() != pool::ITransaction::UPDATE ) return false;
  UserDatabase db( m_session->technologyDispatcher(),
                   m_session->configuration(),
                   m_session->transaction(),
                   m_session->registry(),
                   token.dbID(),
                   pool::DatabaseSpecification::FID );
  pool::DatabaseConnectionPolicy policy; // Keep the default behaviour.
  pool::IDatabase::OpenMode openMode = db.openMode();
  switch ( openMode ) {
  case pool::IDatabase::CLOSED:
    db.setTechnology( token.technology() );
    db.connectForWrite( policy );
    break;
  case pool::IDatabase::READ:
    db.revertMode( pool::IDatabase::UPDATE );
    break;
  default:
    break;
  };
  return db.databaseHandler().updateObject( m_session->configuration().dataTransform(),
                                            object,
                                            type,
                                            token,
                                            handler );
}

bool
pool::PersistencySvc::PersistencySvc::destroyObject( const pool::Token& token )
{
  if ( ! m_session->transaction().isActive() ||
       m_session->transaction().type() != pool::ITransaction::UPDATE ) return false;
  UserDatabase db( m_session->technologyDispatcher(),
                   m_session->configuration(),
                   m_session->transaction(),
                   m_session->registry(),
                   token.dbID(),
                   pool::DatabaseSpecification::FID );
  pool::DatabaseConnectionPolicy policy; // Keep the default behaviour.
  pool::IDatabase::OpenMode openMode = db.openMode();
  switch ( openMode ) {
  case pool::IDatabase::CLOSED:
    db.setTechnology( token.technology() );
    db.connectForWrite( policy );
    break;
  case pool::IDatabase::READ:
    db.revertMode( pool::IDatabase::UPDATE );
    break;
  default:
    break;
  };
  return db.databaseHandler().destroyObject( token );
}

pool::ISession&
pool::PersistencySvc::PersistencySvc::session()
{
  return static_cast< pool::ISession& >( *m_session );
}


bool
pool::PersistencySvc::PersistencySvc::registerObserver( ITransactionObserver& observer )
{
  return m_session->globalTransaction().registerObserver( observer );
}

bool
pool::PersistencySvc::PersistencySvc::deregisterObserver( ITransactionObserver& observer )
{
  return m_session->globalTransaction().deregisterObserver( observer );
}
