import sys, DLFCN
sys.setdlopenflags( DLFCN.RTLD_NOW | DLFCN.RTLD_GLOBAL )
from FileCatalog import *
import PyFCContainer, PyFileCatalog, PyFileCatalogError
class PyFCAction(object):
    def __init__(self):
        self.writecatalog=0
        self.readcatalogs=[]
        self.catalog=0
    def visitFCLeaf(self, catalog):
        self.catalog=catalog
        if catalog.wchild!=0:
            self.writecatalog=catalog.wchild
        else:
            self.readcatalogs.append(catalog.rchildren[0])
    def visitFCComposite(self, catalog):
        self.catalog=catalog
        self.writecatalog=catalog.wchild
        self.readcatalogs=catalog.rchildren
    def isWritableEntry(self, guid):
        if self.writecatalog==0 : return 0
        if self.writecatalog.isReadOnly():
            return 0
        else:
            pfn=self.writecatalog.lookupBestPFN(guid)
            if len(pfn)!=0 :
                return 1
            else:
                return 0
    def registerPFN(self, pfn, filetype):
        if self.writecatalog!=0:
            if self.writecatalog.isReadOnly()==1:
                raise isReadOnlyCatalogError
            return self.writecatalog.registerPFN(pfn,filetype)
    def lookupFileByPFN(self, pfn):
        if self.writecatalog!=0:
            self.writecatalog.lookupFileByPFN(pfn)
            guid=self.writecatalog.lookupFileByPFN(pfn)
            if len(guid)!=0: return guid
        if len(self.readcatalogs)!=0:
            for cat in self.readcatalogs:
                guid=cat.lookupFileByPFN(pfn)
                if len(guid)!=0: return guid
        
    def lookupFileByLFN(self, lfn):
        if self.writecatalog!=0:
            self.writecatalog.lookupFileByLFN(lfn)
            guid=self.writecatalog.lookupFileByLFN(lfn)
            if len(guid)!=0: return guid
        if len(self.readcatalogs)!=0:
            for cat in self.readcatalogs:
                guid=cat.lookupFileByLFN(lfn)
                if len(guid)!=0: return guid
        
    def lookupBestPFN(self, guid, omode=READ, amode=SEQUENTIAL):
        if self.writecatalog!=0:
            pfn=self.writecatalog.lookupBestPFN(guid)
            if len(pfn)!=0: return pfn
        if len(self.readcatalogs)!=0:
            for cat in self.readcatalogs:
                pfn=cat.lookupBestPFN(guid)
                if len(pfn['pfn'])!=0: return pfn
        
    def getMetaDataSpec(self):
        if self.writecatalog!=0:
            return self.writecatalog.getMetaDataSpec()
        else:
            if self.readcatalogs[0]!=0:
                return self.readcatalogs[0].getMetaDataSpec()
    
class PyFCregister(PyFCAction):
    def __init__(self):
        PyFCAction.__init__(self)
    def registerLFN(self, pfn,lfn):
        if self.writecatalog!=0:
            if self.writecatalog.isReadOnly()==1:
                raise isReadOnlyCatalogError 
            self.writecatalog.registerLFN(pfn,lfn)
    def addReplicaPFN(self, pfn, rpfn):
        if self.writecatalog!=0:
            if self.writecatalog.isReadOnly()==1:
                raise isReadOnlyCatalogError 
            self.writecatalog.addReplicaPFN(pfn,rpfn)    
    def renamePFN(self, pfn, npfn):
        if self.writecatalog!=0:
            if self.writecatalog.isReadOnly()==1:
                raise isReadOnlyCatalogError 
            self.writecatalog.renamePFN(pfn,npfn)
    def registerMetaData(self, fid, metadict):
        if self.writecatalog!=0:
            if self.writecatalog.isReadOnly()==1:
                raise isReadOnlyCatalogError 
            metadict['guid']=fid
            self.writecatalog.insertMetaData(fid,metadict)

class PyFClookup(PyFCAction):
    def __init__(self):
        PyFCAction.__init__(self)
    def lookupPFN(self, guid, pfncontainer):
        if len(guid)==0: return
        pfncontainer.setQuery("guid="+"'"+guid+"'")
    def lookupLFN(self, guid, lfncontainer):
        if len(guid)==0: return
        lfncontainer.setQuery("guid="+"'"+guid+"'")
    def lookupPFNByQuery(self, query, pfncontainer):
        pfncontainer.setQuery(query)
    def lookupLFNByQuery(self, query, lfncontainer):
        lfncontainer.setQuery(query)
    def lookupPFNByLFN(self, lfn, pfncontainer):
        if len(lfn)==0: return
        pfncontainer.setQuery("lfname="+"'"+lfn+"'")
    def lookupLFNByPFN(self, pfn, lfncontainer):
        if len(pfn)==0: return
        lfncontainer.setQuery("pfname="+"'"+pfn+"'")
    def lookupMetaDataByQuery(self, query, metadatacontainer):
        metadatacontainer.setQuery(query)
    def lookupFileByQuery(self, query, guidcontainer):
        guidcontainer.setQuery(query)
    def lookupEntryByQuery(self, query, cachesize):
        guidcontainer=PyFCContainer.PyFCContainer(self.catalog,GUID,cachesize)
        self.lookupFileByQuery(query,guidcontainer)
        dictlist=[]
        while guidcontainer.hasNext():
            dict={}
            myguid=guidcontainer.Next()
            #warning:cache uses default size here
            pfns=PyFCContainer.PyFCContainer(self.catalog,PFN)
            lfns=PyFCContainer.PyFCContainer(self.catalog,LFN)
            metas=PyFCContainer.PyFCContainer(self.catalog,META)
            self.lookupPFN(myguid,pfns)
            self.lookupLFN(myguid,lfns)
            self.lookupMetaDataByQuery('guid='+"'"+myguid+"'",metas)
            dict['guid']=myguid
            pfnlist=[]
            while pfns.hasNext():
                pfnentry=pfns.Next()
                pfnlist.append((pfnentry[0],pfnentry[2]))
                dict['pfns']=pfnlist
            lfnlist=[]
            while lfns.hasNext():
                lfnentry=lfns.Next()
                lfnlist.append(lfnentry[0])
                dict['lfns']=lfnlist
            while metas.hasNext():
                metaentry=metas.Next()
                for i in metaentry.iterkeys():
                    if i !='guid':
                        dict[i]=metaentry[i]
            dictlist.append(dict)
        return dictlist
class PyFCAdmin(PyFCAction):
    def __init__(self):
        PyFCAction.__init__(self)
    def createMetaDataSpec(self, metadefdict):
        if self.writecatalog!=0:
            if self.writecatalog.isReadOnly()==1:
                raise isReadOnlyCatalogError 
            self.writecatalog.createMetaDataSpec(metadefdict)
    def updateMetaDataSpec(self, metadefdict, metamode=NO_DELETE):
        if self.writecatalog!=0:
            if self.writecatalog.isReadOnly()==1:
                raise isReadOnlyCatalogError 
            self.writecatalog.updateMetaDataSpec(metadefdict,metamode)
    def dropMetaDataSpec(self):
        if self.writecatalog!=0:
            if self.writecatalog.isReadOnly()==1:
                raise isReadOnlyCatalogError 
            self.writecatalog.dropMetaDataSpec()
    def deletePFN(self, pfn):
        if self.writecatalog!=0:
            if self.writecatalog.isReadOnly()==1:
                raise isReadOnlyCatalogError 
            self.writecatalog.deletePFN(pfn)
    def deleteLFN(self, lfn):
        if self.writecatalog!=0:
            if self.writecatalog.isReadOnly()==1:
                raise isReadOnlyCatalogError 
            self.writecatalog.deleteLFN(lfn)
    def deleteMetaData(self, guid):
        if self.writecatalog!=0:
            if self.writecatalog.isReadOnly()==1:
                raise isReadOnlyCatalogError 
            self.writecatalog.deleteMetaData(guid)
    def deleteEntry(self, guid):
        if self.writecatalog!=0:
            if self.writecatalog.isReadOnly()==1:
                raise isReadOnlyCatalogError 
            self.writecatalog.deleteEntry(guid)
            
