#include "RootCollection.h"

#include "RootCollection/AttributeListLayout.h"

#include "CollectionBase/ICollectionDescription.h"
#include "CollectionBase/CollectionDescription.h"
#include "CollectionBase/CollectionFragment.h"
#include "CollectionBase/CollectionColumn.h"

#include "POOLCore/PoolMessageStream.h"
#include "CoralBase/Attribute.h"
#include "CoralBase/AttributeList.h"

ClassImp(AttributeListLayout)

using namespace pool;
   
AttributeListLayout::AttributeListLayout()
{}

AttributeListLayout::~AttributeListLayout()
{}

AttributeListLayout::AttributeListLayout( const pool::ICollectionDescription& desc )
{
   for( int f_id = 0; f_id < desc.numberOfCollectionFragments(); f_id++ ) {

      for( int col_id = 0; col_id < desc.numberOfTokenColumns( f_id ); col_id++ ) {
         const ICollectionColumn&        column = desc.tokenColumn(col_id, f_id);
         m_layout.push_back( make_pair( column.name(), column.type() ) );
         m_annotations.push_back( column.annotation() );
      }

         
      for( int col_id = 0; col_id < desc.numberOfAttributeColumns( f_id ); col_id++ ) {
         const ICollectionColumn& column = desc.attributeColumn(col_id, f_id);
         m_layout.push_back( make_pair( column.name(), column.type() ) );
         m_annotations.push_back( column.annotation() );
      }
   }

}


void AttributeListLayout::fillDescription( pool::CollectionDescription& desc ) const
{
   size_t        annots_n = m_annotations.size();
   for( size_t i = 0; i < m_layout.size(); i++ ) {
      const std::string& column_name = m_layout[i].first;
      const std::string& annotation = ( i<annots_n? m_annotations[i] : "" ); 
      if( column_name == RootCollection::RootCollection::c_tokenBranchName ) {
         desc.setEventReferenceColumnName( RootCollection::RootCollection::c_tokenBranchName );
      }
      desc.insertColumn( column_name, m_layout[i].second, annotation );
   }
}



void AttributeListLayout::print()const{
  pool::PoolMessageStream log( "AttributeListLayout", seal::Msg::Nil );
  typedef std::vector< std::pair<std::string, std::string> >::const_iterator Iter;
  for(Iter iter=m_layout.begin();
      iter!=m_layout.end();
      ++iter)
    log  << seal::Msg::Info << iter->first << " \t"
         << iter->second << seal::endmsg;
}

