#include "RootCollection.h"
#include "RootCollectionCursor.h"

#include "CoralBase/Attribute.h"

#include <iostream>
using namespace std;


pool::RootCollection::RootCollectionCursor::
RootCollectionCursor(
   const pool::ICollectionDescription& description,
   const pool::CollectionRowBuffer& collectionRowBuffer,
   TTree *tree,
   const TEventList *evl 
   )
      :
      m_description( description ),
      m_collectionRowBuffer( collectionRowBuffer ),
      m_eventList(evl),
      m_idx(-1),
      m_entries( evl? evl->GetN() : tree->GetEntries() ),
      m_attIdx(-23),
      m_tokIdx(-42)
{
   for( coral::AttributeList::iterator attrI = m_collectionRowBuffer.attributeList().begin();
        attrI != m_collectionRowBuffer.attributeList().end();
        ++attrI ) {
      std::string        branchName = attrI->specification().name();
      TBranch* branch = tree->GetBranch( branchName.c_str() );
      if( attrI->specification().type() == typeid(std::string) ) {
         branch->SetAddress( m_charBuffer );
         m_attrBranches.push_back( make_pair(branch, &attrI->data<std::string>()) );
      } else {
         branch->SetAddress( attrI->addressOfData() );         
         m_attrBranches.push_back( make_pair( branch, (std::string*)0 ) );
      }
   }

   for( pool::TokenList::iterator tokenI = m_collectionRowBuffer.tokenList().begin();
        tokenI != m_collectionRowBuffer.tokenList().end();
        ++tokenI ) {
      std::string  columnName = tokenI.tokenName();
      if( columnName == m_description.eventReferenceColumnName() ) {
         // map the event reference column to the constant ROOT branch name
         columnName = RootCollection::c_tokenBranchName;
      }
      TBranch* branch = tree->GetBranch( columnName.c_str() );
      branch->SetAddress( m_charBuffer );
      m_tokenBranches.push_back( make_pair(branch, &*tokenI) );
   }
}


pool::RootCollection::RootCollectionCursor::~RootCollectionCursor()
{
   close();
   if(m_eventList) delete m_eventList;
}


void
pool::RootCollection::RootCollectionCursor::close()
{
}


bool
pool::RootCollection::RootCollectionCursor::next()
{
   if( ++m_idx >= m_entries ) {
      return false;
   }

   Long64_t entry( m_eventList? m_eventList->GetEntry(m_idx) : m_idx );

   // read attributes
   for( AttrBranchVector_t::const_iterator branchI = m_attrBranches.begin(); branchI != m_attrBranches.end(); ++branchI) {
      branchI->first->GetEntry(entry);
      if( branchI->second ) {
         // copy the read string from character buffer to std::string of the coral attribute
         *branchI->second = m_charBuffer;
      }
   }
   // read tokens
   for( TokenBranchVector_t::const_iterator branchI = m_tokenBranches.begin(); branchI != m_tokenBranches.end(); ++branchI) {
      branchI->first->GetEntry(entry);
      branchI->second->fromString( m_charBuffer );
   }

  
   // Get iterator over current row.
//  coral::AttributeList::const_iterator iData = m_cursor.currentRow().begin();

/* 
  cout << " * Cursor next(), values: " << endl;
  for( ; iData != m_cursor.currentRow().end(); ++iData ) {
      std::cout << "[";
      iData->toOutputStream( std::cout );
      std::cout << "] ";
  }
  cout << endl;  
  iData = m_cursor.currentRow().begin();
*/
  
  return true;
}


const pool::CollectionRowBuffer& 
pool::RootCollection::RootCollectionCursor::currentRow() const
{
  return m_collectionRowBuffer;
}


const pool::Token& 
pool::RootCollection::RootCollectionCursor::eventRef() const
{
   return m_collectionRowBuffer.tokenList()[ m_description.eventReferenceColumnName() ];
}


