#include <string>
#include <iostream>

#include "CollectionBase/ICollectionService.h"
#include "CollectionBase/CollectionService.h"
#include "CollectionBase/CollectionDescription.h"
#include "CollectionBase/ICollection.h"
#include "CollectionBase/CollectionRowBuffer.h"
#include "CollectionBase/TokenList.h"
#include "CollectionBase/ICollectionColumn.h"
#include "CollectionBase/ICollectionQuery.h"
#include "CollectionBase/ICollectionCursor.h"
#include "CollectionBase/ICollectionDataEditor.h"
#include "CollectionBase/ICollectionMetadata.h"

#include "POOLCore/POOLContext.h"
#include "POOLCore/Token.h"
#include "SealKernel/Exception.h"

#include "CoralBase/Attribute.h"

using namespace std;
using namespace pool;


/*
void write() {
   
   string connection( "" );
   try{
      cout << "Openning collection 'test_collection' in CREATE_AND_OVERWRITE mode." << endl;
      RootCollection collection( 0, connection, "test_collection", ICollection::CREATE_AND_OVERWRITE );
      cout << "Openning the collection." << endl;
    
      cout << "Specifying attribute list: 'attr1' of type 'int', 'attr2' of type 'std::string' "
           << "and attr3 of type 'double'." << endl;
      AttributeListSpecification& spec = collection.attributeListSpecification();
      spec.push_back( "attr1", "int" );
      spec.push_back( "attr2", "string" );
      spec.push_back( "attr3", "double" );
      spec.push_back( "attr4", "bool" );
    
      cout << "Adding 30 elements to the collection." << endl;
      for( int i=100; i<130; i++ )
      {
         AttributeList attrList( spec );
         attrList[ "attr1" ].setValue( i );
         char s[20];
         sprintf( s, "%s_%d", "testString", i );
         attrList[ "attr2" ].setValue( string( s ) );
         attrList[ "attr3" ].setValue( double( i ) / 3 );
         attrList[ "attr4" ].setValue( bool(i%3) );
         sprintf( s, "%s_%d","testToken", i );
         Token* t = new Token();
         t->fromString( string( "" ) );
         t->setDb( s );
         collection.add( *t, attrList );

         cout << "Token: " << t->toString();
         cout << endl;
         cout << "Attributes: ";
         printAttributes( attrList );
         cout << endl;
         t->release();
      }
      cout << "Done." << endl;
      cout << "Commiting the collection." << endl;
      collection.commit();
      cout << "Closing the collection." << endl;
      collection.close();
    
      cout << "[OVAL] finished hopefully" << endl;
    
   } catch ( seal::Exception sealException )
   {
      cout << "SEAL EXCEPTION: " << sealException.message() << endl;
   } catch ( ... )
   {
      cout << "Unknown exception" << endl;
   }
}



void read()
{
   string connection( "" );
   try
   {
      cout << "Openning collection 'test_collection' in READ mode." << endl;
      RootCollection collection( 0, connection, "test_collection", ICollection::READ );

      cout << "Creating Iterator with empty query." << endl;
      ICollectionIterator* iterator = collection.select( "", "", "" );
      cout << "Contents of the collection: " << endl;
      while( iterator->next() )
      {
         cout << "Token: " << iterator->token()->toString();
         cout << endl;
         cout << "Attributes: ";
         printAttributes( iterator->attributeList() );
         cout << endl;
      }
      cout << "Done." << endl;
      cout << "Closing the collection." << endl;
      collection.close();

      cout << endl << endl;
      cout << "Openning the collection." << endl;
      collection.open();
    
      std::string query = "attr1 > 117 && attr1 <123 && strstr(attr2,\"String_11\")";
      cout << "Creating Iterator with query: '" << query << "'." << endl;
      iterator = collection.select(query, "", "" );
      cout << "Contents of the collection: " << endl;
      while( iterator->next() ) {
         cout << "[OVAL] ";
         cout << "Token: " << iterator->token()->toString();
         cout << endl;
         cout << "[OVAL] ";
         cout << "Attributes: ";
         printAttributes( iterator->attributeList() );
         cout << endl;
      }
      cout << "Done." << endl;
      cout << "Closing the collection." << endl;
      collection.close();
    
      cout << "[OVAL] finished successfully" << endl;
  
   } catch ( seal::Exception sealException )
   {
      cout << "SEAL EXCEPTION: " << sealException.message() << endl;
   } catch ( ... )
   {
      cout << "Unknown exception" << endl;
   }

}

*/


class TestDriver {
public:
  TestDriver( const std::string& name,
              const std::string& connection );

  ~TestDriver() {}

  void write();

  void read();


private:
  std::string m_name;
  std::string m_connection;
};



TestDriver::TestDriver( const std::string& name,
                        const std::string& connection )
      : m_name( name ),  m_connection( connection )
{
   // MN: this line prevents hang on exit when only RootCollections are used
   // no idea why
   pool::POOLContext::loadComponent( "CORAL/Services/ConnectionService" );

   // Load message service.
   pool::POOLContext::loadComponent( "SEAL/Services/MessageService" );
   if( !getenv("POOL_OUTMSG_LEVEL") ) {
      pool::POOLContext::setMessageVerbosityLevel( seal::Msg::Info );
   }
}



void
TestDriver::write()
{
   cout << "Creating CollectionService" << endl;
   pool::ICollectionService *serviceHandle = new pool::CollectionService( pool::POOLContext::context() );
   cout << "Creating Collection Description" << endl;
   pool::CollectionDescription description( m_name, "RootCollection", m_connection );
   description.insertColumn( "attr1", "int", "integer test attribute" );
   description.insertColumn( "attr2", "string", "some string" );
   description.insertColumn( "attr3", "double", "floating point attribute" );
   description.insertColumn( "attr4", "bool" );

   cout << "Creating Collection" << endl;
   pool::ICollection* collection = serviceHandle->create( description, true );
   if ( ! collection )   {
      throw std::runtime_error( "Could not create a relational collection object" );
   }

   cout << "Adding 30 elements to the collection." << endl;
   pool::CollectionRowBuffer rowBuffer = collection->dataEditor().rowBuffer();

   for( int i=100; i<130; i++ )   {
      rowBuffer.attributeList()[ "attr1" ].data<int>() = i ;
      char s[20];
      sprintf( s, "%s_%d", "testString", i );
      rowBuffer.attributeList()[ "attr2" ].data<std::string>() = string( s );
      rowBuffer.attributeList()[ "attr3" ].data<double>() = double( i ) / 3;
      rowBuffer.attributeList()[ "attr4" ].data<bool>() =  bool(i%3);

      sprintf( s, "%s_%d","testToken", i );
      Token* t = new Token();
      t->fromString( string( "" ) );
      t->setDb( s );

      t->setData( &*rowBuffer.tokenList().begin() );

      collection->dataEditor().insertRow( rowBuffer );
      t->release();
   }

   ICollectionMetadata& metadata( collection->metadata() );
   metadata.setValueForKey("Author", "This collection has been created by readTest from RootCollection package");

   const char *author = metadata.getValueForKey("Author");
   if( !author ) {
      throw std::runtime_error("The collection has no Author key in the metadata");
   } else {
      cout << "Collection author metadata inserted: " << author << endl;
   }

   metadata.setValueForKey("random1", "random metadata 1");
   metadata.setValueForKey("random5", "random metadata 5");
   metadata.setValueForKey("random1", "overwritten random metadata 1");

      
   cout << "Done." << endl;
   cout << "Commiting the collection." << endl;
   collection->commit();
   cout << "Closing the collection." << endl;
   collection->close();
   delete collection;

   cout << "[OVAL] finished hopefully" << endl;
}

   

void
TestDriver::read()
{
   /*
   cout << " Retrieve handle to loaded collection service" << endl;
  std::vector< seal::IHandle< pool::ICollectionService > > loadedCollectionServices;
  pool::POOLContext::context()->query( loadedCollectionServices );
  if ( loadedCollectionServices.empty() )   {
    // Load connection service.
    pool::POOLContext::loadComponent( "POOL/Services/CollectionService" );
    pool::POOLContext::context()->query( loadedCollectionServices );
  }
  seal::IHandle< pool::ICollectionService > serviceHandle = loadedCollectionServices.front();
*/

   cout << "Creationg CollectionService" << endl;
   seal::Handle<pool::CollectionService> serviceHandle( new pool::CollectionService( pool::POOLContext::context() ) );
   
  cout << "Getting handle to existing collection ( opened for read-only transactions by default )" << endl;
  pool::ICollection* collection = serviceHandle->handle( m_name, "RootCollection" , m_connection );

  if( ! collection )   {
     throw std::runtime_error( "Could not create a rootCollection object" );
  }

  cout << "Executing query that only selects event reference column (the default)" << endl;
  pool::ICollectionQuery* query1 = collection->newQuery();
//  query1->setCondition( "Xx * Yy < 20 AND Xx > 2" );
  pool::ICollectionCursor& cursor1 = query1->execute();
  cout << "Iterating over query results..." << endl;
  int counter = 0;
  while( cursor1.next() && counter < 10 )  {
     std::cout << "Token : " << cursor1.eventRef().toString() << std::endl;
     coral::AttributeList attributeList = cursor1.currentRow().attributeList();
     std::cout << "Meta data : ";
     for( coral::AttributeList::const_iterator iAttribute = attributeList.begin();
           iAttribute != attributeList.end(); ++iAttribute )
     {
        if ( iAttribute != attributeList.begin() ) std::cout << ", ";
        std::cout << "[";
        iAttribute->toOutputStream( std::cout );
        std::cout << "]";
     }
     cout << endl;
     counter++;
  }
  std::cout << counter << " records read back" << std::endl;
  delete query1;

  
  cout << "Executing query that selects 2 meta data columns." << endl;
  pool::ICollectionQuery* query2 = collection->newQuery();
  query2->setCondition( "attr1 > 115" );
  query2->addToOutputList( "attr1,attr2" );
  pool::ICollectionCursor& cursor2 = query2->execute();
  counter = 0;
  while ( cursor2.next() && counter < 100 )
  {
     //MN: std::cout << "Token : " << cursor2.currentRow().eventRef().toString() << std::endl;
    std::cout << "Token : " << cursor2.eventRef().toString() << std::endl;
    coral::AttributeList attributeList = cursor2.currentRow().attributeList();
    std::cout << "Meta data : ";
    for ( coral::AttributeList::const_iterator iAttribute = attributeList.begin();
          iAttribute != attributeList.end(); ++iAttribute )
    {
      if ( iAttribute != attributeList.begin() ) std::cout << ", ";
      std::cout << "[";
      iAttribute->toOutputStream( std::cout );
      std::cout << "]";
    }
    cout << endl;
    counter++;
  }
  std::cout << counter << " records read back" << std::endl;
  delete query2;


  /*
  // Execute query that selects all meta data.
  pool::ICollectionQuery* query3 = collection->newQuery();
  query3->setCondition( "Xx * Yy < 20 AND Xx > 2" );
  query3->selectAll();
  pool::ICollectionCursor& cursor3 = query3->execute();
  counter = 0;
  while( cursor3.next() )  {
     //MN:  std::cout << "Token : " << cursor3.currentRow().eventRef().toString() << std::endl;
    std::cout << "Token : " << cursor3.eventRef().toString() << std::endl;
    coral::AttributeList attributeList = cursor3.currentRow().attributeList();
    std::cout << "Meta data : ";
    for ( coral::AttributeList::const_iterator iAttribute = attributeList.begin();
          iAttribute != attributeList.end(); ++iAttribute )
    {
      if ( iAttribute != attributeList.begin() ) std::cout << ", ";
      std::cout << "[";
      iAttribute->toOutputStream( std::cout );
      std::cout << "]";
    }
    const pool::TokenList tokens = cursor3.currentRow().tokenList();
    std::cout << endl;
    cout << "Token 2: " << tokens["Ref"].toString() << std::endl;
   
    cout << "-------- " << endl;
    
    counter++;
  }
  std::cout << counter << " records read back" << std::endl;
  delete query3;
  */

  const char *author = collection->metadata().getValueForKey("Author");
  if( !author ) {
     cout << "The collection has no Author key in the metadata" << endl;
  } else {
     cout << "Collection author metadata: " << author << endl;
  }

  std::cout << endl << "Dumping all metadata" << std::endl;
  ICollectionMetadata::const_iterator  mdIter
     = collection->metadata().begin();
  while( mdIter !=  collection->metadata().end() ) {
     cout << "  Metadata Key=" << mdIter.key();
     cout << ",  Value=" << mdIter.value() << endl;
     ++mdIter;
  }
  
  // Close collection.
  collection->close();
  delete collection;
}





int main()
{
   try {
      std::cout << "Read test starting..." << std::endl;
      TestDriver driver( "test_collection", "" );
      driver.write();
      driver.read();

   }
   catch ( std::exception& e ) {
      std::cerr << "---> Exception:" << std::endl;
      std::cerr << e.what() << std::endl;
      return 1;
   }
   catch (...) {
      std::cerr << "Unhandled exception..." << std::endl;
      return 1;
   }
   
   return 0;
}






