//====================================================================
//
//	Package    : RootStorageSvc (The POOL project)
//
//	Author     : M.Frank
//====================================================================
#ifndef POOL_ROOTSTORAGESVC_REFSTREAMER_H
#define POOL_ROOTSTORAGESVC_REFSTREAMER_H 1

#include "RootStorageSvc/StreamerBase.h"

/*
 *   POOL namespace declaration
 */
namespace pool  {

  /**@class RefStreamer RefStreamer.h src/RefStreamer.h
    *
    * Specialized streamer to handle POOL references.
    *
    *  @author  M.Frank
    *  @version 1.0
    *  @date    11/11/2004
    */
  class RefStreamer : public StreamerBase  {
    /// Reference type version
    int        m_version;
    /// reference type
    int        m_refType;
  public:
    /// Initializing constructor
    RefStreamer(const TypeH& typ, TClass* cl, int m_version=0);
    /// Standard destructor
    virtual void operator()(TBuffer &b, void *obj);
    /// Read data from buffer
    void get(TBuffer& refBuffer, void* objp);
    /// Read data from buffer (backwards compatible mode with long oid)
    void getLong(TBuffer& refBuffer, void* objp);
    /// Write data to buffer
    void put(TBuffer& refBuffer, void* objp);
    /// Set reference version number
    void setVersion(int vsn)  {
      m_version = vsn;
    }
    /// Access reference version number
    int version()  const {
      return m_version;
    }
  };
}
#endif // POOL_ROOTSTORAGESVC_REFSTREAMER_H
