//====================================================================
//
//	Package    : RootStorageSvc (The POOL project)
//
//	Author     : M.Frank
//====================================================================

#include "StorageSvc/DbPrint.h"
#include "RootStorageSvc/CINTTypedefs.h"
#include "StorageSvc/DbReflex.h"
#include "TClass.h"
#include "TROOT.h"
#include "Api.h"

#define __SCHEMA_CHANGE__
#include <cstdlib>
#include <fstream>
#ifdef _WIN32
#include <sstream>
#else
#include <iostream>
#endif
#include <memory>

typedef ROOT::Reflex::Type TypeH;

static std::auto_ptr<CINT::Typedefs> s_defs(0);

/// Standard constructor
CINT::Typedefs::Typedefs()  {
  const char* defs = ::getenv("POOL_ROOTDEFS");
  if ( 0 == defs ) defs = ::getenv("POOLDB_ROOTDEFS");
  if ( defs )  {
    char buff[1024];
    const char* idq = defs;
    const char* idx = std::char_traits<char>::find(idq, ::strlen(idq), ';');
    if ( idx == 0 && *idq ) idx = idq;
    while ( idx )  {
      std::string fname = idq;
      if ( idx > idq ) fname[idx-idq]=0;
      std::fstream in(fname.c_str(),std::ios_base::in);
      memset(buff,0,sizeof(buff));
      while ( in.getline(buff,sizeof(buff),'\n').good() )   {
        buff[sizeof(buff)-1]=0;
        if ( ::strlen(buff) )  {
          std::istringstream instr(buff);
          std::string dummy, old, type;
          instr >> dummy;
          if ( dummy == "typedef" )  {
            instr >> old >> type;
            size_t len = (type[type.length()-1]==';') ? type.length()-1 : type.length();
            m_defs[type.substr(0,len)] = old;
          }
          else if ( dummy == "typeremove" )  {
            instr >> type;
            size_t len = (type[type.length()-1]==';') ? type.length()-1 : type.length();
            m_defs.erase(type.substr(0,len));
          }
        }
        memset(buff,0,sizeof(buff));
      }
      idq = ++idx;
      idx = std::char_traits<char>::find(idq, ::strlen(idq), ';');
      if ( idx == 0 && *idq ) idx = idq;
    }
  }
}

/// Standard destructor
CINT::Typedefs::~Typedefs()  {
  m_defs.clear();
}

/// Call to apply typedefs on a per classname basis if requested.
TClass* CINT::Typedefs::applyType(const std::string& type_name)  {
  Definitions::const_iterator i=m_defs.find(type_name);
  if ( i != m_defs.end() )  {
    std::string typ = (*i).first;
    std::string old = (*i).second;
    TClass* cl = gROOT->GetClass(old.c_str(), kTRUE);
    if ( cl )  {
      G__ClassInfo info(old.c_str());
      if ( info.IsValid() )  {
        G__search_typename2(typ.c_str(),'u',info.Tagnum(),0,-1);
        G__setnewtype(-1,NULL,0);
        return cl;
      }
    }
    pool::DbPrint err("CINT::Typedefs");
    err << pool::DbPrint::ERROR
        << "Failed to apply CINT typedef:" << typ << " --> " << old
        << ". Class " << old << " is not availiable." 
        << pool::DbPrint::endmsg;
  }
  return 0;
}

/// Call to apply typedefs on a per classname basis if requested.
TypeH CINT::Typedefs::applyDefinition(const std::string& type_name)  {
  Definitions::const_iterator i=m_defs.find(type_name);
  if ( i != m_defs.end() )  {
    std::string typ = (*i).first;
    std::string old = (*i).second;
    TypeH c = pool::DbReflex::forTypeName(old);
    if ( c )  {
      return c;
    }
    pool::DbPrint err("CINT::Typedefs");
    err << pool::DbPrint::ERROR
        << "Failed to find CINT typedef:" << typ << " --> " << old
        << ". Class " << old << " is not availiable." 
        << pool::DbPrint::endmsg;
  }
  return 0;
}

/// Get the corresponding Class structure from the type def table
TypeH CINT::Typedefs::definition(const std::string& name)   {
  if ( 0 == s_defs.get() )  {
    s_defs = std::auto_ptr<Typedefs>(new Typedefs());
  }
  return s_defs->applyDefinition(name);
}

TClass* CINT::Typedefs::apply(const std::string& cl_name) {
  if ( 0 == s_defs.get() )  {
    s_defs = std::auto_ptr<Typedefs>(new Typedefs());
  }
  return s_defs->applyType(cl_name);
}
