#include "StorageSvc/DbReflex.h"
#include "StorageSvc/IIOHandler.h"
#include "RootStorageSvc/ICINTStreamer.h"
#include "RootStorageSvc/IOHandlerCustomStreamer.h"
#include <stdexcept>
#include "TBuffer.h"
#include "TClass.h"
#include "TFile.h"
using namespace pool;

IOHandlerCustomStreamer::IOHandlerCustomStreamer( const TypeH& typ,
                                                  TClass* cl, 
                                                  IIOHandler* h,
                                                  ICINTStreamer* s) 
: StreamerBase(typ, cl), m_streamer(s), m_ioHandler(h)
{
}

void IOHandlerCustomStreamer::operator()(TBuffer &b, void *obj)  {
  try {
    UInt_t start, count;
    if ( b.IsReading() )   {
      /* Version_t version = */ b.ReadVersion(&start, &count, m_root);
      if ( ((TFile*)b.GetParent())->GetVersion() < 40000 )  {
        b.ReadVersion(&start, &count);
        m_streamer->readData(obj, m_root, b, 0);
        b.CheckByteCount(start, count, m_root);
        m_ioHandler->onReadUpdate(obj);
        return;
      }
      m_streamer->readData(obj, m_root, b, 0);
      m_ioHandler->onReadUpdate(obj);
    }
    else  {
      count = b.WriteVersion(m_root, kTRUE);
      m_ioHandler->onWriteUpdate(obj);
      m_streamer->writeData(obj, m_root, b, 0);
      b.SetByteCount(count, kTRUE);
    }
  }
  catch( const std::exception& e )    {
    std::string err = "Class:" + DbReflex::fullTypeName(m_type) + "> Exception in object I/O";
    err += e.what();
    throw std::runtime_error(err);
  }
  catch( ... )    {
    std::string err = "Class:" + DbReflex::fullTypeName(m_type) + "> Exception in object I/O";
    throw std::runtime_error(err);
  }
}
