#include "StorageSvc/DbReflex.h"
#include "StorageSvc/IIOHandler.h"
#include "RootStorageSvc/IOHandlerStreamer.h"
#include <stdexcept>
#include "TBuffer.h"
#include "TClass.h"
using namespace pool;

IOHandlerStreamer::IOHandlerStreamer(const TypeH& typ, TClass* cl, IIOHandler* h)
: StreamerBase(typ, cl), m_ioHandler(h)
{
}

void IOHandlerStreamer::operator()(TBuffer &b, void *obj)  {
  try {
    if ( b.IsReading() )   {
      UInt_t start, count;
      Version_t version = b.ReadVersion(&start, &count, m_root);
      m_root->ReadBuffer(b, obj, version, start, count);
      m_ioHandler->onReadUpdate(obj);
    }
    else  {
      m_ioHandler->onWriteUpdate(obj);
      m_root->WriteBuffer(b, obj);
    }
  }
  catch( const std::exception& e )    {
    std::string err = "Class:" + DbReflex::fullTypeName(m_type) + "> Exception in object I/O";
    err += e.what();
    throw std::runtime_error(err);
  }
  catch( ... )    {
    std::string err = "Class:" + DbReflex::fullTypeName(m_type) + "> Exception in object I/O";
    throw std::runtime_error(err);
  }
}
