// $Id: DbDictionary.h,v 1.36 2007/11/20 18:10:03 frankb Exp $
//====================================================================
//  Database dictionary definition
//--------------------------------------------------------------------
//
//  Package    : pool (The POOL project)
//
//  Description: After getting annoyed with writing all these
//               #$%&@@! wrappers to hide the LCG dictionary
//               I could not resist to make something more generic.
//               Clearly this is the result of two hours thinking
//               and by far incomplete in order to serve a similar
//               functionality as e.g. boost/python does.
//               Nevertheless, the idea is the same (to be clear: I 
//               I have stolen it from boost/python) and gradually
//               the content of this header file may well be improved.
//
//  @author      M.Frank
//====================================================================
#ifndef POOL_DBDICTIONARY_H
#define POOL_DBDICTIONARY_H

#include "ReflectionBuilder/ReflectionBuilder.h"
#include <stdexcept>
#include <list>

/*
 *   POOL namespace declaration
 */
namespace pool {
  const std::string typeName(const std::type_info& typ);

  /// Const argument reference type declaration
  typedef const std::vector<void*>& arg_type;
  /// Const argument type declaration
  typedef std::vector<std::string>  arg_decl;
  /// Void reflection call definition
  typedef void  (*VOID_CALL)          (void*);
  typedef void* (*VOID_STAR_CALL)     (void*);
  typedef void  (*VOID_ARG_CALL)      (void*, arg_type);
  typedef void* (*VOID_STAR_ARG_CALL) (void*, arg_type);

  /// Specialized class to encapsulate RTTI information
  template <class T>  struct type {    typedef T id;                };
  /// Specialized classes by enumeration
  template <int i> struct tag     {                                 };

  template <class T> inline
  void* invalid_function_call(T        /* pmf  */, 
                              void*    /* o    */,
                              arg_type /* args */)
  {    return 0;                                                    }
  inline void* valid_function_call()   
  {    return (void*)1;                                             }

  inline bool check_args(arg_type args, size_t narg)  
  {    return narg <= args.size();                                  }
  inline bool check_args(void*, arg_type args, size_t narg)
  {    return narg <= args.size();                                  }
  template <class T> inline T from_ref(void* p, type<T>)  
  {    return *(T*)p;                                               }
  template <class T> inline T* from_ref(void* p, type<T*>)  
  {    return (T*)p;                                                }
  template <class T> inline T from_ref_ptr(void* p, type<T>)  
  {    return T(p);                                                 }
  template <class T> inline void* to_ref(const T& o)  {
    static const T ptr = o;
    return (void*)&ptr;
  }

  /// Structure to define the call sequences.
  template <class R>  struct caller {

    template <class T>
    static void* call(R (T::*pmf)(), void* o, arg_type args) {
      if (check_args(o, args, 0))   {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)());
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1>
    static void* call(R (T::*pmf)(A1), void* o, arg_type args) {
      if (check_args(o, args, 1))    {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)(from_ref(args[0], type<A1>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2>
    static void* call(R (T::*pmf)(A1, A2), void* o, arg_type args) {
      if (check_args(o, args, 2))   {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>()),
                                      from_ref(args[1], type<A2>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3>
    static void call(R (T::*pmf)(A1, A2, A3), void* o, arg_type args) {
      if (check_args(o, args, 3))   {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>()),
                                      from_ref(args[1], type<A2>()),
                                      from_ref(args[2], type<A3>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4>
    static void* call(R (T::*pmf)(A1, A2, A3, A4), void* o, arg_type args) {
      if (check_args(o, args, 4))   {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>()),
                                      from_ref(args[1], type<A2>()),
                                      from_ref(args[2], type<A3>()),
                                      from_ref(args[3], type<A4>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5>
    static void* call(R (T::*pmf)(A1, A2, A3, A4, A5), void* o, arg_type args) {
      if (check_args(o, args, 5))   {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>()),
                                      from_ref(args[1], type<A2>()),
                                      from_ref(args[2], type<A3>()),
                                      from_ref(args[3], type<A4>()),
                                      from_ref(args[4], type<A5>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6>
    static void* call(R (T::*pmf)(A1, A2, A3, A4, A5, A6), void* o, arg_type args) {
      if (check_args(o, args, 6))    {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>()),
                                      from_ref(args[1], type<A2>()),
                                      from_ref(args[2], type<A3>()),
                                      from_ref(args[3], type<A4>()),
                                      from_ref(args[4], type<A5>()),
                                      from_ref(args[5], type<A6>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7>
    static void* call(R (T::*pmf)(A1, A2, A3, A4, A5, A6, A7), void* o, arg_type args) {
      if (check_args(o, args, 7))    {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>()),
                                      from_ref(args[1], type<A2>()),
                                      from_ref(args[2], type<A3>()),
                                      from_ref(args[3], type<A4>()),
                                      from_ref(args[4], type<A5>()),
                                      from_ref(args[5], type<A6>()),
                                      from_ref(args[6], type<A7>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8>
    static void* call(R (T::*pmf)(A1, A2, A3, A4, A5, A6, A7, A8), void* o, arg_type args) {
      if (check_args(o, args, 8))  {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>()),
                                      from_ref(args[1], type<A2>()),
                                      from_ref(args[2], type<A3>()),
                                      from_ref(args[3], type<A4>()),
                                      from_ref(args[4], type<A5>()),
                                      from_ref(args[5], type<A6>()),
                                      from_ref(args[6], type<A7>()),
                                      from_ref(args[7], type<A8>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9>
    static void* call(R (T::*pmf)(A1, A2, A3, A4, A5, A6, A7, A8, A9), void* o, arg_type args) {
      if (check_args(o, args, 9))    {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>()),
                                      from_ref(args[1], type<A2>()),
                                      from_ref(args[2], type<A3>()),
                                      from_ref(args[3], type<A4>()),
                                      from_ref(args[4], type<A5>()),
                                      from_ref(args[5], type<A6>()),
                                      from_ref(args[6], type<A7>()),
                                      from_ref(args[7], type<A8>()),
                                      from_ref(args[8], type<A9>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9, class A10>
    static void* call(R (T::*pmf)(A1, A2, A3, A4, A5, A6, A7, A8, A9, A10), void* o, arg_type args) {
      if (check_args(o, args, 10))   {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>()),
                                      from_ref(args[1], type<A2>()),
                                      from_ref(args[2], type<A3>()),
                                      from_ref(args[3], type<A4>()),
                                      from_ref(args[4], type<A5>()),
                                      from_ref(args[5], type<A6>()),
                                      from_ref(args[6], type<A7>()),
                                      from_ref(args[7], type<A8>()),
                                      from_ref(args[8], type<A9>()),
                                      from_ref(args[9], type<A10>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T>
    static void* call(R (T::*pmf)() const, void* o, arg_type args) {
      if (check_args(o, args, 0))   {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)());
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1>
    static void* call(R (T::*pmf)(A1) const, void* o, arg_type args) {
      if (check_args(o, args, 1))    {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2>
    static void* call(R (T::*pmf)(A1, A2) const, void* o, arg_type args) {
      if (check_args(o, args, 2))   {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>()),
                                      from_ref(args[1], type<A2>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3>
    static void call(R (T::*pmf)(A1, A2, A3) const, void* o, arg_type args) {
      if (check_args(o, args, 3))   {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>()),
                                      from_ref(args[1], type<A2>()),
                                      from_ref(args[2], type<A3>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4>
    static void* call(R (T::*pmf)(A1, A2, A3, A4) const, void* o, arg_type args) {
      if (check_args(o, args, 4))   {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>()),
                                      from_ref(args[1], type<A2>()),
                                      from_ref(args[2], type<A3>()),
                                      from_ref(args[3], type<A4>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5>
    static void* call(R (T::*pmf)(A1, A2, A3, A4, A5) const, void* o, arg_type args) {
      if (check_args(o, args, 5))   {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>()),
                                      from_ref(args[1], type<A2>()),
                                      from_ref(args[2], type<A3>()),
                                      from_ref(args[3], type<A4>()),
                                      from_ref(args[4], type<A5>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6>
    static void* call(R (T::*pmf)(A1, A2, A3, A4, A5, A6) const, void* o, arg_type args) {
      if (check_args(o, args, 6))    {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>()),
                                      from_ref(args[1], type<A2>()),
                                      from_ref(args[2], type<A3>()),
                                      from_ref(args[3], type<A4>()),
                                      from_ref(args[4], type<A5>()),
                                      from_ref(args[5], type<A6>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7>
    static void* call(R (T::*pmf)(A1, A2, A3, A4, A5, A6, A7) const, void* o, arg_type args) {
      if (check_args(o, args, 7))    {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>()),
                                      from_ref(args[1], type<A2>()),
                                      from_ref(args[2], type<A3>()),
                                      from_ref(args[3], type<A4>()),
                                      from_ref(args[4], type<A5>()),
                                      from_ref(args[5], type<A6>()),
                                      from_ref(args[6], type<A7>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8>
    static void* call(R (T::*pmf)(A1, A2, A3, A4, A5, A6, A7, A8) const, void* o, arg_type args) {
      if (check_args(o, args, 8))  {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>()),
                                      from_ref(args[1], type<A2>()),
                                      from_ref(args[2], type<A3>()),
                                      from_ref(args[3], type<A4>()),
                                      from_ref(args[4], type<A5>()),
                                      from_ref(args[5], type<A6>()),
                                      from_ref(args[6], type<A7>()),
                                      from_ref(args[7], type<A8>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9>
    static void* call(R (T::*pmf)(A1, A2, A3, A4, A5, A6, A7, A8, A9) const, void* o, arg_type args) {
      if (check_args(o, args, 9))    {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>()),
                                      from_ref(args[1], type<A2>()),
                                      from_ref(args[2], type<A3>()),
                                      from_ref(args[3], type<A4>()),
                                      from_ref(args[4], type<A5>()),
                                      from_ref(args[5], type<A6>()),
                                      from_ref(args[6], type<A7>()),
                                      from_ref(args[7], type<A8>()),
                                      from_ref(args[8], type<A9>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9, class A10>
    static void* call(R (T::*pmf)(A1, A2, A3, A4, A5, A6, A7, A8, A9, A10) const, void* o, arg_type args) {
      if (check_args(o, args, 10))   {
        T* target = from_ref_ptr(o, type<T*>());
        return to_ref((target->*pmf)( from_ref(args[0], type<A1>()),
                                      from_ref(args[1], type<A2>()),
                                      from_ref(args[2], type<A3>()),
                                      from_ref(args[3], type<A4>()),
                                      from_ref(args[4], type<A5>()),
                                      from_ref(args[5], type<A6>()),
                                      from_ref(args[6], type<A7>()),
                                      from_ref(args[7], type<A8>()),
                                      from_ref(args[8], type<A9>()),
                                      from_ref(args[9], type<A10>())));
      }
      return invalid_function_call(pmf, o, args);
    }

    // Free functions
    static void* call(R (*f)(), void* o, arg_type args) {
      if (check_args(o,args, 0))   {
        return to_ref(f());
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1>
    static void* call(R (*f)(A1), void* o, arg_type args) {
      if (check_args(o, args, 1))    {
        return to_ref(f(from_ref(args[0], type<A1>())));
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2>
    static void* call(R (*f)(A1, A2), void* o, arg_type args) {
      if (check_args(o, args, 2))    {
        return to_ref(f(from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>())));
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2, class A3>
    static void* call(R (*f)(A1, A2, A3), void* o, arg_type args) {
      if (check_args(o, args, 3))    {
        return to_ref(f(from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>())));
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2, class A3, class A4>
    static void* call(R (*f)(A1, A2, A3, A4), void* o, arg_type args) {
      if (check_args(o, args, 4))    {
        return to_ref(f(from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>())));
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2, class A3, class A4, class A5>
    static void* call(R (*f)(A1, A2, A3, A4, A5), void* o, arg_type args) {
      if (check_args(o, args, 5))   {
        return to_ref(f(from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>())));
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2, class A3, class A4, class A5, class A6>
    static void* call(R (*f)(A1, A2, A3, A4, A5, A6), void* o, arg_type args) {
      if (check_args(o, args, 6))    {
        return to_ref(f(from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>()),
                        from_ref(args[5], type<A6>())));
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2, class A3, class A4, class A5, class A6, class A7>
    static void* call(R (*f)(A1, A2, A3, A4, A5, A6, A7), void* o, arg_type args) {
      if (check_args(o, args, 7))   {
        return to_ref(f(from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>()),
                        from_ref(args[5], type<A6>()),
                        from_ref(args[6], type<A7>())));
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8>
    static void* call(R (*f)(A1, A2, A3, A4, A5, A6, A7, A8), void* o, arg_type args) {
      if (check_args(o, args, 8))    {
        return to_ref(f(from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>()),
                        from_ref(args[5], type<A6>()),
                        from_ref(args[6], type<A7>()),
                        from_ref(args[7], type<A8>())));
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9>
    static void* call(R (*f)(A1, A2, A3, A4, A5, A6, A7, A8, A9), void* o, arg_type args) {
      if (check_args(o, args, 9))   {
        return to_ref(f(from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>()),
                        from_ref(args[5], type<A6>()),
                        from_ref(args[6], type<A7>()),
                        from_ref(args[7], type<A8>()),
                        from_ref(args[8], type<A9>())));
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9, class A10>
    static void* call(R (*f)(A1, A2, A3, A4, A5, A6, A7, A8, A9, A10), void* o, arg_type args) {
      if (check_args(o, args, 10))   {
        return to_ref(f(from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>()),
                        from_ref(args[5], type<A6>()),
                        from_ref(args[6], type<A7>()),
                        from_ref(args[7], type<A8>()),
                        from_ref(args[8], type<A9>()),
                        from_ref(args[9], type<A10>())));
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9, class A10, class A11>
    static void* call(R (*f)(A1, A2, A3, A4, A5, A6, A7, A8, A9, A10, A11), void* o, arg_type args) {
      if (check_args(o, args, 11))   {
        return to_ref(f(from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>()),
                        from_ref(args[5], type<A6>()),
                        from_ref(args[6], type<A7>()),
                        from_ref(args[7], type<A8>()),
                        from_ref(args[8], type<A9>()),
                        from_ref(args[9], type<A10>()),
                        from_ref(args[10], type<A11>())));
      }
      return invalid_function_call(f, o, args);
    }
  };
                         
  template <> struct caller<void>  {
    template <class T>
    static void* call(void (T::*pmf)(), void* o, arg_type args) {
      if (check_args(o, args, 0))   {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)();
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1>
    static void* call(void (T::*pmf)(A1), void* o, arg_type args) {
      if (check_args(o, args, 1))  {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2>
    static void* call(void (T::*pmf)(A1, A2), void* o, arg_type args) {
      if (check_args(o, args, 2))  {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3>
    static void* call(void (T::*pmf)(A1, A2, A3), void* o, arg_type args) {
      if (check_args(o, args, 3))  {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4>
    static void* call(void (T::*pmf)(A1, A2, A3, A4), void* o, arg_type args) {
      if (check_args(o, args, 4))  {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5>
    static void* call(void (T::*pmf)(A1, A2, A3, A4, A5), void* o, arg_type args) {
      if (check_args(o, args, 5))    {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6>
    static void* call(void (T::*pmf)(A1, A2, A3, A4, A5, A6), void* o, arg_type args) {
      if (check_args(o, args, 6))    {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>()),
                        from_ref(args[5], type<A6>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7>
    static void* call(void (T::*pmf)(A1, A2, A3, A4, A5, A6, A7), void* o, arg_type args) {
      if (check_args(o, args, 7))  {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>()),
                        from_ref(args[5], type<A6>()),
                        from_ref(args[6], type<A7>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8>
    static void* call(void (T::*pmf)(A1, A2, A3, A4, A5, A6, A7, A8), void* o, arg_type args) {
      if (check_args(o, args, 8))    {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>()),
                        from_ref(args[5], type<A6>()),
                        from_ref(args[6], type<A7>()),
                        from_ref(args[7], type<A8>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9>
    static void* call(void (T::*pmf)(A1, A2, A3, A4, A5, A6, A7, A8, A9), void* o, arg_type args) {
      if (check_args(o, args, 9))   {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>()),
                        from_ref(args[5], type<A6>()),
                        from_ref(args[6], type<A7>()),
                        from_ref(args[7], type<A8>()),
                        from_ref(args[8], type<A9>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9, class A10>
    static void* call(void (T::*pmf)(A1, A2, A3, A4, A5, A6, A7, A8, A9, A10), void* o, arg_type args) {
      if (check_args(o, args, 10))  {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>()),
                        from_ref(args[5], type<A6>()),
                        from_ref(args[6], type<A7>()),
                        from_ref(args[7], type<A8>()),
                        from_ref(args[8], type<A9>()),
                        from_ref(args[9], type<A10>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T>
    static void* call(void (T::*pmf)() const, void* o, arg_type args) {
      if (check_args(o, args, 0))    {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)();
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1>
    static void* call(void (T::*pmf)(A1) const, void* o, arg_type args) {
      if (check_args(o, args, 1))  {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)(from_ref(args[0], type<A1>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2>
    static void* call(void (T::*pmf)(A1, A2) const, void* o, arg_type args) {
      if (check_args(o, args, 2))   {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3>
    static void* call(void (T::*pmf)(A1, A2, A3) const, void* o, arg_type args) {
      if (check_args(o, args, 3))   {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4>
    static void* call(void (T::*pmf)(A1, A2, A3, A4) const, void* o, arg_type args) {
      if (check_args(o, args, 4))   {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5>
    static void* call(void (T::*pmf)(A1, A2, A3, A4, A5) const, void* o, arg_type args) {
      if (check_args(o, args, 5))   {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6>
    static void* call(void (T::*pmf)(A1, A2, A3, A4, A5, A6) const, void* o, arg_type args) {
      if (!check_args(o, args, 6))    {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>()),
                        from_ref(args[5], type<A6>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7>
    static void* call(void (T::*pmf)(A1, A2, A3, A4, A5, A6, A7) const, void* o, arg_type args) {
      if (check_args(o, args, 7))   {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>()),
                        from_ref(args[5], type<A6>()),
                        from_ref(args[6], type<A7>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8>
    static void* call(void (T::*pmf)(A1, A2, A3, A4, A5, A6, A7, A8) const, void* o, arg_type args) {
      if (check_args(o, args, 8))   {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>()),
                        from_ref(args[5], type<A6>()),
                        from_ref(args[6], type<A7>()),
                        from_ref(args[7], type<A8>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9>
    static void* call(void (T::*pmf)(A1, A2, A3, A4, A5, A6, A7, A8, A9) const, void* o, arg_type args) {
      if (check_args(o, args, 9))   {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>()),
                        from_ref(args[5], type<A6>()),
                        from_ref(args[6], type<A7>()),
                        from_ref(args[7], type<A8>()),
                        from_ref(args[8], type<A9>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }

    template <class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9, class A10>
    static void* call(void (T::*pmf)(A1, A2, A3, A4, A5, A6, A7, A8, A9, A10) const, void* o, arg_type args) {
      if (check_args(o, args, 10))   {
        T* target = from_ref_ptr(o, type<T*>());
        (target->*pmf)( from_ref(args[0], type<A1>()),
                        from_ref(args[1], type<A2>()),
                        from_ref(args[2], type<A3>()),
                        from_ref(args[3], type<A4>()),
                        from_ref(args[4], type<A5>()),
                        from_ref(args[5], type<A6>()),
                        from_ref(args[6], type<A7>()),
                        from_ref(args[7], type<A8>()),
                        from_ref(args[8], type<A9>()),
                        from_ref(args[9], type<A10>()));
        return valid_function_call();
      }
      return invalid_function_call(pmf, o, args);
    }


    // Free functions
    static void* call(void (*f)(), void* o, arg_type args) {
      if (check_args(args,0))    {
        f();
        return valid_function_call();
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1>
    static void* call(void (*f)(A1), void* o, arg_type args) {
      if (check_args(args, 1))    {
        f(from_ref(args[0], type<A1>()));
        return valid_function_call();
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2>
    static void* call(void (*f)(A1, A2), void* o, arg_type args) {
      if (check_args(args, 2))    {
        f( from_ref(args[0], type<A1>()),
           from_ref(args[1], type<A2>()));
        return valid_function_call();
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2, class A3>
    static void* call(void (*f)(A1, A2, A3), void* o, arg_type args) {
      if (check_args(args, 3))   {
        f( from_ref(args[0], type<A1>()),
           from_ref(args[1], type<A2>()),
           from_ref(args[2], type<A3>()));
        return valid_function_call();
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2, class A3, class A4>
    static void* call(void (*f)(A1, A2, A3, A4), void* o, arg_type args) {
      if (check_args(o, args, 4))    {
        f( from_ref(args[0], type<A1>()),
           from_ref(args[1], type<A2>()),
           from_ref(args[2], type<A3>()),
           from_ref(args[3], type<A4>()));
        return valid_function_call();
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2, class A3, class A4, class A5>
    static void* call(void (*f)(A1, A2, A3, A4, A5), void* o, arg_type args) {
      if (check_args(o, args, 5))    {
        f( from_ref(args[0], type<A1>()),
           from_ref(args[1], type<A2>()),
           from_ref(args[2], type<A3>()),
           from_ref(args[3], type<A4>()),
           from_ref(args[4], type<A5>()));
        return valid_function_call();
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2, class A3, class A4, class A5, class A6>
    static void* call(void (*f)(A1, A2, A3, A4, A5, A6), void* o, arg_type args) {
      if (check_args(o, args, 6))   {
        f( from_ref(args[0], type<A1>()),
           from_ref(args[1], type<A2>()),
           from_ref(args[2], type<A3>()),
           from_ref(args[3], type<A4>()),
           from_ref(args[4], type<A5>()),
           from_ref(args[5], type<A6>()));
        return valid_function_call();
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2, class A3, class A4, class A5, class A6, class A7>
    static void* call(void (*f)(A1, A2, A3, A4, A5, A6, A7), void* o, arg_type args) {
      if (check_args(o, args, 7))    {
        f( from_ref(args[0], type<A1>()),
           from_ref(args[1], type<A2>()),
           from_ref(args[2], type<A3>()),
           from_ref(args[3], type<A4>()),
           from_ref(args[4], type<A5>()),
           from_ref(args[5], type<A6>()),
           from_ref(args[6], type<A7>()));
        return valid_function_call();
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8>
    static void* call(void (*f)(A1, A2, A3, A4, A5, A6, A7, A8), void* o, arg_type args) {
      if (check_args(o, args, 8))   {
        f(from_ref(args[0], type<A1>()),
          from_ref(args[1], type<A2>()),
          from_ref(args[2], type<A3>()),
          from_ref(args[3], type<A4>()),
          from_ref(args[4], type<A5>()),
          from_ref(args[5], type<A6>()),
          from_ref(args[6], type<A7>()),
          from_ref(args[7], type<A8>()));
        return valid_function_call();
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9>
    static void* call(void (*f)(A1, A2, A3, A4, A5, A6, A7, A8, A9), void* o, arg_type args) {
      if (check_args(o, args, 9)) {
        f(from_ref(args[0], type<A1>()),
          from_ref(args[1], type<A2>()),
          from_ref(args[2], type<A3>()),
          from_ref(args[3], type<A4>()),
          from_ref(args[4], type<A5>()),
          from_ref(args[5], type<A6>()),
          from_ref(args[6], type<A7>()),
          from_ref(args[7], type<A8>()),
          from_ref(args[8], type<A9>()));
        return valid_function_call();
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9, class A10>
    static void* call(void (*f)(A1, A2, A3, A4, A5, A6, A7, A8, A9, A10), void* o, arg_type args) {
      if (check_args(o, args, 10))   {
        f(from_ref(args[0], type<A1>()),
          from_ref(args[1], type<A2>()),
          from_ref(args[2], type<A3>()),
          from_ref(args[3], type<A4>()),
          from_ref(args[4], type<A5>()),
          from_ref(args[5], type<A6>()),
          from_ref(args[6], type<A7>()),
          from_ref(args[7], type<A8>()),
          from_ref(args[8], type<A9>()),
          from_ref(args[9], type<A10>()));
        return valid_function_call();
      }
      return invalid_function_call(f, o, args);
    }

    template <class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9, class A10, class A11>
    static void* call(void (*f)(A1, A2, A3, A4, A5, A6, A7, A8, A9, A10, A11), void* o, arg_type args) {
      if (check_args(o, args, 11))   {
        f(from_ref(args[0], type<A1>()),
          from_ref(args[1], type<A2>()),
          from_ref(args[2], type<A3>()),
          from_ref(args[3], type<A4>()),
          from_ref(args[4], type<A5>()),
          from_ref(args[5], type<A6>()),
          from_ref(args[6], type<A7>()),
          from_ref(args[7], type<A8>()),
          from_ref(args[8], type<A9>()),
          from_ref(args[9], type<A10>()),
          from_ref(args[10],type<A11>()));
        return valid_function_call();
      }
      return invalid_function_call(f, o, args);
    }

  };

  // Return value extraction:
  // free functions
  template <class R> inline
  type<R> return_type(R (*)()) { return type<R>(); }

  template <class R, class A1> inline
  type<R> return_type(R (*)(A1)) { return type<R>(); }

  template <class R, class A1, class A2> inline
  type<R> return_type(R (*)(A1, A2)) { return type<R>(); }

  template <class R, class A1, class A2, class A3> inline
  type<R> return_type(R (*)(A1, A2, A3)) { return type<R>(); }

  template <class R, class A1, class A2, class A3, class A4> inline
  type<R> return_type(R (*)(A1, A2, A3, A4)) { return type<R>(); }

  template <class R, class A1, class A2, class A3, class A4, class A5> inline
  type<R> return_type(R (*)(A1, A2, A3, A4, A5)) { return type<R>(); }

  template <class R, class A1, class A2, class A3, class A4, class A5, class A6> inline
  type<R> return_type(R (*)(A1, A2, A3, A4, A5, A6)) { return type<R>(); }

  template <class R, class A1, class A2, class A3, class A4, class A5, class A6, class A7> inline
  type<R> return_type(R (*)(A1, A2, A3, A4, A5, A6, A7)) { return type<R>(); }

  template <class R, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8> inline
  type<R> return_type(R (*)(A1, A2, A3, A4, A5, A6, A7, A8)) { return type<R>(); }

  template <class R, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9> inline
  type<R> return_type(R (*)(A1, A2, A3, A4, A5, A6, A7, A8, A9)) { return type<R>(); }

  template <class R, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9, class A10> inline
  type<R> return_type(R (*)(A1, A2, A3, A4, A5, A6, A7, A8, A9, A10)) { return type<R>(); }

  // TODO(?): handle 'const void'

  // member functions
  template <class R, class T> inline
  type<R> return_type(R (T::*)()) { return type<R>(); }

  template <class R, class T, class A1> inline
  type<R> return_type(R (T::*)(A1)) { return type<R>(); }

  template <class R, class T, class A1, class A2> inline
  type<R> return_type(R (T::*)(A1, A2)) { return type<R>(); }

  template <class R, class T, class A1, class A2, class A3> inline
  type<R> return_type(R (T::*)(A1, A2, A3)) { return type<R>(); }

  template <class R, class T, class A1, class A2, class A3, class A4> inline
  type<R> return_type(R (T::*)(A1, A2, A3, A4)) { return type<R>(); }

  template <class R, class T, class A1, class A2, class A3, class A4, class A5> inline
  type<R> return_type(R (T::*)(A1, A2, A3, A4, A5)) { return type<R>(); }

  template <class R, class T, class A1, class A2, class A3, class A4, class A5, class A6> inline
  type<R> return_type(R (T::*)(A1, A2, A3, A4, A5, A6)) { return type<R>(); }

  template <class R, class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7> inline
  type<R> return_type(R (T::*)(A1, A2, A3, A4, A5, A6, A7)) { return type<R>(); }

  template <class R, class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8> inline
  type<R> return_type(R (T::*)(A1, A2, A3, A4, A5, A6, A7, A8)) { return type<R>(); }

  template <class R, class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9> inline
  type<R> return_type(R (T::*)(A1, A2, A3, A4, A5, A6, A7, A8, A9)) { return type<R>(); }

  template <class R, class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9, class A10> inline
  type<R> return_type(R (T::*)(A1, A2, A3, A4, A5, A6, A7, A8, A9, A10)) { return type<R>(); }

  template <class R, class T> inline
  type<R> return_type(R (T::*)() const) { return type<R>(); }

  template <class R, class T, class A1> inline
  type<R> return_type(R (T::*)(A1) const) { return type<R>(); }

  template <class R, class T, class A1, class A2> inline
  type<R> return_type(R (T::*)(A1, A2) const) { return type<R>(); }

  template <class R, class T, class A1, class A2, class A3> inline
  type<R> return_type(R (T::*)(A1, A2, A3) const) { return type<R>(); }

  template <class R, class T, class A1, class A2, class A3, class A4> inline
  type<R> return_type(R (T::*)(A1, A2, A3, A4) const) { return type<R>(); }

  template <class R, class T, class A1, class A2, class A3, class A4, class A5> inline
  type<R> return_type(R (T::*)(A1, A2, A3, A4, A5) const) { return type<R>(); }

  template <class R, class T, class A1, class A2, class A3, class A4, class A5, class A6> inline
  type<R> return_type(R (T::*)(A1, A2, A3, A4, A5, A6) const) { return type<R>(); }

  template <class R, class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7> inline
  type<R> return_type(R (T::*)(A1, A2, A3, A4, A5, A6, A7) const) { return type<R>(); }

  template <class R, class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8> inline
  type<R> return_type(R (T::*)(A1, A2, A3, A4, A5, A6, A7, A8) const) { return type<R>(); }

  template <class R, class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9> inline
  type<R> return_type(R (T::*)(A1, A2, A3, A4, A5, A6, A7, A8, A9) const) { return type<R>(); }

  template <class R, class T, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9, class A10> inline
  type<R> return_type(R (T::*)(A1, A2, A3, A4, A5, A6, A7, A8, A9, A10) const) { return type<R>(); }

  arg_decl declare_argument_type(  arg_decl& args, const std::type_info& typ);
  arg_decl declare_argument_types( const std::type_info& a1  = typeid(void),
                                   const std::type_info& a2  = typeid(void),
                                   const std::type_info& a3  = typeid(void),
                                   const std::type_info& a4  = typeid(void),
                                   const std::type_info& a5  = typeid(void),
                                   const std::type_info& a6  = typeid(void),
                                   const std::type_info& a7  = typeid(void),
                                   const std::type_info& a8  = typeid(void),
                                   const std::type_info& a9  = typeid(void),
                                   const std::type_info& a10 = typeid(void),
                                   const std::type_info& a11 = typeid(void) );

  template<class R, class T> inline
  const arg_decl signature( R (T::*)() )  {
    return declare_argument_types();
  }
  template<class R, class T, class A1> inline
  const arg_decl signature( R (T::*)(A1) )  {
    return declare_argument_types(typeid(A1));
  }
  template<class R, class T, class A1, class A2> inline
  const arg_decl signature( R (T::*)(A1,A2) )  {
    return declare_argument_types(typeid(A1), typeid(A2));
  }
  template<class R, class T, class A1, class A2, class A3> inline
  const arg_decl signature( R (T::*)(A1,A2,A3) )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3));
  }
  template<class R, class T, class A1, class A2, class A3, class A4> inline 
  const arg_decl signature( R (T::*)(A1,A2,A3,A4) )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4));
  }
  template<class R, class T, class A1, class A2, class A3, class A4, class A5> inline
  const arg_decl signature( R (T::*)(A1,A2,A3,A4,A5) )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4),
                                  typeid(A5));
  }
  template<class R, class T, class A1, class A2, class A3, class A4, class A5, class A6> inline 
  const arg_decl signature( R (T::*)(A1,A2,A3,A4,A5,A6) )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4),
                                  typeid(A5), typeid(A6));
  }
  template<class R, class T, class A1, class A2, class A3, class A4, class A5, class A6, 
           class A7> inline
  const arg_decl signature( R (T::*)(A1,A2,A3,A4,A5,A6,A7) )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4),
                                  typeid(A5), typeid(A6), typeid(A7));
  }
  template<class R, class T, class A1, class A2, class A3, class A4, class A5, class A6, 
           class A7, class A8> inline
  const arg_decl 
  signature( R (T::*)(A1,A2,A3,A4,A5,A6,A7,A8) )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4),
                                  typeid(A5), typeid(A6), typeid(A7), typeid(A8));
  }
  template<class R, class T, class A1, class A2, class A3, class A4, class A5, class A6, 
           class A7, class A8, class A9> inline 
  const arg_decl
  signature( R (T::*)(A1,A2,A3,A4,A5,A6,A7,A8,A9) )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4),
                                  typeid(A5), typeid(A6), typeid(A7), typeid(A8),
                                  typeid(A9));
  }
  template<class R, class T, class A1, class A2, class A3, class A4, class A5, class A6, 
           class A7, class A8, class A9, class A10> inline
  const arg_decl
  signature( R (T::*)(A1,A2,A3,A4,A5,A6,A7,A8,A9,A10) )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4),
                                  typeid(A5), typeid(A6), typeid(A7), typeid(A8),
                                  typeid(A9), typeid(A10));
  }

  template<class R, class T> inline
  const arg_decl signature( R (T::*)() const )  {
    return declare_argument_types();
  }
  template<class R, class T, class A1> inline
  const arg_decl signature( R (T::*)(A1) const )  {
    return declare_argument_types(typeid(A1));
  }
  template<class R, class T, class A1, class A2> inline
  const arg_decl signature( R (T::*)(A1,A2) const )  {
    return declare_argument_types(typeid(A1), typeid(A2));
  }
  template<class R, class T, class A1, class A2, class A3> inline
  const arg_decl signature( R (T::*)(A1,A2,A3) const )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3));
  }
  template<class R, class T, class A1, class A2, class A3, class A4> inline 
  const arg_decl signature( R (T::*)(A1,A2,A3,A4) const )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4));
  }
  template<class R, class T, class A1, class A2, class A3, class A4, class A5> inline
  const arg_decl signature( R (T::*)(A1,A2,A3,A4,A5) const )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4),
                                  typeid(A5));
  }
  template<class R, class T, class A1, class A2, class A3, class A4, class A5, class A6> inline 
  const arg_decl signature( R (T::*)(A1,A2,A3,A4,A5,A6) const )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4),
                                  typeid(A5), typeid(A6));
  }
  template<class R, class T, class A1, class A2, class A3, class A4, class A5, class A6, 
           class A7> inline
  const arg_decl signature( R (T::*)(A1,A2,A3,A4,A5,A6,A7) const )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4),
                                  typeid(A5), typeid(A6), typeid(A7));
  }
  template<class R, class T, class A1, class A2, class A3, class A4, class A5, class A6, 
           class A7, class A8> inline
  const arg_decl 
  signature( R (T::*)(A1,A2,A3,A4,A5,A6,A7,A8) const )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4),
                                  typeid(A5), typeid(A6), typeid(A7), typeid(A8));
  }
  template<class R, class T, class A1, class A2, class A3, class A4, class A5, class A6, 
           class A7, class A8, class A9> inline 
  const arg_decl
  signature( R (T::*)(A1,A2,A3,A4,A5,A6,A7,A8,A9) const )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4),
                                  typeid(A5), typeid(A6), typeid(A7), typeid(A8),
                                  typeid(A9));
  }
  template<class R, class T, class A1, class A2, class A3, class A4, class A5, class A6, 
           class A7, class A8, class A9, class A10> inline
  const arg_decl
  signature( R (T::*)(A1,A2,A3,A4,A5,A6,A7,A8,A9,A10) const )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4),
                                  typeid(A5), typeid(A6), typeid(A7), typeid(A8),
                                  typeid(A9), typeid(A10));
  }

  template<class R> inline
  const arg_decl signature( R (*)() )  {
    return declare_argument_types();
  }
  template<class R, class A1> inline
  const arg_decl signature( R (*)(A1) )  {
    return declare_argument_types(typeid(A1));
  }
  template<class R, class A1, class A2> inline
  const arg_decl signature( R (*)(A1,A2) )  {
    return declare_argument_types(typeid(A1), typeid(A2));
  }
  template<class R, class A1, class A2, class A3> inline
  const arg_decl signature( R (*)(A1,A2,A3) )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3));
  }
  template<class R, class A1, class A2, class A3, class A4> inline 
  const arg_decl signature( R (*)(A1,A2,A3,A4) )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4));
  }
  template<class R, class A1, class A2, class A3, class A4, class A5> inline
  const arg_decl signature( R (*)(A1,A2,A3,A4,A5) )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4),
                                  typeid(A5));
  }
  template<class R, class A1, class A2, class A3, class A4, class A5, class A6> inline 
  const arg_decl signature( R (*)(A1,A2,A3,A4,A5,A6) )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4),
                                  typeid(A5), typeid(A6));
  }
  template<class R, class A1, class A2, class A3, class A4, class A5, class A6, 
           class A7> inline
  const arg_decl signature( R (*)(A1,A2,A3,A4,A5,A6,A7) )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4),
                                  typeid(A5), typeid(A6), typeid(A7));
  }
  template<class R, class A1, class A2, class A3, class A4, class A5, class A6, 
           class A7, class A8> inline
  const arg_decl 
  signature( R (*)(A1,A2,A3,A4,A5,A6,A7,A8) )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4),
                                  typeid(A5), typeid(A6), typeid(A7), typeid(A8));
  }
  template<class R, class A1, class A2, class A3, class A4, class A5, class A6, 
           class A7, class A8, class A9> inline 
  const arg_decl
  signature( R (*)(A1,A2,A3,A4,A5,A6,A7,A8,A9) )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4),
                                  typeid(A5), typeid(A6), typeid(A7), typeid(A8),
                                  typeid(A9));
  }
  template<class R, class A1, class A2, class A3, class A4, class A5, class A6, 
           class A7, class A8, class A9, class A10> inline
  const arg_decl
  signature( R (*)(A1,A2,A3,A4,A5,A6,A7,A8,A9,A10) )  {
    return declare_argument_types(typeid(A1), typeid(A2), typeid(A3), typeid(A4),
                                  typeid(A5), typeid(A6), typeid(A7), typeid(A8),
                                  typeid(A9), typeid(A10));
  }

  /// Invoke empty destructor
  template <class T> struct CTOR0 {
    static void* call(void* mem)  { return new(mem) T();            }
  };

  /// Invoke destructor
  template <class T> struct DTOR  {
    static void call(void* mem)   { if ( mem ) ((T*)mem)->~T();     }
  };

  /// Calculate the offset between any two classes
  template <class T, class B> struct offset {
    static int call(void* p)
    { return ((char*)((T*)p)) - ((char*)((B*)((T*)p)));             }
  };

  class DictBase;

  /// Callback structure with arguments
  template <class TAG, class F, class R> struct call_with_args  {
    typedef F fun_type;
    fun_type  m_pmf;

    void set(fun_type pmf)                 { m_pmf = pmf;        }
    static call_with_args& instance()   
    {  static call_with_args inst; return inst;                  }
    static void void_func_call(void* o, arg_type args)
    {  caller<R>::call(instance().m_pmf, o, args);               }
    static void* ret_func_call(void* o, arg_type args)
    {  return caller<R>::call(instance().m_pmf, o, args);        }
  };
  /// Callback structure without arguments
  template <class TAG, class F, class R> struct call_sans_args  {
    typedef F fun_type;
    fun_type  m_pmf;

    void set(fun_type pmf)                 { m_pmf = pmf;        }
    static call_sans_args& instance()   
    {  static call_sans_args inst; return inst;                  }
    static void void_func_call(void* o)    {  
      std::vector<void*> args;
      caller<R>::call(instance().m_pmf, o, args);
    }
    static void* ret_func_call(void* o)    {  
      std::vector<void*> args;
      return caller<R>::call(instance().m_pmf, o, args);
    }
  };

  class DictBase {
  public:
    typedef seal::reflect::ClassBuilder ClassBuilder;
    ClassBuilder* c; 
    std::string   name;
    DictBase(const std::type_info& typ);
    virtual ~DictBase() {}
    void setProperty(const std::string& nam, const std::string& val);
    void publish()  const;
    void finalize();
    void declareMethodRaw(const std::string& a1, const std::string& a2, const arg_decl& arg_types, 
                          const std::type_info& r_typ, 
                          void* void_call, 
                          void* ret_call);
    void declareMethodRaw(const std::string& a1, const std::string& a2, 
                          const std::type_info& r_typ, 
                          void* void_call, 
                          void* ret_call);
    virtual void i_create(const std::string& nam, 
      const std::string& dsc, 
      const std::type_info& typ, 
      size_t siz,
      bool primitive = false,
      const seal::reflect::CONTAINER contType = seal::reflect::NOCONTAINER, 
      int arraySize = 0,
      int modifiers = seal::reflect::NOMODIFIER);

    virtual void i_create_and_finalize( 
      const std::string& nam,
      const std::string& dsc,
      const std::type_info& typ,
      size_t siz,
      bool primitive = false,
      const seal::reflect::CONTAINER contType = seal::reflect::NOCONTAINER, 
      int arraySize = 0,
      int modifiers = seal::reflect::NOMODIFIER);

    static void i_createGenericContainer(
      const std::type_info& typ,
      size_t siz,
      const seal::reflect::CONTAINER contType = seal::reflect::NOCONTAINER, 
      void* (*ctor)(void*)=0,
      void  (*dtor)(void*)=0,
      void* (*elements)(void*, arg_type)=0,
      void* (*allocate)(void*, arg_type)=0
      );
  };

  /** class DictDefs.
    * @author  M.Frank
    * @date    11/12/2002
    * @version 1.0
    */
  template <class T> class DictDefs  : public DictBase  {
  public:
    typedef CTOR0 < T > ctor;
    typedef DTOR  < T > dtor;
    static T* obj()                           {    return (T*)0x0;            }
    /// Default constructor
    DictDefs() : DictBase(typeid(T))          {                               }
    /// Default destructor
    virtual ~DictDefs() { }

    virtual void create(const std::string& nam, const std::string& dsc, int modifiers = seal::reflect::PUBLIC)
    { i_create(nam.c_str(), dsc, typeid(T), sizeof(T), false, seal::reflect::NOCONTAINER, 0, modifiers);                  }
    virtual void createClass(const std::string& nam, const std::string& dsc, int modifiers = seal::reflect::PUBLIC)
    { i_create(nam.c_str(), dsc, typeid(T), sizeof(T), false, seal::reflect::NOCONTAINER, 0, modifiers);                  }
    virtual void createContainer(const std::string& nam, const std::string& dsc, const seal::reflect::CONTAINER typ, int modifiers = seal::reflect::PUBLIC)
    { i_create(nam.c_str(), dsc, typeid(T), sizeof(T), false, typ, 0, modifiers);      }
    virtual void createArray(const std::string& nam, const std::string& dsc, bool prim, int siz, int modifiers = seal::reflect::PUBLIC)
    { i_create(nam, dsc, typeid(T), sizeof(T), prim, seal::reflect::NOCONTAINER, siz, modifiers);    }

    void makeContainer( const std::string& ns, const std::string& nam, const std::string& comp,const seal::reflect::CONTAINER typ) {
      std::string n = ns;
      if ( n.length()>0 ) n += "::";
      std::string cn = nam + "<" + comp + ",std::allocator<" + comp + "> >";
      createContainer(n+cn, "Standard "+nam+" of type "+cn , typ);
      addConstructor(nam, "Standard constructor for "+cn);
      addDestructor ("~"+nam,"Standard destructor for "+cn);
    }
    template <class B> void declareBase(type<B> /* t */, const std::string& nam)
    { c->addSuperClass(nam.c_str(),offset<T,B>::call((void*)0x1000),offset<T,B>::call);}
    /// Add single field to the class definition
    template <class D> void addField(const std::string& a1, const std::string& a2, const std::string& a3, const D& a4) 
    {      const D* p = &a4; c->addField(a1.c_str(), a2.c_str(), a3.c_str(),int(p));      }
    /// Add empty constructor to class definition
    template <class A, class B> void addConstructor(A a1, B a2) 
    {      c->addMethod(a1,a2,a1,CTOR0<T>::call);                 }
    /// Add destructor to class definition
    template <class A, class B> void addDestructor(A a1, B a2) 
    {      c->addMethod(a1,a2,DTOR<T>::call);                     }
    /// Add a single method to the class description; We allow max.100 methods!
    template <class TAG, class F> void addMethod(TAG t, const std::string& a1, const std::string& a2, F pmf)
    { declareMethod(t, a1, a2, return_type(pmf), signature(pmf), pmf);    }
    template <typename TAG, typename F, typename R>
    void declareMethod(TAG                /* t   */, 
                       const std::string&    a1, 
                       const std::string&    a2, 
                       R                  /* ret */, 
                       const arg_decl&       arg_types, 
                       F                     pmf )
    {
      //std::cout << typeid(T).name() << "::" << typeid(pmf).name() << std::endl;
      if ( arg_types.size()==0 )    {
        typedef typename R::id r_type;
        typedef call_sans_args< TAG, F, r_type> wrap_;
        wrap_::instance().set(pmf);
        declareMethodRaw(a1, a2, typeid(r_type), (void*)wrap_::void_func_call, (void*)wrap_::ret_func_call);
      }
      else  {
        typedef typename R::id r_type;
        typedef call_with_args< TAG, F, r_type> wrap_;
        wrap_::instance().set(pmf);
        declareMethodRaw(a1, a2, arg_types, typeid(r_type), (void*)wrap_::void_func_call, (void*)wrap_::ret_func_call);
      }
    }
  };

  template <class T> class Dict {};

  /** Derive object types: class ArrayType.
    * @author  M.Frank
    * @date    11/12/2002
    * @version 1.0
    */
  template <class T> class SimpleType : public DictBase  {
  public:
    /// Default constructor
    SimpleType(bool prim) : DictBase(typeid(T))     {
      std::string nam = pool::typeName(typeid(T));
      i_create_and_finalize(nam,"Simple type "+nam,typeid(T),sizeof(T),
        prim,seal::reflect::NOCONTAINER,1);
    }
    /// Default destructor
    virtual ~SimpleType() { }
  };

  /** Derive object types: class ArrayType.
    * @author  M.Frank
    * @date    11/12/2002
    * @version 1.0
    */
  template <class T> class ArrayType  {
  public:
    /// Default constructor
    ArrayType(bool prim, int siz) {
      DictBase d(typeid(T*));
      std::string nam = pool::typeName(typeid(T*));
      d.i_create_and_finalize(nam+"[]","Generic array of type "+nam,typeid(T*),
        sizeof(T), prim, seal::reflect::ARRAY, siz);
    }
    /// Default constructor
    ArrayType(const std::string& nam, bool prim, int siz) {
      DictBase d(typeid(T[1]));
      d.i_create_and_finalize(nam,"Array of type "+nam,typeid(T*),sizeof(T), 
        prim, seal::reflect::ARRAY, siz);
    }
    /// Default destructor
    virtual ~ArrayType() { }
    /// Standard function
    virtual void publish() { }
  };
}

#define START_MULTI_DICT_MAP(ID, WRAP)                      \
namespace pool {                                            \
template <>                                                 \
class Dict< WRAP >:public DictDefs< WRAP >{public:Dict();}; \
static Dict<WRAP > dict_##ID;                               \
Dict< WRAP >::Dict() {

#define START_DICT_MAP(WRAP)           START_MULTI_DICT_MAP(0,  WRAP)
#define BEGIN_MULTI_DICT_MAP(ID, WRAP) START_MULTI_DICT_MAP(ID, WRAP)

#define END_DICT_MAP   finalize(); }}


#endif // POOL_DBDICTIONARY_H
