// $Id: DbHeap.h,v 1.11 2007/11/20 16:43:06 frankb Exp $
//====================================================================
//  DbHeap class definitions
//--------------------------------------------------------------------
//
//  Package    : StorageSvc  (The POOL project)
//  @author      M.Frank
//====================================================================
#ifndef POOL_STORAGESVC_DBHEAP_H
#define POOL_STORAGESVC_DBHEAP_H

// Framework include files
#include "StorageSvc/pool.h"
#include "StorageSvc/DbHandleBase.h"

/*
 *  POOL namespace declaration
 */
namespace pool  {

  // Forward declarations
  class DbContainer;
  template <class T> class DbObjectHandle;

  /** Db objects: class DbHeap

      Description:
      Handle managing a persistent memory.

      @author  M.Frank
      @version 1.0
  */
  class DbHeap  {
  private:
    /// Heap handle
    void*   m_handle;
    /// External heap identifier
    void*   m_identifier;
    /// Initial heap size
    size_t  m_initial;
    /// Maximal allowed heap size
    size_t  m_total;

  private:
    /// Allocate a chunk of raw memory from the local heap
    static void* allocateRawMemory (void* handle, size_t siz);
    /// Free a chunk of raw memory from the local heap
    static DbStatus freeRawMemory    (void* handle, void* ptr, size_t siz);

  public:
    /// Standard Constructor
    DbHeap();
    /// Standard Destructor
    virtual ~DbHeap();
    /// Create the heap
    DbStatus create(void* identifier, size_t intitial, size_t total);
    /// Release the created heap
    DbStatus release();
    /// Access the object guard size
    static size_t guardSize();
    /// Number of chunks currently allocated from the heap
    static long long int numInstances();
    /// Access the container of an object
    static DbContainer& container(const DbObject* pObj);
    /// Free an object, which was initially allocated from the heap
    static DbStatus free(void* ptr, DbContainer* cntH);
    /// Free an object on the heap
    static DbObject* allocate(size_t siz,
                              DbContainer* cntH,
                              const DbLink* linkH,
                              DbObjectHandle<DbObject>* objH);
  };
}       // End namespace pool
#endif  // POOL_STORAGESVC_DBHEAP_H
