// $Id: DbObjectGuard.h,v 1.13 2007/11/20 16:43:06 frankb Exp $
//====================================================================
//  DbObjectGuard definition
//--------------------------------------------------------------------
//
//  Package    : StorageSvc (The POOL project)
//
//  Description: Guard containing object persistency information
//
//  @author      M.Frank
//====================================================================
#ifndef POOL_DBOBJECTGUARD_H
#define POOL_DBOBJECTGUARD_H 1

// Framework include files
#include "StorageSvc/DbContainer.h"

/*
 * POOL namespace declaration
 */
namespace pool    {

  /** @class DbObjectGuard DbObjectGuard.h StorageSvc/DbObjectGuard.h 
    *
    *  Description:
    *  class containing persistency information for managing 
    *  basic persistable objects.
    *
    *  @author  M.Frank
    *  @version 1.0
    */
  class DbObjectGuard   {
  private:
    typedef void (*DeleteObject)(DbObject* pObj);
    /// Size of memory chunk
    size_t                        m_size;
    /// Reference counter
    int                           m_refCount;
    /// Installation for object deletion: call of destructor
    DeleteObject                  m_destruct;
    /// Reference to persistent container
    DbContainer                   m_container;
    /// OID
    DbLink                        m_oid;
   public:
    /// Destructor
    virtual ~DbObjectGuard();
    /// Standard constructor
    DbObjectGuard(int type = 0, int siz = 0);
    /// Copy constructor
    DbObjectGuard(const DbContainer& cntH, int siz = 0);
    /// Access reference counter
    int refCount()   const            { return m_refCount;        }
    /// Access memory chunk size
    size_t size()   const             { return m_size;            }
    /// Increase reference count
    int addRef()                      { return ++m_refCount;      }
    /// Decrease reference count
    int release()                     { return --m_refCount;      }
    /// Access oid
    DbLink& oid()                     { return m_oid;             }
    /// Access Storage Type
    const DbType& type()  const       { return m_container.type();}
    /// Accessor: get object container
    DbContainer& container()          { return m_container;       }
    /// Accessor: get object delete function
    DeleteObject freeHandler()        { return m_destruct;        }
    /// Install object delete function
    void installFreeHandler(DeleteObject del) { m_destruct = del; }
  };
}       // End namespace pool
#endif  // POOL_DBOBJECTGUARD_H
