//====================================================================
//
//  Package    : StorageSvc (The POOL project)
//
//  Author     : M.Frank
//====================================================================
#ifndef POOL_STORAGESVC_IIOHANDLERFACTORY_H
#define POOL_STORAGESVC_IIOHANDLERFACTORY_H

#include "StorageSvc/IIOHandler.h"
#include "PluginManager/PluginFactory.h"

// Forward declarations
namespace ROOT { namespace Reflex {  class Type;   }}

/*
 *    POOL namespace declaration
 */
namespace pool {

  /**@class IIOHandlerFactory IIOHandlerFactory.h StorageSvc/IIOHandlerFactory.h
    *
    *  Implement SEAL plugin model for SEAL 1.4.0
    *
    *  @author  M.Frank
    *  @version 1.0
    */
  class IIOHandlerFactory 
    : public seal::PluginFactory< IIOHandler* (const ROOT::Reflex::Type& classType) >  {
  public:
    static IIOHandlerFactory* get();
    IIOHandlerFactory();
  };
}      // End namespace pool


// Reflex object factory definition
#include "Reflex/PluginService.h"
namespace {
  template < typename P > class Factory<P, pool::IIOHandler*(ROOT::Reflex::Type)> {
    public: static void* Func(void*,const std::vector<void*>& arg,void*) 
    { return (pool::IIOHandler*)new P(*(ROOT::Reflex::Type*)arg[0]);   }
  };
}
#define DECLARE_IOHANDLER(x) PLUGINSVC_FACTORY(x,pool::IIOHandler*(ROOT::Reflex::Type))

#endif // POOL_STORAGESVC_IIOHANDLERFACTORY_H
