#include "StorageSvc/DbDataHandlerGuard.h"
#include "StorageSvc/DbObjectCallBack.h"
#include "StorageSvc/DbDefaultDataHandler.h"
using namespace pool;

namespace  {
  static DataCallBack* s_Caller = 0;
  static std::vector< DataCallBack* > s_Callers(1, static_cast< DataCallBack* >( 0 ) );

  void pushDbDataHandlerGuard(DataCallBack* cb)  {
    s_Callers.push_back( cb );
    s_Caller = cb;
  }
  void popDbDataHandlerGuard()  {
    s_Callers.pop_back();
    s_Caller = s_Callers.empty() ? 0 : s_Callers.back();
  }
}

/// Standard Constrcutor; sets the caller environment
DbDataHandlerGuard::DbDataHandlerGuard(DataCallBack* caller,DbDatabase* dbH)
: m_handler(0)
{
  if ( caller )   {
    pushDbDataHandlerGuard(caller);
    DataHandler* handler = caller->handler();
    if ( handler )  {
      if ( handler->quickCast() == 0xFEEDFEED )  {
        m_handler = (DbDefaultDataHandler*)handler;
        if ( m_handler )  {
          m_handler->setIOContext(dbH);
          return;
        }
      }
    }
  }
}

/// Standard Constrcutor; releases the caller environment
DbDataHandlerGuard::~DbDataHandlerGuard()   {
  if ( m_handler )   {
    m_handler->setIOContext(0);
  }
  popDbDataHandlerGuard();
}

/// Access to current handler
DataCallBack* DbDataHandlerGuard::caller()  {
  return s_Caller;
}
