#include "POOLCore/PoolMessageStream.h"
#include "StorageSvc/DbPrint.h"

namespace pool  {
  void setLogLevel(seal::Msg::Level lvl);
  seal::Msg::Level getLogLevel();

  /** @class DbPrintReporter DbPrint.h StorageSvc/DbPrint.h
    *
    * Default POOL reporter implementation
    *
    * @author   Markus Frank
    * @version  1.0
    * @date     15/07/2002
    */
  class DbPrintReporter : public IMsgReporter  {
  public:
    /// Default constructor
    DbPrintReporter()           {  
      // std::cout << "Installing PoolMessageStream reporter...." << (int)pool::getLogLevel() << std::endl;
      DbPrint::installMsgReporter(this);   
    }
    /// Destructor
    virtual ~DbPrintReporter()  {                                                 }
    /// Release reference to reporter
    virtual void release()                                 {  delete this;        }
    /// Access output level
    virtual DbPrint::MsgLevel outputLevel() const      
    {  return DbPrint::MsgLevel(pool::getLogLevel());                             }
    /// Modify output level
    virtual void setOutputLevel(DbPrint::MsgLevel lvl) 
    {  pool::setLogLevel((seal::Msg::Level)lvl);                                  }
    /// Report a message
    virtual void report(int lvl, const std::string& src, const std::string& msg)  {
      PoolMessageStream str(src);
      str << (seal::Msg::Level)lvl << msg << seal::endmsg;
    }
  };
  static DbPrintReporter* s_reporter = new DbPrintReporter;
  IMsgReporter* getDbPrintReporter()  {    return s_reporter;           }
}
