// $Id: DbPrint.cpp,v 1.12 2007/11/27 20:50:55 frankb Exp $
//====================================================================
//  Implementation file of a DbPrint object
//--------------------------------------------------------------------
//
//  Package: pool/StorageSvc (The POOL project)
//
//  Description: Generic data persistency for LHC
//
//  @author      M.Frank
//====================================================================
#include "StorageSvc/DbPrint.h"
#include <iostream>

#include "POOLCore/PoolMessageStream.h"
namespace pool  {
  void setLogLevel(seal::Msg::Level lvl);
  seal::Msg::Level getLogLevel();
}

// POOL namespace
using namespace pool;

// Preset default message reporter to None
static std::auto_ptr<IMsgReporter> s_msgReporter(new DbMsgReporter);

/// Default constructor
DbMsgReporter::DbMsgReporter() {
}

/// Access output level
DbPrint::MsgLevel DbMsgReporter::outputLevel() const      {
  return DbPrint::MsgLevel(pool::getLogLevel());
}

/// Modify output level
void DbMsgReporter::setOutputLevel(DbPrint::MsgLevel lvl) { 
  pool::setLogLevel((seal::Msg::Level)lvl); 
} 

/// report mesage to stdout
void DbMsgReporter::report(int lvl, const std::string& src, const std::string& msg)  {
  const char* level = "UNKNOWN";
  switch ( lvl )    {
#define SET(x)  case DbPrint::x:  level=#x;  break 
    SET( NIL );
    SET( VERBOSE );
    SET( DEBUG );
    SET( INFO );
    SET( WARNING );
    SET( ERROR );
    SET( FATAL );
    case DbPrint::ALWAYS:
      level="SUCCESS";
      break;
    default:
      level="UNKNOWN";
      break;
    break;
#undef SET
  }
  size_t len = src.length();
  std::cout << std::setw(32) << std::left << src.substr(0,len<32 ? len : 32)
            << " " << std::setw(12) << std::left << level
            << " " << msg << std::endl;
}
/// Install new message reporter object
void DbPrint::installMsgReporter(IMsgReporter* reporter)   {
  if ( reporter )  {
    if ( s_msgReporter.get() )  {
      s_msgReporter->release();
      s_msgReporter.release();
    }
    s_msgReporter = std::auto_ptr<IMsgReporter>(reporter);
    return;
  }
  s_msgReporter = std::auto_ptr<IMsgReporter>(new DbMsgReporter);
}

/// Preset default verbosity if no message service is present.
void DbPrint::setMsgVerbosity(MsgLevel lvl)  {
  if ( s_msgReporter.get() ) s_msgReporter->setOutputLevel(lvl);
}

/// Standard initializing constructor
DbPrint::DbPrint(const std::string& source) 
  : m_act(false), m_source(source), m_currLevel(NIL)
{
  m_level = DbPrint::MsgLevel(pool::getLogLevel());
}

/// Propagate message to reporter
void DbPrint::doOutput()  {
  // std::cout << m_source << " " << m_level << " " << m_currLevel << " " 
  //           << (int)pool::getLogLevel() << " " << m_stream.str() << std::endl;
  s_msgReporter->report(m_currLevel,m_source,m_stream.str());
  m_stream.str("");
}

/// Print data of type long long int
void DbPrint::prtLongLong(const long long int arg)  {
#ifdef _WIN32
  char buf[128];
  ::sprintf(buf,m_stream.flags()&std::ios::hex ? "%I64x" : "%I64d", arg);
  m_stream << buf;
#else
  m_stream << arg;
#endif
}

/// Print data of type unsigned long long int
void DbPrint::prtLongLong(const unsigned long long int arg)  {
#ifdef _WIN32
  char buf[128];
  ::sprintf(buf,m_stream.flags()&std::ios::hex ? "%I64x" : "%I64d", arg);
  m_stream << buf;
#else
  m_stream << arg;
#endif
}
