// $Id: poolDb.cpp,v 1.27 2008/01/11 13:17:14 frankb Exp $
//====================================================================
//  DbFileFactory implementation
//--------------------------------------------------------------------
//
//  Package    : System (The POOL project)
//
//  Description: Management of the open Db files
//
//  @author      M.Frank
//====================================================================

#include "StorageSvc/pool.h"
#include "StorageSvc/DbType.h"
#include <stdexcept>
#include <iostream>
#include <iomanip>
#include <cstdio>
#include <map>

static bool s_debug = (0 != ::getenv("POOL_TRACE")) || (0 != ::getenv("POOLDB_TRACE"));

static void startDebugger()  {
#ifdef WIN32
    _asm int 3
#endif
}

bool pool::doTrace() {
  return s_debug;
}

void pool::enableBreak(bool yes_no)   {
  s_debug = yes_no;
}

/// Debug break with printout and exception chaining
void pool::debugBreak( const std::string& src, 
                       const std::string& msg,
                       const std::exception& e,
                       bool rethrow)
{
  std::string m = "Unknown Source> ";
  m += e.what();
  m += "\n";
  m += src + "> " + msg;
  std::cout << "ERROR (pool): " << std::endl << m << std::endl;
  debugBreak();
  if ( rethrow )    {
    throw std::runtime_error(m);
  }
}

void pool::debugBreak(const std::string& src, const std::string& msg, bool rethrow) {
  std::string m = src + "> " + msg;
  std::cout << "ERROR (pool): " << std::endl << m << std::endl;
  debugBreak();
  if ( rethrow )    {
    throw std::runtime_error(m);
  }
}

void pool::debugBreak()   {
  if ( s_debug )    {
    startDebugger();
  }
  else  {
    //std::cout << "debugbreak (poolDb) However, breaks are DISABLED" 
    //          << std::endl;
  }
}

/// Translate acce mode to string
const char* pool::accessMode(pool::DbAccessMode mode)   {
  if      ( mode & pool::READ      ) 
    return "READ     ";
  else if ( mode & pool::UPDATE && mode & pool::CREATE )
    return "CREA/UPDA";
  else if ( mode & pool::UPDATE    ) 
    return "UPDATE   ";
  else if ( mode & pool::CREATE    )
    return "CREATE   ";
  else if ( mode & pool::NOT_OPEN  ) 
    return "NOT_OPEN ";
  else                                 
    return "UNKNOWN  ";
}

/// Bad pointer assignment occured. Do printout for debugging puposes
void pool::badAssignMessage(const char* msg)   {
  std::cout << "ERROR (pool): "
            << "--> Invalid pointer assignment "
            << msg << std::endl;
}

/// Last resort error handler. Throws exception if sc.severity==ERROR
void pool::unexpectedProblem(const DbStatus& sc, const char* msg) {
  std::cout << "ERROR (pool): " << msg << std::endl;
  switch(sc.severity()) {
  case DbStatus::INFO:
  case DbStatus::SUCCESS:
  case DbStatus::WARNING:
    break;
  case DbStatus::ERROR:
    /// Severe problem: throw exception
    break;
  }
}

