#include "POOLCore/Token.h"
#include "PersistencySvc/Placement.h"
#include "PersistencySvc/ITransaction.h"
#include "DataSvc/IDataSvc.h"
#include "DataSvc/Ref.h"
#include "DataSvc/RefException.h"
#include "StorageSvc/DbType.h"
#include "StorageSvc/poolDb.h"
#include <stdexcept>
#include <fstream>
#include <vector>
#include "PersistencySvc/ISession.h"
#include "PersistencySvc/DatabaseConnectionPolicy.h"
#include "TestBase/TestBase.h"

#include "DataModel.h"

#include "TROOT.h"
#include "TClass.h"
#include "TStreamerInfo.h"
#include "TStreamerElement.h"

namespace pool  {


  class TestDriver : public TestBase  {

    typedef ext::BColl<ext::B,ext::Obj> Base;
    typedef ext::TColl<ext::B,ext::Obj> ObColl;
    typedef ext::TColl<ext::B, Base> BaseColl;
    typedef ext::Id Id;

    private:
    static const std::string m_dbName1;
    static const std::string m_dbName2;
    static const DbType m_type1;
    static const DbType m_type2;
    char* mode; 
    public: 
    TestDriver(char* aMode) : TestBase("data/fileCatalog.xml"),mode(aMode){

      std::vector< std::string > libraries;
      libraries.push_back( "DataRegression_DataSvc_CMSCollDataModelDict" );
      init( libraries );
      setMessageVerbosityLevel(seal::Msg::Verbose);
    };

    /// write objects to the persistent store
    void write(unsigned int ntimes){

      std::cout << "starting WRITE procedure\n";
      pool::DatabaseConnectionPolicy policy;
      policy.setWriteModeForNonExisting( pool::DatabaseConnectionPolicy::CREATE );
      policy.setWriteModeForExisting( pool::DatabaseConnectionPolicy::OVERWRITE );
      m_dataSvc->session().setDefaultConnectionPolicy( policy );

      std::ofstream tokensFile("data/tokens.txt");

      Placement place1, place2;  
      place1.setDatabase(m_dbName1,pool::DatabaseSpecification::PFN );
      place1.setContainerName("test1_cont_1");
      place1.setTechnology(m_type1.type());
      place2.setDatabase(m_dbName2,pool::DatabaseSpecification::PFN );
      place2.setContainerName("test2_cont_1");
      place2.setTechnology(m_type2.type());

      std::cout << "size of Coll " << sizeof(BaseColl) << std::endl;
      std::cout << "size of Elem " << sizeof(BaseColl::Elem) << std::endl;

      // Start an update transaction
      if ( ! ( m_dataSvc->transaction().start( pool::ITransaction::UPDATE ) ) ) {
        printError("test ERROR: Could not start an UPDATE transaction");
      }  

      unsigned int i;
      for (i = 1; i <= ntimes; ++i )   {

        pool::Ref<BaseColl> o1(m_dataSvc, new BaseColl);
        try{
          o1.mark_write(place1);
          tokensFile << o1.toString() << std::endl;
        } catch (const RefException& e){
          printException(e);        
        }
        for (int j=0; j<10; j++) {
          pool::Ref<Base> oo(m_dataSvc, new ObColl);
          try{
            oo.mark_write(place2);
          } catch (const RefException& re){
            printException(re);
          }
          o1->me.push_back(BaseColl::Elem(ext::B(Id(j,j),Id(j,j)),oo));
        }
        std::cout << "size " << o1->me.size() << std::endl;
        std::cout << "capacity " << o1->me.capacity() << std::endl;

      }
      printM("Commit transactions...");
      if ( ! m_dataSvc->transaction().commit() ) {
        printError("test ERROR: Could not commit the WRITE transaction." );
      }
      gROOT->GetClass(
             "pair<ext::B,pool::Ref<ext::BColl<ext::B,ext::Obj> > >")->GetStreamerInfo()->ls();

      std::cout << "Written "<< i << " objects in the database\n";
      tokensFile.close();
      m_dataSvc->clearCache();
      return;
    }

    /// read objects from the persistent store
    void read() {

      std::cout << "starting READ procedure\n";
      std::vector<std::string> tokens;
      char buff[1024];
      std::ifstream tokensFile("data/tokens.txt",std::ios::in);
      if (tokensFile.bad()) {
        printError("test ERROR: Error opening file for read");
      }

      while(!tokensFile.getline(buff, sizeof(buff)).eof()) {
        std::cout << buff << '\n' ;
        std::string s(buff);
        tokens.push_back(s);
      }
      tokensFile.close();

      if ( ! m_dataSvc->transaction().start( pool::ITransaction::READ ) ) {
        printError("test ERROR: Could not start a READ transaction." );
      }

      unsigned int i=0;
      std::vector<std::string>::iterator iter;
      for(iter=tokens.begin();iter!=tokens.end();++iter){

        // Read first object
        pool::Ref<BaseColl> ref(m_dataSvc,*iter);

        if(ref){
           gROOT->GetClass(
             "pair<ext::B,pool::Ref<ext::BColl<ext::B,ext::Obj> > >")->GetStreamerInfo()->ls();
	  //std::pair<ext::X,pool::Ref<ext::B> > p;
	  std::vector<BaseColl::Elem>::iterator it;

	  //std::vector<std::pair<ext::X,pool::Ref<const ext::B<int> > > >::iterator it;
          for(it = ref->me.begin();it!=ref->me.end();++it){
            int ar = (*it).first.a.r;
	    std::cout << OVALTAG << "ar="<<ar<< std::endl;
            int ae = (*it).first.a.e;
	    std::cout << OVALTAG << "ae="<<ae<< std::endl;
            int br = (*it).first.b.r;
	    std::cout << OVALTAG << "br="<<br<< std::endl;
            int be = (*it).first.b.e;
	    std::cout << OVALTAG << "be="<<be<< std::endl;
            unsigned int s = (*it).first.s;
	    std::cout << OVALTAG << "s="<<s<< std::endl;
            unsigned int cs = (*it).first.cs;
	    std::cout << OVALTAG << "cs="<<cs<< std::endl;

            unsigned int sizeBColl = (*it).second->size();
	    std::cout << OVALTAG << "sizeBColl="<<sizeBColl<< std::endl;

          }
        }
        i++;
      }
        
      if ( ! m_dataSvc->transaction().commit() ) {
        printError("test ERROR: Could not commit the READ transaction." );
      }

      std::cout << "Object read back: "<< i << std::endl;

      printM("Clear cache...");
      m_dataSvc->clearCache();
      return;
    }

    void execute(){
      if(mode){
        if(mode[0] == 'W'){
          write(1);
        } else if(mode[0] == 'R'){
          read();
        }
      }else {
	std::cout << "No mode provided\n";
      }
    }
  };

}

const std::string pool::TestDriver::m_dbName1 = "data/test1.root";
const std::string pool::TestDriver::m_dbName2 = "data/test2.root";
const pool::DbType pool::TestDriver::m_type1 = pool::ROOTTREE_StorageType;
const pool::DbType pool::TestDriver::m_type2 = pool::ROOTTREE_StorageType;

int main(int argn, char** args)     {
  if(argn>1){
    pool::TestDriver theTest(args[1]);
    theTest.run();    
  } else {
    std::cout << "Wrong number of arguments provided\n";
    return 1;
  }
  return 0;
}
