#include "PersistencySvc/Placement.h"
#include "PersistencySvc/ITransaction.h"
#include "POOLCore/Token.h"
#include "DataSvc/IDataSvc.h"
#include "DataSvc/ICacheSvc.h"
#include "DataSvc/Ref.h"
#include "DataSvc/RefException.h"
#include "ListClass.h"
#include "MemberClass.h"
#include "LoopClass.h"
#include "DataSvc_CircularRef.h"
//#include "../../DataSvc/src/CacheSvc.h"

#ifndef WIN32
# include <unistd.h>
#endif

pool::CircularRefTest::CircularRefTest(std::string& dbName, const DbType& storage_type)
      :TestBase(),m_dbName(dbName),m_type(storage_type), tok1(0),tok2(0),
                                                         tok3(0),tok4(0){
  m_fid = fileId(m_dbName,m_type);
  if(m_fid == ""){
    printError("test ERROR: cannot retrieve file id");
    return;
  }
}

void pool::CircularRefTest::write(){
 
  Placement place1, place2;
  place1.setDatabase(m_dbName,pool::DatabaseSpecification::PFN );
  place1.setContainerName("db_cont_1");
  place1.setTechnology(m_type.type());
  place2.setDatabase(m_dbName,pool::DatabaseSpecification::PFN );
  place2.setContainerName("db_cont_2");
  place2.setTechnology(m_type.type());

  // Start an update transaction
  if ( ! ( m_dataSvc->transaction().start( pool::ITransaction::UPDATE ) ) ) {
    printError("test ERROR: Could not start an UPDATE transaction");
  }  

  Ref<MemberClass> refMember(m_dataSvc,new MemberClass);
  Ref<ListClass> refList(m_dataSvc,new ListClass);

  refList->m_list.push_back(refMember);
  refMember->belongsToList = refList;

  refMember.markWrite(place1);
  tok1 = const_cast<Token*>(refMember.token());
  tok1->addRef();

  refList.markWrite(place2);  
  tok2 = const_cast<Token*>(refList.token());
  tok2->addRef();

  if(!refMember.token()){
    printError("test ERROR: in refMember token is missing (1)");
  }
  if(!refMember->belongsToList.token()){
    printError("test ERROR: in refMember->list token is missing (1)");
  } 
  refMember.reset();

  printM("Commit transactions...");
  if ( ! m_dataSvc->transaction().commit() ) {
    printError("test ERROR: Could not commit the WRITE transaction." );
  }


  refMember.reset();
 
  if ( ! ( m_dataSvc->transaction().start( pool::ITransaction::READ ) ) ) {
    printError("test ERROR: Could not start an UPDATE transaction");
  }
   
  if(!refList.token()){
    printError("test ERROR: in refList token is missing (2)");
  }
  if(!(refList->m_list[0].token())){
    printError("test ERROR: in refList->member token is missing (2)");
  }

  if(!refMember.token()){
    printError("test ERROR: in refMember token is missing (2)");
  }
  if(!refMember->belongsToList.token()){
    printError("test ERROR: in refMember->list token is missing (2)");
  } 


  printM("Commit transactions...");
  if ( ! m_dataSvc->transaction().commit() ) {
    printError("test ERROR: Could not commit the WRITE transaction." );
  }

  refMember.reset();

  return;
}

void pool::CircularRefTest::writeOnScope(unsigned int nw){
 
  Placement place1, place2;
  place1.setDatabase(m_dbName,pool::DatabaseSpecification::PFN );
  place1.setContainerName("db_cont_1");
  place1.setTechnology(m_type.type());
  place2.setDatabase(m_dbName,pool::DatabaseSpecification::PFN );
  place2.setContainerName("db_cont_2");
  place2.setTechnology(m_type.type());

  // Start an update transaction
  if ( ! ( m_dataSvc->transaction().start( pool::ITransaction::UPDATE ) ) ) {
    printError("test ERROR: Could not start an UPDATE transaction");
  }  

  for(unsigned i=0;i<nw;i++) {
     
  Ref<MemberClass> refMember(m_dataSvc,new MemberClass);
  Ref<ListClass> refList(m_dataSvc,new ListClass);

  refList->m_list.push_back(refMember);
  refMember->belongsToList = refList;

  refMember.markWrite(place1);
  tok1 = const_cast<Token*>(refMember.token());

  refList.markWrite(place2);  
  tok2 = const_cast<Token*>(refList.token());

  if(!refMember.token()){
    printError("test ERROR: in refMember token is missing (1)");
  }
  if(!refMember->belongsToList.token()){
    printError("test ERROR: in refMember->list token is missing (1)");
  } 
  refMember.reset();

  }

  std::cout << OVALTAG << "Written "<<nw << " objects on DB"<<std::endl;

  printM("Commit transactions...");
  if ( ! m_dataSvc->transaction().commit() ) {
    printError("test ERROR: Could not commit the WRITE transaction." );
  }

  m_dataSvc->cacheSvc().resetCache();

  return;
}

/// read objects from the persistent store
void pool::CircularRefTest::read(){
  if(SharedAnyPtr::numberOfInstances()){
    SharedAnyPtr::dumpRegister();
  }
  if ( ! m_dataSvc->transaction().start( pool::ITransaction::READ ) ) {
    printError("test ERROR: Could not start a READ transaction." );
  }

  Ref<MemberClass> refMember(m_dataSvc,*tok1);
  tok1->release();
  Ref<ListClass> refList(m_dataSvc,*tok2);
  tok2->release();
  if(refMember->belongsToList.ptr() != refList.ptr()){
    printError("test ERROR: refMember pointers are different");
  }

  if((refList->m_list[0]).ptr() != refMember.ptr()){
    printError("test ERROR: refList pointers are different");
  }

  if ( ! m_dataSvc->transaction().commit() ) {
    printError("test ERROR: Could not commit the READ transaction." );
  }
      
  refMember.reset();
  //m_dataSvc->cacheSvc().resetCache();
 
  if ( ! ( m_dataSvc->transaction().start( pool::ITransaction::READ ) ) ) {
    printError("test ERROR: Could not start an UPDATE transaction");
  }   

  if(!refList.token()){
    printError("test ERROR: in refList token is missing (2)");
  }
  if(!(refList->m_list[0].token())){
    printError("test ERROR: in refList->member token is missing (2)");
  }

  if(!refMember.token()){
    printError("test ERROR: in refMember token is missing (2)");
  }
  if(!refMember->belongsToList.token()){
    printError("test ERROR: in refMember->list token is missing (2)");
  } 


  printM("Commit transactions...");
  if ( ! m_dataSvc->transaction().commit() ) {
    printError("test ERROR: Could not commit the WRITE transaction." );
  }

  refMember.reset();
  return;
}

void pool::CircularRefTest::writeLoop(){

  printM("Writing procedure for LoopClass");
  Placement place1, place2, place3;
  place1.setDatabase(m_dbName,pool::DatabaseSpecification::PFN );
  place1.setContainerName("db_cont_3");
  place1.setTechnology(m_type.type());
  place2.setDatabase(m_dbName,pool::DatabaseSpecification::PFN );
  place2.setContainerName("db_cont_4");
  place2.setTechnology(m_type.type());
  place3.setDatabase(m_dbName,pool::DatabaseSpecification::PFN );
  place3.setContainerName("db_cont_5");
  place3.setTechnology(m_type.type());

  // Start an update transaction
  if ( ! ( m_dataSvc->transaction().start( pool::ITransaction::UPDATE ) ) ) {
    printError("test ERROR: Could not start an UPDATE transaction");
  }  

  Ref<LoopClass> refLoop(m_dataSvc,new LoopClass);
  refLoop->m_loop = refLoop;

  refLoop.markWrite(place1);
  tok1 = const_cast<Token*>(refLoop.token());
  tok1->addRef();

  pool::Ref<LoopClass> a2(m_dataSvc,new LoopClass);
  pool::Ref<LoopClass> a3(m_dataSvc,new LoopClass);
  a2->m_loop=refLoop;
  a3->m_loop=a2;

  a2.markWrite(place2);
  tok2 = const_cast<Token*>(a2.token());
  tok2->addRef();

  a3.markWrite(place3);
  tok3 = const_cast<Token*>(a3.token());
  tok3->addRef();

  printM("Commit LoopClass transactions...");
  if ( ! m_dataSvc->transaction().commit() ) {
    printError("test ERROR: Could not commit the WRITE transaction." );
  }

  //refLoop.reset();
  m_dataSvc->resetCache(); 

  if ( ! ( m_dataSvc->transaction().start( pool::ITransaction::READ ) ) ) {
    printError("test ERROR: Could not start an UPDATE transaction");
  }
   
  if(!refLoop.token()){
    printError("test ERROR: in refLoop token is missing (2)");
  }
  if(!(refLoop->m_loop.token())){
    printError("test ERROR: in refLoop->loop token is missing (2)");
  }

  if(!a2->m_loop.token()){
    printError("test ERROR: in a2->loop token is missing (2)");
  }
  if(!a3->m_loop.token()){
    printError("test ERROR: in a3->loop token is missing (2)");
  } 

  printM("Commit transactions...");
  if ( ! m_dataSvc->transaction().commit() ) {
    printError("test ERROR: Could not commit the WRITE transaction." );
  }

  //m_dataSvc->resetCache(); 

  printM("write LoopClass completed");
  
}

/// read objects from the persistent store
void pool::CircularRefTest::readLoop(){
  printM("Reading procedure for LoopClass");
  if(SharedAnyPtr::numberOfInstances()){
    SharedAnyPtr::dumpRegister();
  }
  if ( ! m_dataSvc->transaction().start( pool::ITransaction::READ ) ) {
    printError("test ERROR: Could not start a READ transaction." );
  }

  Ref<LoopClass> refLoop(m_dataSvc,*tok1);
  tok1->release();
  Ref<LoopClass> a2(m_dataSvc,*tok2);
  tok2->release();
  Ref<LoopClass> a3(m_dataSvc,*tok3);
  tok3->release();

  if(refLoop->m_loop.ptr() != refLoop.ptr()){
    printError("test ERROR: refLoop pointers are different");
  }

  if(a2->m_loop.ptr() != refLoop.ptr()){
    printError("test ERROR: a2 pointers are different");
  }

  if(a3->m_loop.ptr() != a2.ptr()){
    printError("test ERROR: a3 pointers are different");
  }

  if ( ! m_dataSvc->transaction().commit() ) {
    printError("test ERROR: Could not commit the READ transaction." );
  }
      
  m_dataSvc->resetCache(); 
  //refLoop.reset();
  //m_dataSvc->cacheSvc().resetCache();
 
  if ( ! ( m_dataSvc->transaction().start( pool::ITransaction::READ ) ) ) {
    printError("test ERROR: Could not start an UPDATE transaction");
  }   

  if(!refLoop.token()){
    printError("test ERROR: in refLoop token is missing (2)");
  }
  if(!(refLoop->m_loop.token())){
    printError("test ERROR: in refLoop->loop token is missing (2)");
  }
  if(!(a2->m_loop.token())){
    printError("test ERROR: in a2->loop token is missing (2)");
  }
  if(!(a3->m_loop.token())){
    printError("test ERROR: in a3->loop token is missing (2)");
  }

  printM("Commit transactions...");
  if ( ! m_dataSvc->transaction().commit() ) {
    printError("test ERROR: Could not commit the WRITE transaction." );
  }

  m_dataSvc->resetCache(); 
  //refLoop.reset();
  return;
}

void pool::CircularRefTest::execute(){
  write();
  read();
  writeOnScope(1000);
  writeLoop();
  readLoop();
}

int main(int,char**)     {
  typedef pool::CircularRefTest Test;
  std::string file = "test1.root";
  std::vector< std::string > libraries;
  libraries.push_back( "test_DataSvc_CircularRefDict" );
  Test::init( libraries );
  Test theTest(file, pool::ROOTKEY_StorageType);
  theTest.run();
  return 0;
}
