#include "PluginManager/PluginManager.h"
#include "POOLCore/Token.h"
#include "FileCatalog/URIParser.h"
#include "FileCatalog/FCSystemTools.h"
#include "FileCatalog/IFileCatalog.h"
#include "StorageSvc/DbType.h"
#include "PersistencySvc/DatabaseConnectionPolicy.h"
#include "PersistencySvc/ISession.h"
#include "PersistencySvc/ITransaction.h"
#include "PersistencySvc/IDatabase.h"
#include "PersistencySvc/Placement.h"
#include "DataSvc/DataSvcFactory.h"
#include "DataSvc/IDataSvc.h"
#include "DataSvc/ICacheSvc.h"
#include "DataSvc/Ref.h"
#include <map>
#include<utility>
#include<vector>
#include<string>
#include "TestRefClasses/TestClass.h"
#include "TestBase/TestKernel.h"
#include <stdexcept>

#include "SealBase/SharedLibrary.h"
#include "SealBase/SharedLibraryError.h"

int main(int) {


  pool::TestKernel testSvc;

  seal::PluginManager::get()->initialise();

  try {  
    seal::SharedLibrary::load( seal::SharedLibrary::libname( "test_TestRefClassesDict" ) );
  } catch ( seal::SharedLibraryError *error) {
    std::cout << "test ERROR: "<< error->explainSelf() << std::endl;
  }


  pool::URIParser p;
  p.parse();
  
  // get hold of a factory for file catalogs
//   pool::IFileCatalog* cat = pool::FCSystemTools::createCatalog(p.prefix());
  pool::IFileCatalog* cat = new pool::IFileCatalog();
  cat->setWriteCatalog( p.contactstring() );
  
  if (!cat) {
    throw std::runtime_error( "Could not create a file catalog" );
  }

  cat->connect();
  
  cat->start();
  
  pool::IDataSvc *svc = pool::DataSvcFactory::instance(cat);
  // Define the policy for the implicit file handling
  pool::DatabaseConnectionPolicy policy;
  policy.setWriteModeForNonExisting(pool::DatabaseConnectionPolicy::CREATE);
  // policy.setWriteModeForExisting(pool::DatabaseConnectionPolicy::OVERWRITE);
  policy.setWriteModeForExisting(pool::DatabaseConnectionPolicy::UPDATE);
  svc->session().setDefaultConnectionPolicy(policy);
  
  svc->transaction().start(pool::ITransaction::UPDATE);
  pool::Ref<TestClass> o1(svc, new TestClass);
  {
    pool::Placement place;
    place.setDatabase("DN", pool::DatabaseSpecification::PFN); 
    place.setContainerName("L1");
    place.setTechnology(pool::ROOTKEY_StorageType.type()); 
    try{  
      o1.markWrite(place);
    } catch (const pool::Exception& e){
      std::cout << e.what() << std::endl;
    }

  }
  pool::Ref<TestClass> o2(svc, new TestClass);
  {
    pool::Placement place;
    place.setDatabase("DN", pool::DatabaseSpecification::PFN); 
    place.setContainerName("L");
    place.setTechnology(pool::ROOTKEY_StorageType.type()); 
    // This will also register the file. For this to occur, the placement object must use a PFN.
    o2.markWrite(place);
  }
  if(svc->cacheSvc().numberOfMarkedObjects()!=2){
    testSvc.printError("test ERROR wrong number of marked object");
    std::cout << "N marked  = " 
            << svc->cacheSvc().numberOfMarkedObjects() << std::endl;
  }

  // svc->cacheSvc().resetCache();
  o1.reset();
  if(svc->cacheSvc().numberOfMarkedObjects()!=2){
    testSvc.printError("test ERROR wrong number of marked object");
    std::cout << "N marked after reset = " 
            << svc->cacheSvc().numberOfMarkedObjects() << std::endl;
  }
  std::cout << "o1 " << o1.toString() << std::endl;
  std::cout << "o1 p " << o1.isNull() << std::endl;
  std::cout << "o2 p " << o2.isNull() << std::endl;
  if(o1.isNull()){
    testSvc.printError("test ERROR (1): o1 isNull after reset");
  }
  if(o2.isNull()){
    testSvc.printError("test ERROR (2): o2 isNull after reset");
  }
  svc->transaction().commit();
  if(svc->cacheSvc().numberOfMarkedObjects()!=0){
    testSvc.printError("test ERROR wrong number of marked object");
    std::cout << "N marked after commit  = " 
            << svc->cacheSvc().numberOfMarkedObjects() << std::endl;
  }
  svc->session().disconnectAll();
  svc->cacheSvc().resetCache();
  svc->transaction().start(pool::ITransaction::UPDATE);
  // std::cout << "o1 p " << o1.isNull() << std::endl;
  std::cout << "o1 " << o1.toString() << std::endl;
  std::cout << "o1 p " << o1.isNull() << std::endl;
  if(o1.isNull()){
    testSvc.printError("test ERROR (3): o1 isNull after reset");
  }
  o1.markDelete();  
  if(svc->cacheSvc().numberOfMarkedObjects()!=1){
    testSvc.printError("test ERROR wrong number of marked object");
    std::cout << "N marked  = " 
            << svc->cacheSvc().numberOfMarkedObjects() << std::endl;
  }
  o1.reset();
  if(svc->cacheSvc().numberOfMarkedObjects()!=1){
    testSvc.printError("test ERROR wrong number of marked object");
    std::cout << "N marked after reset = " 
            << svc->cacheSvc().numberOfMarkedObjects() << std::endl;
  }
  std::cout << "o2 d " << o2.toString() << std::endl;
  std::cout << "o2 t " << o2.token() << std::endl;
  std::cout << "o2 p " << o2.isNull() << std::endl;
  if(o2.isNull()){
    testSvc.printError("test ERROR (4): o2 isNull after reset");
  }
  svc->transaction().commit();
  if(svc->cacheSvc().numberOfMarkedObjects()!=0){
    testSvc.printError("test ERROR wrong number of marked object");
    std::cout << "N marked after commit  = " 
            << svc->cacheSvc().numberOfMarkedObjects() << std::endl;
  }
  svc->session().disconnectAll();

  svc->transaction().start(pool::ITransaction::UPDATE);
  std::cout << "here we do not die anymore..." << std::endl;
  std::cout << "o2 d " << o2.toString() << std::endl;
  std::cout << "o2 t " << o2.token() << std::endl;
  std::cout << "o2 p " << o2.isNull() << std::endl;
  std::cout << "o2 ptr="<< o2.ptr() << std::endl;
  if(o2.isNull()){
    testSvc.printError("test ERROR (5): o2 isNull after reset");
  }
  if(svc->cacheSvc().numberOfMarkedObjects()!=0){
    testSvc.printError("test ERROR wrong number of marked object");
    std::cout << "N marked  = " 
            << svc->cacheSvc().numberOfMarkedObjects() << std::endl;
  }
  svc->transaction().commit();
  if(svc->cacheSvc().numberOfMarkedObjects()!=0){
    testSvc.printError("test ERROR wrong number of marked object");
    std::cout << "N marked after commit = " 
            << svc->cacheSvc().numberOfMarkedObjects() << std::endl;
  }
  svc->session().disconnectAll();

  svc->cacheSvc().resetCache(); // here is the problem...

  svc->transaction().start(pool::ITransaction::UPDATE);
  std::cout << "o1 d " << o1.toString() << std::endl;
  std::cout << "o1 t " << o1.token() << std::endl;
  std::cout << "o1 p " << o1.isNull() << std::endl;
  if(!o1.isNull()){
    testSvc.printError("test ERROR (6): o1 isNull false after delete");
  }
  std::cout << "o2 d " << o2.toString() << std::endl;
  std::cout << "o2 t " << o2.token() << std::endl;
  std::cout << "o2 p " << o2.isNull() << std::endl;  /// this is true???
  if(o2.isNull()){
    testSvc.printError("test ERROR (7): o2 isNull after reset");
  }
  svc->transaction().commit();
  svc->session().disconnectAll();

  try {
    svc->transaction().start(pool::ITransaction::UPDATE);
    std::cout << "o2 d " << o2.toString() << std::endl;
    std::cout << "o2 t " << o2.token() << std::endl;
    std::cout << "o2 p " << o2.isNull() << std::endl;
    if(o2.isNull()){
      testSvc.printError("test ERROR (8): o2 isNull after reset");
    }
    o2.markUpdate();
    std::cout << "after update 1" << std::endl;
    o2.markDelete();  
    std::cout << "after delete 1" << std::endl;
    o2.markUpdate(); 
    std::cout << "after update 2" << std::endl;
    o2.markDelete();  
    std::cout << "after delete 2" << std::endl;
    if(svc->cacheSvc().numberOfMarkedObjects()!=1){
      testSvc.printError("test ERROR wrong number of marked object");
      std::cout << "N marked after commit = " 
            << svc->cacheSvc().numberOfMarkedObjects() << std::endl;
    }
    svc->transaction().commit();
    if(svc->cacheSvc().numberOfMarkedObjects()!=0){
      testSvc.printError("test ERROR wrong number of marked object");
      std::cout << "N marked after commit = " 
            << svc->cacheSvc().numberOfMarkedObjects() << std::endl;
    }
    svc->session().disconnectAll();
  }
  catch(const pool::Exception& er){
    testSvc.printError("test ERROR: exception->");
    std::cerr << er.what() << std::endl;
  }
  cat->commit();

  delete svc;
  delete cat;
  
  testSvc.dumpError();

}

