#ifndef CATALOGPOPULATOR_H
#include "CMSFileManagerBase/CatalogPopulator.h"
#endif
#include "FileCatalog/IFCAction.h"
CatalogPopulator::CatalogPopulator(){
  m_lfns.reserve(100);
}
CatalogPopulator::~CatalogPopulator(){
  if(m_catalog !=0){
    delete m_catalog;
  }
  if(m_cmspec !=0){
    delete m_cmspec;
  }
}
void CatalogPopulator::openCatalog(const std::string& contactstr){
  m_catalog=new pool::IFileCatalog;
  m_catalog->setWriteCatalog(contactstr);
  m_catalog->connect();
}
void CatalogPopulator::populateCatalogFromfile(const std::string& iname){
  std::ifstream ifile(iname.c_str(), std::ios_base::in);
  if( !ifile ){
    std::cerr<< "can't open input file \""<<iname<<"\"";
  }
  std::string lfn;
  while( std::getline( ifile, lfn, '\n' ) ){
    m_lfns.push_back(lfn);
  }
  ifile.close();
  
  std::vector<std::string>::iterator i, ibeg,iend;
  ibeg=m_lfns.begin(); iend=m_lfns.end();
  pool::FileCatalog::FileID fid;
  m_catalog->start();
  std::string pfnentry,lfnentry;
  pool::FCregister r;
  m_catalog->setAction(r);
  for(i=ibeg; i!=iend; ++i){
    pool::MetaDataEntry attrs(*m_cmspec);
    parselfn(*i,attrs);
    pfnentry=std::string("pfn:")+(*i);
    lfnentry=std::string("lfn:")+(*i);
    r.registerPFN(pfnentry,"cmsraw",fid);
    r.registerLFN(pfnentry,lfnentry);
    r.registerMetaData(fid,attrs);
  }
  m_catalog->commit();
}
void CatalogPopulator::defineSchema(){
  m_cmspec=new pool::MetaDataEntry;
  m_cmspec->addAttributeSpec("FileCategory","string");
  m_cmspec->addAttributeSpec("DataType","string");
  m_cmspec->addAttributeSpec("jobid","string");
  m_cmspec->addAttributeSpec("runid","string");
  m_cmspec->addAttributeSpec("owner", "string");
  m_cmspec->addAttributeSpec("dataset", "string");
  m_catalog->start();
  pool::FCAdmin a;
  m_catalog->setAction(a);
  a.dropMetaDataSpec();
  a.createMetaDataSpec(*m_cmspec);
  m_catalog->commit();
}

void CatalogPopulator::closeCatalog() {
  m_catalog->disconnect();
}

void CatalogPopulator::parselfn(const std::string& lfn, pool::MetaDataEntry& attrs){
  if( isMETA(lfn) ){
    parsemeta(lfn,attrs);
  }else if( isEVD(lfn) ){
      parseevd(lfn,attrs);
  }else{
    parseother(lfn,attrs);
    }
}
bool CatalogPopulator::isMETA(const std::string& lfn) const{
  return lfn.find("META")!=std::string::npos;
}
bool CatalogPopulator::isEVD(const std::string& lfn) const{
  return lfn.find("EVD")!=std::string::npos;
}
void CatalogPopulator::parseseparator( const std::string& lfn, const std::string& separators, std::vector<std::string>& words ){
  std::string::size_type begIdx, endIdx;
  begIdx=lfn.find_first_not_of(separators);
  while( begIdx != std::string::npos){
    endIdx = lfn.find_first_of(separators,begIdx);
    std::string datatype=lfn.substr(begIdx,endIdx-begIdx);
    words.push_back(datatype);
      begIdx=lfn.find_first_not_of(separators,endIdx); 
  }
}
void CatalogPopulator::parsemeta(const std::string& lfn, pool::MetaDataEntry& attrs){
  std::string separators(".");
  std::vector<std::string> words;
  parseseparator(lfn,separators,words);
  attrs.setAttributeValue("FileCategory",std::string("META"));
  attrs.setAttributeValue("DataType",words[0]);
  attrs.setAttributeValue("owner",words[2]);
}
void CatalogPopulator::parseevd(const std::string& lfn, pool::MetaDataEntry& attrs){
  std::string separators(".");
  std::vector<std::string> words;
  std::vector<std::string> subwords;
  parseseparator(lfn,separators,words);
  parseseparator(words[0],"_",subwords);
  attrs.setAttributeValue("FileCategory",std::string("EVD"));
  attrs.setAttributeValue("DataType",subwords[1]);
  if( words.size()>4 ){
    attrs.setAttributeValue("jobid",words[1]);
    attrs.setAttributeValue("runid",words[2]);
    attrs.setAttributeValue("dataset",words[3]);
    attrs.setAttributeValue("owner",words[4]);
  }else{
    attrs.setAttributeValue("dataset",words[1]);
    attrs.setAttributeValue("owner",words[2]);
  }
}
void CatalogPopulator::parseother(const std::string& lfn, pool::MetaDataEntry& attrs){
  std::string separators("_");
  std::vector<std::string> words;
  std::string::size_type begIdx, endIdx;
  begIdx=lfn.find_first_not_of(separators);
  if( begIdx != std::string::npos ){
    endIdx = lfn.find_first_of(separators,begIdx);
    if( endIdx != std::string::npos ){
      std::string datatype=lfn.substr(begIdx,endIdx-begIdx);
      words.push_back(datatype);
      datatype=lfn.substr(endIdx+1);
      words.push_back(datatype);
    }
  }
  if(words.size()>0){
    attrs.setAttributeValue("FileCategory",words[0]);
    attrs.setAttributeValue("owner",words[1]);
  }
}


