#ifndef SIMPLEASCIICOLLECTION_H
#define SIMPLEASCIICOLLECTION_H

#include <vector>
#include <string>
#include <iostream>
#include <fstream>

namespace pool {
 /** @class SimpleAsciiCollection SimpleAsciiCollection.h Tests/Libraries/StressTestBase/SimpleAsciiCollection.h

    SimpleAsciiCollection provides a simple interface for creating, reading and writing
    to an ascii text file that contains std::string entries line by line. It is intended as
    a simple way of storing externalised pool Tokens.

    @author Tim Barrass
    @date 8/5/2003
*/
struct SimpleAsciiCollection {
  /** Construct with default collection file
  */
  SimpleAsciiCollection();
  
  /** Construct with the name of the collection file
  */
  SimpleAsciiCollection(std::string nuCollectionFile);

  /** Add a token (as string) to the collection in memory
    @return int 0 success
    @param nuToken The new token as string
  */
  int add(std::string nuToken);

  /** Read the collection file into memory
    @return 0 is success
  */
  int read();

  /** Get collection vector
  */
  const std::vector<std::string>& collection() const { return m_collection; }

  /** Write the collection from memory to file
    @return 0 is success
  */  
  int write();
    
  virtual ~SimpleAsciiCollection();

  std::vector<std::string> m_collection;
  std::string m_collectionFile;
};

} // ns pool

#endif  // SIMPLEASCIICOLLECTION_H
