#ifndef TRIVIALBUILDER_H
#define TRIVIALBUILDER_H

#include "StressTestBase/Wrapper.h"
#include "StressTestBase/IPersistentObjectBuilder.h"
#include "StressTestBase/SimpleAsciiCollection.h"

#include "POOLCore/Token.h"
#include "StorageSvc/DbType.h"
#include "POOLCore/Guid.h"
#include "PersistencySvc/Placement.h"
#include "PersistencySvc/ISession.h"
#include "PersistencySvc/DatabaseConnectionPolicy.h"

#include <string>

template <typename T>
/** @class TrivialBuilder TrivialBuilder.h Tests/Libraries/StressTestBase/TrivialBuilder.h

    TrivialBuilder is a simple object implementing IPersistentObjectBuilder. It can be used
    to persist simple objects- where simple implies that the object contains no persistent
    references to other objects.

    @author Tim Barrass
    @date 8/5/2003
*/
class TrivialBuilder : virtual public IPersistentObjectBuilder {
public:
	virtual ~TrivialBuilder() {};

  TrivialBuilder(   std::string nuDbName,
                    pool::DbType nuType,
                    T &theObject,
                    pool::IDataSvc *nuContext ) :     m_dbName(nuDbName),
                                                      m_type(nuType),
                                                      m_context(nuContext)

  {
    m_place.setDatabase(nuDbName,pool::DatabaseSpecification::PFN);
    m_place.setContainerName("default_container");
    m_place.setTechnology(nuType.type());

    // I don't want to own their object so ...
    pool::Ref<T> tmp(m_context,new T(theObject));
    m_ref = tmp;
    // and ref takes care of memory managment ...
  }
  
  TrivialBuilder(const TrivialBuilder& b) :			m_dbName(b.dbName()),
                                                m_type(b.storageType()),
                                                m_ref(b.ref()),
                                                m_place(b.place()),
                                                m_context(b.context()) {};

  virtual IPersistentObjectBuilder* clone() const { return new TrivialBuilder(*this); }

  void buildPersistent() {
    ref().mark_write(place());
  }

  const std::string& dbName() const { return m_dbName; }
  const pool::DbType& storageType() const { return m_type; }
  pool::IDataSvc* context() const { return m_context; }

protected:
  const pool::Ref<T> ref() const { return m_ref; }
  const pool::Placement place() const {return m_place; }

private:
  std::string m_dbName;
  pool::DbType m_type;
  pool::Ref<T> m_ref;
  pool::Placement m_place;
  pool::IDataSvc *m_context;
};

#endif  // TRIVIALBUILDER_H
