#ifndef WRAPPER_H
#define WRAPPER_H

template <class T>
/** @class PtrWrapper Wrapper.h Tests/Libraries/StressTestBase/Wrapper.h

    PtrWrapper is effectively a simple smart pointer. It is intended to allow
    proper memory management of polymorphic objects in STL containers. It is
    derived directly from an example in
    http://www.xraylith.wisc.edu/~khan/software/stl/STL.newbie.html

    @author Tim Barrass
    @date 8/5/2003
*/
class PtrWrapper {
public:
    PtrWrapper(T* t_ptr = 0) : t_ptr_(t_ptr) { }
    PtrWrapper(const PtrWrapper<T>& w) {
        t_ptr_ = w() ? w()->clone() : 0;
    }
    ~PtrWrapper() { delete t_ptr_; }
    const T* operator()()  const { return t_ptr_; }
    T* operator()()  { return t_ptr_; }
    PtrWrapper<T>& operator= (const PtrWrapper<T>& w) {
        delete t_ptr_;
        t_ptr_ = w()->clone();
        return *this;
    }
private:
    T* t_ptr_;
};

template <class T>
bool operator== (const PtrWrapper<T>& w1, const PtrWrapper<T>& w2) {
    return false;
}

template <class T>
bool operator< (const PtrWrapper<T>& w1, const PtrWrapper<T>& w2) {
    return false;
}

#endif  // WRAPPER_H

