#include "SealBase/SharedLibrary.h"
#include "POOLCore/POOLContext.h"
#include "FileCatalog/URIParser.h"
#include "FileCatalog/IFileCatalog.h"
#include "DataSvc/DataSvcFactory.h"
#include "PersistencySvc/ISession.h"
#include "PersistencySvc/DatabaseConnectionPolicy.h"
#include "PersistencySvc/Placement.h"
#include "StorageSvc/DbType.h"
#include "PersistencySvc/ITransaction.h"
#include "DataSvc/Ref.h"
#include "DataSvc/IDataSvc.h"
#include "AttributeList/AttributeList.h"

#include <iostream> 
#include <vector>
#include <string>

using namespace std;
using namespace pool;


#include "Event.h"
#include "MultiCollection/MultiCollection.h"


const AttributeList& getEventMetaData(){
  
  static int cnt = 0;
  static AttributeList* al = 0;
  static AttributeListSpecification als;
  
  if(!al){
    als.push_back("NMuon", "int");
    als.push_back("NZ0","int");
    als.push_back("NHiggs","int");
    al = new AttributeList(als);
  }
  
  ++cnt;
    
  (*al)["NMuon"].setValue(cnt);
  (*al)["NZ0"].setValue(cnt/2);
  (*al)["NHiggs"].setValue(cnt/4);
  
  return *al; 
}



const AttributeList& getRunMetaData(){
  
  static int cnt = 0;
  static AttributeList* al = 0;
  static AttributeListSpecification als;
  static const string crew[] = {"Wallace, Gromit",
				"Wallace, Gromit, Feathers"};
  
  if(!al){
    //als.push_back("NEvents", "int");
    als.push_back("Lumi","double");
    als.push_back("ShiftCrew","string");
    al = new AttributeList(als);
  }
  
  ++cnt;
  
  //(*al)["NEvents"].setValue(cnt*23);
  (*al)["Lumi"].setValue(cnt*23/42.0);
  (*al)["ShiftCrew"].setValue(crew[cnt%2]);
  
  return *al; 
}

void printAttributes( const AttributeList& attrList ){
  const AttributeListSpecification& attrListSpec = attrList.attributeListSpecification();
  for( unsigned int i = 0; i < attrListSpec.size(); ++i ){
    const AttributeSpecification& attrSpec = attrListSpec[i];
    string attrName = attrSpec.name();
    string attrValue = attrList[attrName].getValueAsString();
    cout << attrName << "=" << attrValue << "\t";
  }
  cout << endl;
}




int main(){
  
  const int nRuns = 5;
  const int nEvents = 3;
  
  const string evtCollType[] = { "RootCollection",
				 "MySQLCollection" };
  
  const string runCollType[] = { "RootMultiCollection",
				 "MySQLMultiCollection" };
  
  const string evtCollNamePrefix = "EvtColl_";
  
  const string connection[] = { "",
				"mysql://jwojcies:jWfC2003@lxshare070d.cern.ch/jacekdb" };
  
  const string goodRunQuery[] = { "!strstr(ShiftCrew,\"Feathers\")",
				  "NOT ( ShiftCrew like '%Feathers%' )" };
  
  const string badRunQuery[] = { "strstr(ShiftCrew,\"Feathers\")",
				 "ShiftCrew like '%Feathers%'" };
  
  const string muonQuery[] = { "NMuon % 2 == 0",
			       "NMuon % 2 = 0" };
  
  const string higgsQuery[] = { "NHiggs % 2 == 0",
				"NHiggs % 2 = 0" };
  
  try{
    pool::POOLContext::loadComponent( "SEAL/Services/Message Service" );
    pool::POOLContext::setMessageVerbosityLevel( seal::Msg::Info );
    const string plaformDependentName = seal::SharedLibrary::libname("MultiCollection_BasicFunctionalityDict");
    seal::SharedLibrary::load(plaformDependentName);
    
//     IFileCatalog* fileCatalog = IFileCatalogPluginFactory::get()->create("XMLFileCatalog");
//     fileCatalog->connect( "file:PoolFileCatalog.xml");
    pool::URIParser p("file:PoolFileCatalog.xml");
    pool::IFileCatalog* fileCatalog = new pool::IFileCatalog();
    p.parse();
    fileCatalog->addReadCatalog( p.contactstring() );
    fileCatalog->connect();
    
    IDataSvc* dataSvc = pool::DataSvcFactory::instance( fileCatalog );
    
    {  
      fileCatalog->start();
      
      pool::DatabaseConnectionPolicy policy;
      policy.setWriteModeForNonExisting( pool::DatabaseConnectionPolicy::CREATE );
      policy.setWriteModeForExisting( pool::DatabaseConnectionPolicy::OVERWRITE );
      dataSvc->session().setDefaultConnectionPolicy( policy );
      
      pool::Placement placement;
      placement.setDatabase( "testFile.pool", pool::DatabaseSpecification::PFN );
      placement.setContainerName( "TestContainer" );
      placement.setTechnology( pool::ROOTTREE_StorageType.type() );    
      
      
      // fill a vector with references to the events we want to store in the collections 
      vector< Ref<Event> > EventRefVec;
      
      dataSvc->transaction().start( pool::ITransaction::UPDATE );
      
      for(int i=0; i<nEvents*nRuns; ++i){
	Ref<Event> ref(dataSvc, new Event(true));
	ref.markWrite(placement);
	EventRefVec.push_back(ref);
      }
      
      dataSvc->transaction().commit();
      fileCatalog->commit();
      
      fileCatalog->start();
      dataSvc->transaction().start( pool::ITransaction::READ );
      
      
     


      
      
      // i started with basic
      for(int i=0; i<2; ++i){
	
	cout << "\n#########################################################" << endl;
	if(i==0) cout << "using ROOT implementation" << endl;
	if(i==1) cout << "using MYSQL implementation" << endl;
	cout << "#########################################################\n" << endl;
	

	// setup an iterator for the Ref<Event>s
	vector< Ref<Event> >::const_iterator EventRefIter = EventRefVec.begin();
	
	{
	  cout << "\n### Create " << nRuns
	       << " event collections, add " << nEvents
	       << "\n### entries to each of it and add the 5 event "
	       << "\n### collections to one run collection named \"AllRuns\""
	       << endl; 
	  
	  // Create the MultiCollection i.e. (collection of event collections)
	  MultiCollection<Event> runColl( dataSvc, 
					  runCollType[i], 
					  connection[i], 
					  "AllRuns", 
					  ICollection::CREATE_AND_OVERWRITE );	
	  
	  // Create the event collections
	  for(int run=1; run<=nRuns; ++run){
	    
	    ostringstream oss;
	    oss << evtCollNamePrefix << run;
	    
	    Collection<Event> evtColl( dataSvc, 
				       evtCollType[i], 
				       connection[i], 
				       oss.str(), 
				       ICollection::CREATE_AND_OVERWRITE );
	    // Add some entries to the event collections
	    for (int i=0; i<nEvents; ++i)
	      evtColl.add(*EventRefIter++, getEventMetaData());
	    
	    evtColl.commit();
	    
	    runColl.add(evtColl,getRunMetaData());
	  }
	  // save the collection
	  runColl.commit();
	}
	
	
	
	{
	  cout << "\n### Open the created \"AllRuns\" run collection and loop over the runs" << endl;
	  MultiCollection<Event> coll( dataSvc, 
				       runCollType[i], 
				       connection[i], 
				       "AllRuns", 
				       ICollection::READ);	
	  
	  
	  MultiCollection<Event>::Iterator runIter = coll.selectObjCollections();
	  
	  while( runIter.next() ){
	    printAttributes( runIter.attributeList());
	  }
	}
	
	
	
	{
	  cout << "\n### Open the created \"AllRuns\" run collection and loop over the events" << endl;
	  Collection<Event> coll( dataSvc, 
				  runCollType[i], 
				  connection[i], 
				  "AllRuns", 
				  ICollection::READ);
	  
	  Collection<Event>::Iterator evtIter = coll.select();
	
	  while( evtIter.next() ){
	    evtIter->print();
	    printAttributes(evtIter.attributeList());
	  }
	}
	
	
	
	{
	  cout << "\n### Create a new run collection named \"GoodRuns\", select from \"AllRuns\" \n"
	       << "### only those runs where \"feathers\" is not member of the shift crew and \n"
	       << "### add these runs to \"GoodRuns\""
	       << endl;
	  
	  MultiCollection<Event> srcColl( dataSvc, 
					  runCollType[i], 
					  connection[i], 
					  "AllRuns", 
					  ICollection::READ);
	  
	  MultiCollection<Event> dstColl( dataSvc, 
					  runCollType[i], 
					  connection[i], 
					  "GoodRuns", 
					  ICollection::CREATE_AND_OVERWRITE);
	  
	  MultiCollection<Event>::Iterator srcIter = srcColl.selectObjCollections(goodRunQuery[i]);
	  
	  while( srcIter.next() ){
	    dstColl.add( *srcIter, srcIter.attributeList());
	  }
	  dstColl.commit();
	}
	
	
	
	{
	  cout << "\n### Open the created \"GoodRuns\" run collection and loop over the runs" << endl;
	  MultiCollection<Event> coll( dataSvc, 
				       runCollType[i], 
				       connection[i], 
				       "GoodRuns", 
				       ICollection::READ);	
	  
	  MultiCollection<Event>::Iterator runIter = coll.selectObjCollections();
	  
	  while( runIter.next() ){
	    printAttributes( runIter.attributeList());
	  }
	}
	
	
	
	{
	  cout << "\n### Open the created \"GoodRuns\" run collection and select"
	       << "\n### the events with an even number of muons" << endl;
	  
	  Collection<Event> coll( dataSvc, 
				  runCollType[i], 
				  connection[i], 
				  "GoodRuns", 
				  ICollection::READ);
	  
	  Collection<Event>::Iterator evtIter = coll.select(muonQuery[i]);
	  
	  while( evtIter.next() ){
	    evtIter->print();
	    printAttributes(evtIter.attributeList());
	  }
	}
	
	
	
	{
	  cout << "\n### Open the \"AllRuns\" run collection and select"
	       << "\n### the events with an even number of muons only from those runs"
	       << "\n### where \"feathers\" is not member of the shift crew"
	       << "\n### result should be identical to the last test "
	       << endl;
	  
	  Collection<Event> coll( dataSvc, 
				  runCollType[i], 
				  connection[i], 
				  "AllRuns", 
				  ICollection::READ);
	  
	  Collection<Event>::Iterator evtIter = coll.select( muonQuery[i], goodRunQuery[i]);
	  
	  while( evtIter.next() ){
	    evtIter->print();
	    printAttributes(evtIter.attributeList());
	  }
	}
	
	
	
	{
	  cout << "\n### Create a new run collection named \"BadRuns\", select from \"AllRuns\""
	       << "\n### only those runs where \"feathers\" is member of the shift crew"
	       << "\n### and add these runs to \"BadRuns\""
	       << endl;
	  
	  MultiCollection<Event> srcColl( dataSvc, 
					  runCollType[i], 
					  connection[i], 
					  "AllRuns", 
					  ICollection::READ);
	  
	  MultiCollection<Event> dstColl( dataSvc, 
					  runCollType[i], 
					  connection[i], 
					  "BadRuns", 
					  ICollection::CREATE_AND_OVERWRITE);
	  
	  MultiCollection<Event>::Iterator srcIter = srcColl.selectObjCollections(badRunQuery[i]);
	  
	  while( srcIter.next() ){
	    dstColl.add( *srcIter, srcIter.attributeList());
	  }
	  dstColl.commit();
	}
	
	
	{
	  cout << "\n### Create a new run collection named \"GoodNBadRuns\" and add the two run collections" 
	       << "\n### \"GoodRuns\" and \"BadRuns\" to it."
	       << "\n### The content of \"GoodNBadRuns\" and \"AllRuns\" should be the same"
	       << endl;
	  
	  MultiCollection<Event> dstColl( dataSvc, 
					  runCollType[i], 
					  connection[i], 
					  "GoodNBadRuns", 
					  ICollection::CREATE_AND_OVERWRITE);	
	  
	  Collection<Event> goodColl( dataSvc, 
				      runCollType[i], 
				      connection[i], 
				      "GoodRuns", 
				      ICollection::READ);
	  
	  Collection<Event> badColl( dataSvc, 
				     runCollType[i], 
				     connection[i], 
				     "BadRuns", 
				     ICollection::READ);
	  
	  
	  dstColl.add( goodColl, getRunMetaData());
	  dstColl.add( badColl, getRunMetaData());
	  
	  dstColl.commit();
	}
	
	
	
	{
	  cout << "\n### Open the created \"GoodNBadRuns\" run collection and loop over the runs" << endl;
	  MultiCollection<Event> coll( dataSvc, 
				       runCollType[i], 
				       connection[i], 
				       "GoodNBadRuns", 
				       ICollection::READ);	
	  
	  MultiCollection<Event>::Iterator runIter = coll.selectObjCollections();
	  
	  while( runIter.next() ){
	    printAttributes( runIter.attributeList());
	  }
	}
	
	
	
	{
	  cout << "\n### Open the created \"GoodNBadRuns\" run collection and select the events \n"
	       << "### with an even number of higgs only from those runs"
	       << "\n### where \"feathers\" is member of the shift crew" 
	       << endl;
	  
	  Collection<Event> coll( dataSvc, 
				  runCollType[i], 
				  connection[i], 
				  "GoodNBadRuns", 
				  ICollection::READ);
	  
	  Collection<Event>::Iterator evtIter = coll.select( higgsQuery[i], badRunQuery[i]);
	  
	  while( evtIter.next() ){
	    cout << "[OVAL] "; 
	    evtIter->print();
	    printAttributes(evtIter.attributeList());
	  }
	}
	
	
	
	{
	  cout << "\n### Open the created \"AllRuns\" run collection and select the events \n"
	       << "### with an even number of higgs only from those runs"
	       << "\n### where \"feathers\" is not member of the shift crew" 
	       << endl;
	  
	  Collection<Event> coll( dataSvc, 
				  runCollType[i], 
				  connection[i], 
				  "AllRuns", 
				  ICollection::READ);
	  
	  Collection<Event>::Iterator evtIter = coll.select( higgsQuery[i], badRunQuery[i]);
	  
	  while( evtIter.next() ){
	    cout << "[OVAL] "; 
	    evtIter->print();
	    printAttributes(evtIter.attributeList());
	  }
	}
      }
      
      dataSvc->transaction().commit();
    }

    delete dataSvc; 
    delete fileCatalog;
    
  }catch( std::exception& exception ) {
    cout << "Exception: " << exception.what() << endl;
  }
} 


