#include "RefHandler.h"
#include "DataRef.h"
#include "IDataLink.h"
#include "ITokenCache.h"
#include <stdexcept>

RefHandler::RefHandler( ITokenCache& tokenCache ):
  m_tokenCache( tokenCache )
{}

RefHandler::~RefHandler()
{}

seal::Status
RefHandler::saveReference( pool::Reference* pRef, pool::Token::OID_t* data )
{
  DataRef* ref = dynamic_cast< DataRef* >( pRef );
  if ( ! ref ) {
    throw std::runtime_error("Could not dynamic cast to DataRef");
    return seal::Status::ERROR;
  }
  const std::string& key = ref->key();
  if ( key.empty() ) {
    throw std::runtime_error( "empty key at save reference" );
  }
  TargetObject* object = m_tokenCache.lookup( key );
  if ( ! object ) {
    throw std::runtime_error( "No object pointed by the link" );
    data->first = data->second = 0xFFFFFFFF;
    return seal::Status::ERROR;
  }
  pool::Token* token = m_tokenCache.saveObject( object );
  if ( ! token ) {
    throw std::runtime_error( "Object not written" );
    data->first = data->second = 0xFFFFFFFF;
    return seal::Status::ERROR;
  }
  seal::Status sc = this->saveToken( token, data );
  return sc;
}

seal::Status
RefHandler::loadReference( pool::Reference* pRef, pool::Token::OID_t* data)
{
  pool::Token *token = 0;
  seal::Status sc = createToken( data, token );
  if ( sc.isSuccess() ) {
    DataRef* ref = dynamic_cast< DataRef* >( pRef );
    if ( ! ref ) {
      throw std::runtime_error("Could not dynamic cast to TheRef");
      return seal::Status::ERROR;
    }
    ref->setCache( static_cast< IDataCache* >( &m_tokenCache ) );
    ref->setCacheLocationIndex( m_tokenCache.registerToken( token ) );
  }
  return sc;
}
