//$Id: FCdescribeMetaData.cpp,v 1.9.2.1 2007/12/20 10:15:59 frankb Exp $
/**FCdescribeMetaData.cpp -- FileCatalog command line tool to describe metadata definition of the catalog
   @author: Zhen Xie
   @date: 02/03/2005 Z.X.
   set default logging to Warning if no POOL_OUTMSG_LEVEL is set; 
   separate logging stream to std::cerr, output stream to std::cout.
*/
#include "FileCatalog/CommandLine.h"
#include "FileCatalog/IFileCatalog.h"
#include "FileCatalog/FCException.h"
#include "FileCatalog/IFCAction.h"
#include "FileCatalog/FCEntry.h"
#include "AttributeList/AttributeListSpecification.h"
#include "POOLCore/POOLContext.h"
#include "POOLCore/Exception.h"
#include "POOLCore/PoolMessageStream.h"
#include "SealKernel/Context.h"
#include "SealKernel/MessageStream.h"
#include <memory>
using namespace pool;

void printUsage(){
  std::cout<<"usage: FCdescribeMetadata [-u contactstring -h]" <<std::endl;
  std::cout<<"format of metadata description: "<<std::endl;
  std::cout<<"(jobid,string),(owner,string),(jobtype,string)"<<std::endl;
}

int main(int argc, char** argv) {
  pool::POOLContext::loadComponent( "SEAL/Services/MessageService" );
  if(!::getenv( "POOL_OUTMSG_LEVEL" )){ //if not set, default to warning
    pool::POOLContext::setMessageVerbosityLevel(seal::Msg::Warning);
  }else{
    pool::PoolMessageStream pms("get threshold");
    pool::POOLContext::setMessageVerbosityLevel( pms.threshold() );
  }
  seal::IHandle<seal::IMessageService> mesgsvc =
    pool::POOLContext::context()->query<seal::IMessageService>("SEAL/Services/MessageService");
  if( mesgsvc ){
    //all logging go to cerr
    mesgsvc->setOutputStream( std::cerr, seal::Msg::Nil );
    mesgsvc->setOutputStream( std::cerr, seal::Msg::Verbose );
    mesgsvc->setOutputStream( std::cerr, seal::Msg::Debug );
    mesgsvc->setOutputStream( std::cerr, seal::Msg::Info );
    mesgsvc->setOutputStream( std::cerr, seal::Msg::Fatal );
    mesgsvc->setOutputStream( std::cerr, seal::Msg::Error );
    mesgsvc->setOutputStream( std::cerr, seal::Msg::Warning );
  } 
  std::string myuri;
  std::string metadef;
  try{
    CommandLine commands(argc,argv);
    if( commands.Exists("u") ){
      myuri=commands.GetByName("u");
    } 
    if( commands.Exists("h") ){
      printUsage();
      exit(0);
    }
  }catch(std::string& strError){
    std::cerr << "Error: command parsing error "<<strError<<std::endl;
    exit(-1);
  }
  try{
    std::auto_ptr<IFileCatalog> mycatalog(new IFileCatalog);
    mycatalog->addReadCatalog(myuri);
    FClookup l;
    mycatalog->setAction(l);
    mycatalog->connect();
    mycatalog->start();
    MetaDataEntry spec;
    l.getMetaDataSpec(spec);
    AttributeListSpecification myspec=spec.spec();
    for(AttributeListSpecification::iterator ispec=myspec.begin();ispec!=myspec.end(); ++ispec){
      if( (*ispec).name()!="guid" ){
        if(ispec!=(myspec.end()-1)){
          std::cout<<"("<<(*ispec).name()<<","<<(*ispec).type_name()<<"),";
        }else{
          std::cout<<"("<<(*ispec).name()<<","<<(*ispec).type_name()<<")";
        }
      }
    }  
    std::cout<<std::endl; 
    mycatalog->commit();  
    mycatalog->disconnect();
  }catch (const pool::Exception& er){
    //er.printOut(std::cerr);
    //std::cerr << std::endl;
    std::cerr<<er.what()<<std::endl;
    exit(1);
  }catch (const std::exception& er){
    std::cerr<<er.what()<<std::endl;
    exit(1);
  }
}


