/*
  Concrete implementation of the XML Meta Data catalog. 
  Auhor: Maria Girone
*/
#ifndef POOL_XMLMETADATACATALOG_H
#include "XMLCatalog/XMLMetaDataCatalog.h"
#endif
#ifndef POOL_XMLFILECATALOG_H
#include "XMLCatalog/XMLFileCatalog.h"
#endif
#include "POOLCore/PoolMessageStream.h"
#include "FileCatalog/FCException.h"
#include "PoolXMLFileCatalog.h"


#include <set>
#include <exception>


pool::XMLMetaDataCatalog::XMLMetaDataCatalog(FCImpl* fc) : FCMetaImpl(fc) {
    x_fc=static_cast<XMLFileCatalog*>(m_fc);  
}

pool::XMLMetaDataCatalog::~XMLMetaDataCatalog()
{
}

void
pool::XMLMetaDataCatalog::createMetaDataSpec(MetaDataEntry& meta) const 
{
  PoolMessageStream xmllog( "XMLMetaDataCatalog", seal::Msg::Nil );
  MetaDataEntry m_spec;
  this->getMetaDataSpec(m_spec);
  if ( m_spec.spec().size() > 1 ){    
    throw FCduplicatemetadataspecException
      ("XMLMetaDataCatalog::createMetaDataSpec");
    return;
  }
  
  try{
    x_fc->ClassHandle();
    PoolXMLFileCatalog* _xmlcatalog = x_fc->castConnectionHandle(x_fc->XMLcon);
    AttributeListSpecification schemadef = meta.spec();
    AttributeListSpecification::iterator isp;    
    std::vector<std::string> meta_attri; 
    std::vector<std::string> meta_type; 
    for (isp = schemadef.begin(); isp!= schemadef.end(); isp++){
      if ( isp->name() != "guid" ) {
        meta_attri.push_back(isp->name());
        meta_type.push_back(isp->type_name());
      }
    }
    _xmlcatalog->setMeta(meta_attri,meta_type);
    x_fc->to_be_updated = true;
  }
  catch( std::exception& er){
    std::string message("Cannot create metadata schema");
    xmllog << seal::Msg::Fatal << message << seal::endmsg;
    throw FCbackendException("XMLMetaDataCatalog::createMetaDataSpec", er.what());        
  }
}

void 
pool::XMLMetaDataCatalog::updateMetaDataSpec(MetaDataEntry& meta,
                                         const FileCatalog::MetaUpdateMode metamode) const
{
  PoolMessageStream xmllog( "XMLMetaDataCatalog", seal::Msg::Nil );
  try{
    x_fc->ClassHandle();
    AttributeListSpecification schemadef = meta.spec();
    MetaDataEntry oldmeta;
    this->getMetaDataSpec(oldmeta);
    AttributeListSpecification oldspec=oldmeta.spec();
    if( oldspec==schemadef ){
      xmllog<<seal::Msg::Info
            <<"old schema equals the new one, do nothing"
            <<seal::endmsg;
      return;
    }

    PoolXMLFileCatalog* _xmlcatalog = x_fc->castConnectionHandle(x_fc->XMLcon);

    if( metamode==FileCatalog::DELETE_REDUNDANT ){
      for( AttributeListSpecification::iterator it=oldspec.begin(); 
           it!=oldspec.end(); ++it){
        std::string colname=it->name();
        size_t idx=schemadef.index( colname );
        if( idx == pool::IDX_NOT_FOUND )
          _xmlcatalog->dropGuidMetas(colname);        
      }
    }
    
    std::vector<std::string> meta_attri; 
    std::vector<std::string> meta_type; 


    for( AttributeListSpecification::iterator it=schemadef.begin(); 
         it!=schemadef.end(); ++it){
      size_t idx=oldspec.index( it->name() ) ;
      if( idx == pool::IDX_NOT_FOUND ){
        meta_attri.push_back(it->name());
        meta_type.push_back(it->type_name());
      }
    }
    if (meta_attri.size() > 0 ){
      x_fc->to_be_updated = true;
      _xmlcatalog->setMeta(meta_attri,meta_type);
    }
    x_fc->to_be_updated = true;
  }

  catch( std::exception& er)
  {
    std::string message("Cannot update metadata schema");
    xmllog << seal::Msg::Fatal << message << seal::endmsg;
    throw FCbackendException("XMLMetaDataCatalog::updateMetaDataSpec", er.what());        
  }
}

void
pool::XMLMetaDataCatalog::getMetaDataSpec(MetaDataEntry& meta) const
{
  PoolMessageStream xmllog( "XMLMetaDataCatalog", seal::Msg::Nil );
  try{
    x_fc->ClassHandle();
    PoolXMLFileCatalog* _xmlcatalog = x_fc->castConnectionHandle(x_fc->XMLcon);

    std::map<std::string,pool::typename_string_t> guidMeta = 
      _xmlcatalog->getMeta();
    std::map<std::string,pool::typename_string_t>::iterator iMeta;

    if(!guidMeta.empty()){
      for (iMeta = guidMeta.begin(); iMeta!=guidMeta.end(); iMeta++){
        if (iMeta->first!="guid")
          meta.addAttributeSpec(iMeta->first,iMeta->second);
      }
    }
  }
  catch( std::exception& er ){
    std::string message("Cannot get metadata schema");
    xmllog << seal::Msg::Fatal << message << seal::endmsg;
    throw FCbackendException("XMLMetaDataCatalog::getMetaDataSpec", er.what());
  }  
}

void pool::XMLMetaDataCatalog::dropMetaDataSpec() const
{
  x_fc->ClassHandle();
  PoolXMLFileCatalog* _xmlcatalog = x_fc->castConnectionHandle(x_fc->XMLcon);
  _xmlcatalog->dropGuidMetas();        
  x_fc->to_be_updated = true;

}

void
pool::XMLMetaDataCatalog::insertMetaData(MetaDataEntry& mentry) const
{
  PoolMessageStream xmllog( "XMLMetaDataCatalog", seal::Msg::Nil );
  try{    
    x_fc->ClassHandle();
    PoolXMLFileCatalog* _xmlcatalog = x_fc->castConnectionHandle(x_fc->XMLcon);
    FileCatalog::FileID fid = mentry.guid();
    //ZX: begin fix, seg fault when use attributeList->spec(don't know why), use spec directly
    AttributeListSpecification arow = mentry.spec();
    AttributeListSpecification::iterator it;
    if (arow.size()==0) return;
    for(it=arow.begin(); it!=arow.end(); ++it){
      if ((*it).name() != "guid"){
        if ( _xmlcatalog->getGuidMeta(fid,(*it).name()).empty() ){
          _xmlcatalog->setGuidMeta(fid,(*it).name(),
                                   mentry.getAttributeValueAsString((*it).name()));
        }else{
          throw FCduplicatemetadataException("XMLCatalog::insertMetaData"); 
        }
      }
    }
    //end fix
    x_fc->to_be_updated = true;
  }//ZX: begin fix should rethrow this exception
  catch( pool::FCduplicatemetadataException& /* m */ ){
    throw FCduplicatemetadataException("XMLCatalog::insertMetaData"); 
  }//ZX: end fix
  catch( std::exception& er)
  {
    std::string message("Cannot insert MetaData");
    xmllog << seal::Msg::Fatal << message << seal::endmsg;
    throw FCbackendException("XMLMetaDataCatalog::insertMetaData", er.what());
  }
}

void
pool::XMLMetaDataCatalog::deleteMetaData(const FileCatalog::FileID& fid) const
{
  x_fc->ClassHandle();
  PoolXMLFileCatalog* _xmlcatalog = x_fc->castConnectionHandle(x_fc->XMLcon);
  if ( !_xmlcatalog->existsFileID(fid) )
    throw FCnonexistentFileException("XMLMetaDataCatalog::deleteMetaData",fid);
 
  _xmlcatalog->dropMetas(fid);
  x_fc->to_be_updated = true;
}

bool
pool::XMLMetaDataCatalog::retrievePFN(const std::string& query, 
                                      FCBuf<PFNEntry>& buf,
                                      const size_t& start)
{
  x_fc->ClassHandle();
  buf.clear();
  PoolXMLFileCatalog* _xmlcatalog = x_fc->castConnectionHandle(x_fc->XMLcon);
  std::vector<PoolXMLFileCatalog::poolFNs> files;
  //ZX: begin fix, if query is empty, should retrieve all; should pass buf.maxsize() instead of buf.size(). 
  std::string myquery(query);
  if (query.empty()){
    myquery="guid like %";
  }
  files = _xmlcatalog->getAllpfnsbyq(myquery,start,buf.maxsize());   
  //end fix
  for (unsigned int i = 0 ; i < files.size(); i++){ 
    PFNEntry* pentry = new 
      PFNEntry(files[i].fname, files[i].guid, files[i].filetype);
    buf.push_back(*pentry);
  }
  return (files.size() > 0);
}

bool
pool::XMLMetaDataCatalog::retrieveLFN(const std::string& query,
                                      FCBuf<LFNEntry>& buf,
                                      const size_t& start) 
{
  x_fc->ClassHandle();
  buf.clear();
  PoolXMLFileCatalog* _xmlcatalog = x_fc->castConnectionHandle(x_fc->XMLcon);
  std::vector<PoolXMLFileCatalog::poolFNs> files;
  //ZX: begin fix, if query is empty, should retrieve all; should pass buf.maxsize() instead of buf.size(). 
  std::string myquery(query);
  if (query.empty()){
    myquery="guid like %";
  }
  files = _xmlcatalog->getAlllfnsbyq(myquery,start,buf.maxsize());
  //end fix
  
  
  for (unsigned int i = 0 ; i < files.size(); i++){ 
    LFNEntry* lentry = new 
      LFNEntry(files[i].fname, files[i].guid);
    buf.push_back(*lentry);
  }
  return (files.size() > 0);
}

bool
pool::XMLMetaDataCatalog::retrieveMetaData(const std::string& query,
                                           FCBuf<MetaDataEntry>& buf,
                                           const size_t& start) 
{
  x_fc->ClassHandle();
  buf.clear();
  PoolXMLFileCatalog* _xmlcatalog = x_fc->castConnectionHandle(x_fc->XMLcon);
  std::vector<std::vector<PoolXMLFileCatalog::poolMeta> > files;
  //ZX: begin fix, if query is empty, should retrieve all; should pass buf.maxsize() instead of buf.size(). 
  std::string myquery(query);
  if (query.empty()){
    myquery="guid like %";
  }
  files = _xmlcatalog->getAllMetaDatabyq(myquery,start,buf.maxsize());
  //end fix
  for (unsigned int i = 0 ; i < files.size(); i++){ 
    //ZX: begin fix, mentry should be created inside loop
    MetaDataEntry mentry;
    this->getMetaDataSpec(mentry);
    //ZX: end fix
    std::vector<PoolXMLFileCatalog::poolMeta> mymeta = files[i];
    std::vector<PoolXMLFileCatalog::poolMeta>::iterator idata = mymeta.begin();
    for (unsigned int i=0; i<mymeta.size(); i++){
      try{
        std::string name=idata->first;
        if (name == "guid") {
          mentry.setGuidValue(idata->second);
        }else{
          //ZX: begin fix, should match type
          std::string type=(mentry.spec()[name]).type_name();
          if(type=="string"){
            mentry.setAttributeValue(name,idata->second);
          }else if(type=="int"){
            int value=::atoi((idata->second).c_str());
            mentry.setAttributeValue(name,value);
          }
          //ZX: end fix
        }
      }
      catch( std::exception& e)
      {
        throw e;
      }
      idata++;
    }
    buf.push_back(mentry);    
  }
  return (files.size() > 0);
}  

bool
pool::XMLMetaDataCatalog::retrieveGuid(const std::string& query, 
                                       FCBuf<FileCatalog::FileID>& buf, 
                                       const size_t& start ){
  x_fc->ClassHandle();
  buf.clear();
  PoolXMLFileCatalog* _xmlcatalog = x_fc->castConnectionHandle(x_fc->XMLcon);
  std::string myquery(query);
  std::vector<FileCatalog::FileID> fids;
  if( query.empty() ){
    myquery="guid like %";
  }  
  fids= _xmlcatalog->getAllGuidbyq(myquery,start,buf.maxsize());
  std::copy(fids.begin(),fids.end(),std::back_inserter(buf.getData())); 
  return (buf.size()>0);
}
//end fix



/*
void
pool::XMLMetaDataCatalog::ClassHandle() const
{
  PoolMessageStream xmllog( "XMLMetaDataCatalog", seal::Msg::Nil );
  if ( !is_started) {
    xmllog<< seal::Msg::Error 
          <<"Catalog not started, please use XMLMetaDataCatalog::start()"
          <<seal::endmsg;
    throw FCbackendException("XMLMetaDataCatalog::ClassHandle",
                             "start() not called");
  }
}




pool::PoolXMLFileCatalog*
pool::XMLMetaDataCatalog::castConnectionHandle(PoolXMLFileCatalog* con)
{
  PoolXMLFileCatalog* x = (con ? con : lastConnection);
  if (!x) 
    throw FCconnectionException("XMLMetaDataCatalog::castConnectionHandle",
                                "cannot find a connection");  
  return x;
}
*/

