#include <vector>
#include <fstream>
#include <iostream>
#include <string>
#include <xercesc/dom/DOM.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/framework/LocalFileFormatTarget.hpp>
#include <xercesc/util/XMLUni.hpp>
#include <xercesc/util/XMLURL.hpp>

#include "src/DTDRedirect.h"
#include "FileCatalog/CommandLine.h"

inline std::string _toString(const XMLCh *toTranscode)
{
  std::string tmp(XMLString::transcode(toTranscode));
  return tmp;
}
  
inline XMLCh*  _toDOMS( std::string temp ){
  XMLCh* buff = XMLString::transcode(temp.c_str());    
  return  buff;
}

int main(int argc, char** argv) {

  std::string  suri;
  std::string  duri;

  try{
    pool::CommandLine commands(argc,argv);
    if( commands.Exists("u") ){
      suri=commands.GetByName("u");
    }  
    if( commands.Exists("d") ){
      duri=commands.GetByName("d");
    }
    if(commands.Exists("h" ) ){
      std::cout<<"Usage:\nXMLmigrate_POOL1toPOOL1.4 -u source.xml -d destination.xml - No prefix is needed to the .xml files."<<std::endl;
      exit(0);
    }

  } catch(std::string& strError){
    std::cout << "error "<<strError<<std::endl;
    exit(0);
  }
  
  if( duri.empty() || suri.empty()  ) {
    std::cout<<"Usage:\nXMLmigrate_POOL1toPOOL1.4 -u source.xml -d destination.xml - No prefix is needed to the .xml files."<<std::endl;
    exit(0);
  }
  
  
  
  XMLPlatformUtils::Initialize();  
  
  XercesDOMParser* parser = new XercesDOMParser;     
  parser->setValidationScheme(XercesDOMParser::Val_Auto);
  parser->setDoNamespaces(false);      
  DTDRedirect dtdinmem;
  parser->setEntityResolver(&dtdinmem);
  parser->parse(suri.c_str());
  DOMDocument* doc = parser->getDocument();

  
  unsigned int Num_of_Tags  = 
    doc->getElementsByTagName(_toDOMS("pfn"))->getLength();
  for (unsigned int i=0; i<Num_of_Tags; i++){
    DOMNode* MyNode = 
      doc->getElementsByTagName(_toDOMS("pfn"))->item(i);
    
    DOMElement* nodelem = static_cast<DOMElement *>(MyNode);
    nodelem->removeAttribute(_toDOMS("file_status"));
    nodelem->removeAttribute(_toDOMS("job_status"));
  }

  DOMImplementation *impl = 
    DOMImplementationRegistry::getDOMImplementation(_toDOMS("LS"));
  DOMWriter         *theSerializer = 
    ((DOMImplementationLS*)impl)->createDOMWriter();
  XMLFormatTarget *myFormTarget = new LocalFileFormatTarget(duri.c_str());
  theSerializer->setFeature(XMLUni::fgDOMWRTFormatPrettyPrint, true);
  theSerializer->writeNode(myFormTarget, *doc);
  theSerializer->release();
  delete  myFormTarget;
  theSerializer=0;
  myFormTarget=0;    

  delete parser;
  XMLPlatformUtils::Terminate(); 
}
