/*
 * Copyright (C) 2003-2004 by CERN
 */

/*
 * @(#)$RCSfile: gfal.cpp,v $ $Revision: 1.2 $ $Date: 2005/02/15 08:50:31 $ CERN
Jean-Philippe Baud
 */

#include <sys/types.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#if defined(_WIN32)
#include <io.h>
#else
#include <unistd.h>
#endif
#include "LFCCatalog/gfal_api.h"
#if !defined(OFF_MAX)
#define OFF_MAX 2147483647
#endif

int
gfal_errmsg (char *errbuf, int errbufsz, const char *errmsg)
{
  if (errbuf == NULL)
    fprintf (stderr, "%s\n", errmsg);
  else {
    strncpy (errbuf, errmsg, errbufsz - 1);
    *(errbuf+errbufsz-1) = '\0';
  }
  return (0);
}

int
getdomainnm (char *name, size_t namelen)
{
  FILE *fd;
  char line[300];
  char *p;

  if ((fd = fopen ("/etc/resolv.conf", "r")) != NULL) {
    while (fgets (line, sizeof(line), fd) != NULL) {
      if ((strncmp (line, "domain", 6) == 0 ||
           strncmp (line, "search", 6) == 0) && line[6] == ' ')
        {
          fclose (fd);
          p = line + 6;
          while (*p == ' ')
            p++;
          if (*p)
            *(p + strlen (p) - 1) = '\0';
          if (strlen (p) > namelen) {
            errno = EINVAL;
            return (-1);
          }
          strcpy (name, p);
          return (0);
        }
    }
    fclose (fd);
  }
  return (-1);
}


char *
getbestfile(char **surls, int size, char *errbuf, int errbufsz)
{
  char dname[64];
  int first;
  int i;
  char  *p1, *p2;
  char *p;
  int ret;

  /* skip entries not in the form srm: or sfn:
   * take entry on same domain if it exists else
   * take the first supported entry
   */
  first = -1;
  *dname = '\0';
  (void) getdomainnm (dname, sizeof(dname));
  for (i = 0; i < size; i++) {
    p = surls[i];
    if (strncmp (p, "srm://", 6) && strncmp (p, "sfn://", 6))
      continue;
    if ((p1 = strchr (p + 6, '/')) == NULL) continue;
    *p1 = '\0';
    if ((p2 = strchr (p + 6, ':')))
      *p2 = '\0';
    if ((p = strchr (p + 6, '.')) == NULL) continue;
    if (first < 0) first = i;
    ret = strcmp (p + 1, dname);
    *p1 = '/';
    if (p2) *p2 = ':';
    if (ret == 0) break;	/* domains match ==> local replica */
  }
  if (i == size) {	/* no entry on same domain */
    if (first < 0) {	/* only non suported entries */
      gfal_errmsg(errbuf, errbufsz, "Only non supported entries. No replica entry starting with \"srm://\" or \"sfn://\".");
      errno = EINVAL;
      return (NULL);
    }
    i = first;
  }
  return surls[i];
}
