#ifndef OBJECTRELATIONALACCESS_OBJECTRELATIONDEFAULTMAPPINGBUILDER_H
#define OBJECTRELATIONALACCESS_OBJECTRELATIONDEFAULTMAPPINGBUILDER_H

#include <string>
#include <vector>

namespace ROOT {
  namespace Reflex{
    class Type;
  }
}

namespace pool {
  class ObjectRelationalMapping;
  class ObjectRelationalMappingElement;
  class PoolMessageStream;
  /**
     @class ObjectRelationalDefaultMappingBuilder ObjectRelationalDefaultMappingBuilder.h ObjectRelationalAccess/ObjectRelationalDefaultMappingBuilder.h

     Class responsible for the construction of a mapping given the C++ class definitions, following default rules.
  */

  class ObjectRelationalDefaultMappingBuilder {
  public:
    /// Constructor
    ObjectRelationalDefaultMappingBuilder();
    /// Destructor
    ~ObjectRelationalDefaultMappingBuilder();

    /**
       Processes a class given its name. Returns the success (or not) of the operation.
       @param mapping The mapping to be modified
       @param classDictionary The reflection information of the class to be processed
    */
    bool processClass( ObjectRelationalMapping& mapping,
                       const ROOT::Reflex::Type& classDictionary,
                       bool arraysInBlobs = false );

    bool buildCArrayElementTree( ObjectRelationalMappingElement& topElement );
    
    /// Returns the current list of involved tables
    const std::vector<std::string>& tablesInvolved() const;

    static size_t MaxColumnsPerTable;
    
    static size_t MaxSizeForInlineCArrays;

    static size_t DefaultSizeForInlineCArrays;
    
    // 
    static size_t sizeInColumns(const ROOT::Reflex::Type& topLevelClassType);
  private:
    /// The tables that are involved in the mapping
    std::vector<std::string> m_tablesInvolved;
    pool::PoolMessageStream& m_log;
    /// A flag indicating whether the arrays should be blobified
    bool m_arraysInBlobs;

    private:
    
    bool processObject( ObjectRelationalMappingElement& mappingElement,
                        const ROOT::Reflex::Type& classDictionary );

    bool processItem( ObjectRelationalMappingElement& parentelement,
                      const std::string& attributeName,
                      const ROOT::Reflex::Type& attributeType );

    bool processPrimitive( ObjectRelationalMappingElement& parentelement,
                           const std::string& attributeName,
                           const ROOT::Reflex::Type& attributeType );
    
    bool processBlob( ObjectRelationalMappingElement& parentelement,
                      const std::string& attributeName,
                      const ROOT::Reflex::Type& attributeType );

    bool processEmbeddedClass( ObjectRelationalMappingElement& parentelement,
                               const std::string& attributeName,
                               const ROOT::Reflex::Type& attributeType,
                               bool isRef );


    bool processArray( ObjectRelationalMappingElement& parentelement,
                       const std::string& attributeName, 
                       const ROOT::Reflex::Type& attributeType );

    bool processCArray( ObjectRelationalMappingElement& parentelement,
                        const std::string& attributeName,
                        const ROOT::Reflex::Type& attributeType );
    
    bool processCArrayItem( ObjectRelationalMappingElement& parentelement,
                            const std::string& attributeName,
                            const ROOT::Reflex::Type& attributeType,
                            const ROOT::Reflex::Type& arrayElementType );
    
    bool processInlineCArrayItem( ObjectRelationalMappingElement& parentelement,
                                  const std::string& attributeName,
                                  const ROOT::Reflex::Type& attributeType,
                                  const ROOT::Reflex::Type& arrayElementType );

    static void _sizeInColumns(const ROOT::Reflex::Type& typ, size_t& sz);
  };

}

#endif
