#ifndef OBJECTRELATIONALACCESS_OBJECTRELATIONALMAPPING_H
#define OBJECTRELATIONALACCESS_OBJECTRELATIONALMAPPING_H

#include "ObjectRelationalMappingElement.h"

namespace pool {

  /**
   * @class ObjectRelationalMapping ObjectRelationalMapping.h ObjectRelationalAccess/ObjectRelationalMapping.h
   * 
   * The structure holding an object/relational mapping.
   */
  
  class ObjectRelationalMapping
  {
    
    /** Public methods: */
  public:
    /// Constructor
    explicit ObjectRelationalMapping( const std::string& version );

    /// Destructor
    ~ObjectRelationalMapping() {}

    void setVersion( const std::string& version);
    
    /**
     * Returns the version of the mapping
     */
    const std::string& version() const;

    std::set<std::string> involvedTables();
    
    /// Iterator definition
    typedef std::map< std::string, ObjectRelationalMappingElement >::iterator iterator;
    typedef std::map< std::string, ObjectRelationalMappingElement >::const_iterator const_iterator;

    /// Returns an iterator in the beginning of the sequence
    iterator begin();
    const_iterator begin() const;

    /// Returns an iterator in the end of the sequence
    iterator end();
    const_iterator end() const;

    /// Retrieves a sub-element
    iterator find( const std::string& key );
    const_iterator find( const std::string& key ) const;

    /**
     * Appends a new element into the structure, and returns the new element
     * @param className The class name of the new element
     * @param tableName The table name of the new element
     */
    ObjectRelationalMappingElement& appendElement( const std::string& className,
                                                   const std::string& tableName );

    void override(const ObjectRelationalMapping& source);

  private:
    /**
     * The mapping version
     */
    std::string m_version;

    /**
     * The top-level mapping elements
     */
    std::map< std::string, ObjectRelationalMappingElement > m_elements;
  };
}

#include "ObjectRelationalMapping.inl"

#endif // OBJECTRELATIONALACCESS_OBJECTRELATIONALMAPPING_H
