#ifndef OBJECTRELATIONALACCESS_OBJECTRELATIONALMAPPINGUTILITIES_H
#define OBJECTRELATIONALACCESS_OBJECTRELATIONALMAPPINGUTILITIES_H

#include "ObjectRelationalAccess/ObjectRelationalUtilities.h"
#include "SealKernel/PropertyManager.h"

namespace coral {
  class ISessionProxy;
}

namespace pool {

  class MappingBuilder;
  class ObjectRelationalDatabaseDescriptor;
  
  class ObjectRelationalMappingUtilities: public ObjectRelationalUtilities  {

    public:
    ObjectRelationalMappingUtilities();

    ObjectRelationalMappingUtilities(coral::ISessionProxy* session);

    virtual ~ObjectRelationalMappingUtilities();

    void buildAndMaterializeMapping(const std::string& fileName, const std::string& outFile, bool evolve, bool dryRun=false);

    void buildAndMaterializeMapping(const ObjectRelationalDatabaseDescriptor& descriptor, const std::string& outFile, bool evolve, bool dryRun=false);

    void buildAndMaterializeMappingFromBuffer(const char* buffer, bool evolve, bool dryRun=false);

    void retrieveMapping(const std::string& version,const std::string& fileName);

    std::map<std::pair<std::string,std::string>, std::string > mappingForClassVersions();
    
    std::map<std::string,std::string> listMappings();

    std::map<std::string, std::string> containerTableMap();

    void selectMapping(const std::string& version, std::map<std::pair<std::string,std::string>,std::pair<std::string,std::string> >& mappingStruct);

    bool existsMapping(const std::string& version);

    bool removeMapping(const std::string& version, bool removeTables=false, bool dryRun=false);

    ObjectRelationalDatabaseDescriptor* loadDescriptor(const std::string fileName);

    ObjectRelationalDatabaseDescriptor* loadDescriptorFromBuffer(const char* buffer);

    static void prettyPrint(const std::map<std::string,std::string>& data, const std::string& firstColumn,const std::string& secondColumn, std::ostream& out );

    static void prettyPrint(const std::map<std::pair<std::string,std::string>,std::string>& data,
                            const std::string& firstColumn,const std::string& secondColumn, const std::string& thirdColumn, std::ostream& out );

    static void prettyPrint(const std::map<std::pair<std::string,std::string>,std::pair<std::string,std::string> >& data,
                            const std::string& firstColumn,const std::string& secondColumn,
                            const std::string& thirdColumn,const std::string& fourthColumn, std::ostream& out );

    static std::string keyOfMappingList();

    static std::string valueOfMappingList();

    static std::string keyFirstOfClassVersionList();
    
    static std::string keySecondOfClassVersionList();

    static std::string valueOfClassVersionList();

    static std::string keyOfContainerTableMap();

    static std::string valueOfContainerTableMap();
    
  };

}

#endif
