#ifndef OBJECTRELATIONALACCESS_OBJECTRELATIONALNAMES_H
#define OBJECTRELATIONALACCESS_OBJECTRELATIONALNAMES_H

#include <string>

namespace pool {

  namespace ObjectRelationalAccess {

    /**
     * @class ObjectRelationalNames ObjectRelationalNames.h
     *
     * Class holding the string constants used in the implemenation classes of the ObjectRelationalAccess package.
     */

    class ObjectRelationalNames {
    public:
      /// Dummy constructor
      ObjectRelationalNames(){}

      /// Returns the name of the package
      static std::string moduleName();

      /// Returns the name of the object mapping element type
      static std::string objectMappingElementType();
      /// Returns the name of the primitive mapping element type
      static std::string primitiveMappingElementType();
      /// Returns the name of the array mapping element type
      static std::string arrayMappingElementType();
      /// Returns the name of the array mapping element type
      static std::string CArrayMappingElementType();
      /// Returns the name of the inline array mapping element type
      static std::string inlineCArrayMappingElementType();
      /// Returns the name of the POOL reference mapping element type
      static std::string poolReferenceMappingElementType();
      /// Returns the name of the pointer mapping element type
      static std::string pointerMappingElementType();
      /// Returns the name of the reference mapping element type
      static std::string referenceMappingElementType();
      /// Returns the name of the blob mapping element type
      static std::string blobMappingElementType();
      /// Checks if the provided element type is valid
      static bool isValidMappingElementType( const std::string& elementType );

      /// Returns the name of the table which holds the versions of the mappings.
      static std::string mappingVersionTable();
      /// Returns the column name which holds the version in the table which holds the versions of the mappings.
      static std::string versionFieldInMappingVersionTable();
      /// Returns the column name which holds the container name in the table which holds the versions of the classes.
      static std::string containerNameFieldInMappingVersionTable();

      /// Returns the name of the table which holds the versions of the classes.
      static std::string classVersionTable();
      /// Returns the column name which holds the class version in the table which holds the versions of the classes.
      static std::string classVersionFieldInClassVersionTable();
      /// Returns the column name which holds the container name in the table which holds the versions of the classes.
      static std::string containerNameFieldInClassVersionTable();
      /// Returns the column name which holds the unique key (class+container name) in the table which holds the versions of the classes.
      ///static std::string classAndContainerFieldInClassVersionTable();
      /// Returns the column name which holds the version in the table which holds the versions of the classes.
      static std::string mappingVersionFieldInClassVersionTable();

      /// Returns the name of the table which holds the mapping elements.
      static std::string mappingElementTable();
      /// Returns the field which holds the element type in the table which holds the mapping elements.
      static std::string elementTypeFieldInMappingElementTable();
      /// Returns the field which holds the scope name in the table which holds the mapping elements.
      static std::string scopeNameFieldInMappingElementTable();
      /// Returns the field which holds the variable name in the table which holds the mapping elements.
      static std::string variableNameFieldInMappingElementTable();
      /// Returns the field which holds the variable type in the table which holds the mapping elements.
      static std::string variableTypeFieldInMappingElementTable();
      /// Returns the field which holds the associated table name in the table which holds the mapping elements.
      static std::string tableNameFieldInMappingElementTable();
      /// Returns the field which holds the version in the table which holds the mapping elements.
      static std::string versionFieldInMappingElementTable();

      /// Returns the name of the table which holds the names of the associated columns in a mapping element.
      static std::string mappingElementColumnsTable();
      /// Returns the field which holds the mapping version in the table which holds the names of the associated columns in a mapping element.
      static std::string versionFieldInMappingElementColumnsTable();
      /// Returns the field which holds the scope name in the table which holds the names of the associated columns in a mapping element.
      static std::string scopeNameFieldInMappingElementColumnsTable();
      /// Returns the field which holds the variable name in the table which holds the names of the associated columns in a mapping element.
      static std::string variableNameFieldInMappingElementColumnsTable();
      /// Returns the field which holds the column name in the table which holds the names of the associated columns in a mapping element.
      static std::string columnFieldInMappingElementColumnsTable();
      /// Returns the field which holds the position of the column in a mapping element in the table which holds the names of the associated columns in a mapping element.
      static std::string positionFieldInMappingElementColumnsTable();
      /// Returns the name of the foreign key constraint in the table which holds the names of the associated columns in a mapping element.
      static std::string foreignKeyNameInMappingElementColumnsTable();

      /// Returns the name of the index for the identity of an object mapped to a table
      static std::string indexNameForIdentity( const std::string& tableName );
      /// Returns the name of the foreign key to the identity of the parrent table
      static std::string fkNameForIdentity( const std::string& tableName );

      static std::string newMappingVersionForClass( const std::string& className, int iteration );

      static std::string classVersionPropertyNameInDictionary();

      static std::string defaultClassVersion(const std::string& className);
    };

  }

}

#endif
