#ifndef OBJECTRELATIONALACCESS_OBJECTRELATIONALTABLEVARIABLENAMERATIONALISER_H
#define OBJECTRELATIONALACCESS_OBJECTRELATIONALTABLEVARIABLENAMERATIONALISER_H

#include <string>
#include <set>
#include <map>

namespace pool {

  class ObjectRelationalMappingElement;

  /**
     @class ObjectRelationalTableVariableNameRationalizer ObjectRelationalTableVariableNameRationalizer.h ObjectRelationalAccess/ObjectRelationalTableVariableNameRationalizer.h

     Utility class to provide valid table variable names taking into account the maximum size allowed for a table name.
   */

  class ObjectRelationalTableVariableNameRationalizer {
  public:
    /// The default maximum size of a table variable name
    static const unsigned int defaultMaximumSizeForTableVariableName = 30;

    /// Constructor using the default values 
    ObjectRelationalTableVariableNameRationalizer();

    /// Constructor
    ObjectRelationalTableVariableNameRationalizer( unsigned int maximumSizeForTableVariableName );

    /// Destructor
    ~ObjectRelationalTableVariableNameRationalizer();

    /// Returns a valid name for a new table variable given a suggestion
    std::string validNameForNewTableVariable( const std::string& tableName,
                                              const std::string& suggestion );

    /// Rationalizes the table entries in a mapping element recursively
    void rationalizeMappingElement( ObjectRelationalMappingElement& element );

  private:
    /// The maximum size of a table name
    unsigned int m_maximumSizeForTableVariableName;

    /// The set of the valid table names so far
    std::map< std::string, std::set< std::string > > m_validTableVariableNames;

    /// The suggested to valid name resolutions
    std::map< std::string, std::map< std::string, std::string > > m_suggestedToValidName;
  };

}

#endif
