#ifndef OBJECTRELATIONALACCESS_OBJECTRELATIONALUTILITIES_H
#define OBJECTRELATIONALACCESS_OBJECTRELATIONALUTILITIES_H

#include <map>
#include <string>
#include <vector>
#include "SealKernel/MessageStream.h"
#include "SealKernel/Service.h"
#include "RelationalAccess/IAuthenticationService.h"
#include "RelationalAccess/AccessMode.h"

namespace coral 
{
  class ISessionProxy;
  class AuthenticationCredentials;
  class IAuthenticationCredentials;
}

namespace pool {
  
  class ObjectRelationalUtilities  {

    public:
    ObjectRelationalUtilities();

    ObjectRelationalUtilities(coral::ISessionProxy* session);

    virtual ~ObjectRelationalUtilities();

    void initialize();
    
    void connect(const std::string& connectionString, coral::AccessMode accessMode);

    coral::ISessionProxy* session();

    void setProperty(const std::string& propertyName,const std::string& propertyValue);

    void setFlag(const std::string& flagName,bool set);

    std::string getProperty(const std::string& propertyName) const;
    
    bool getFlag(const std::string& flagName) const;
    
    void loadDictionaries(const std::string& dictionaries);
    
    void setMessageVerbosityLevel(seal::Msg::Level level );

    seal::Msg::Level messageVerbosityLevel() const;

    static const std::string& connectionStringProperty();

    static const std::string& userNameProperty();

    static const std::string& passwordProperty();

    static const std::string& authenticationProperty();

    static const std::string& dictionaryProperty();

    static const std::string& verbosityProperty();

    static const std::string& debugVerbosityPropertyValue();

    static const std::string& infoVerbosityPropertyValue();

    class LocalAuthenticationService : public seal::Service,
      virtual public coral::IAuthenticationService
      {
      DECLARE_SEAL_COMPONENT;

      public:
        /// Standard Constructor
        LocalAuthenticationService( seal::Context* c );
      
        /// Standard Constructor with a component key
        LocalAuthenticationService( seal::Context* c, const std::string& key );

        /// Standard Destructor
        virtual ~LocalAuthenticationService();

        void setCredentials(const std::string& userName, const std::string& password);

      public:
        /**
         * Returns a reference to the credentials object
         */
        const coral::IAuthenticationCredentials& credentials( const std::string& ) const;
        const coral::IAuthenticationCredentials& credentials( const std::string&, const std::string& ) const;

      private:
        mutable coral::AuthenticationCredentials* m_credentials;
    };

    private:

    void loadLibraries(std::vector<std::string> libraries);

    std::map<std::string,std::string> m_properties;
    
    std::map<std::string,bool> m_flags;

    coral::ISessionProxy* m_session;

    bool m_sessionOwned;

    seal::Msg::Level m_verbosityLevel;
    
  };

}

#endif
