#include "ObjectRelationalAccess/ObjectRelationalMappingDescriptorElement.h"
#include "ObjectRelationalAccess/ObjectRelationalException.h"
#include "ObjectRelationalAccess/ObjectRelationalElementTypeNames.h"
#include "ObjectRelationalAccess/ObjectRelationalMappingElement.h"
#include "ObjectRelationalAccess/ObjectRelationalDefaultMappingRules.h"
#include "ObjectRelationalAccess/ObjectRelationalDefaultMappingBuilder.h"
#include "ObjectRelationalAccess/ObjectRelationalTableVariableNameRationalizer.h"
#include <sstream>
#include "StringTokenize.h"

const std::string& pool::ObjectRelationalMappingDescriptorElement::mappingTag(){
  static std::string s_mappingTag("Mapping");
  return s_mappingTag;
}

const std::string& pool::ObjectRelationalMappingDescriptorElement::classTag(){
  static std::string s_classTag("Class");
  return s_classTag;
}

const std::string& pool::ObjectRelationalMappingDescriptorElement::containerTag(){
  static std::string s_classTag("Container");
  return s_classTag;
}

const std::string& pool::ObjectRelationalMappingDescriptorElement::versionAttr(){
  static std::string s_versionAttr("version");
  return s_versionAttr;
}

const std::string& pool::ObjectRelationalMappingDescriptorElement::mappingVersionAttr(){
  static std::string s_mappingVersionAttr("mapping_version");
  return s_mappingVersionAttr;
}

const std::string& pool::ObjectRelationalMappingDescriptorElement::useMappingAttr(){
  static std::string s_useMappingAttr("use_mapping");
  return s_useMappingAttr;
}

const std::string& pool::ObjectRelationalMappingDescriptorElement::baseMappingAttr(){
  static std::string s_baseMappingAttr("base_mapping");
  return s_baseMappingAttr;
}

const std::string& pool::ObjectRelationalMappingDescriptorElement::defaultVersionAttrValue(){
  static std::string s_defaultVersionAttrValue("DEFAULT");
  return s_defaultVersionAttrValue;
}

const std::string& pool::ObjectRelationalMappingDescriptorElement::nameAttr(){
  static std::string s_nameAttr("name");
  return s_nameAttr;
}

const std::string& pool::ObjectRelationalMappingDescriptorElement::tableNameAttr(){
  static std::string s_tableNameAttr("table");
  return s_tableNameAttr;
}

const std::string& pool::ObjectRelationalMappingDescriptorElement::idColumnsAttr(){
  static std::string s_idColumnsAttr("id_columns");
  return s_idColumnsAttr;
}

const std::string& pool::ObjectRelationalMappingDescriptorElement::columnAttr(){
  static std::string s_columnAttr("column");
  return s_columnAttr;
}

const std::string& pool::ObjectRelationalMappingDescriptorElement::positionColumnAttr(){
  static std::string s_positionColumnAttr("position_column");
  return s_positionColumnAttr;
}

const std::string& pool::ObjectRelationalMappingDescriptorElement::refTableColumnAttr(){
  static std::string s_refTableColumnAttr("reference_table_column");
return s_refTableColumnAttr;
}

const std::string& pool::ObjectRelationalMappingDescriptorElement::poolContainerNameAttr(){
  static std::string s_poolContainerNameAttr("pool_container_name");
return s_poolContainerNameAttr;  
}


pool::ObjectRelationalMappingDescriptorElement::ObjectRelationalMappingDescriptorElement(const std::string& type) :  
                                                   m_type(type), 
                                                   m_attributes(),
                                                   m_children(),
                                                   m_parent(0),
                                                   m_defaultTableName(""){
}

pool::ObjectRelationalMappingDescriptorElement::ObjectRelationalMappingDescriptorElement(const ObjectRelationalMappingElement& element): m_type(ObjectRelationalMappingElement::elementTypeAsString(element.elementType())),
                                                                                                         m_attributes(),
                                                                                                         m_children(),
                                                                                                         m_parent(0),
                                                                                                         m_defaultTableName(""){
  if(m_type==ObjectRelationalElementTypeNames::arrayMappingElementType()) m_type=containerTag();

  setAttribute(tableNameAttr(),element.tableName());
  setAttribute(nameAttr(),element.variableName());
  setColumnNames(element);
  ObjectRelationalMappingElement::const_iterator iter;
  for(iter=element.begin();iter!=element.end();iter++){
    ObjectRelationalMappingDescriptorElement*  child = new ObjectRelationalMappingDescriptorElement((*iter).second);
    child->m_parent = this;  
    m_children.push_back(child);    
  }
}

pool::ObjectRelationalMappingDescriptorElement::~ObjectRelationalMappingDescriptorElement(){
  std::vector<ObjectRelationalMappingDescriptorElement*>::const_iterator iter;
  for(iter = m_children.begin(); iter!=m_children.end(); iter++){
     delete (*iter);
  }
  m_children.clear();
}

pool::ObjectRelationalMappingDescriptorElement::ObjectRelationalMappingDescriptorElement(const ObjectRelationalMappingDescriptorElement& aMappingDescriptorElement):  m_type(aMappingDescriptorElement.m_type),
		m_attributes(aMappingDescriptorElement.m_attributes),
                m_children(aMappingDescriptorElement.m_children),
                                                                                                                                                                      m_parent(0) {
}

pool::ObjectRelationalMappingDescriptorElement& pool::ObjectRelationalMappingDescriptorElement::operator=(const ObjectRelationalMappingDescriptorElement& aMappingDescriptorElement){
  m_type = aMappingDescriptorElement.m_type;
  m_attributes = aMappingDescriptorElement.m_attributes;
  m_children = aMappingDescriptorElement.m_children;
  return *this;
}
 
void pool::ObjectRelationalMappingDescriptorElement::setAttribute(const std::string& attributeName,const std::string& attributeValue){
  if(m_attributes.find(attributeName)==m_attributes.end()){
    m_attributes.insert(std::make_pair(attributeName,attributeValue));
  } else {
    m_attributes[attributeName] = attributeValue;
  }
}

const std::map<std::string,std::string>& pool::ObjectRelationalMappingDescriptorElement::attributes() const {
  return m_attributes;
}

std::string pool::ObjectRelationalMappingDescriptorElement::attribute(const std::string& attributeName) const {
  std::map<std::string,std::string>::const_iterator entry = m_attributes.find(attributeName);
  if(entry==m_attributes.end()){ 
    std::stringstream sout;
    sout << "Attribute " << attributeName << " not found.";
    throw ObjectRelationalException(sout.str(),"ObjectRelationalMappingDescriptorElement::attribute(const std::string&)");
  }
  return (*entry).second;
}

std::string pool::ObjectRelationalMappingDescriptorElement::name() const {
  return attribute(nameAttr());
}

void pool::ObjectRelationalMappingDescriptorElement::setType(std::string type){
  m_type = type;
}

std::string pool::ObjectRelationalMappingDescriptorElement::type() const{
  return m_type;
}

std::string pool::ObjectRelationalMappingDescriptorElement::objectRelationalMappingElementType() const{
  if(m_type==classTag()) {
    return ObjectRelationalElementTypeNames::objectMappingElementType();
  }  else if ( m_type==containerTag() || m_type==ObjectRelationalElementTypeNames::arrayMappingElementType()){
    return ObjectRelationalElementTypeNames::arrayMappingElementType();
  }  
  return m_type;
}

pool::ObjectRelationalMappingDescriptorElement* pool::ObjectRelationalMappingDescriptorElement::addChild(const std::string& type){
  ObjectRelationalMappingDescriptorElement* child = new ObjectRelationalMappingDescriptorElement(type);
  child->m_parent = this;  
  m_children.push_back(child);
  return child;
}

void pool::ObjectRelationalMappingDescriptorElement::importChild(ObjectRelationalMappingDescriptorElement* element){  
  element->m_parent = this;  
  m_children.push_back(element);
  return;  
}

const std::vector<pool::ObjectRelationalMappingDescriptorElement*>& pool::ObjectRelationalMappingDescriptorElement::children() const {
  return m_children;
}

bool pool::ObjectRelationalMappingDescriptorElement::alter(ObjectRelationalMappingElement& element,
                                                           ObjectRelationalTableVariableNameRationalizer& rationalizer,
                                                           bool ignoreMismatches) const {
  
  // check if the element type matches
  if(element.elementType()!=ObjectRelationalMappingElement::elementTypeFromString(objectRelationalMappingElementType())){
    if(objectRelationalMappingElementType()==pool::ObjectRelationalElementTypeNames::blobMappingElementType())
    {
      if(m_defaultTableName.empty())
      {
         std::stringstream sout;
         sout << "Cannot retrieve default table for Blob type element:"<<name();
         throw ObjectRelationalException(sout.str(),"ObjectRelationalMappingDescriptorElement::alter");
      }      
      element.alterType(pool::ObjectRelationalElementTypeNames::blobMappingElementType());
      element.alterTableName(m_defaultTableName);
    }
    else if(objectRelationalMappingElementType()==pool::ObjectRelationalElementTypeNames::CArrayMappingElementType() &&
            element.elementType()==pool::ObjectRelationalMappingElement::InlineCArray )
    {
      element.alterType(pool::ObjectRelationalElementTypeNames::CArrayMappingElementType());
      std::vector<std::string> elementsToDelete;
      for(pool::ObjectRelationalMappingElement::iterator iE=element.begin();
          iE!=element.end();++iE){
        elementsToDelete.push_back(iE->second.variableName());
      }
      for(std::vector<std::string>::iterator iV=elementsToDelete.begin();
          iV!=elementsToDelete.end();++iV){
        element.removeSubElement(*iV);
      }
      ObjectRelationalDefaultMappingBuilder builder;
      builder.buildCArrayElementTree( element );
    }
    else 
    {
      if(!ignoreMismatches)
      {
        std::stringstream sout;
        sout << "Type mismatch for element name="<<name()<<": expected="<<objectRelationalMappingElementType() << " - found="<<pool::ObjectRelationalMappingElement::elementTypeAsString(element.elementType());
        throw ObjectRelationalException(sout.str(),"ObjectRelationalMappingDescriptorElement::alter");
      }
    }
  }
  
  // alter parameters for the current field
  std::map<std::string,std::string>::const_iterator entry = m_attributes.find(tableNameAttr());
  if(entry!=m_attributes.end()){ 
    std::string tableName = (*entry).second;
    if(!tableName.empty()){
      element.alterTableName(tableName);
    }
  }
  // alter column names
  this->alterColumnNames(element, rationalizer);
  // alter parameters for the fields in the nested scope
  std::vector<ObjectRelationalMappingDescriptorElement*>::const_iterator iter;
  for(iter = m_children.begin(); iter!=m_children.end(); iter++){
    std::string name = (*iter)->name();
    ObjectRelationalMappingElement::iterator elem_iter =  element.find(name);
    if(elem_iter!=element.end()){
      // get the default table as the table name of the parent element
      (*iter)->m_defaultTableName = element.tableName();
      (*iter)->alter((*elem_iter).second, rationalizer, ignoreMismatches);
    } else {
      if(!ignoreMismatches)
      {
        std::stringstream sout;
        sout << "Variable name=\"" << name << "\" specified for object \"" << this->name()
             << "\" does not exist (not found in default mapping).";
        throw ObjectRelationalException(sout.str(),"ObjectRelationalMappingDescriptorElement::alter");
      }
    }
  }
  return true;
}

pool::ObjectRelationalMappingDescriptorElement* pool::ObjectRelationalMappingDescriptorElement::parent() const {
  return m_parent;
}

void pool::ObjectRelationalMappingDescriptorElement::alterColumnNames(ObjectRelationalMappingElement& toElement,
                                                      ObjectRelationalTableVariableNameRationalizer& rationalizer) const {
  std::vector<std::string> columnNames;
  // only for PoolReferences: retrieve and keep the two hidden column names 
  const std::vector<std::string>& elementCols = toElement.columnNames();
  unsigned int ncols = elementCols.size();
  // reconstruct columns array depending on the type
  ObjectRelationalMappingElement::ElementType elementType = ObjectRelationalMappingElement::elementTypeFromString(objectRelationalMappingElementType());
  std::map<std::string,std::string>::const_iterator entry;
  switch ( elementType ) {
  case ObjectRelationalMappingElement::Object :
    entry = m_attributes.find(idColumnsAttr());
    if(entry!=m_attributes.end()){
      stringTokenize((*entry).second,columnNames);
    }
    break;
  case ObjectRelationalMappingElement::InlineCArray :
    entry = m_attributes.find(idColumnsAttr());
    if(entry!=m_attributes.end()){
      stringTokenize((*entry).second,columnNames);
    }
    break;
  case ObjectRelationalMappingElement::Primitive :
    entry = m_attributes.find(columnAttr());
    if(entry!=m_attributes.end()){
      columnNames.push_back((*entry).second);
    }
    break;
  case ObjectRelationalMappingElement::Array :
    entry = m_attributes.find(positionColumnAttr());
    if(entry!=m_attributes.end()){
      columnNames.push_back((*entry).second);
    }
    entry = m_attributes.find(idColumnsAttr());
    if(entry!=m_attributes.end()){
      stringTokenize((*entry).second,columnNames);    
    }
    break;
  case ObjectRelationalMappingElement::CArray :
    entry = m_attributes.find(positionColumnAttr());
    if(entry!=m_attributes.end()){
      columnNames.push_back((*entry).second);
    }
    entry = m_attributes.find(idColumnsAttr());
    if(entry!=m_attributes.end()){
      stringTokenize((*entry).second,columnNames);    
    }
    break;
  case ObjectRelationalMappingElement::PoolReference :
    entry = m_attributes.find(idColumnsAttr());
    if(entry!=m_attributes.end()){
      stringTokenize((*entry).second,columnNames);
    }
    // restore the hidden columns
    if(ncols>1){
      columnNames.push_back(elementCols[ncols-2]);
      columnNames.push_back(elementCols[ncols-1]);
    }
    break;
  case ObjectRelationalMappingElement::Reference :
    // not yet supported by seal??
    break;
  case ObjectRelationalMappingElement::Pointer :
    entry = m_attributes.find(positionColumnAttr());
    entry = m_attributes.find(refTableColumnAttr());
    if(entry!=m_attributes.end()){    
      columnNames.push_back((*entry).second);
    }
    entry = m_attributes.find(idColumnsAttr());
    if(entry!=m_attributes.end()){
      stringTokenize((*entry).second,columnNames);        
    }
    break;
  case ObjectRelationalMappingElement::Blob :
    entry = m_attributes.find(columnAttr());
    if(entry!=m_attributes.end()){
      columnNames.push_back((*entry).second);
    } else {
      /// handling default column name for blobs
      std::string blobColName = pool::ObjectRelationalDefaultMappingRules::columnNameForVariable(toElement.variableName(),toElement.scopeName());
      columnNames.push_back( rationalizer.validNameForNewTableVariable( toElement.tableName(),blobColName ));
    }
    break;
  case ObjectRelationalMappingElement::Undefined :
    throw ObjectRelationalException("Unsupported type for custom mapping","ObjectRelationalMappingDescriptorElement::getColumnNames");
    break;
  };
  // apply the new columns
  if(columnNames.size()!=0){
    toElement.setColumnNames(columnNames);
  }
}

void pool::ObjectRelationalMappingDescriptorElement::setColumnNames(const ObjectRelationalMappingElement& fromElement) {
  ObjectRelationalMappingElement::ElementType elementType = fromElement.elementType();
  std::vector<std::string> columnNames = fromElement.columnNames();
  std::vector<std::string>::const_iterator iter;
  std::stringstream sout;
  unsigned int i=0;
  switch ( elementType ) {
  case ObjectRelationalMappingElement::Object :
    for(iter=columnNames.begin();iter!=columnNames.end();iter++){
      if(i>0) sout << " ";
      sout << (*iter);
      i++; 
    }
    if(i)setAttribute(idColumnsAttr(),sout.str());
    break;
  case ObjectRelationalMappingElement::InlineCArray :
    for(iter=columnNames.begin();iter!=columnNames.end();iter++){
      if(i>0) sout << " ";
      sout << (*iter);
      i++; 
    }
    if(i)setAttribute(idColumnsAttr(),sout.str());
    break;
  case ObjectRelationalMappingElement::Primitive :
    iter=columnNames.begin();
    if(iter!=columnNames.end()){
      setAttribute(columnAttr(),(*iter));
    }
    break;
  case ObjectRelationalMappingElement::Blob :
    iter=columnNames.begin();
    if(iter!=columnNames.end()){
      setAttribute(columnAttr(),(*iter));
    }
    break;
  case ObjectRelationalMappingElement::Array :
    for(iter=columnNames.begin();iter!=columnNames.end();iter++){
      if(i==0) {
        setAttribute(positionColumnAttr(),(*iter));
      } else {
        if(i>1) sout << " ";
        sout << (*iter);
      }
      i++; 
    }
    if(i>1) setAttribute(idColumnsAttr(),sout.str()); 
    break;
  case ObjectRelationalMappingElement::CArray :
    for(iter=columnNames.begin();iter!=columnNames.end();iter++){
      if(i==0) {
        setAttribute(positionColumnAttr(),(*iter));
      } else {
        if(i>1) sout << " ";
        sout << (*iter);
      }
      i++; 
    }
    if(i>1) setAttribute(idColumnsAttr(),sout.str()); 
    break;
  case ObjectRelationalMappingElement::PoolReference :
    columnNames.pop_back();
    columnNames.pop_back();
    for(iter=columnNames.begin();iter!=columnNames.end();iter++){
      if(i>0) sout << " ";
      sout << (*iter);
      i++; 
    }
    if(i)setAttribute(idColumnsAttr(),sout.str());
    break;
  case ObjectRelationalMappingElement::Reference :
    //
    break;
  case ObjectRelationalMappingElement::Pointer :
    for(iter=columnNames.begin();iter!=columnNames.end();iter++){
      if(i==0) {
        setAttribute(refTableColumnAttr(),(*iter));
      } else {
        if(i>1) sout << " ";
        sout << (*iter);
      }
      i++; 
    }
    if(i>1) setAttribute(idColumnsAttr(),sout.str()); 
    break;
  case ObjectRelationalMappingElement::Undefined :
    throw ObjectRelationalException("Unsupported type for custom mapping","ObjectRelationalMappingDescriptorElement::setColumnNames");
    break;
  };
}

