#include "ObjectRelationalAccess/ObjectRelationalMappingUtilities.h"
#include "ObjectRelationalAccess/ObjectRelationalDatabaseDescriptor.h"
#include "ObjectRelationalAccess/ObjectRelationalException.h"
#include "MappingBuilder.h"
#include "XMLDatabaseManagerDriver.h"
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ITransaction.h"
#include "POOLCore/POOLContext.h"
#include "SealKernel/Context.h"
#include <iostream>
#include <iomanip>

pool::ObjectRelationalMappingUtilities::ObjectRelationalMappingUtilities() : ObjectRelationalUtilities(){}

pool::ObjectRelationalMappingUtilities::ObjectRelationalMappingUtilities(coral::ISessionProxy* session) : ObjectRelationalUtilities(session){}

pool::ObjectRelationalMappingUtilities::~ObjectRelationalMappingUtilities(){
}

void pool::ObjectRelationalMappingUtilities::buildAndMaterializeMapping(const std::string& fileName,
                                                                        const std::string& outFile,
                                                                        bool evolve,
                                                                        bool dryRun ){
  initialize();
  coral::ISessionProxy* session = this->session();
  if(!session) throw pool::ObjectRelationalException("No valid connection to database.",
                                                     "ObjectRelationalMappingUtilities::buildAndMaterializeMapping");
  pool::MappingBuilder builder(POOLContext::context(),session->nominalSchema());
  std::auto_ptr<ObjectRelationalDatabaseDescriptor> dbstruct(loadDescriptor(fileName));
  // loop over containers
  for(std::vector<ObjectRelationalContainerDescriptor*>::const_iterator iContainer=dbstruct->containers().begin();
      iContainer!=dbstruct->containers().end();
      iContainer++){
    for(std::vector<ObjectRelationalClassDescriptor*>::const_iterator iClass=(*iContainer)->classStores().begin();
        iClass!=(*iContainer)->classStores().end();
        iClass++){
      const ObjectRelationalMappingDescriptor* mappDescr = (*iClass)->mapping();
      if(mappDescr){
        builder.buildMapping(mappDescr);
      }
    }
  }
  builder.finalizeMapping(evolve,dryRun);
  if(!outFile.empty()){
    builder.exportMapping(outFile);
  }
}

void pool::ObjectRelationalMappingUtilities::buildAndMaterializeMapping(const ObjectRelationalDatabaseDescriptor& dbstruct,
                                                                        const std::string& outFile,
                                                                        bool evolve,
                                                                        bool dryRun ){
  initialize();
  coral::ISessionProxy* session = this->session();
  if(!session) throw pool::ObjectRelationalException("No valid connection to database.",
                                                     "ObjectRelationalMappingUtilities::buildAndMaterializeMapping");
  pool::MappingBuilder builder(POOLContext::context(),session->nominalSchema());
  // loop over containers
  for(std::vector<ObjectRelationalContainerDescriptor*>::const_iterator iContainer=dbstruct.containers().begin();
      iContainer!=dbstruct.containers().end();
      iContainer++){
    for(std::vector<ObjectRelationalClassDescriptor*>::const_iterator iClass=(*iContainer)->classStores().begin();
        iClass!=(*iContainer)->classStores().end();
        iClass++){
      const ObjectRelationalMappingDescriptor* mappDescr = (*iClass)->mapping();
      if(mappDescr){
        builder.buildMapping(mappDescr);
      }
    }
  }
  builder.finalizeMapping(evolve,dryRun);
  if(!outFile.empty()){
    builder.exportMapping(outFile);
  }
}


void pool::ObjectRelationalMappingUtilities::buildAndMaterializeMappingFromBuffer(const char* buffer,
                                                                                  bool evolve,
                                                                                  bool dryRun){
  initialize();
  coral::ISessionProxy* session = this->session();
  if(!session) throw pool::ObjectRelationalException("No valid connection to database.",
                                                     "ObjectRelationalMappingUtilities::buildAndMaterializeMappingFromBuffer");
  pool::MappingBuilder builder(POOLContext::context(),session->nominalSchema());
  std::auto_ptr<ObjectRelationalDatabaseDescriptor> dbstruct(loadDescriptorFromBuffer(buffer));
  // loop over containers
  for(std::vector<ObjectRelationalContainerDescriptor*>::const_iterator iContainer=dbstruct->containers().begin();
      iContainer!=dbstruct->containers().end();
      iContainer++){
    for(std::vector<ObjectRelationalClassDescriptor*>::const_iterator iClass=(*iContainer)->classStores().begin();
        iClass!=(*iContainer)->classStores().end();
        iClass++){
      const ObjectRelationalMappingDescriptor* mappDescr = (*iClass)->mapping();
      if(mappDescr){
        builder.buildMapping(mappDescr);
      }
    }
  }
  builder.finalizeMapping(evolve,dryRun);
}

void pool::ObjectRelationalMappingUtilities::retrieveMapping(const std::string& version,const std::string& fileName){
  initialize();
  coral::ISessionProxy* session = this->session();
  if(!session) throw pool::ObjectRelationalException("No valid connection to database.","ObjectRelationalMappingUtilities::retrieveMapping");
  pool::MappingBuilder builder(POOLContext::context(),session->nominalSchema());
  builder.retrieveMapping(version,fileName);
}

std::map<std::pair<std::string,std::string>, std::string >
pool::ObjectRelationalMappingUtilities::mappingForClassVersions(){
  initialize();
  coral::ISessionProxy* session = this->session();
  if(!session) throw pool::ObjectRelationalException("No valid connection to database.","ObjectRelationalMappingUtilities::mappingForClassVersions");
  pool::MappingBuilder builder(POOLContext::context(),session->nominalSchema());
  return builder.mappingForClassVersions();  
}

std::map<std::string,std::string>
pool::ObjectRelationalMappingUtilities::listMappings(){
  initialize();
  coral::ISessionProxy* session = this->session();
  if(!session) throw pool::ObjectRelationalException("No valid connection to database.","ObjectRelationalMappingUtilities::listMappings");
  pool::MappingBuilder builder(POOLContext::context(),session->nominalSchema());
  return builder.mappingVersions();
}

std::map<std::string, std::string>
pool::ObjectRelationalMappingUtilities::containerTableMap(){
   initialize();
  coral::ISessionProxy* session = this->session();
  if(!session) throw pool::ObjectRelationalException("No valid connection to database.","ObjectRelationalMappingUtilities::listMappings");
  pool::MappingBuilder builder(POOLContext::context(),session->nominalSchema());
  return builder.containerTableMap(); 
}

void
pool::ObjectRelationalMappingUtilities::selectMapping(const std::string& version,
                                                      std::map<std::pair<std::string,std::string>,std::pair<std::string,std::string> >& mappingStruct){
  initialize();
  coral::ISessionProxy* session = this->session();
  if(!session) throw pool::ObjectRelationalException("No valid connection to database.","ObjectRelationalMappingUtilities::listMappings");
  pool::MappingBuilder builder(POOLContext::context(),session->nominalSchema());
  builder.selectMapping(version,mappingStruct);
}

bool pool::ObjectRelationalMappingUtilities::existsMapping(const std::string& version){
  initialize();
  coral::ISessionProxy* session = this->session();
  if(!session) throw pool::ObjectRelationalException("No valid connection to database.","ObjectRelationalMappingUtilities::existsMapping");
  pool::MappingBuilder builder(POOLContext::context(),session->nominalSchema());
  std::map<std::string,std::string> versionList = builder.mappingVersions();
  return (versionList.find(version)!=versionList.end());
}

bool pool::ObjectRelationalMappingUtilities::removeMapping(const std::string& version, bool removeTables, bool dryRun){
  initialize();
  coral::ISessionProxy* session = this->session();
  if(!session) throw pool::ObjectRelationalException("No valid connection to database.","ObjectRelationalMappingUtilities::removeMapping");
  bool removed = false;
  pool::MappingBuilder builder(POOLContext::context(),session->nominalSchema());
  removed = builder.removeMapping(version,removeTables,dryRun);
  return removed;
}

pool::ObjectRelationalDatabaseDescriptor*
pool::ObjectRelationalMappingUtilities::loadDescriptor(const std::string fileName){
  XMLDatabaseManagerDriver driver;
  ObjectRelationalDatabaseDescriptor*  dbstruct = driver.read(fileName);
  return dbstruct;
}

pool::ObjectRelationalDatabaseDescriptor*
pool::ObjectRelationalMappingUtilities::loadDescriptorFromBuffer(const char* buffer){
  XMLDatabaseManagerDriver driver;
  ObjectRelationalDatabaseDescriptor*  dbstruct = driver.readFromBuffer(buffer);
  return dbstruct;
}

void
pool::ObjectRelationalMappingUtilities::prettyPrint(const std::map<std::string,std::string>& data,
                                                    const std::string& firstColumn,const std::string& secondColumn, std::ostream& out ){
  size_t firstColMax = firstColumn.length();
  size_t secondColMax = secondColumn.length();
  // first find the max lenghts
  for(std::map<std::string,std::string>::const_iterator iR = data.begin();iR!=data.end();++iR){
    if(iR->first.length() > firstColMax) firstColMax = iR->first.length();
    if(iR->second.length() > secondColMax) secondColMax = iR->second.length();
  }

  // then print formatted
  std::string space("   ");
  out << std::setiosflags(std::ios_base::left);
  out << space << std::setw(firstColMax)<<firstColumn;
  out << space << std::setw(secondColMax)<<secondColumn;
  out << std::endl;
  out << space << std::setfill('-')<<std::setw(firstColMax)<<"";
  out << space <<std::setw(secondColMax)<<"";
  out << std::endl;
  out << std::setfill(' ');
  for(std::map<std::string,std::string>::const_iterator iR = data.begin();iR!=data.end();++iR){
    out << space << std::setw(firstColMax)<<iR->first;
    out << space << std::setw(secondColMax)<<iR->second;
    out << std::endl;
  }
}

void
pool::ObjectRelationalMappingUtilities::prettyPrint(const std::map<std::pair<std::string,std::string>,std::string>& data,
                                                    const std::string& firstColumn,
                                                    const std::string& secondColumn,
                                                    const std::string& thirdColumn,
                                                    std::ostream& out ){
  size_t firstColMax = firstColumn.length();
  size_t secondColMax = secondColumn.length();
  size_t thirdColMax = thirdColumn.length();
  // first find the max lenghts
  for(std::map<std::pair<std::string,std::string>, std::string>::const_iterator iR = data.begin();iR!=data.end();++iR){
    if(iR->first.first.length() > firstColMax) firstColMax = iR->first.first.length();
    if(iR->first.second.length() > secondColMax) secondColMax = iR->first.second.length();
    if(iR->second.length() > thirdColMax) thirdColMax = iR->second.length();
  }

  // then print formatted
  std::string space("   ");
  out << std::setiosflags(std::ios_base::left);
  out << space << std::setw(firstColMax)<<firstColumn;
  out << space << std::setw(secondColMax)<<secondColumn;
  out << space << std::setw(thirdColMax)<<thirdColumn;
  out << std::endl;
  out << space << std::setfill('-')<<std::setw(firstColMax)<<"";
  out << space <<std::setw(secondColMax)<<"";
  out << space <<std::setw(thirdColMax)<<"";
  out << std::endl;
  out << std::setfill(' ');
  for(std::map<std::pair<std::string,std::string>, std::string>::const_iterator iR = data.begin();iR!=data.end();++iR){
    out << space << std::setw(firstColMax)<<iR->first.first;
    out << space << std::setw(secondColMax)<<iR->first.second;
     out << space << std::setw(thirdColMax)<<iR->second;
   out << std::endl;
  }
}

void
pool::ObjectRelationalMappingUtilities::prettyPrint(const std::map<std::pair<std::string,std::string>,std::pair<std::string,std::string> >& data,
                 const std::string& firstColumn,const std::string& secondColumn,
                 const std::string& thirdColumn,const std::string& fourthColumn, std::ostream& out ){
  
  size_t firstColMax = firstColumn.length();
  size_t secondColMax = secondColumn.length();
  size_t thirdColMax = thirdColumn.length();
  size_t fourthColMax = fourthColumn.length();
  // first find the max lenghts
  for(std::map<std::pair<std::string,std::string>, std::pair<std::string,std::string> >::const_iterator iR = data.begin();iR!=data.end();++iR){
    if(iR->first.first.length() > firstColMax) firstColMax = iR->first.first.length();
    if(iR->first.second.length() > secondColMax) secondColMax = iR->first.second.length();
    if(iR->second.first.length() > thirdColMax) thirdColMax = iR->second.first.length();
    if(iR->second.second.length() > fourthColMax) fourthColMax = iR->second.second.length();
  }

  // then print formatted
  std::string space("   ");
  out << std::setiosflags(std::ios_base::left);
  out << space << std::setw(firstColMax)<<firstColumn;
  out << space << std::setw(secondColMax)<<secondColumn;
  out << space << std::setw(thirdColMax)<<thirdColumn;
  out << space << std::setw(fourthColMax)<<fourthColumn;
  out << std::endl;
  out << space << std::setfill('-')<<std::setw(firstColMax)<<"";
  out << space <<std::setw(secondColMax)<<"";
  out << space <<std::setw(thirdColMax)<<"";
  out << space <<std::setw(fourthColMax)<<"";
  out << std::endl;
  out << std::setfill(' ');
  for(std::map<std::pair<std::string,std::string>,std::pair<std::string,std::string> >::const_iterator iR = data.begin();iR!=data.end();++iR){
    out << space << std::setw(firstColMax)<<iR->first.first;
    out << space << std::setw(secondColMax)<<iR->first.second;
     out << space << std::setw(thirdColMax)<<iR->second.first;
     out << space << std::setw(fourthColMax)<<iR->second.second;
   out << std::endl;
  }
}

std::string pool::ObjectRelationalMappingUtilities::keyOfMappingList(){
  static std::string s_ret("mapping_version");
  return s_ret;
}

std::string pool::ObjectRelationalMappingUtilities::valueOfMappingList(){
  static std::string s_ret("container_name");
  return s_ret;
}

std::string pool::ObjectRelationalMappingUtilities::keyFirstOfClassVersionList(){
  static std::string s_ret("class_version");
  return s_ret;
}

std::string pool::ObjectRelationalMappingUtilities::keySecondOfClassVersionList(){
  static std::string s_ret("container_name");
  return s_ret;
}

std::string pool::ObjectRelationalMappingUtilities::valueOfClassVersionList(){
  static std::string s_ret("mapping_version");
  return s_ret;
}

std::string pool::ObjectRelationalMappingUtilities::keyOfContainerTableMap(){
  static std::string s_ret("table_name");
  return s_ret;
}

std::string pool::ObjectRelationalMappingUtilities::valueOfContainerTableMap(){
  static std::string s_ret("container_name");
  return s_ret;
}

