#include "ObjectRelationalAccess/ObjectRelationalTableNameRationalizer.h"
#include "ObjectRelationalAccess/ObjectRelationalMappingElement.h"
#include "RelationalAccess/ISchema.h"
#include <sstream>
#include <iostream>

pool::ObjectRelationalTableNameRationalizer::ObjectRelationalTableNameRationalizer( coral::ISchema& schema ):
  m_schema( schema ),
  m_maximumSizeForTableName( pool::ObjectRelationalTableNameRationalizer::defaultMaximumSizeForTableName ),
  m_validTableNames(),
  m_suggestedToValidName()
{}

pool::ObjectRelationalTableNameRationalizer::ObjectRelationalTableNameRationalizer( coral::ISchema& schema,
                                                                                    unsigned int maximumSizeForTableName ):
  m_schema( schema ),
  m_maximumSizeForTableName( maximumSizeForTableName ),
  m_validTableNames(),
  m_suggestedToValidName()
{}

pool::ObjectRelationalTableNameRationalizer::~ObjectRelationalTableNameRationalizer()
{}

std::string
pool::ObjectRelationalTableNameRationalizer::validNameForNewTable( const std::string& suggestion )
{
  // Check if this is an already validated name
  std::map< std::string, std::string >::const_iterator iValidName = m_suggestedToValidName.find( suggestion );
  if ( iValidName != m_suggestedToValidName.end() ) return iValidName->second;

  // Check name as is first
  if (  suggestion.size() <= m_maximumSizeForTableName &&
        ! m_schema.existsTable( suggestion ) &&
        ! m_schema.existsView( suggestion ) ) {
    m_suggestedToValidName.insert( std::make_pair( suggestion, suggestion ) );
    m_validTableNames.insert( suggestion );
    return suggestion;
  }

  for ( unsigned int iTrial = 1; static_cast<unsigned int>( iTrial / 10 ) + 1 < m_maximumSizeForTableName; ++iTrial ) {
    // Calculate the prefix
    unsigned int digitsForPrefix = iTrial / 10 + 1;
    unsigned int prefixSize = m_maximumSizeForTableName - digitsForPrefix - 1;
    std::ostringstream os;
    os << suggestion.substr( 0, prefixSize ) << "_" << iTrial;
    std::string candidateName = os.str();
    if ( ! m_schema.existsTable( candidateName ) &&
         ! m_schema.existsView( candidateName ) &&
         m_validTableNames.find( candidateName ) == m_validTableNames.end() ) {
      m_suggestedToValidName.insert( std::make_pair( suggestion, candidateName ) );
      m_validTableNames.insert( candidateName );
      return candidateName;
    }
  }

  return "";
}


void
pool::ObjectRelationalTableNameRationalizer::rationalizeMappingElement( pool::ObjectRelationalMappingElement& element )
{
  element.alterTableName( this->validNameForNewTable( element.tableName() ) );
  for ( pool::ObjectRelationalMappingElement::iterator iElement = element.begin();
        iElement != element.end(); ++iElement ) {
    this->rationalizeMappingElement( iElement->second );
  }
}
