#include "TableDescriptionStack.h"
#include "ObjectRelationalAccess/ObjectRelationalNames.h"
#include "SealKernel/MessageStream.h"
#include "POOLCore/POOLContext.h"
#include "RelationalAccess/TableDescription.h"


pool::ObjectRelationalAccess::TableDescriptionStack::TableDescriptionStack():
  m_tableNames(),
  m_descriptions(),
  m_log( new seal::MessageStream( pool::POOLContext::context(),
                                  pool::ObjectRelationalAccess::ObjectRelationalNames::moduleName() ) )
{}


pool::ObjectRelationalAccess::TableDescriptionStack::~TableDescriptionStack()
{
  for ( std::map< std::string, coral::TableDescription* >::iterator iDescription = m_descriptions.begin();
        iDescription != m_descriptions.end(); ++iDescription ) {
    delete iDescription->second;
  }
  delete m_log;
}


coral::TableDescription&
pool::ObjectRelationalAccess::TableDescriptionStack::description( const std::string& tableName )
{
  std::map< std::string, coral::TableDescription* >::iterator iDescription = m_descriptions.find( tableName );
  if ( iDescription != m_descriptions.end() ) {
    return *( iDescription->second );
  }
  coral::TableDescription* newDescription = new coral::TableDescription("ObjectRelationalAccess");
  newDescription->setName(tableName);
  m_tableNames.push_back( tableName );
  m_descriptions.insert( std::make_pair( tableName, newDescription ) );
  return *newDescription;
}


const coral::TableDescription&
pool::ObjectRelationalAccess::TableDescriptionStack::description( const std::string& tableName ) const
{
  return *( m_descriptions.find( tableName )->second );
}


pool::ObjectRelationalAccess::TableDescriptionStack::const_iterator
pool::ObjectRelationalAccess::TableDescriptionStack::begin() const
{
  return m_tableNames.begin();
}


pool::ObjectRelationalAccess::TableDescriptionStack::const_iterator
pool::ObjectRelationalAccess::TableDescriptionStack::end() const
{
  return m_tableNames.end();
}
