#include <iostream>
#include <stdexcept>
#include "TestDriver.h"
#include "ObjectRelationalAccess/ObjectRelationalMappingPersistency.h"
#include "ObjectRelationalAccess/ObjectRelationalMapping.h"
#include "ObjectRelationalAccess/ObjectRelationalElementTypeNames.h"

void
TestDriver::removeOldMappings( coral::ISchema& schema )
{
  pool::ObjectRelationalMappingPersistency mappingPersistency( schema );
  std::map<std::string,std::string> storedVersions = mappingPersistency.retrieveMappingVersions();
  for ( std::map<std::string,std::string>::const_iterator iVersion = storedVersions.begin();
        iVersion != storedVersions.end(); ++iVersion ) {
    std::cout << "Removing old version : " << iVersion->first << std::endl;
    mappingPersistency.removeMapping( iVersion->first);
    mappingPersistency.removeMaterializedTables( iVersion->first);
  }
}


void
TestDriver::writeMapping( coral::ISchema& schema )
{
  std::vector< std::string > columns;
  const std::string version1 = "Version1";
  pool::ObjectRelationalMapping mapping1( version1 );

  // Defining a class 
  const std::string className1 = "MyClassA";
  const std::string tableName1 = "T_A";
  pool::ObjectRelationalMappingElement& class1 = mapping1.appendElement( className1, tableName1 );
  const std::string idColumn1 = "ID";
  columns.push_back( idColumn1 );
  class1.setColumnNames( columns );
  columns.clear();

  // Defining an integer data member
  const std::string variableName1_a = "a";
  const std::string variableType1_a = "int";
  const std::string variableColumn1_a = "A";
  pool::ObjectRelationalMappingElement& variable1_a = class1.appendSubElement( pool::ObjectRelationalElementTypeNames::primitiveMappingElementType(),
                                                                               variableName1_a, variableType1_a, tableName1 );
  columns.push_back( variableColumn1_a );
  variable1_a.setColumnNames( columns );
  columns.clear();

  // Defining a float data member
  const std::string variableName1_b = "b";
  const std::string variableType1_b = "float";
  const std::string variableColumn1_b = "B";
  pool::ObjectRelationalMappingElement& variable1_b = class1.appendSubElement( pool::ObjectRelationalElementTypeNames::primitiveMappingElementType(),
                                                                               variableName1_b, variableType1_b, tableName1 );
  columns.push_back( variableColumn1_b );
  variable1_b.setColumnNames( columns );
  columns.clear();

  // Defining a vector<float> as a data member.
  const std::string variableName1_c = "c";
  const std::string variableType1_c = "std::vector<float>";
  const std::string tableName1_c = "T_A_C";
  const std::string idColumn1_c = "ID";
  const std::string posColumn1_c = "POS";
  pool::ObjectRelationalMappingElement& variable1_c = class1.appendSubElement( pool::ObjectRelationalElementTypeNames::arrayMappingElementType(),
                                                                               variableName1_c, variableType1_c, tableName1_c );
  columns.push_back( posColumn1_c );
  columns.push_back( idColumn1_c );
  variable1_c.setColumnNames( columns );
  columns.clear();

  const std::string variableTypeElement1_c = "float";
  const std::string variableColumnElement1_c = "VAL_C";
  pool::ObjectRelationalMappingElement& element1_c = variable1_c.appendSubElement( pool::ObjectRelationalElementTypeNames::primitiveMappingElementType(),
                                                                                   variableTypeElement1_c, variableTypeElement1_c, tableName1_c );
  columns.push_back( variableColumnElement1_c );
  element1_c.setColumnNames( columns );
  columns.clear();


  // Defining a vector<MyClassB{int a, vector<double> b}> as a data member.
  const std::string variableName1_d = "d";
  const std::string variableType1_d = "std::vector<MyClassB>";
  const std::string tableName1_d = "T_A_D";
  const std::string idColumn1_d = "ID";
  const std::string posColumn1_d = "POS";
  pool::ObjectRelationalMappingElement& variable1_d = class1.appendSubElement( pool::ObjectRelationalElementTypeNames::arrayMappingElementType(),
                                                                               variableName1_d, variableType1_d, tableName1_d );
  columns.push_back( posColumn1_d );
  columns.push_back( idColumn1_d );
  variable1_d.setColumnNames( columns );
  columns.clear();

  const std::string variableTypeElement1_d = "MyClassB";
  pool::ObjectRelationalMappingElement& myClassB = variable1_d.appendSubElement( pool::ObjectRelationalElementTypeNames::objectMappingElementType(),
                                                                                   variableTypeElement1_d, variableTypeElement1_d, tableName1_d );
  columns.push_back( posColumn1_d );
  columns.push_back( idColumn1_d );
  myClassB.setColumnNames( columns );
  columns.clear();

  const std::string variableName_MyClassB_a = "a";
  const std::string variableType_MyClassB_a = "int";
  const std::string variableColumn_MyClassB_a = "A";
  pool::ObjectRelationalMappingElement& myClassB_a = myClassB.appendSubElement( pool::ObjectRelationalElementTypeNames::primitiveMappingElementType(),
                                                                                variableName_MyClassB_a, variableType_MyClassB_a, tableName1_d );
  columns.push_back( variableColumn_MyClassB_a );
  myClassB_a.setColumnNames( columns );
  columns.clear();

  const std::string variableName_MyClassB_b = "b";
  const std::string variableType_MyClassB_b = "std::vector<double>";
  const std::string tableName_MyClassB_b = "T_A_D_B";
  const std::string id1Column_MyClassB_b = "ID_ID";
  const std::string id2Column_MyClassB_b = "ID_POS";
  const std::string posColumn_MyClassB_b = "POS";
  pool::ObjectRelationalMappingElement& myClassB_b = myClassB.appendSubElement( pool::ObjectRelationalElementTypeNames::arrayMappingElementType(),
                                                                                variableName_MyClassB_b, variableType_MyClassB_b, tableName_MyClassB_b );
  columns.push_back( posColumn_MyClassB_b );
  columns.push_back( id1Column_MyClassB_b );
  columns.push_back( id2Column_MyClassB_b );
  myClassB_b.setColumnNames( columns );
  columns.clear();

  const std::string variableTypeElement_MyClassB_b = "double";
  const std::string variableColumnElement_MyClassB_b = "VAL_B";
  pool::ObjectRelationalMappingElement& element_MyClassB_b = myClassB_b.appendSubElement( pool::ObjectRelationalElementTypeNames::primitiveMappingElementType(),
                                                                                          variableTypeElement_MyClassB_b, variableTypeElement_MyClassB_b, tableName_MyClassB_b );
  columns.push_back( variableColumnElement_MyClassB_b );
  element_MyClassB_b.setColumnNames( columns );
  columns.clear();

  // Alter a table.
  class1.alterTableName( "T_AA" ) ;

  // Store the first mapping
  std::cout << "Storing the mapping of version \"" << version1 << "\"" << std::endl;
  pool::ObjectRelationalMappingPersistency mappingPersistency( schema );
  if ( ! mappingPersistency.storeMapping( mapping1, className1+"_DEFAULT" ) ) {
    throw std::runtime_error( "Could not store mapping version1" );
  }

  // Store a second mapping.
  const std::string version2 = "Version2";
  pool::ObjectRelationalMapping mapping2( version2 );
  pool::ObjectRelationalMappingElement& class2 = mapping2.appendElement( className1, tableName1 );
  columns.push_back( idColumn1 );
  class2.setColumnNames( columns );
  columns.clear();

  pool::ObjectRelationalMappingElement& variable2_a = class1.appendSubElement( pool::ObjectRelationalElementTypeNames::primitiveMappingElementType(),
                                                                               variableName1_a, variableType1_a, tableName1 );
  columns.push_back( variableColumn1_a );
  variable2_a.setColumnNames( columns );
  columns.clear();
  std::cout << "Storing the mapping of version \"" << version2 << "\"" << std::endl;
  if ( ! mappingPersistency.storeMapping( mapping2, className1+"_DEFAULT" ) ) {
    throw std::runtime_error( "Could not store mapping version2" );
  }

}


void
TestDriver::readMapping( coral::ISchema& schema )
{
  const std::string version1 = "Version1";
  std::cout << "Reading the mapping of version \"" << version1 << "\"" << std::endl;
  pool::ObjectRelationalMappingPersistency mappingPersistency( schema );
  const pool::ObjectRelationalMapping& mapping = mappingPersistency.retrieveMapping( version1 );
  for ( pool::ObjectRelationalMapping::const_iterator i = mapping.begin();
        i != mapping.end(); ++i ) {
    this->printElement( i->second, 0 );
  }
}


void
TestDriver::printElement( const pool::ObjectRelationalMappingElement& element,
                          int level )
{
  std::cout << " ";
  for ( int l = 0; l < level; ++l ) std::cout << "  ";
  if ( ! element.scopeName().empty() ) std::cout << element.scopeName() << "::";
  std::cout << element.variableName()
            << " ( " << pool::ObjectRelationalMappingElement::elementTypeAsString( element.elementType() ) << " : " << element.variableType()
            << " ) -> " << element.tableName() << " ( ";
  for ( std::vector< std::string >::const_iterator c = element.columnNames().begin();
        c != element.columnNames().end(); ++c ) {
    if ( c != element.columnNames().begin() ) std::cout << ", ";
    std::cout << *c;
  }
  std::cout << " )" << std::endl;

  for ( pool::ObjectRelationalMappingElement::const_iterator i = element.begin();
        i != element.end(); ++i ) {
    this->printElement( i->second, level + 1 );
  }
}
