#include <iostream>
#include <stdexcept>
#include <memory>
#include <cstdlib>

#include "PluginManager/PluginManager.h"
#include "SealKernel/Context.h"
#include "SealKernel/ComponentLoader.h"

#include "POOLCore/POOLContext.h"
#include "RelationalAccess/IConnectionService.h"
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ITransaction.h"
#include "ObjectRelationalAccess/ObjectRelationalMappingPersistency.h"
#include "TestDriver.h"

int main( int, char** )
{
  try {
    seal::PluginManager* pm = seal::PluginManager::get();
    pm->initialise();

    pool::POOLContext::loadComponent( "SEAL/Services/MessageService" );
    pool::POOLContext::setMessageVerbosityLevel( seal::Msg::Info );
    pool::POOLContext::loadComponent( "CORAL/Services/XMLAuthenticationService");
    pool::POOLContext::loadComponent( "CORAL/Services/ConnectionService");

    //const std::string userNameEnv = "POOL_AUTH_USER=ioannis";
    const std::string userNameEnv = "CORAL_AUTH_USER=giacomo";
    ::putenv( const_cast<char*>( userNameEnv.c_str() ) );
    //const std::string passwdEnv = "POOL_AUTH_PASSWORD=ioannis";
    const std::string passwdEnv = "CORAL_AUTH_PASSWORD=govi10gdb";
    ::putenv( const_cast<char*>( passwdEnv.c_str() ) );

    seal::IHandle<coral::IConnectionService> serviceHandle =
      pool::POOLContext::context()->query<coral::IConnectionService>( "CORAL/Services/ConnectionService" );

    if ( ! serviceHandle )
    {
      throw std::runtime_error( "Could not retrieve the connection service" );
    }

    //std::string connectionString = "sqlite_file:mappingPersistency.db";
    std::string connectionString = "oracle://devdb10/giacomo";

    std::cout << "Connecting..." << std::endl;
    coral::ISessionProxy* session = serviceHandle->connect( connectionString );

    TestDriver driver;
    
    // removing old mappings
    session->transaction().start(); 
    driver.removeOldMappings( session->nominalSchema() );
    session->transaction().commit();

    // writing the mappings
    session->transaction().start();
    driver.writeMapping( session->nominalSchema() );
    session->transaction().commit();

    // reading back a mapping
    session->transaction().start();
    driver.readMapping( session->nominalSchema() );
    session->transaction().commit();

    std::cout << "Disconnecting..." << std::endl;
    delete session;
    std::cout << "Exiting..." << std::endl;
  }
  catch ( std::exception& e ) {
    std::cerr << "Standard C++ exception : " << e.what() << std::endl;
    return 1;
  }
  catch ( ... ) {
    std::cerr << "Unknown exception ... " << std::endl;
    return 1;
  }
  return 0;
}
