#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ITransaction.h"
#include "POOLCore/CommandOptions.h"
#include "ObjectRelationalAccess/ObjectRelationalMappingUtilities.h"
#include <exception>

int main (int argc, char** argv)
{
  std::vector<pool::CommandOptions::Option> ops;
  pool::CommandOptions::Option opt;
  opt.flag = "-v";
  opt.property = "version";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the version label of the mapping to find";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-l";
  opt.property = "list";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "the versions list of the mappings";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-cv";
  opt.property = "classVersions";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "the mappings for the different class versions";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-tm";
  opt.property = "tableMap";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "the containers associated to the existing tables";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-c";
  opt.property = "connectionString";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the connection string";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-u";
  opt.property = "user";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user name";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-p";
  opt.property = "password";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user password";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-dump";
  opt.property = "dump";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "dump the mapping";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-debug";
  opt.property = "debug";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the debug verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-info";
  opt.property = "info";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the info verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-h";
  opt.property = "help";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "display the help";
  opt.required = false;
  ops.push_back(opt);
  pool::CommandOptions cmd(ops);
  try{
    cmd.parse(argc,argv);
    const std::map<std::string,std::string>& ops = cmd.userOptions();
    std::map<std::string,std::string>::const_iterator iter = ops.find("help");
    if(iter!=ops.end() || ops.size()==0){
      cmd.help();
    } else {
      pool::ObjectRelationalMappingUtilities utility;
      iter = ops.find("debug");
      if(iter!=ops.end()){
        utility.setProperty(pool::ObjectRelationalMappingUtilities::verbosityProperty(),
                            pool::ObjectRelationalMappingUtilities::debugVerbosityPropertyValue());
      } else {
        iter = ops.find("info");
        if(iter!=ops.end()){
          utility.setProperty(pool::ObjectRelationalMappingUtilities::verbosityProperty(),
                              pool::ObjectRelationalMappingUtilities::infoVerbosityPropertyValue());
        }
      }
      std::string theVersion("");
      iter = ops.find("version");
      if(iter!=ops.end()){
        theVersion = iter->second;
      }
      bool classVersions = false;
      iter = ops.find("classVersions");
      if(iter!=ops.end()){
        classVersions = true;
      } 
      bool tableMap = false;
      iter = ops.find("tableMap");
      if(iter!=ops.end()){
        tableMap = true;
      }
      bool versionList = true;
      iter = ops.find("list");
      if(iter==ops.end() && (classVersions|tableMap)){
        versionList = false;
      }
      bool dump = false;
      iter = ops.find("dump");
      if(iter!=ops.end()){
        dump = true;
        if(theVersion.empty()){
          std::cout << "   Required parameter \"version\" (mapping version) has not been specified." << std::endl;
          return 1;
        }
      }
      std::string theConnectionString("");
      iter = ops.find("connectionString");
      if(iter!=ops.end()){
        theConnectionString = iter->second;
      } 
      iter = ops.find("user");
      if(iter!=ops.end()){
        std::string theUserName = iter->second;
        utility.setProperty(pool::ObjectRelationalMappingUtilities::userNameProperty(),theUserName);
      } 
      iter = ops.find("password");
      if(iter!=ops.end()){
        std::string thePassword = (*iter).second;
        utility.setProperty(pool::ObjectRelationalMappingUtilities::passwordProperty(),thePassword);
      }
      utility.initialize();
      utility.connect(theConnectionString,coral::ReadOnly);
      utility.session()->transaction().start(true);
      try {
        if(!theVersion.empty()){
          if(!utility.existsMapping(theVersion)){
              std::cout << "   Mapping version " << theVersion << " not found in the database." << std::endl;
              return 1;
          }
        }
        if(versionList){
          std::map<std::string,std::string> vlist = utility.listMappings();
          if(!theVersion.empty()){
            std::map<std::string,std::string>::const_iterator iM = vlist.find(theVersion);
            if(iM==vlist.end()){
              std::cout << "   Mapping version " << theVersion << " not found in the database." << std::endl;
              return 1;
            } else {
              std::pair<std::string,std::string> entry = *iM;
              vlist.clear();
              vlist.insert(entry);
            }
          }
          pool::ObjectRelationalMappingUtilities::prettyPrint(vlist,pool::ObjectRelationalMappingUtilities::keyOfMappingList(),
                                                              pool::ObjectRelationalMappingUtilities::valueOfMappingList(),std::cout);
          std::cout << std::endl;
        }
        if(classVersions){
          std::map<std::pair<std::string,std::string>,std::string > classVersionList = utility.mappingForClassVersions();
          if(!theVersion.empty()){
            std::map<std::pair<std::string,std::string>,std::string > newList;
            for(std::map<std::pair<std::string,std::string>,std::string >::const_iterator iM = classVersionList.begin();
                iM!=classVersionList.end();++iM){
              if(iM->second == theVersion){
                newList.insert(*iM);
              }
            }
            classVersionList = newList;
          }
          if(classVersionList.size()){
            pool::ObjectRelationalMappingUtilities::prettyPrint(classVersionList,
                                                                pool::ObjectRelationalMappingUtilities::keyFirstOfClassVersionList(),
                                                                pool::ObjectRelationalMappingUtilities::keySecondOfClassVersionList(),
                                                                pool::ObjectRelationalMappingUtilities::valueOfClassVersionList(),
                                                                std::cout);
          }else{
            std::cout << "   No class version matching the selection."<<std::endl;
          }
          std::cout << std::endl;
        }
        if( tableMap ){
          std::map<std::string,std::string> vlist = utility.containerTableMap();
          pool::ObjectRelationalMappingUtilities::prettyPrint(vlist,pool::ObjectRelationalMappingUtilities::keyOfContainerTableMap(),
                                                              pool::ObjectRelationalMappingUtilities::valueOfContainerTableMap(),std::cout);
          std::cout << std::endl;
        }
        if( dump ) {
          std::map<std::pair<std::string,std::string>,std::pair<std::string,std::string> > mappingStruct;
          utility.selectMapping(theVersion,mappingStruct);
          pool::ObjectRelationalMappingUtilities::prettyPrint(mappingStruct,
                                                              std::string("scope"), std::string("variable_name"),
                                                              std::string("table_name"), std::string("column_name"),std::cout);
          
        }
        utility.session()->transaction().commit();
      } catch (const std::exception& e){
        std::cout << "ERROR: " << e.what() << std::endl;
        utility.session()->transaction().rollback();
        return 1;
      }
    }
    return 0;
  } catch (const std::exception& e){
    std::cout << "ERROR: " << e.what() << std::endl;
    return 1;
  }
}

