#include "RelationalAccess/IConnectionService.h"
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/ISchema.h"
#include "POOLCore/POOLContext.h"
#include "SealKernel/Context.h"
#include "../../src/RelationalFileCatalogNames.h"

#include <stdexcept>
#include <memory>

void
resetCatalog( const std::string& databaseConnection )
{
  seal::IHandle<coral::IConnectionService> serviceHandle = pool::POOLContext::context()->query<coral::IConnectionService>( "CORAL/Services/ConnectionService" );
  if ( ! serviceHandle ) {
    pool::POOLContext::loadComponent( "CORAL/Services/ConnectionService" );
    serviceHandle = pool::POOLContext::context()->query<coral::IConnectionService>( "CORAL/Services/ConnectionService" );
  }

  std::auto_ptr< coral::ISessionProxy > session( serviceHandle->connect( databaseConnection, coral::Update ) );

  coral::ITransaction& transaction = session->transaction();
  transaction.start();

  coral::ISchema& nominalSchema = session->nominalSchema();
  nominalSchema.dropIfExistsTable( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() );
  nominalSchema.dropIfExistsTable( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() );
  nominalSchema.dropIfExistsTable( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() );
  nominalSchema.dropIfExistsTable( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataDescriptionTable() );

  transaction.commit();
}
