#include <iostream>
#include <stdexcept>
#include <memory>
#include <cstdlib>

#include "FileCatalog/IFileCatalog.h"
#include "FileCatalog/URIParser.h"
#include "POOLCore/POOLContext.h"
#include "POOLCore/PoolMessageStream.h"

void resetCatalog( const std::string& databaseConnection );

int main( int, char** )
{
  try {
    pool::PoolMessageStream get("get");
    
    pool::POOLContext::loadComponent( "SEAL/Services/MessageService" );
    pool::POOLContext::setMessageVerbosityLevel( seal::Msg::Warning );

    ::putenv( "CORAL_AUTH_USER=ral_writer" );
    ::putenv( "CORAL_AUTH_PASSWORD=ral" );

    const std::string databaseConnection = "oracle://devdb10/ral_writer";

    resetCatalog( databaseConnection );

    pool::IFileCatalog fileCatalog;
    pool::URIParser p( "relationalcatalog_" + databaseConnection );
    p.parse();

    fileCatalog.setWriteCatalog( p.contactstring() );
    fileCatalog.connect();
    fileCatalog.start();

    fileCatalog.commit();
    fileCatalog.disconnect();
  }
  catch ( std::exception& e ) {
    std::cerr << e.what() << std::endl;
    return 1;
  }
  catch ( ... ) {
    std::cerr << "Unknown exception..." << std::endl;
    return 1;
  }
  return 0;
} 
