#include <iostream>
#include <stdexcept>
#include <memory>
#include <cstdlib>

#include "FileCatalog/IFileCatalog.h"
#include "FileCatalog/IFCAction.h"
#include "FileCatalog/URIParser.h"
#include "FileCatalog/FCEntry.h"
#include "FileCatalog/IFCContainer.h"
#include "POOLCore/POOLContext.h"

void resetCatalog( const std::string& databaseConnection );

int main( int, char** )
{
  try {
    pool::POOLContext::loadComponent( "SEAL/Services/MessageService" );
    pool::POOLContext::setMessageVerbosityLevel( seal::Msg::Info );

    ::putenv( "CORAL_AUTH_USER=ral_writer" );
    ::putenv( "CORAL_AUTH_PASSWORD=ral" );

    const std::string databaseConnection = "oracle://devdb10/ral_writer";

    resetCatalog( databaseConnection );

    pool::IFileCatalog fileCatalog;
    pool::URIParser p( "relationalcatalog_" + databaseConnection );
    p.parse();
    fileCatalog.setWriteCatalog( p.contactstring() );
    fileCatalog.connect();
    fileCatalog.start();

    const std::string fileType = "default";
    const std::string fileName1 = "my.first.file";
    const std::string logicalFileName1 = "/my/first/file";
    const std::string fileName2 = "my.second.file";
    const std::string fileName3 = "my.third.file";
    const std::string logicalFileName3 = "/my/third/file";
    const std::string logicalFileName3_1 = "/my/third/file/again";

    pool::FCregister registerAction;
    fileCatalog.setAction( registerAction );

    pool::FileCatalog::FileID fid;
    registerAction.registerPFN( fileName1, fileType, fid );
    registerAction.registerPFN( fileName2, fileType, fid );
    pool::FileCatalog::FileID fid3;
    registerAction.registerPFN( fileName3, fileType, fid3 );
    registerAction.registerLFN( fileName1, logicalFileName1 );
    registerAction.registerLFN( fileName3, logicalFileName3 );
    registerAction.registerLFN( fileName3, logicalFileName3_1 );

    pool::FCAdmin adminAction;
    fileCatalog.setAction( adminAction );
    pool::MetaDataEntry metaEntry;
    metaEntry.addAttributeSpec( "x", "double" );
    metaEntry.addAttributeSpec( "y", "float" );
    metaEntry.addAttributeSpec( "id", "int" );
    adminAction.createMetaDataSpec( metaEntry );

    fileCatalog.setAction( registerAction );
    metaEntry.setAttributeValue<double>( "x", 2.22 );
    metaEntry.setAttributeValue<float>( "y", 2.2 );
    metaEntry.setAttributeValue<int>( "id", 2 );
    registerAction.registerMetaData( fid, metaEntry );

    metaEntry.setAttributeValue<double>( "x", 3.33 );
    metaEntry.setAttributeValue<float>( "y", 3.3 );
    metaEntry.setAttributeValue<int>( "id", 3 );
    registerAction.registerMetaData( fid3, metaEntry );

    fileCatalog.commit();
    fileCatalog.disconnect();

    // Starting a new session.
    fileCatalog.connect();
    fileCatalog.start();

    pool::FClookup lookup;
    fileCatalog.setAction( lookup );

    pool::LFNContainer buf( &fileCatalog );
    std::string query = "x > 3";
    lookup.lookupLFNByQuery( query, buf );
    int lfn1 = 0;
    int lfn2 = 0;
    while ( buf.hasNext() ) {
      const pool::LFNEntry& entry = buf.Next();
      std::string lfn = entry.lfname();
      if ( lfn == logicalFileName3 ) {
        lfn1++;
      }
      else if ( lfn == logicalFileName3_1 ) {
        lfn2++;
      }
      else {
        throw std::runtime_error( "Retrieved unexpected LFN : \"" + lfn + "\"" );
      }
    }

    if ( lfn1 != 1 || lfn2 != 1 ) {
      throw std::runtime_error( "Failed query" );
    }
  
    fileCatalog.commit();
    fileCatalog.disconnect();
  }
  catch ( std::exception& e ) {
    std::cerr << e.what() << std::endl;
    return 1;
  }
  catch ( ... ) {
    std::cerr << "Unknown exception..." << std::endl;
    return 1;
  }
  return 0;
} 
