#include <iostream>
#include <stdexcept>
#include <memory>
#include <cstdlib>

#include "FileCatalog/IFileCatalog.h"
#include "FileCatalog/IFCAction.h"
#include "FileCatalog/URIParser.h"
#include "POOLCore/POOLContext.h"

void resetCatalog( const std::string& databaseConnection );

int main( int, char** )
{
  try {
    pool::POOLContext::loadComponent( "SEAL/Services/MessageService" );
    pool::POOLContext::setMessageVerbosityLevel( seal::Msg::Info );

    ::putenv( "CORAL_AUTH_USER=ral_writer" );
    ::putenv( "CORAL_AUTH_PASSWORD=ral" );

    const std::string databaseConnection = "oracle://devdb10/ral_writer";

    resetCatalog( databaseConnection );

    pool::IFileCatalog fileCatalog;
    pool::URIParser p( "relationalcatalog_" + databaseConnection );
    p.parse();

    fileCatalog.setWriteCatalog( p.contactstring() );
    fileCatalog.connect();
    fileCatalog.start();

    const std::string fileType = "default";
    const std::string fileName1 = "my.first.file";
    const std::string logicalFileName1 = "/my/first/file";
    const std::string fileName2 = "my.second.file";
    const std::string fileName3 = "my.third.file";
    const std::string logicalFileName3 = "/my/third/file";

    pool::FCregister registerAction;
    fileCatalog.setAction( registerAction );

    std::string fid;
    registerAction.registerPFN( fileName1, fileType, fid );
    registerAction.registerPFN( fileName2, fileType, fid );
    std::string fid3;
    registerAction.registerPFN( fileName3, fileType, fid3 );
    registerAction.registerLFN( fileName1, logicalFileName1 );
    registerAction.registerLFN( fileName3, logicalFileName3 );

    fileCatalog.commit();
    fileCatalog.disconnect();

    // Starting a new session.
    fileCatalog.connect();
    fileCatalog.start();

    pool::IFCAction action;
    fileCatalog.setAction( action );
    fid = "";
    action.lookupFileByLFN( logicalFileName3, fid );
    if ( fid != fid3 ) {
      throw std::runtime_error( "Failed the query" );
    }
    std::string pfn = "";
    std::string type = "";
    action.lookupBestPFN( fid, pool::FileCatalog::READ, pool::FileCatalog::RANDOM, pfn, type );
    if ( pfn != fileName3 || type != fileType ) {
      throw std::runtime_error( "Failed the query" );
    }
    fid3 = "";
    action.lookupFileByPFN( fileName3, fid3, type);
    if ( fid != fid3 || type != fileType ) {
      throw std::runtime_error( "Failed the query" );
    }

    fileCatalog.commit();
    fileCatalog.disconnect();


  }
  catch ( std::exception& e ) {
    std::cerr << e.what() << std::endl;
    return 1;
  }
  catch ( ... ) {
    std::cerr << "Unknown exception..." << std::endl;
    return 1;
  }
  return 0;
} 
