#ifndef POOL_IBLOBSTREAMINGSERVICE_H
#define POOL_IBLOBSTREAMINGSERVICE_H

#include <string>

namespace coral {
  class Blob;
}

namespace ROOT {
  namespace Reflex{
    class Type;
  }
}

namespace pool {

  /// Interface for streaming objects into Blobs
  class IBlobWriter
  {
  public:
    /// Empty destructor
    virtual ~IBlobWriter() {}

    /// Streams an object an returns by constant reference the underlying blob
    virtual const coral::Blob& write( const void* addressOfInputData ) = 0;
  };

  /// Interface for constructing objects from Blobs
  class IBlobReader
  {
  public:
    /// Empty destructor
    virtual ~IBlobReader() {}

    /// Reads an object from a Blob and fills-in the container
    virtual void read( const coral::Blob& blobData,
                       void* addressOfContainer ) const = 0;
  };

  /// Interface for a Streaming Service. Any implementation should be a seal::Service.
  class IBlobStreamingService
  {
  public:
    /// Empty destructor
    virtual ~IBlobStreamingService() {}

    /// Returns a new streamer for writing into a BLOB
    virtual IBlobWriter* newWriter( const ROOT::Reflex::Type& classDictionary,
                                    const std::string& version ) const = 0;

    /// Returns a new streamer for reading from a BLOB
    virtual IBlobReader* newReader( const ROOT::Reflex::Type& classDictionary,
                                    const std::string& version ) const = 0;
  };

}

#endif
