#ifndef RELATIONALSTORAGESERVICE_RELATIONALPOOLDATABASEUTILITIES_H
#define RELATIONALSTORAGESERVICE_RELATIONALPOOLDATABASEUTILITIES_H

#include "ObjectRelationalAccess/ObjectRelationalUtilities.h"
#include <map>

namespace pool {
  
  class IStorageSvc;
  class DbDatabase;

  class PoolDatabaseManager;
  class ObjectRelationalDatabaseDescriptor;

  class RelationalPoolDatabaseUtilities : public pool::ObjectRelationalUtilities {

    public:
    RelationalPoolDatabaseUtilities();

    virtual ~RelationalPoolDatabaseUtilities();

    void setUpDatabase(const std::string& driverFile, const std::string& outFile, bool evolve, bool dry=false);

    void updateDatabase(bool dry=false);

    void updateContainer(const std::string& contName,bool dry=false);

    void dropContainer(const std::string& contName,bool full=false);

    void init();

    class ContainerInfo {
       public:
 
      ContainerInfo():name(""),containerClassName(""),technology(""),mappingVersion(""),className(""),size(0),classes(){}
      virtual ~ContainerInfo(){}
      ContainerInfo(const ContainerInfo& cp):name(cp.name),containerClassName(cp.containerClassName),technology(cp.technology),mappingVersion(cp.mappingVersion),className(cp.className),size(cp.size),classes(cp.classes){}
      ContainerInfo& operator=(const ContainerInfo& cp){
        name = cp.name;
        containerClassName = cp.containerClassName;
        technology = cp.technology;
        mappingVersion= cp.mappingVersion;
        className = cp.className;
        size = cp.size;
        classes = cp.classes;
        return *this;
      }
      std::string name;
      std::string containerClassName;
      std::string technology;
      std::string mappingVersion;
      std::string className;
      unsigned int size;
      std::map<std::string, unsigned int> classes;
    
    };

    class DatabaseInfo {
       public:
 
      DatabaseInfo():name(""),connectionString(""),containers(){}
      virtual ~DatabaseInfo(){}
      DatabaseInfo(const DatabaseInfo& cp):name(cp.name),
                                           connectionString(cp.connectionString),
                                           containers(cp.containers){}
      DatabaseInfo& operator=(const DatabaseInfo& cp){
        name = cp.name;
        connectionString = cp.connectionString;
        containers = cp.containers;
        return *this;
      }
      std::string name;
      std::string connectionString;
      std::map<std::string,ContainerInfo> containers;
    
    };

    DatabaseInfo queryDatabase();

    void updatePoolSchema();
    
    private:

    void setUpDatabase(const pool::ObjectRelationalDatabaseDescriptor& dbstruct);

    void updateContainer(pool::DbDatabase& db, const ContainerInfo& info, bool dry);

    std::string m_connectionString;
  };

}
#endif
