#include "BaseContainerIteratorHandler.h"
#include "RelationalStorageServiceException.h"
#include "Reflex/Object.h"

pool::RelationalStorageService::BaseContainerIteratorHandler::BaseContainerIteratorHandler( const ROOT::Reflex::Environ<long>& collEnv,
                                                                                            ROOT::Reflex::CollFuncTable* collProxy,
                                                                                            const ROOT::Reflex::Type& iteratorReturnType ):
  m_returnType(iteratorReturnType),
  m_collEnv(collEnv),
  m_collProxy(collProxy),
  m_currentElement(0){

  // retrieve the first element
  if(m_collProxy){    
    m_currentElement = m_collProxy->first_func(&m_collEnv);
  }
}

pool::RelationalStorageService::BaseContainerIteratorHandler::~BaseContainerIteratorHandler(){}

void
pool::RelationalStorageService::BaseContainerIteratorHandler::increment()
{
  if(m_collProxy){
    // this is required! It sets the number of memory slots (of size sizeof(Class)) to be used for the step 
    m_collEnv.idx = 1;
    m_currentElement = m_collProxy->next_func(&m_collEnv);
  }
}


void*
pool::RelationalStorageService::BaseContainerIteratorHandler::object()
{
  return m_currentElement;
}


ROOT::Reflex::Type
pool::RelationalStorageService::BaseContainerIteratorHandler::returnType() const
{
  return m_returnType;
}
