#ifndef BASECONTAINER_ITERATOR_HANDLER_H
#define BASECONTAINER_ITERATOR_HANDLER_H

#include "Reflex/Type.h"
#include "Reflex/Member.h"
#include "Reflex/Builder/CollectionProxy.h"
#include "IContainerIteratorHandler.h"

namespace pool {

  namespace RelationalStorageService {

    class BaseContainerIteratorHandler : virtual public IContainerIteratorHandler {
    public:
      /// Constructor
      BaseContainerIteratorHandler( const ROOT::Reflex::Environ<long>& collEnv,
                                    ROOT::Reflex::CollFuncTable* collProxy,
                                    const ROOT::Reflex::Type& iteratorReturnType );

      /// Destructor
      ~BaseContainerIteratorHandler();

      /// Increments itself
      void increment();

      /// Returns the current object
      void* object();

      /// Returns the return type of the iterator dereference method
      ROOT::Reflex::Type returnType() const;

    private:

      /// The return type of the iterator dereference method
      ROOT::Reflex::Type m_returnType;
      
      /// Structure containing parameters of the collection instance  
      ROOT::Reflex::Environ<long> m_collEnv;

      /// Proxy of the generic collection
      ROOT::Reflex::CollFuncTable* m_collProxy;

      /// Current element object pointer
      void* m_currentElement;
    };

  }

}

#endif
