#ifndef CARRAY_HANDLER_H
#define CARRAY_HANDLER_H

#include "IContainerHandler.h"
#include "Reflex/Reflex.h"

namespace pool {

  namespace RelationalStorageService {

    class CArrayHandler : virtual public IContainerHandler {

    public:
      /// Constructor
      explicit CArrayHandler( const ROOT::Reflex::Type& dictionary );

      /// Destructor
      ~CArrayHandler();

      /// Returns the size of the container
      size_t size( const void* address );

      /// Returns an initialized iterator
      IContainerIteratorHandler* iterate( const void* address );

      /// Appends a new element and returns its address of the object reference
      void appendNewElement( void* address, void* data );

      /// Clear the content of the container
      void clear( const void* address );
      
      /// Returns the iterator return type
      ROOT::Reflex::Type iteratorReturnType() const;

      /// Returns the associativeness of the container
      bool isAssociative() const { return false; }
      
    private:
      /// The dictionary information
      ROOT::Reflex::Type m_type;

      /// The iterator return type
      ROOT::Reflex::Type m_elementType;
    };

  }

}

#endif
