#ifndef RELATIONALSTORAGESERVICE_EXISTINGDATASCANNER_H
#define RELATIONALSTORAGESERVICE_EXISTINGDATASCANNER_H

#include <string>
#include <vector>
#include <map>

namespace coral {

  class ISchema;
  class IQuery;
  class ICursor;
  
}

namespace pool {

  class ObjectRelationalMappingElement;

  namespace RelationalStorageService {

    class ExistingDataScanner {
    public:

      class Entry {
        public:

        Entry():className(""),mappingVersion(""),targetOid(0){}

	      std::string className;

        std::string mappingVersion;

        int targetOid;
      };

      /// Constructor
      ExistingDataScanner( coral::ISchema& schema );

      /// Destructor
      ~ExistingDataScanner();

      void prepareScan( const ObjectRelationalMappingElement& mappingElement, std::string version = "" );

      void prepareScanForNewRows( const std::string& containerTable, const ObjectRelationalMappingElement& mappingElement, std::string version = "" );

      void setChunkSize( unsigned int chunkSize ); 

      bool scan();

      bool scanAll();
 
      const std::vector<Entry>& entries() const;

      unsigned long writtenObjects() const;
      
      unsigned long deletedObjects() const;

      unsigned long storedObjects() const;

      unsigned long storedObjects(const std::string& tableName) const;

      const std::map<std::string,std::pair<unsigned long,int> >& tableMap() const;

      bool isChunkFull() const;

      void resetChunk();

      void reset();

      unsigned int chunkSize() const;

    private:

      coral::ISchema& m_schema;

      coral::IQuery* m_query;      

      coral::ICursor* m_resultSet;      

      std::string m_mappingVersion;

      std::string m_className;

      std::string m_tableName;

      unsigned int m_maxChunkSize;

      unsigned int m_chunkSize;

      std::vector<Entry> m_entries;

      unsigned long m_writtenObjects;

      unsigned long m_deletedObjects;

      unsigned long m_totalRows;

      std::map<std::string,std::pair<unsigned long,int> > m_tableMap;

    };

  }

}

#endif
