#ifndef RELATIONALSTORAGESERVICE_MAPPING_MANAGER_H
#define RELATIONALSTORAGESERVICE_MAPPING_MANAGER_H

#include "Reflex/Type.h"
#include <string>
#include <map>

namespace seal {
  class Context;
}

namespace coral {

  class ISchema;
  
}

namespace pool {
  
  class ObjectRelationalMappingPersistency;

  namespace RelationalStorageService {

    class MappingInformation;

    /**
       @class MappingManager MappingManager.h
       Utility class to manage the object-relational mappings for the C++ classes.
    */

    class MappingManager {
    public:
      /// Constructor
      MappingManager( coral::ISchema& schema,
                      seal::Context* serviceContext,
                      const std::string& serviceName );

      /// Destructor
      ~MappingManager();

      /// Retrieves a valid mapping for a given class.
      const MappingInformation& mappingForNewHomogeneousContainer( const std::string& containerName, const ROOT::Reflex::Type& dictionary );

      /// Retrieves a valid mapping for a given class to be used by an homogeneous container
      const MappingInformation& existingMappingForHomogeneousContainer( const std::string& containerName,
                                                                        const std::string& className,
                                                                        const std::string& nominalVersion );

      /// Retrieves a valid mapping for a given class to be used by a polymorphic container.
      const MappingInformation& mappingForPolymorphicContainer( const std::string& containerName, const ROOT::Reflex::Type& dictionary );

      /// Retrieves a valid mapping for a given class to be used by a polymorphic container
      const MappingInformation& existingMappingForPolymorphicContainer( const std::string& containerName,
                                                                        const ROOT::Reflex::Type& dictionary,
                                                                        const std::string& nominalVersion );

      /// Clears the internal state
      void clear();
      public:

      static std::string versionOfClass( const std::string& className);

    private:
      /// Checks if a table is empty
      bool isTableEmpty( const std::string& tableName );

    private:
      /// The schema in use
      coral::ISchema& m_schema;
      /// The service context
      seal::Context* m_serviceContext;
      /// The service name
      std::string m_serviceName;
      /// The mapping persistency service
      pool::ObjectRelationalMappingPersistency* m_persistency;
      /// The map of mapping information objects for homogeneous containers
      std::map< const ROOT::Reflex::Type, std::map< std::string, MappingInformation* > > m_classMappingsForHomogeneousContainers;
      /// The map of mapping information objects for polymorphic containers.
      std::map< const ROOT::Reflex::Type, std::map< std::string, MappingInformation* > > m_classMappingsForPolymorphicContainers;
    };

  }

}

#endif
