#ifndef POOL_RELATIONALSTORAGESERVICE_MAPPINGSEQUENCE_H
#define POOL_RELATIONALSTORAGESERVICE_MAPPINGSEQUENCE_H

#include <string>
#include <map>

namespace seal {
  class Context;
}

namespace coral {

  class ISchema;
  class AttributeList;
}

namespace pool {
  
  namespace RelationalStorageService {

    class MappingSequence {
    public:
      /// Constructor
      MappingSequence( coral::ISchema& schema,
                       seal::Context* serviceContext,
                       const std::string& serviceName );

      /// Destructor
      ~MappingSequence();

      /// Increments and returns a new valid oid for a table
      int incrementOid( const std::string& tableName );

      /// Updates the last used Oid
      void updateOid( const std::string& tableName, int lastOid );

      /// Clears the internal state
      void clear();

    private:
      /// Creates the table holding the sequences
      void createTheSequencesTable();

      /// Locks an entry in the table
      bool lockEntry( const std::string& sequenceName,
                      int& lastOid );

    private:
      /// The schema in use
      coral::ISchema& m_schema;

      /// The parent service context
      seal::Context* m_serviceContext;

      /// The parent service name
      std::string m_serviceName;

      /// Map of oids used.
      std::map< std::string, int > m_tableToOid;

      /// Flag indicating whether the sequence table exists
      bool m_sequenceTableExists;

      /// The where clause pinning a sequence entry
      std::string m_whereClause;

      /// The data for the where clause
      coral::AttributeList* m_whereData;

      /// The set clause for updating a sequence entry
      std::string m_setClause;
    };

  }

}

#endif
