#ifndef RELATIONALSTORAGESERVICE_OBJECTREADER_H
#define RELATIONALSTORAGESERVICE_OBJECTREADER_H

#include "Reflex/Type.h"
#include <vector>

namespace coral {

  class ISchema;
}

namespace pool {

  class ObjectRelationalMappingElement;
  class DataCallBack;
  class DbDatabase;

  namespace RelationalStorageService {

    class ObjectRelationalReadDataBuffer;

    class ObjectReader {
    public:
      /// Constructor
      ObjectReader( coral::ISchema& schema,
                    const pool::ObjectRelationalMappingElement& mapping,
                    pool::DbDatabase* databaseHandle );

      ObjectReader( coral::ISchema& schema,
                    const ROOT::Reflex::Type& dictionary,
                    const pool::ObjectRelationalMappingElement& mapping,
                    pool::DbDatabase* databaseHandle );

      /// Destructor
      ~ObjectReader();

      /// Reads an object
      bool readObject( void* objectData,
                       int oid,
                       bool anyObject,
                       pool::DataCallBack& callback );

    private:
      /// Reads an array
      bool readArray( const std::vector<int>& identity,
                      ObjectRelationalReadDataBuffer& buffer,
                      void* address,
                      pool::DataCallBack& callback );

    private:
      /// The map of mapping elements to element entries
      ObjectRelationalReadDataBuffer* m_dataBuffer;
      /// The dictionary for a pool reference
      const ROOT::Reflex::Type m_refDict;
      /// The parent database handle
      pool::DbDatabase* m_databaseContext;
    };

  }

}

#endif
