#ifndef OBJECTRELATIONALREADDATABUFFER_H
#define OBJECTRELATIONALREADDATABUFFER_H

#include <vector>
#include <map>
#include <string>
#include "Reflex/Type.h"

namespace coral {

  class ITable;
  class ISchema;
  class AttributeList;
  class AttributeListSpecification;
  class Blob;
}

namespace pool {
  class ObjectRelationalMappingElement;
  class IBlobReader;

  namespace RelationalStorageService {

    class IContainerHandler;

    class ObjectRelationalReadDataBuffer {
    public:
      /// Constuctor
      ObjectRelationalReadDataBuffer();

      /// Destructor
      ~ObjectRelationalReadDataBuffer();

      /// Builds the structure given a mapping element and the dictionary information
      bool build( coral::ISchema& schema,
                  const pool::ObjectRelationalMappingElement& mapping,
                  const ROOT::Reflex::Type& dictionary );

      /// Returns the data specification
      const coral::AttributeListSpecification& dataSpecification() const;

      /// Returns the offset vector
      const std::vector< int >& offsets() const;

      /// Returns the number of items belonging to the key (in case of an associative container)
      int keyItems() const;

      /// Returns the offset w.r.t. the parent object
      int offsetToParent() const;

      /// Returns the associoted table handle
      coral::ITable& table();

      /// Returns the associated dictionary information
      ROOT::Reflex::Type dictionary() const;

      /// Returns the relevant mapping element
      const pool::ObjectRelationalMappingElement& mapping() const;

      /// Returns the container handler
      IContainerHandler& containerHandler();

      /// Returns a flag indicating whether the starting address is the key of an associative container
      bool isKeyData() const;

      /// Returns the where clause for pinning the id
      std::string whereClauseForId() const;

      /// Returns the data for the where clause.
      coral::AttributeList& whereDataForId();

      /// Returns the original version of the specified data, using the blob reader registered in the pool context 
      void readBlob(int offset, const coral::Blob& data, void* startAddress);

      public:
      typedef std::vector< ObjectRelationalReadDataBuffer* >::iterator iterator;
      iterator begin();
      iterator end();

    private:
      /// builds a buffer entry
      int buildBuffer( const pool::ObjectRelationalMappingElement& mapping,
                       const ROOT::Reflex::Type& dictionary,
                       coral::ISchema& schema,
                       int offset,
                       bool keyData );

      /// builds an array buffer
      bool buildArrayBuffer( const pool::ObjectRelationalMappingElement& mapping,
                             const ROOT::Reflex::Type& dictionary,
                             coral::ISchema& schema,
                             int offset,
                             bool keyData );

      /// builds an array buffer
      bool buildCArrayBuffer( const pool::ObjectRelationalMappingElement& mapping,
                              const ROOT::Reflex::Type& dictionary,
                              coral::ISchema& schema,
                              int offset,
                              bool keyData );
    private:
      coral::AttributeListSpecification* m_spec;
      std::vector< int > m_offsets;
      coral::ITable* m_table;
      ROOT::Reflex::Type m_dictionary;
      int m_offsetToParent;
      int m_keyItems;
      const pool::ObjectRelationalMappingElement* m_mapping;
      bool m_keyData;
      IContainerHandler* m_containerHandler;
      std::vector< ObjectRelationalReadDataBuffer* > m_dependentBuffers;
      coral::AttributeList* m_whereData;
      std::string m_whereClause;
      std::map< int, pool::IBlobReader* > m_blobReaders;
    };

  }

}

#endif
