#ifndef RELATIONALSTORAGESERVICE_OBJECTWRITER_H
#define RELATIONALSTORAGESERVICE_OBJECTWRITER_H

#include "Reflex/Type.h"
#include <vector>

namespace coral {
  
  class ISchema;
}

namespace pool {

  class ObjectRelationalMappingElement;
  class DataCallBack;
  class DbDatabase;

  namespace RelationalStorageService {

    class ObjectRelationalDataBuffer;

    class ObjectWriter {
    public:
      /// Constructor
      ObjectWriter( coral::ISchema& schema,
                    const ROOT::Reflex::Type& dictionary,
                    const pool::ObjectRelationalMappingElement& mapping,
                    pool::DbDatabase* databaseHandle );

      /// Destructor
      ~ObjectWriter();

      /// Writes an object
      bool write( int oid,
                  const void* data,
                  pool::DataCallBack& callback );

    private:

      /// Writes an array
      bool writeArray( const std::vector<int>& identity,
                       ObjectRelationalDataBuffer& buffer,
                       const void* address,
                       pool::DataCallBack& callback );
      
    private:
      /// The map of mapping elements to element entries
      ObjectRelationalDataBuffer* m_dataBuffer;
      /// The mapping of the top level class
      const pool::ObjectRelationalMappingElement& m_mapping;
      /// The dictionary for a pool reference
      const ROOT::Reflex::Type m_refDict;
      /// The parent database handle
      pool::DbDatabase* m_databaseContext;
    };

  }

}

#endif
