#ifndef RELATIONALSTORAGESERVICE_H
#define RELATIONALSTORAGESERVICE_H

#include <string>
#include <map>

namespace seal {
  class Context;
}

namespace coral {

  class ISessionProxy;
}

namespace pool {

  class DbTypeInfo;

  namespace RelationalStorageService {

    class PoolBaseContainerManager;
    class MappingManager;
    class MappingSequence;

    class PoolDatabaseManager {
    public:
      /// Constructor
      PoolDatabaseManager( coral::ISessionProxy* session,
                           seal::Context* context );

      /// Destructor
      ~PoolDatabaseManager();

      /// Checks whether the database exists
      bool existsDatabase();

      /// Drops a database
      void dropDatabase();

      /// Creates (and opens) a database
      void createDatabase();

      /// Creates a new homogeneous container
      PoolBaseContainerManager& createHomogeneousContainer( const std::string& containerName,
                                                            const pool::DbTypeInfo& info );

      /// Creates a new polymporphic container
      PoolBaseContainerManager& createPolymorphicContainer( const std::string& containerName );

      /// Checks for the existence of a container
      bool existsContainer( const std::string& containerName );

      /// Returns the container handle
      PoolBaseContainerManager& containerHandle( const std::string& containerName );

      /// Collects the container names to be frees one by one
      void endOfContainerTransaction( const std::string& containerName );

      /// Frees a container handle
      void freeContainerHandle( const std::string& containerName );

      /// Signals the end of a transaction
      void signalEndOfTransaction();

      /// Re-initializes the connection
      void reopen( coral::ISessionProxy* session );

      /// Drop the container (deleting the involved mapping versions)
      void dropContainer(const std::string& containerName);

      void updateMappingMetadata();

    private:
      /// Refreshes the map of containers
      void refreshContainers();

      /// Increments the number of containers in the database
     unsigned long incrementNumberOfContainers();

    private:
      /// Reference to the current session
      coral::ISessionProxy* m_session;
      /// Pointer to the service conext
      seal::Context* m_serviceContext;
      /// Flag indicating whether the cached list of containers is up-to-date
      bool m_containersUpToDate;
      /// The map of containers
      std::map< std::string, PoolBaseContainerManager* > m_containers;
      /// The mapping manager
      MappingManager* m_mappingManager;
      /// The mapping sequence object
      MappingSequence* m_mappingSequence;
      /// the next available contaier id
      int m_nextContainerId;
    };

  }

}

#endif
