#include "PoolPrimitivesContainerManager.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/ITableDataEditor.h"
#include "RelationalAccess/IQuery.h"
#include "RelationalAccess/ICursor.h"
#include "RelationalStorageServiceNames.h"
#include "CoralBase/Attribute.h"
#include "CoralBase/AttributeList.h"
#include "CoralBase/AttributeListSpecification.h"
#include "Reflex/Type.h"
#include "RelationalAccess/ITableDescription.h"
#include "RelationalStorageServiceNames.h"

pool::RelationalStorageService::PoolPrimitivesContainerManager::PoolPrimitivesContainerManager( int containerId,
                                                                                                coral::ISchema& schema,
                                                                                                const std::string& tableName,
                                                                                                seal::Context* context,
                                                                                                const std::string& serviceName ):
  pool::RelationalStorageService::PoolBaseContainerManager( containerId, pool::RelationalStorageService::RelationalStorageServiceNames::primitivesContainerTypeValueInContainerHeaderTable(), schema, context, serviceName ),
  m_tableName( tableName ),
  m_schema( schema ),
  m_specForInput( 0 ),
  m_inputDataBuffer( 0 ),
  m_specForIdOnReading( 0 ),
  m_idOnReading( 0 ),
  m_whereClauseForPinningId( pool::RelationalStorageService::RelationalStorageServiceNames::idVariableNameInPrimitivesContainerTable() +
                             " = :" + pool::RelationalStorageService::RelationalStorageServiceNames::idVariableNameInPrimitivesContainerTable() ),
  m_whereClauseForLowId( pool::RelationalStorageService::RelationalStorageServiceNames::idVariableNameInPrimitivesContainerTable() +
                         " >= :" + pool::RelationalStorageService::RelationalStorageServiceNames::idVariableNameInPrimitivesContainerTable() ),
  m_typeName( "" ),
  m_specForUpdate( 0 ),
  m_updateDataBuffer( 0 ),
  m_setClause( pool::RelationalStorageService::RelationalStorageServiceNames::dataVariableNameInPrimitivesContainerTable() +
               " =:" + pool::RelationalStorageService::RelationalStorageServiceNames::dataVariableNameInPrimitivesContainerTable() )
{}


pool::RelationalStorageService::PoolPrimitivesContainerManager::~PoolPrimitivesContainerManager()
{
  if ( m_updateDataBuffer ) delete m_updateDataBuffer;
  if ( m_inputDataBuffer ) delete m_inputDataBuffer;
  if ( m_idOnReading ) delete m_idOnReading;
}

std::string
pool::RelationalStorageService::PoolPrimitivesContainerManager::mappingVersion() const{
  return std::string("");
}

std::string
pool::RelationalStorageService::PoolPrimitivesContainerManager::className() const{
  return std::string("");
}

bool
pool::RelationalStorageService::PoolPrimitivesContainerManager::dropContainer()
{
  this->schema().dropIfExistsTable( m_tableName );
  return this->removeContainerEntry();
}


bool
pool::RelationalStorageService::PoolPrimitivesContainerManager::writeObject( const void* data,
                                                                             int oid,
                                                                             pool::DataCallBack& /*callback*/,
                                                                             const std::string& type_name,
                                                                             const ROOT::Reflex::Type& /* dictionary */,
                                                                             pool::DbDatabase* /*databaseHandle*/ )
{
  // Prepare the input data buffer
  if ( ! m_specForInput ) this->prepareInputDataBuffer( type_name );
  else if ( type_name != m_typeName ) return false;

  // Increment the last oid
  this->incrementNumberOfObjectsWritten();

  // Set the values in the data buffer
  m_idAttribute->data<int>() = oid;
  m_dataAttribute->setValueFromAddress( data );

  // Write the object row
  m_schema.tableHandle( m_tableName ).dataEditor().insertRow( *m_inputDataBuffer );
  return true;
}


bool
pool::RelationalStorageService::PoolPrimitivesContainerManager::updateObject( const void* data,
                                                                              int oid,
                                                                              pool::DataCallBack& /*callback*/,
                                                                              const std::string& type_name,
                                                                              const ROOT::Reflex::Type& /*dictionary*/,
                                                                              pool::DbDatabase* /*databaseHandle*/ )
{
  // Prepare the input data buffer
  if ( ! m_updateDataBuffer ) this->prepareUpdateDataBuffer( type_name );
  else if ( type_name != m_typeName ) return false;

  // Set the values in the data buffer
  coral::AttributeList::iterator iDataBuffer = m_updateDataBuffer->begin();
  iDataBuffer->setValueFromAddress( data );
  ++iDataBuffer;
  iDataBuffer->data<int>() = oid;

  // Update the corresponding row
  return ( m_schema.tableHandle( m_tableName ).dataEditor().updateRows( m_setClause, m_whereClauseForPinningId, *m_updateDataBuffer ) != 0 );
}



bool
pool::RelationalStorageService::PoolPrimitivesContainerManager::readObject( void** data,
                                                                            int oid,
                                                                            pool::DataCallBack& /*callback*/,
                                                                            const std::string& type_name,
                                                                            const ROOT::Reflex::Type& /*dictionary*/,
                                                                            pool::DbDatabase* /*databaseHandle*/ )
{
  // Prepare the input data buffer
  this->prepareOutputDataBuffer( type_name );

  // Set the oid
  m_idOnReading->begin()->data<int>() = oid;

  // Create the query
  coral::IQuery* query = m_schema.tableHandle( m_tableName ).newQuery();
  query->addToOutputList( pool::RelationalStorageService::RelationalStorageServiceNames::dataVariableNameInPrimitivesContainerTable() );
  query->defineOutputType( pool::RelationalStorageService::RelationalStorageServiceNames::dataVariableNameInPrimitivesContainerTable(), m_typeName );
  query->setCondition( m_whereClauseForPinningId, *m_idOnReading );

  bool success = true;
  coral::ICursor& cursor = query->execute();
  if ( cursor.next() ) {
    cursor.currentRow().begin()->copyValueToAddress(*data);
  }
  else {
    success = false;
  }

  delete query;

  return success;
}


bool
pool::RelationalStorageService::PoolPrimitivesContainerManager::readAnyObject( void** data,
                                                                               int oid,
                                                                               pool::DataCallBack& /*callback*/,
                                                                               const std::string& type_name,
                                                                               const ROOT::Reflex::Type& /*dictionary*/,
                                                                               pool::DbDatabase* /*databaseHandle*/ )
{
  if ( ! m_inputDataBuffer ) this->prepareInputDataBuffer( type_name );
  else if ( type_name != m_typeName ) return false;
  this->prepareOutputDataBuffer( type_name );

  // Set the oid
  m_idOnReading->begin()->data<int>() = oid;

  // Create the query
  coral::IQuery* query = m_schema.tableHandle( m_tableName ).newQuery();

  query->addToOutputList( pool::RelationalStorageService::RelationalStorageServiceNames::idVariableNameInPrimitivesContainerTable() );
  query->defineOutputType( pool::RelationalStorageService::RelationalStorageServiceNames::idVariableNameInPrimitivesContainerTable(), coral::AttributeSpecification::typeNameForType<int>() );
  
  query->addToOutputList( pool::RelationalStorageService::RelationalStorageServiceNames::dataVariableNameInPrimitivesContainerTable() );
  query->defineOutputType( pool::RelationalStorageService::RelationalStorageServiceNames::dataVariableNameInPrimitivesContainerTable(), m_typeName );
  
  query->setCondition( m_whereClauseForLowId, *m_idOnReading );
  query->addToOrderList( pool::RelationalStorageService::RelationalStorageServiceNames::idVariableNameInPrimitivesContainerTable() );
  query->limitReturnedRows( 1, 0 );

  bool success = true;
  coral::ICursor& cursor = query->execute();
  if ( cursor.next() ) {
    const coral::AttributeList& row = cursor.currentRow();
    coral::AttributeList::const_iterator iAttribute = row.begin();
    oid = iAttribute->data< int >();
    ++iAttribute;
    iAttribute->copyValueToAddress(*data);
  }
  else {
    success = false;
  }

  delete query;
  return success;
}


bool
pool::RelationalStorageService::PoolPrimitivesContainerManager::deleteObject( int oid )
{
  // Increment the number of objects deleted
  this->incrementNumberOfObjectsDeleted();

  if ( ! m_idOnReading ) this->prepareIdForReading();
  m_idOnReading->begin()->data<int>() = oid;
  return ( m_schema.tableHandle( m_tableName ).dataEditor().deleteRows( m_whereClauseForPinningId, *m_idOnReading ) > 0 );
}

bool 
pool::RelationalStorageService::PoolPrimitivesContainerManager::updateContent(const std::vector<std::pair<std::string,std::string> >&, bool, bool){
  return false;
}

bool
pool::RelationalStorageService::PoolPrimitivesContainerManager::existsOid( int oid,
                                                                           int& oidOut,
                                                                           ROOT::Reflex::Type& dictionary )
{
  // Prepare the input data buffer
  if ( ! m_idOnReading ) this->prepareIdForReading();

  // Set the oid
  m_idOnReading->begin()->data<int>() =  oid;

  // Create the query
  coral::IQuery* query = m_schema.tableHandle( m_tableName ).newQuery();
  query->setCondition( m_whereClauseForLowId, *m_idOnReading );
  query->addToOrderList( pool::RelationalStorageService::RelationalStorageServiceNames::idVariableNameInPrimitivesContainerTable() );
  query->limitReturnedRows( 1, 0 );

  coral::ICursor& cursor = query->execute();
  bool success = false;
  if ( cursor.next() ) {
    oidOut = cursor.currentRow().begin()->data<int>();
    success = true;
  }
  
  dictionary = ROOT::Reflex::Type();

  delete query;
  return success;
}


void
pool::RelationalStorageService::PoolPrimitivesContainerManager::prepareInputDataBuffer( const std::string& type_name )
{
  m_specForInput = new coral::AttributeListSpecification;
  m_specForInput->extend<int>( pool::RelationalStorageService::RelationalStorageServiceNames::idVariableNameInPrimitivesContainerTable() );
  m_specForInput->extend( pool::RelationalStorageService::RelationalStorageServiceNames::dataVariableNameInPrimitivesContainerTable(),type_name );
  m_inputDataBuffer = new coral::AttributeList( *m_specForInput );
  coral::AttributeList::iterator iAttribute = m_inputDataBuffer->begin();
  m_idAttribute = &( *iAttribute );
  ++iAttribute;
  m_dataAttribute = &( *iAttribute );
  m_typeName = type_name;
}


void
pool::RelationalStorageService::PoolPrimitivesContainerManager::prepareUpdateDataBuffer( const std::string& type_name )
{
  m_specForUpdate = new coral::AttributeListSpecification;
  m_specForUpdate->extend( pool::RelationalStorageService::RelationalStorageServiceNames::dataVariableNameInPrimitivesContainerTable(), type_name );
  m_specForUpdate->extend<int>( pool::RelationalStorageService::RelationalStorageServiceNames::idVariableNameInPrimitivesContainerTable() );
  m_updateDataBuffer = new coral::AttributeList( *m_specForUpdate );
  m_typeName = type_name;
}


void
pool::RelationalStorageService::PoolPrimitivesContainerManager::prepareOutputDataBuffer( const std::string& type_name )
{
  m_typeName = type_name;
  if ( ! m_idOnReading ) this->prepareIdForReading();
}

void
pool::RelationalStorageService::PoolPrimitivesContainerManager::prepareIdForReading()
{
  m_specForIdOnReading = new coral::AttributeListSpecification;
  m_specForIdOnReading->extend<int>( pool::RelationalStorageService::RelationalStorageServiceNames::idVariableNameInPrimitivesContainerTable() );
  m_idOnReading = new coral::AttributeList( *m_specForIdOnReading );
}
