#include "RelationalPoolPolymorphicContainer.h"
#include "PoolDatabaseManager.h"
#include "PoolBaseContainerManager.h"
#include "StorageSvc/DbSelect.h"
#include "Reflex/Type.h"
#include "POOLCore/Guid.h"
#include "StorageSvc/DbReflex.h"
#include "RelationalAccess/SessionException.h"
#include "SealKernel/MessageStream.h"

pool::RelationalStorageService::RelationalPoolPolymorphicContainer::RelationalPoolPolymorphicContainer( pool::IOODatabase* idb,
                                                                                                        const std::string& name,
                                                                                                        seal::Context* context ):
  pool::RelationalStorageService::RelationalPoolContainer( idb, name, context )
{}


pool::RelationalStorageService::RelationalPoolPolymorphicContainer::~RelationalPoolPolymorphicContainer()
{}


pool::DbStatus
pool::RelationalStorageService::RelationalPoolPolymorphicContainer::fetch( pool::DbSelect& selection )
{
  if ( selection.criteria().length() == 0 || selection.criteria() == "*" )  {
    selection.link().second++;
    pool::RelationalStorageService::PoolBaseContainerManager& containerManager = this->containerManager();

    ROOT::Reflex::Type dictionary;
    try 
    {
      
      if ( containerManager.existsOid( selection.link().second,
                                       selection.link().second,
                                       dictionary ) ) { 
        if(dictionary){
          pool::Guid guid = pool::DbReflex::guid(dictionary);
          selection.setShapeID( guid );
        }
        return pool::Success;
      }
      else {
        return pool::Error;
      }
    }
    catch (const coral::SessionException& e)
    {
      seal::MessageStream log( m_context, m_serviceName );
      log << seal::Msg::Error << e.what() << seal::flush;
      return pool::Error;
    }
    

  }
  return pool::Error;
}


pool::DbStatus
pool::RelationalStorageService::RelationalPoolPolymorphicContainer::fetch( const pool::DbLink& linkH,
                                                                           pool::DbLink& stmt )
{
  return this->RelationalPoolContainer::fetch( linkH, stmt );
}


pool::RelationalStorageService::PoolBaseContainerManager*
pool::RelationalStorageService::RelationalPoolPolymorphicContainer::createContainerManager( pool::RelationalStorageService::PoolDatabaseManager& databaseManager,
                                                                                            const std::string& containerName,
                                                                                            const pool::DbTypeInfo*)
{
  return &( databaseManager.createPolymorphicContainer( containerName ) );
}
