#include <iostream>
#include <stdexcept>
#include "POOLCore/POOLContext.h"

#include "StorageSvc/IStorageSvc.h"
#include "StorageSvc/IStorageSvcFactory.h"

#include "StorageSvc/DbType.h"
#include "StorageSvc/poolDb.h"

#include <iostream>
#include <sstream>
#include <memory>


int main( int, char** ) 
{
  try{
    pool::POOLContext::loadComponent( "SEAL/Services/MessageService" );
    pool::POOLContext::setMessageVerbosityLevel( seal::Msg::Info );
    
    pool::IStorageSvcFactory* fac = pool::IStorageSvcFactory::get();
    if ( ! fac ) {
      throw std::runtime_error( "Could not retrieve a StorageSvcFactory" );
    }
    pool::IStorageSvc* storSvc = fac->create("pool_DbStorageSvc", 0, "StorageSvc");
    if ( ! storSvc ) {
      throw std::runtime_error( "Could not create a StorageSvc object" );
    }
    storSvc->addRef();
    pool::Session* sessionHandle = 0;
    if ( ! ( storSvc->startSession( poolDb::CREATE, pool::POOL_RDBMS_StorageType.type(), sessionHandle ).isSuccess() ) ) {
      throw std::runtime_error( "Could not start a session." );
    }

    std::cout << "Successfully loaded the component" << std::endl;

    storSvc->release();
    std::cout << "exiting..." << std::endl;
  }
  catch( std::exception& e ) {
    std::cerr << e.what() << std::endl;
    return 1;
  }
  catch( ... ) {
    std::cerr << "Funny error" << std::endl;
    return 1;
  }
  return 0;
}
