#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ITransaction.h"
#include "POOLCore/CommandOptions.h"
#include "RelationalStorageService/RelationalPoolDatabaseUtilities.h"
#include <exception>

int main (int argc, char** argv)
{
  std::vector<pool::CommandOptions::Option> ops;
  pool::CommandOptions::Option opt;
  opt.flag = "-n";
  opt.property = "name";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the name of the container to remove";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-c";
  opt.property = "connectionString";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the connection string";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-u";
  opt.property = "user";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user name";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-p";
  opt.property = "password";
  ops.push_back(opt);
  opt.flag = "-full";
  opt.property = "full";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "remove the related data tables and mappings";
  opt.required = false;
  ops.push_back(opt);
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user password";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-debug";
  opt.property = "debug";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the debug verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-info";
  opt.property = "info";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the info verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-h";
  opt.property = "help";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "display the help";
  opt.required = false;
  ops.push_back(opt);
  pool::CommandOptions cmd(ops);
  try{
    cmd.parse(argc,argv);
    const std::map<std::string,std::string>& ops = cmd.userOptions();
    std::map<std::string,std::string>::const_iterator iter = ops.find("help");
    if(iter!=ops.end() || ops.size()==0){
      cmd.help();
    } else {
      pool::RelationalPoolDatabaseUtilities utils;
      iter = ops.find("debug");
      if(iter!=ops.end()){
        utils.setProperty(pool::ObjectRelationalUtilities::verbosityProperty(),
                          pool::ObjectRelationalUtilities::debugVerbosityPropertyValue());
      } else {
        iter = ops.find("info");
        if(iter!=ops.end()){
          utils.setProperty(pool::ObjectRelationalUtilities::verbosityProperty(),
                            pool::ObjectRelationalUtilities::infoVerbosityPropertyValue());
        }
      }
      std::string contName;
      iter = ops.find("name");
      if(iter!=ops.end()){
        contName = iter->second;
      }
      std::string theConnectionString("");
      iter = ops.find("connectionString");
      if(iter!=ops.end()){
        theConnectionString = iter->second;
        utils.setProperty(pool::ObjectRelationalUtilities::connectionStringProperty(),theConnectionString);
      } 
      iter = ops.find("user");
      if(iter!=ops.end()){
        std::string theUserName = iter->second;
        utils.setProperty(pool::ObjectRelationalUtilities::userNameProperty(),theUserName);
      } 
      iter = ops.find("password");
      if(iter!=ops.end()){
        std::string thePassword = (*iter).second;
        utils.setProperty(pool::ObjectRelationalUtilities::passwordProperty(),thePassword);
      }
      bool full = false;
      iter = ops.find("full");
      if(iter!=ops.end()){
        full = true;
      }      
      utils.initialize();
      if(!theVersion.empty()){
        try {
          std::cout << "Dropping container \""<<contName<<"\" from database \""<<theConnectionString<<"\"" << std::endl;
          utils.dropContainer(cntName,full);
          std::cout << "Operation completed. " << std::endl;
        } catch (const std::exception& e){
          std::cout << "ERROR: " << e.what() << std::endl;
          return 1;
        }
      }
    }
    return 0;
  } catch (const std::exception& e){
    std::cout << "ERROR: " << e.what() << std::endl;
    return 1;
  }
}

