#include "POOLCore/CommandOptions.h"
#include "RelationalStorageService/RelationalPoolDatabaseUtilities.h"

#include "SealBase/SharedLibrary.h"
#include "SealBase/SharedLibraryError.h"
#include <exception>

int main (int argc, char** argv)
{
  std::vector<pool::CommandOptions::Option> ops;
  pool::CommandOptions::Option opt;
  opt.flag = "-c";
  opt.property = "connectionString";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the connection string";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-u";
  opt.property = "user";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user name";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-p";
  opt.property = "password";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user password";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-d";
  opt.property = "dictionary";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the dictionary libraries";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-cont";
  opt.property = "container";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the container name";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-dry";
  opt.property = "dryrun";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "only show the changes";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-debug";
  opt.property = "debug";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the debug verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-info";
  opt.property = "info";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the info verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-h";
  opt.property = "help";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "display the help";
  opt.required = false;  
  ops.push_back(opt);
  pool::CommandOptions cmd(ops);
  try{
    cmd.parse(argc,argv);
    const std::map<std::string,std::string>& ops = cmd.userOptions();
    std::map<std::string,std::string>::const_iterator iter = ops.find("help");
    if(iter!=ops.end() || ops.size()==0){
      cmd.help();
    } else {
      pool::RelationalPoolDatabaseUtilities utility;
      bool debug = false;
      iter = ops.find("debug");
      if(iter!=ops.end()){
        debug = true;
        utility.setProperty(pool::ObjectRelationalUtilities::verbosityProperty(),
                            pool::ObjectRelationalUtilities::debugVerbosityPropertyValue());
      }
      if(!debug){
        iter = ops.find("info");
        if(iter!=ops.end()){
          utility.setProperty(pool::ObjectRelationalUtilities::verbosityProperty(),
                              pool::ObjectRelationalUtilities::infoVerbosityPropertyValue());
        }
      }
      iter = ops.find("dictionary");
      if(iter!=ops.end()){
        std::string theDictionaryLibraries = (*iter).second;
        utility.loadDictionaries(theDictionaryLibraries);
      } 

      iter = ops.find("connectionString");
      std::string theConnectionString("");
      if(iter!=ops.end()){
        theConnectionString = (*iter).second;
        utility.setProperty(pool::ObjectRelationalUtilities::connectionStringProperty(),theConnectionString);
      } 
      iter = ops.find("user");
      if(iter!=ops.end()){
        std::string theUserName = (*iter).second;
        utility.setProperty(pool::ObjectRelationalUtilities::userNameProperty(),theUserName);
      } 
      iter = ops.find("password");
      if(iter!=ops.end()){
        std::string thePassword = (*iter).second;
        utility.setProperty(pool::ObjectRelationalUtilities::passwordProperty(),thePassword);
      } 
      std::string contName("");
      iter = ops.find("container");
      if(iter!=ops.end()){
        contName = (*iter).second;
      } 
      bool dry = false;
      iter = ops.find("dryrun");
      if(iter!=ops.end()){
        dry = true;
      } 
      if(contName==""){
        utility.updateDatabase(dry);
      } else {
        utility.updateContainer(contName,dry);
      }
    }
    return 0;
  } catch (const std::exception& e){
    std::cout << "ERROR: " << e.what() << std::endl;
    return 1;
  }
}

