#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ITransaction.h"
#include "POOLCore/Exception.h"
#include "POOLCore/CommandOptions.h"
#include "ObjectRelationalAccess/ObjectRelationalMappingPersistency.h"
#include "RelationalStorageService/RelationalPoolDatabaseUtilities.h"
#include "ObjectRelationalAccess/ObjectRelationalNames.h"
#include "../src/RelationalStorageServiceNames.h"
#include "../src/PoolDatabaseManager.h"

int main (int argc, char** argv)
{
  std::vector<pool::CommandOptions::Option> ops;
  pool::CommandOptions::Option opt;
  opt.flag = "-d";
  opt.property = "dictionary";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the dictionary libraries";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-c";
  opt.property = "connectionString";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the connection string";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-u";
  opt.property = "user";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user name";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-p";
  opt.property = "password";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user password";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-debug";
  opt.property = "debug";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the debug verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-info";
  opt.property = "info";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the info verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-h";
  opt.property = "help";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "display the help";
  opt.required = false;  
  ops.push_back(opt);
  pool::CommandOptions cmd(ops);
  try{
    cmd.parse(argc,argv);
    const std::map<std::string,std::string>& ops = cmd.userOptions();
    std::map<std::string,std::string>::const_iterator iter = ops.find("help");
    if(iter!=ops.end() || ops.size()==0){
      cmd.help();
    } else {
      pool::RelationalPoolDatabaseUtilities utility;
      iter = ops.find("debug");
      if(iter!=ops.end()){
        utility.setProperty(pool::RelationalPoolDatabaseUtilities::verbosityProperty(),
                            pool::RelationalPoolDatabaseUtilities::debugVerbosityPropertyValue());
      } else {
        iter = ops.find("info");
        if(iter!=ops.end()){
          utility.setProperty(pool::RelationalPoolDatabaseUtilities::verbosityProperty(),
                              pool::RelationalPoolDatabaseUtilities::infoVerbosityPropertyValue());
        }
      }
      iter = ops.find("dictionary");
      if(iter!=ops.end()){
        std::string theDictionaryLibraries = (*iter).second;
        utility.loadDictionaries(theDictionaryLibraries);
      } 
      std::string theConnectionString(""); 
      iter = ops.find("connectionString");
      if(iter!=ops.end()){
        theConnectionString = (*iter).second;
      }
      if(theConnectionString.empty()){
        throw pool::Exception("Connection string not provided.", "", "pool_upgrade_pool_schema" );
      }
      
      iter = ops.find("user");
      if(iter!=ops.end()){
        std::string theUserName = (*iter).second;
        utility.setProperty(pool::RelationalPoolDatabaseUtilities::userNameProperty(),theUserName);
      } 
      iter = ops.find("password");
      if(iter!=ops.end()){
        std::string thePassword = (*iter).second;
        utility.setProperty(pool::RelationalPoolDatabaseUtilities::passwordProperty(),thePassword);
      } 
      utility.initialize();
      utility.connect(theConnectionString,coral::Update);
      utility.session()->transaction().start();
      try {
        utility.updatePoolSchema();
        std::cout << "Mapping tables upgraded." << std::endl;
        utility.session()->transaction().commit();
      } catch (const std::exception& e){
        std::cout << "ERROR: " << e.what() << std::endl;
        utility.session()->transaction().rollback();
        return 1;
      }
    }
    return 0;
  } catch (const std::exception& e){
    std::cout << "ERROR: " << e.what() << std::endl;
    return 1;
  }
}

