// $Id: ProcessInfo.h,v 1.1 2003/07/24 16:30:11 rado Exp $
#ifndef POOLBENCH_PROCESSINFO_H
#define POOLBENCH_PROCESSINFO_H 1

#include <cstdio>

namespace pool
{
  
  /** Class @ProcessInfo (so far only Linux is supported)
   * Exploits the /proc filesystem, actually the /proc/PID/statm
   */
  class ProcessInfo
  {      
    public:
      /// Return the total program size of the current process
      static size_t memSize();
      /// Return the program resident set size
      static size_t memRSS();
      /// Return the program's shared segment size
      static size_t memShared();
      /// Return the size program's text(code) segment
      static size_t memText();
      /// Return the size of program's data segment
      static size_t memData();
      /// Return the size of the segment of loaded libraries
      static size_t memLibs();
      /// Return the size of the dirty pages
      static size_t memDirty();
      /// Enable/disable automatic update for each read-out method
      void setAutoUpdate( bool );
      /// Return the current auto-update setting
      bool autoUpdate() const;
      /// Update the memory info snapshot (platform specific)
      static int update();
      
    private:
      /// Update before reading-out the memory snapshots values if needed
      static void updateBeforeRead();
      
    private:
      static bool s_autoUpdate;
  };

  inline bool ProcessInfo::autoUpdate() const {
    return s_autoUpdate;
  }

  inline void ProcessInfo::setAutoUpdate( bool autoUp ) {
    s_autoUpdate = autoUp;
  }
  
  inline void ProcessInfo::updateBeforeRead() {
    if( ProcessInfo::s_autoUpdate ) {
      ProcessInfo::update();
    }
  }
  
} 

#endif // POOLBENCH_PROCESSINFO_H
